/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms;


 

import java.util.LinkedList;
import java.util.List;


import org.biffo.dms.entity.user.Users.UsersAccessType;
import org.biffo.dms.web.administration.AdministrationPage;
import org.biffo.dms.web.config.ConfigPage;
import org.biffo.dms.web.documents.DocumentsPage;
import org.biffo.dms.web.fragments.FragmentsPage;
import org.biffo.dms.web.home.HomePage;
import org.biffo.dms.web.inbox.InboxPage;
 
import org.biffo.dms.web.logs.normal.LogsPage;
import org.biffo.dms.web.modules.ModulesPage;
import org.biffo.dms.web.processes.ProcessesPage;
import org.biffo.dms.web.tags.TagsPage;


public class DmsWebApplicationPages {
	
	public  enum ApplicationPage{
		HOME(0,"Home", UsersAccessType.USER, HomePage.class, "&#xe88a;"), 
		INBOX(1,"Inbox", UsersAccessType.USER, InboxPage.class, "&#xe156;"), 
		DOCUMENTS(2,"Dokumente", UsersAccessType.USER, DocumentsPage.class, "&#xe873;"),
		TAGS(4,"Tags", UsersAccessType.USER, TagsPage.class, "&#xf05b;"), 
		ARCHIVE(5,"Archive", UsersAccessType.USER, DocumentsPage.class, "&#xe1a1;"),
		DELETED(6,"Deleted", UsersAccessType.USER, DocumentsPage.class, "&#xe872;"),
		FRAGMENTS(31,"Fragments", UsersAccessType.USER, FragmentsPage.class, "&#xf8ef;"),
		SPACER(8,null, UsersAccessType.USER, null, null),
		ADMINSTRATION(9,"Administration", UsersAccessType.ADMIN, AdministrationPage.class, "&#xf02e;"),
		PROCESSES(10,"Processes", UsersAccessType.ADMIN, ProcessesPage.class, "&#xe7ef;"),
		LOGS(30,"Logfile", UsersAccessType.USER, LogsPage.class, "&#xf071;"), 
		MODULES(31,"Modules", UsersAccessType.ADMIN, ModulesPage.class, "&#xe8f0;"),
		CONFIG(50,"Config", UsersAccessType.ADMIN, ConfigPage.class, "&#xe8b9;"),
		
		LOGOUT(99,"Logout", UsersAccessType.USER, null, "&#xe9ba;"),
		;
		
		private final int _code;
	    private String _description;
	    private final UsersAccessType _usersAccessType;
	    private final Class<?> _class;
	    private final String _image;

	    ApplicationPage(int code, String description, UsersAccessType accessType, Class<?> clazz, String image){
	    	 _code = code;
	    	 _description = description;
	    	 _usersAccessType = accessType;
	    	 _class = clazz;
	    	 _image = image;
	    	 
	    }
	    public String getDescription(){
	    	return _description;
	    }
	    public void setDescription(String description){
	    	 _description = description;
	    }

	    
	    public int getCode(){
	    	return _code;
	    }
	    public UsersAccessType getAccessType(){
	    	return _usersAccessType;
	    }
	    public Class<?> getWebPageClass(){
	    	return _class;
	    }
	    public String getImage() {
	    	return _image;
	    }
	    
	}
	
	
	public static List<String> getApplicationPages(){
		List<String> lstRC = new LinkedList<String>();
		for(ApplicationPage applicationPage :ApplicationPage.values()){
			lstRC.add(applicationPage.getDescription());
		}
		return lstRC;
	}
	
	
}
