/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms;

import java.io.Serializable;
import java.util.Date;

import org.biffo.dms.Types.EntityType;
import org.biffo.dms.entity.IEntity;
import org.biffo.dms.entity.Document.Document;
import org.biffo.dms.entity.Fragments.Fragment;
import org.biffo.dms.entity.Logentries.LogEntry;
import org.biffo.dms.entity.Logentries.StatusType;
import org.biffo.dms.entity.Tags.Tag;
import org.biffo.dms.entity.Tasks.Task;
import org.biffo.dms.entity.Variables.Variable;
import org.biffo.dms.entity.Variables.Impl.ImportServerImpl;
import org.biffo.dms.entity.user.Tenant.Tenant;
import org.biffo.dms.entity.user.Usergroups.UserGroup;
import org.biffo.dms.entity.user.Users.User;
import com.google.gson.Gson;

 
 

public class DmsWebMessageHandler implements Serializable{

	
	private static final long serialVersionUID = 1L;
	
	private DmsProcessingMapper dmp;
	
	public DmsWebMessageHandler(DmsProcessingMapper dmp) {
		this.dmp = dmp;
	}
	
	public void setProcessingMapper(DmsProcessingMapper dmp) {
		this.dmp = dmp;
	}
	public void createMessage(DmsException dmsException, DmsWebSession session) {
		
		LogEntry ule = new LogEntry();
		if(null==session) ule.setIdUser(99999);
        else ule.setIdUser(session.getCurrentUserId());
        ule.setIdProcess(-1);
        ule.setMessageCode(dmsException.getCode());
        
        ule.setClassName(dmsException.getClassName());
        ule.setFunctionName(dmsException.getFunctionName());
        
        ule.setErstellungsDatum(new Date());
        if(null==session)ule.setErstelltVon("System");
        else ule.setErstelltVon(session.getCurrentUser());
        ule.setAenderungsDatum(ule.getErstellungsDatum());
        ule.setGeaendertVon(ule.getErstelltVon());
        
        
        
        if(dmsException.getCategory().toLowerCase().equals("w")) {
        	ule.setStatusType(StatusType.WARN);
        }else if(dmsException.getCategory().toLowerCase().equals("e")) {
        	ule.setStatusType(StatusType.ERROR);
        }else if(dmsException.getCategory().toLowerCase().equals("i")) {
        	ule.setStatusType(StatusType.INFO);
        }
        
        ule.setEntityType(getEntityTypeFromLogObject(dmsException.getLogObject()));
        
        if(ule.getEntityType()!=EntityType.UNKNOWN) {
        	if(ule.getEntityType()==EntityType.DOCUMENTS) {
        		IEntity[] entities = (IEntity[])dmsException.getLogObject();
        		ule.setIdEntity(entities[0].getId());
        	}else if(ule.getEntityType()==EntityType.USER) {
        		User user = (User)dmsException.getLogObject();
        		ule.setIdEntity(user.getId());
        	}else if(ule.getEntityType()==EntityType.SERVERS) {
        		ImportServerImpl server = (ImportServerImpl)dmsException.getLogObject();
        		ule.setIdEntity(server.getId());
        	}else {
        		IEntity entity = (IEntity)dmsException.getLogObject();
        		ule.setIdEntity(entity.getId());
        	}
        }
        
        ule.setLogText1(dmsException.getMessage());
        if(dmsException.getLogObject()!=null)ule.setLogObject(new Gson().toJson(dmsException.getLogObject()));
        if(null!=dmsException.getExceptionobject()) {
        	ule.setException(dmsException.getExceptionobject().toString());
        }

        try {
        	dmp.getLogDB().addEntity(ule);
        }catch(Exception e) {
        } 
	}
	
	
	private EntityType getEntityTypeFromLogObject(Object logObject) {
		if(logObject instanceof Document)return EntityType.DOCUMENT;
		if(logObject instanceof Document[]) return EntityType.DOCUMENTS;
		
		if(logObject instanceof Tag) return EntityType.TAG;
   		if(logObject instanceof User) return EntityType.USER;
   		if(logObject instanceof ImportServerImpl) return EntityType.SERVERS;
   		
   		if(logObject instanceof Fragment) return EntityType.FRAGMENT;
   		if(logObject instanceof Task) return EntityType.TASKS;
   		if(logObject instanceof Tenant) return EntityType.TENANT;
   		if(logObject instanceof Variable) return EntityType.VARIABLE;
   		if(logObject instanceof UserGroup) return EntityType.USERGROUP;
		
		return EntityType.UNKNOWN;
	}
	
	 
	
	 
	
	public void createError(String message, DmsWebSession session, Exception exception) {
		
		LogEntry ule = new LogEntry();
        ule.setIdUser(session.getCurrentUserId());
        ule.setIdProcess(-1);
        ule.setStatusType(StatusType.ERROR);
        ule.setErstellungsDatum(new Date());
        ule.setErstelltVon(session.getCurrentUser());
        ule.setAenderungsDatum(ule.getErstellungsDatum());
        ule.setGeaendertVon(ule.getErstelltVon());
        ule.setLogText1(message);
        if(null!=exception) ule.setException(exception.toString());
        try {
        	dmp.getLogDB().addEntity(ule);
        }catch(Exception e) {
        } 
	}
	
}
