/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms;


import org.apache.wicket.authroles.authentication.AuthenticatedWebSession;
import org.apache.wicket.authroles.authorization.strategies.role.Roles;
import org.apache.wicket.request.Request;
import org.biffo.dms.entity.Relation.RelationFilterClass;
import org.biffo.dms.entity.Relation.RelationType;
import org.biffo.dms.entity.user.Users.User;
import org.biffo.dms.entity.user.Users.UsersAccessType;
import org.biffo.dms.session.ILocale;
import org.biffo.dms.session.parameters.DmsWebSessionParameters;
import org.biffo.dms.session.parameters.DocumentViewParameters;
import org.biffo.dms.session.parameters.SessionParameterTypes;
import org.biffo.dms.user.UserHandler;
import org.biffo.dms.user.UserLogObject;

public class DmsWebSession extends AuthenticatedWebSession {
 
	private static final String _className = "DmsWebSession";
	private static final long serialVersionUID = 1L;
	private final UserHandler _uh = new UserHandler();
	
	private String _currentUser=null;
	private int _userId=-1;
	private UsersAccessType _usersAccessType=UsersAccessType.USER;
	
	private int _errorCode;
	private UserLogObject _errorObject;
	private int _tenantId=-1;
	private String _sessionId;
	
	private DmsWebSessionParameters _sessionParameters = new DmsWebSessionParameters(-1);

	public DmsWebSession(Request request) {
		super(request);
		 
	}
	
	@Override
	public void onInvalidate() {
		super.onInvalidate();
	}
	
		
	
	
	@Override 
	public boolean authenticate(String username, String password) { 
		_sessionId = getSession().getId();
		
		if(DmsWebApplication.getApplication().isNoConfig()) {
			if(username!=null && password!=null && username.equals("setup") && password.equals("setup")) {
				return true;
			}
		}else {
			try{
				User user = _uh.getUser(username, password, getClientInfo().getProperties().getRemoteAddress(), _sessionId);
				_currentUser = user.getBenutzerName();
				_usersAccessType = user.getAccessType();
				_userId = user.getId();
				
				if(user.getSessionParameters()!=null) {
					_sessionParameters = new DmsWebSessionParameters(_userId,(DmsWebSessionParameters) user.getSessionParameters());
				}else {
					_sessionParameters = new DmsWebSessionParameters(_userId);
				}
				_sessionParameters.setUsersRestrictions(user);
				return true;
			
			}catch(DmsException e){
				if(e.getLogObject()!=null && e.getLogObject() instanceof UserLogObject ) {
					_errorObject = (UserLogObject)e.getLogObject();
				}
				_errorCode = e.getCode();
			}
		}
		return false;
	}

	@Override 
	public Roles getRoles() { 
		return null; 
	}
	
	public String getCurrentUser() {return _currentUser;}
	public int getCurrentUserId() {return _userId;}
	
	public int getCurrentTenantId() {return _tenantId;}

	public UsersAccessType getUserAccessType(){return _usersAccessType;}
	
	public static DmsWebSession getSession(){
		return (DmsWebSession)get();
	}

	public int getErrorCode() {
		return _errorCode;
	}
	public UserLogObject getErrorLogObject() {
		return _errorObject;
	}
	
	public ILocale getLocaleParams() {
		try {
			Class<?> act = Class.forName("org.biffo.dms.session.de_DE");
			Object object = act.getDeclaredConstructor().newInstance();
			return (ILocale) object;
		} catch (Exception e) {
			DmsException dmsException = new DmsException(
				1000, 
				"E", 
				_className, 
				"getLocaleParams",
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsException, this);
		}
		return null;
	}
	
	public void clearSearchParameters() {
		if(null!=getSessionParameters().getParametersByType(SessionParameterTypes.DOCUMENT_PDF_VIEW)) {
			((DocumentViewParameters)getSessionParameters().getParametersByType(SessionParameterTypes.DOCUMENT_PDF_VIEW)).setActualViewParams(null);
		}
	}
	
	
	public boolean isDocumentAvailable(int documentId) {
		
		if(_usersAccessType==UsersAccessType.ADMIN) return true;
		try {
			RelationFilterClass rfc = new RelationFilterClass();
			rfc.setRelationType(RelationType.DOCUMENT_TO_USER);
			rfc.setIdrelation1(documentId);
			rfc.setIdrelation2(_userId);
			if(StaticLocators.getRelations().getNumberOfEntities(rfc)>0)return true;
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
				1000, 
				"E", 
				_className, 
				"isDocumentAvailable",
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsException, this);
		}
		return false;
	}

	public DmsWebSessionParameters getSessionParameters() {return _sessionParameters;}
	
	
}
