/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.auth;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.form.AjaxSubmitLink;
import org.apache.wicket.markup.html.WebPage;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.PasswordTextField;
import org.apache.wicket.markup.html.form.StatelessForm;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.util.string.Strings;
import org.biffo.dms.DmsWebApplication;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.web.helpers.ErrorLabel;
import org.biffo.dms.web.helpers.ResourceText;

public class SignInPage extends WebPage{

	private static final long serialVersionUID = 1L;
	
	private String username, password;
	private TextField<String> txtUsername;
	private PasswordTextField txtPpassword;
	private ErrorLabel _username_info, _password_info;
	
	private int _errorCode;
	
	
	public SignInPage() {
		super();
	}
	
	public SignInPage(PageParameters parameters) {
        super(parameters);
	}
	
	@Override 
	protected void onInitialize() { 
	
		super.onInitialize();
		setVersioned(false);
		setOutputMarkupId(true);
		
		StatelessForm<Void> form = new StatelessForm<Void>("form"){
			
			private static final long serialVersionUID = 1L;
			@Override 
			protected void onSubmit() {
				_errorCode=0;
				boolean authResult= DmsWebSession.getSession().signIn(username, password);
				if(authResult){
					setResponsePage(DmsWebApplication.get().getHomePage());
					//Page responsePage = RedirectPageHelpers.getRedirectPage(getPageParameters());
					/*if(null==responsePage) {
						setResponsePage(MainApplication.get().getHomePage());
					}else {
						setResponsePage(responsePage);
					}*/
				}else {
					_errorCode = DmsWebSession.getSession().getErrorCode();
				}
			 }
		 };
		 
		 form.setDefaultModel(new CompoundPropertyModel<SignInPage>(this));
		 
		 txtUsername = new TextField<String>("username");
		 txtUsername.setOutputMarkupId(true);
		 txtUsername.setRequired(false);
		 txtUsername.add(new AttributeModifier("autofocus",""));
		 form.add(txtUsername);
		 
		 _username_info = new ErrorLabel("username_info");
	     form.add(_username_info);
		 
		 txtPpassword = new PasswordTextField("password");
		 txtPpassword.setRequired(false);
		 txtPpassword.setOutputMarkupId(true);
		 form.add(txtPpassword);
		 
		 _password_info = new ErrorLabel("password_info");
	     form.add(_password_info);
		 
		 final AjaxSubmitLink submit = new AjaxSubmitLink("submit", form) {

			private static final long serialVersionUID = 1L;
			@Override
			protected void onSubmit(AjaxRequestTarget target, Form<?> form) {
				super.onSubmit(target, form);
			}
			
			@Override
			protected void onAfterSubmit(AjaxRequestTarget target, Form<?> form) {
				
				checkShowError(target);
				target.add(form);
				target.focusComponent(txtUsername);
			}
		 };
		 form.add(submit);
		 form.setDefaultButton(submit);
		 
		 Label lblVersion = new Label("version", "v"+DmsWebApplication.getApplication().getApplicationVersion());
		 form.add(lblVersion);
		 
		 
		 add(form); 
	}
	
	private boolean checkShowError(AjaxRequestTarget target) {
		
		boolean error=false;
		
		if(Strings.isEmpty(username)){
			_username_info.setErrorText(ResourceText.getText("errorValueUsername", this));
			_username_info.setVisible(true);
			error = true;
		}else {
			_username_info.setVisible(false);
		}
		
		if(Strings.isEmpty(password)){
			_password_info.setErrorText(ResourceText.getText("errorValuePassword", this));
			_password_info.setVisible(true);
			error = true;
		}else {
			_password_info.setVisible(false);
		}
		
		if(_errorCode!=0 && _errorCode!=3009) {
			
			if(_errorCode==3011) {
				_password_info.setErrorText(ResourceText.getText(""+_errorCode, this));
				_password_info.setVisible(true);
			}else {
				_username_info.setErrorText(ResourceText.getText(""+_errorCode, this));
				_username_info.setVisible(true);
			}
			error = true;
		}
		
		target.add(_username_info);
		target.add(_password_info);		
		
		return error;
		
	}
	
	
}
