/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
*
*	ColorPicker Component for Tag Color Selection 
*	The ColorPicker is based on the work of:
*	Wheel Color Picker for jQuery
*
*	https://raffer.one/projects/jquery-wheelcolorpicker
*
*	Author : Fajar Chandra
*	Date   : 2019.11.05
*
*	Copyright © 2011-2019 Fajar Chandra. All rights reserved.
*	Released under MIT License.
*	http://www.opensource.org/licenses/mit-license.php
*
**********************************************************************/
package org.biffo.dms.components.colorpicker;


import org.apache.wicket.Component;
import org.apache.wicket.ajax.AbstractDefaultAjaxBehavior;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.ComponentTag;
import org.apache.wicket.markup.head.CssReferenceHeaderItem;
import org.apache.wicket.markup.head.IHeaderResponse;
import org.apache.wicket.markup.head.JavaScriptReferenceHeaderItem;
import org.apache.wicket.markup.head.OnDomReadyHeaderItem;
import org.apache.wicket.markup.html.WebComponent;
import org.apache.wicket.model.IModel;
import org.apache.wicket.request.cycle.RequestCycle;
import org.apache.wicket.request.resource.CssResourceReference;
import org.apache.wicket.request.resource.JavaScriptResourceReference;

public class ColorPicker extends WebComponent{

	private static final JavaScriptResourceReference COLORPICKER_JS = new JavaScriptResourceReference(ColorPicker.class, "jquery.wheelcolorpicker.js");
	private static final CssResourceReference COLORPICKER_CSS = new CssResourceReference(ColorPicker.class, "wheelcolorpicker.css");
 
	private static final long serialVersionUID = 1L;
	private String color;
	
	@Override
	public void renderHead(IHeaderResponse response) {
    	super.renderHead(response);

    	response.render(OnDomReadyHeaderItem.forScript("$('#"+getMarkupId()+"').wheelColorPicker({live: true,preview: true});"));
    	if(color!=null) {
    		response.render(OnDomReadyHeaderItem.forScript("$('#"+getMarkupId()+"').wheelColorPicker('color', '#"+color+"');"));
    	}
    	
    	StringBuilder sb = new StringBuilder();
    	sb.append("$('#"+getMarkupId()+"').on('change', function(e){");
		sb.append("var color = $("+getMarkupId()+").wheelColorPicker('getValue');");
		sb.append("Wicket.Ajax.get({ u: cb"+getMarkupId()+", ep: {'event': 'change', 'value': color }});");
		sb.append("});");
		response.render(OnDomReadyHeaderItem.forScript(sb.toString()));
    	
		response.render(JavaScriptReferenceHeaderItem.forReference(COLORPICKER_JS));
    	response.render(CssReferenceHeaderItem.forReference(COLORPICKER_CSS));
    	
	}
	
	@Override
	protected void onComponentTag(final ComponentTag tag) {
		
		checkComponentTag(tag, "input");
		if(!isEnabled())tag.put("disabled","disabled");
		super.onComponentTag(tag);
	}
	
	protected class ColorPickerAjaxBehavior extends AbstractDefaultAjaxBehavior{
 
		private static final long serialVersionUID = 1L;
		
		
        @Override
        public void renderHead(Component component, IHeaderResponse response){
            super.renderHead(component, response);
            String callbackUrl = getCallbackUrl().toString();
            String callbackVar = "cb"+ColorPicker.this.getMarkupId();
            response.render(OnDomReadyHeaderItem.forScript(callbackVar+" = '" + callbackUrl + "';"));
        }

		@Override
		protected void respond(final AjaxRequestTarget target){
			RequestCycle requestCycle = RequestCycle.get();
			String event = requestCycle.getRequest().getRequestParameters().getParameterValue("event").toString(null);
			if(event!=null) {
				if(event.equals("change")) {
					String value = requestCycle.getRequest().getRequestParameters().getParameterValue("value").toString(null);
					color = value;
					ColorPicker.this.setDefaultModelObject(value);
					OnChange(target, event, value);
				}
			}
		}		
	}

	
	public ColorPicker(String id) {
		this(id, null);
	}

	public ColorPicker(final String id, final IModel<?> model) {
		super(id, model);
		setOutputMarkupId(true);
		color = getDefaultModelObjectAsString();
		add(new ColorPickerAjaxBehavior());
	}
	
	protected void OnChange(AjaxRequestTarget target, String event, String value){}

	public String getColor() {
		return color;
	}

}
