/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.components.custommodal;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.ajax.markup.html.modal.ModalWindow;
import org.apache.wicket.markup.head.CssReferenceHeaderItem;
import org.apache.wicket.markup.head.IHeaderResponse;
import org.apache.wicket.markup.head.JavaScriptReferenceHeaderItem;
import org.apache.wicket.request.resource.CssResourceReference;
import org.apache.wicket.request.resource.JavaScriptResourceReference;
import org.apache.wicket.request.resource.ResourceReference;


public class CustomModalWindow extends ModalWindow{
	  
	private static final long serialVersionUID = 1L;
	private static ResourceReference CSS = new CssResourceReference(CustomModalWindow.class, "custom-modal.css");
	private static ResourceReference JAVASCRIPT = new JavaScriptResourceReference(CustomModalWindow.class, "custom-modal.js");
	  
	  
	@Override
	public void renderHead(IHeaderResponse response) {
		super.renderHead(response);
		response.render(JavaScriptReferenceHeaderItem.forReference(JAVASCRIPT));
    	response.render(CssReferenceHeaderItem.forReference(CSS));
	}
	
	@Override
	protected void onBeforeRender() {
		super.onBeforeRender();
	}
	
	
	
	public CustomModalWindow(String id){
	    
		super(id);
	    setCssClassName("custom");
	    
	    setCloseButtonCallback(new CloseButtonCallback(){

	    	private static final long serialVersionUID = 1L;

			public boolean onCloseButtonClicked(AjaxRequestTarget target){
	    		return onCloseClicked(target);
	    	}
	    });
	    
	    setWindowClosedCallback(new WindowClosedCallback(){

			private static final long serialVersionUID = 1L;

			public void onClose(AjaxRequestTarget target){
	    		CustomModalWindow.this.onClose(target);
	    	}
	    });
	  }
	  
	  protected void onClose(AjaxRequestTarget target){}
	  protected boolean onCloseClicked(AjaxRequestTarget target){
		  return true;
	  }
}
