/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
*
*	DatePicker Component
* 	This Component is based on the work of
* 
*	Datepicker v1.0.10
*	https://fengyuanchen.github.io/datepicker
*
* 	Copyright 2014-present Chen Fengyuan
* 	Released under the MIT license
*
* 	Date: 2020-09-29T14:46:10.983Z
*
*
**********************************************************************/
package org.biffo.dms.components.datepicker;

 
import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.Date;
 

import org.apache.wicket.Component;
import org.apache.wicket.ajax.AbstractDefaultAjaxBehavior;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.head.CssReferenceHeaderItem;
import org.apache.wicket.markup.head.IHeaderResponse;
import org.apache.wicket.markup.head.JavaScriptHeaderItem;
import org.apache.wicket.markup.head.JavaScriptReferenceHeaderItem;
import org.apache.wicket.markup.head.OnDomReadyHeaderItem;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.cycle.RequestCycle;
import org.apache.wicket.request.resource.CssResourceReference;
import org.apache.wicket.request.resource.JavaScriptResourceReference;

/**

*/
public class DatePicker extends   Panel{
	
	private static final JavaScriptResourceReference DATETIMEPICKER_JS = new JavaScriptResourceReference(DatePicker.class, "datepicker.min.js");
	private static final CssResourceReference DATETIMEPICKER_CSS = new CssResourceReference(DatePicker.class, "datepicker.css");
	
	private static final long serialVersionUID = 1L;
	
	private TextField<String> _dateTimePicker;
	private Label _clearbtn;
	private boolean _withButtonClear;
	private final DatePickerOptions datePickerOptions = new DatePickerOptions();
	public DatePickerOptions getDatePickerOptions(){return datePickerOptions;}
	
	private String _componentId;
	private boolean active=true;
	
	private class DateHelper implements Serializable{
 
		private static final long serialVersionUID = 1L;
		private SimpleDateFormat sdf;
		private Date date;
		private String dateFormat="MM/dd/yyyy";
		private String datePattern;
		
		public DateHelper() {
			sdf = new SimpleDateFormat(dateFormat);
			datePattern = getPattern(dateFormat);
		}

		@SuppressWarnings("unused")
		public String getDateFormat() {return dateFormat;}
		public void setDateFormat(String dateFormat) {
			sdf = new SimpleDateFormat(dateFormat);
			this.dateFormat = dateFormat;
			datePattern = getPattern(this.dateFormat);
		}
		
		public boolean isValidStringDate(String input) {
			if(null!=input && !input.isEmpty()) {
				try {
					if(input.matches(datePattern)) {
						sdf.parse(input);
						return true;	
					}
					return false;
				}catch(Exception e) {
					return false;
				}
			}
			return false;
		}

		
		public void setStringDate(String input){
			if(null!=input && !input.isEmpty()) {
				try {
					date = sdf.parse(input);
				}catch(Exception e) {
				}
			}else {
				date = null;
			}
			
		}

		public String getStringDate(){
			if(null!=date) return sdf.format(date);
			return null;
		}

		
		public void setDate(Date date){this.date = date;}
		public Date getDate(){return date;}
		
		public String toString() {
			return getStringDate();
		}
		
		private String getPattern(String format) {
			
			format=format.replaceAll("dd", "\\\\d{1,2}");
			format=format.replaceAll("MM", "\\\\d{1,2}");
			format=format.replaceAll("yyyy", "\\\\d{4}");
			format=format.replaceAll("\\.", "\\\\.");
			format=format.replaceAll("/", "\\/");
			return format;
		}
		
	}
	
	
	@Override
	protected void onBeforeRender() {
		if(null!=datePickerOptions.getFormat())dh.setDateFormat(datePickerOptions.getFormat());
		if(active) {
			_dateTimePicker.setEnabled(true);
			if(_withButtonClear)_clearbtn.setVisible(true);
		}else {
			_dateTimePicker.setEnabled(false);
			if(_withButtonClear)_clearbtn.setVisible(false);
		}
		
		super.onBeforeRender();
	}
	
	@Override
	public void renderHead(IHeaderResponse response) {
    	
		super.renderHead(response);
		String options = datePickerOptions.toString();
		StringBuilder sb = new StringBuilder();
    	sb.append("$('#"+_componentId+"').datepicker(");
    	if(null!=options)sb.append(options);
    	sb.append(");");
    	
    	if(dh.getDate()==null) {
    		sb.append("$('#"+_componentId+"').datepicker('setDate', null);");
    	}else {
    		sb.append("$('#"+_componentId+"').datepicker('setDate', '"+dh.getStringDate()+"');");
    	}
		response.render(OnDomReadyHeaderItem.forScript(sb.toString()));
		
		
		sb = new StringBuilder();
		sb.append("$('#"+_componentId+"').on('pick.datepicker', function (e) {");
		sb.append("var datum = $('#"+_componentId+"').datepicker('formatDate', e.date);");
		sb.append("Wicket.Ajax.get({ u: cb"+_componentId+", ep: {'event': 'change', 'value': JSON.stringify(datum) }});");
		sb.append("});");
    	response.render(OnDomReadyHeaderItem.forScript(sb.toString()));
		
    	sb = new StringBuilder();
		sb.append("$('#"+_componentId+"').on('keyup.datepicker', function (e) {");
		sb.append("var datum = $('#"+_componentId+"').datepicker('getValue');");
		sb.append("Wicket.Ajax.get({ u: cb"+_componentId+", ep: {'event': 'keyup', 'value': JSON.stringify(datum) }});");
		sb.append("});");
    	response.render(OnDomReadyHeaderItem.forScript(sb.toString()));
    	
    	sb = new StringBuilder();
    	sb.append("$('#"+_componentId+"').on('focusout', function() {");
		sb.append("var datum = $('#"+_componentId+"').datepicker('getValue', true);");
    	sb.append("Wicket.Ajax.get({ u: cb"+_componentId+", ep: {'event': 'change', 'value': JSON.stringify(datum) }});");
    	sb.append("});");
    	response.render(OnDomReadyHeaderItem.forScript(sb.toString()));
    	if(_withButtonClear) {
	    	sb = new StringBuilder();
			sb.append("$('#"+_clearbtn.getMarkupId()+"').on('touchstart click', function (e) {");
			sb.append("$('#"+_componentId+"').datepicker('clear');");
			sb.append("Wicket.Ajax.get({ u: cb"+_componentId+", ep: {'event': 'clear', 'value': null }});");
			sb.append("});");
	    	response.render(OnDomReadyHeaderItem.forScript(sb.toString()));
    	}
		response.render(JavaScriptReferenceHeaderItem.forReference(DATETIMEPICKER_JS));
    	response.render(CssReferenceHeaderItem.forReference(DATETIMEPICKER_CSS));
    	
    	
	}
	 
	private DateHelper dh;
	
	public DatePicker(String id, IModel<Date> model) {
		this(id, model, true);
	}
	
	public DatePicker(String id, IModel<Date> model, boolean withButtonClear) {
		super(id, model);
		setOutputMarkupId(true);
		
		_withButtonClear = withButtonClear;
		dh = new DateHelper();
		dh.setDate(model.getObject());
		
		_dateTimePicker = new TextField<String>("datetimeinput", new PropertyModel<String>(dh, "StringDate"));
		_dateTimePicker.setOutputMarkupId(true);
		_componentId=_dateTimePicker.getMarkupId();
		add(_dateTimePicker);
		
		_clearbtn = new Label("clearbtn", "&times;");
		_clearbtn.setEscapeModelStrings(false);
		_clearbtn.setOutputMarkupId(true);
		if(!withButtonClear)_clearbtn.setVisible(false);
		add(_clearbtn);
		
		if(active) {
			_dateTimePicker.setEnabled(true);
			_clearbtn.setEnabled(true);
		}else {
			_dateTimePicker.setEnabled(false);
			_clearbtn.setEnabled(false);
		}
		
		add(new DateTimePickerAjaxBehavior());
	}
	
	
	protected class DateTimePickerAjaxBehavior extends AbstractDefaultAjaxBehavior{
		 
		private static final long serialVersionUID = 1L;
		
        @Override
        public void renderHead(Component component, IHeaderResponse response){
            super.renderHead(component, response);
            String callbackUrl = getCallbackUrl().toString();
            String callbackVar = "cb"+DatePicker.this._componentId;
            response.render(OnDomReadyHeaderItem.forScript(callbackVar+" = '" + callbackUrl + "';"));
            response.render(JavaScriptHeaderItem.forScript("var cb"+_componentId+";", "dt"+_componentId));
        }

        @Override
		protected void respond(final AjaxRequestTarget target){
			RequestCycle requestCycle = RequestCycle.get();
			String event = requestCycle.getRequest().getRequestParameters().getParameterValue("event").toString(null);
			boolean process=true;
			if(event!=null) {
				String value = requestCycle.getRequest().getRequestParameters().getParameterValue("value").toString(null);
				if(value!=null && !value.isEmpty() && !value.equals("\"\"")) {
					value = value.replaceAll("\"", "").trim();
				}else {
					process=false;
				}
				if(event.equals("change") && process) {
					if(dh.isValidStringDate(value)) {
						dh.setStringDate(value);
						setDefaultModelObject(dh.getDate());
						OnChange(target, dh.getDate());
					}
				}else if(event.equals("keyup") && process) {
					if(dh.isValidStringDate(value)) {
						dh.setStringDate(value);
						setDefaultModelObject(dh.getDate());
					}else {
						dh.setStringDate(null);
						setDefaultModelObject(null);
					}

				}else if(event.equals("clear")) {
					dh.setStringDate(null);
					setDefaultModelObject(null);
					OnClear(target, dh.getDate());
				}
			}
		}	
	}
	
	@Override
	protected void onModelChanged() {
		try {
			dh.setDate((Date)getDefaultModelObject());
		}catch(Exception e) {
		}
		super.onModelChanged();
	}
 	
	public Date getDate() {
		return dh.getDate();
	}
	
	
	public void clearDate(final AjaxRequestTarget target) {
		dh.setStringDate(null);
		setDefaultModelObject(dh.getDate());
		if(null!=target)OnClear(target, dh.getDate());
	}

	public void show(final AjaxRequestTarget target) {
		target.prependJavaScript("cb"+_componentId+".show();");
	}

	public void hide(final AjaxRequestTarget target) {
		target.prependJavaScript("$('#"+_componentId+"').datepicker('hide');");
	}

	public boolean isActive() {return active;}
	public void setActive(boolean active) {this.active = active;}
	
	protected void OnChange(AjaxRequestTarget target, Date value){}
	protected void OnClear(AjaxRequestTarget target, Date value){}

}
