/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.components.datepicker;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.List;

import org.apache.wicket.ajax.json.JSONObject;
import org.apache.wicket.ajax.json.JsonFunction;

public class DatePickerOptions extends JSONObject implements Serializable{
	 
	
	private static final long serialVersionUID = 1L;

	private Boolean autoShow;
	private Boolean autoHide;
	private Boolean inline;
	private Boolean autoPick;
	private String language;
	private String format;
	
	private String date;
	private String startDate;

	private String endDate;
	private Integer startView;
	
	private Integer weekStart;
	private Boolean yearFirst;
	private String yearSuffix;
	
	private List<Object> days;
	private List<Object> daysShort;
	private List<Object> daysMin;
	private List<Object> months;
	private List<Object> monthsShort;
	
	
	public DatePickerOptions() {}
	
	public Boolean getAutoShow() {return autoShow;}
	public void setAutoShow(Boolean autoShow) {this.autoShow = autoShow;}
	
	public Boolean getAutoHide() {return autoHide;}
	public void setAutoHide(Boolean autoHide) {this.autoHide = autoHide;}
	
	public Boolean getInline() {return inline;}
	public void setInline(Boolean inline) {this.inline = inline;}
	
	public Boolean getAutoPick() {return autoPick;}
	public void setAutoPick(Boolean autoPick) {this.autoPick = autoPick;}
	
	public String getLanguage() {return language;}
	public void setLanguage(String language) {this.language = language;}
	
	public String getFormat() {return format;}
	public void setFormat(String format) {this.format = format;}
	
	public String getDate() {return date;}
	public void setDate(String date) {this.date = date;}
	
	public String getStartDate() {return startDate;}
	public void setStartDate(String startDate) {this.startDate = startDate;}
	
	public String getEndDate() {return endDate;}
	public void setEndDate(String endDate) {this.endDate = endDate;}
	
	public Integer getStartView() {return startView;}
	public void setStartView(Integer startView) {this.startView = startView;}
	
	public Integer getWeekStart() {return weekStart;}
	public void setWeekStart(Integer weekStart) {this.weekStart = weekStart;}
	
	public Boolean getYearFirst() {return yearFirst;}
	public void setYearFirst(Boolean yearFirst) {this.yearFirst = yearFirst;}
	
	public String getYearSuffix() {return yearSuffix;}
	public void setYearSuffix(String yearSuffix) {this.yearSuffix = yearSuffix;}
	
	public List<Object> getDays() {return days;}
	public DatePickerOptions setDays(List<Object> days) {
		if(this.days==null)this.days = new LinkedList<>();
		this.days.addAll(days);
		return this;
	}

	public List<Object> getDaysShort() {return daysShort;}
	public DatePickerOptions setDaysShort(List<Object> daysShort) {
		if(this.daysShort==null)this.daysShort = new LinkedList<>();
		this.daysShort.addAll(daysShort);
		return this;
	}
	
	public List<Object> getDaysMin() {return daysMin;}
	public DatePickerOptions setDaysMin(List<Object> daysMin) {
		if(this.daysMin==null)this.daysMin = new LinkedList<>();
		this.daysMin.addAll(daysMin);
		return this;
	}
	
	public List<Object> getMonths() {return months;}
	public DatePickerOptions setMonths(List<Object> months) {
		if(this.months==null)this.months = new LinkedList<>();
		this.months.addAll(months);
		return this;
	}
	
	public List<Object> getMonthsShort() {return monthsShort;}
	public DatePickerOptions setMonthsShort(List<Object> monthsShort) {
		if(this.monthsShort==null)this.monthsShort= new LinkedList<>();
		this.monthsShort.addAll(monthsShort);
		return this;
	}
	
	@Override
	public String toString() {
		try {
			
			if(null==date) {
				put("date",new JsonFunction("null"));
			}else {
				put("date",date);
			}
			if(null!=autoShow)put("autoShow",autoShow);
			if(null!=autoHide)put("autoHide",autoHide);
			if(null!=inline)put("inline",inline);
			if(null!=autoPick)put("autoPick",autoPick);			

			if(null!=language)put("language",language);
			if(null!=format)put("format",format.toLowerCase());
			 
			if(null!=startDate)put("startDate",startDate);
 
			if(null!=endDate)put("endDate",endDate);
			if(null!=startView)put("startView",startView);
			if(null!=weekStart)put("weekStart",weekStart);
			if(null!=yearFirst)put("yearFirst",yearFirst);
	 
			if(null!=yearSuffix)put("yearSuffix",yearSuffix);
			if(null!=days)put("days",days);
			if(null!=daysShort)put("daysShort",daysShort);
			if(null!=daysMin)put("daysMin",daysMin);
 
			if(null!=months)put("months",months);
			if(null!=monthsShort)put("monthsShort",monthsShort);
 	
			if(length()>0)return super.toString();
		}catch(Exception e) {
		}
		return null;
	}
	
	
	



	 

 

	
	 
	
 
	

}
