/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.components.infobox;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.ComponentTag;
import org.apache.wicket.markup.MarkupStream;
import org.apache.wicket.markup.head.IHeaderResponse;
import org.apache.wicket.markup.head.JavaScriptReferenceHeaderItem;
import org.apache.wicket.markup.head.OnDomReadyHeaderItem;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.request.resource.JavaScriptResourceReference;

public class InfoboxPanel extends Panel{

 
	private static final long serialVersionUID = 1L;
	private static final JavaScriptResourceReference INFOBOXPANEL_JS = new JavaScriptResourceReference(InfoboxPanel.class, "InfoboxPanel.js");
	
	public enum INFOBOX_STATUS{
		success,
		warning,
		error
	}
	
	
	private Label lbl;
	private String message;
	private boolean shown=false;
	private boolean preMessage=false;
	
	@Override
	public void renderHead(IHeaderResponse response) {
		super.renderHead(response);
		response.render(JavaScriptReferenceHeaderItem.forReference(INFOBOXPANEL_JS));
		if(preMessage) {
			response.render(OnDomReadyHeaderItem.forScript("showInfoBoxNotification('" + getMarkupId() + "', 3000);"));
			preMessage=false;
		}
	}

	
	public InfoboxPanel(String id) {
		super(id);
		lbl=new Label("text", message);
		lbl.setEscapeModelStrings(false);
		lbl.setOutputMarkupId(true);
		add(lbl);
	}
	
	public InfoboxPanel(String id, String message, INFOBOX_STATUS status) {
		super(id);
		this.message = message;
		lbl=new Label("text", message);
		lbl.setOutputMarkupId(true);
		lbl.setEscapeModelStrings(false);
		add(new AttributeModifier("class", "infobox "+status));
		add(lbl);
		preMessage=true;
		shown=true;
	}
	
	
	@Override
	protected void onAfterRender() {
		super.onAfterRender();
		if(shown) {
			shown=false;
		}
	}
	
	@Override
	protected void onBeforeRender() {
		if(!shown)setVisible(false);
		super.onBeforeRender();
	}

	@Override
	protected void onComponentTag(ComponentTag arg0) {
		super.onComponentTag(arg0);
	}
	
	@Override
	public void onComponentTagBody(MarkupStream markupStream, ComponentTag openTag) {
		super.onComponentTagBody(markupStream, openTag);
	}
	
	public void setMessage(String message) {
		lbl.setDefaultModelObject(message);
	}
	
	public void setPreMessage(boolean preMessage) {
		this.preMessage = preMessage;
	}
	
	
	public void refresh(AjaxRequestTarget target, String message, INFOBOX_STATUS status) {
		setMessage(message);
		add(new AttributeModifier("class", "infobox "+status));
		if(null!=target) {
			target.add(this);
			target.appendJavaScript("showInfoBoxNotification('" + getMarkupId() + "', 3000);");
		}
		shown=true;
	}
	
	
	

}
