/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.components.modulebox;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.ajax.AjaxEventBehavior;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.ComponentTag;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.util.value.IValueMap;
import org.biffo.dms.entity.Variables.Impl.ApiModuleImpl;
 
 

public class ModuleBoxPanel extends Panel {
	
	private static final long serialVersionUID = 1L;
	
	private final ApiModuleImpl _module;
	private boolean _selected;
	private boolean _withEvent=false;
	
	private String moduleClassName = "module";

	
	
	public ModuleBoxPanel(String id, final ApiModuleImpl module) {
		this(id, module, false);
	}
	
	
	public ModuleBoxPanel(String id, final ApiModuleImpl module, boolean withEvent) {
		super(id);
		_module = module;
		_withEvent = withEvent;
	}
	
	private void replaceComponentTag(ComponentTag module) {
	
		final IValueMap moduleAttributes = module.getAttributes();
		moduleAttributes.clear();
		moduleAttributes.put("wicket:id", getMarkupId());
		module.setName("div");

		 
		
		if(_selected)moduleAttributes.put("class", moduleClassName +" active");
		else moduleAttributes.put("class", moduleClassName);
		moduleAttributes.put("id", getMarkupId());
		
	}
	
	@Override
	protected void onBeforeRender() {
		Label lblmodulevalue = new Label("modulevalue", _module.getClassname());
		lblmodulevalue.setOutputMarkupId(true);
		lblmodulevalue.add(new AttributeModifier("class", moduleClassName+"value"));
		add(lblmodulevalue);
		
		if(_withEvent) {
			add(new AjaxEventBehavior("onclick") {
				private static final long serialVersionUID = 1L;

				protected void onEvent(AjaxRequestTarget target) {
					if(_selected)_selected=false;
					else _selected=true;
					onSelect("click", _selected, target);
				}
			});
			add(new AjaxEventBehavior("dblclick") {
				private static final long serialVersionUID = 1L;

				protected void onEvent(AjaxRequestTarget target) {
					if(_selected)_selected=false;
					onSelect("dblclick", _selected, target);
				}
			});
			
		}
		super.onBeforeRender();
	}
	
	@Override
	protected void onComponentTag(ComponentTag module) {
		replaceComponentTag(module);
	}
	
	
	public ApiModuleImpl getModule() {return _module;}
	public boolean isSelected() {return _selected;}
	public void setSelected(boolean selected) {_selected=selected;}
	public boolean isWithEvent() {return _withEvent;}
	public void setWithEvent(boolean _withEvent) {this._withEvent = _withEvent;}
	
	
	protected void onSelect(String event, boolean selection, AjaxRequestTarget target) {}

	public String getModuleClassName() {
		return moduleClassName;
	}

	public void setModuleClassName(String moduleClassName) {
		this.moduleClassName = moduleClassName;
	}




}
