/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.components.pagging;

import org.apache.wicket.Page;
import org.apache.wicket.markup.ComponentTag;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.navigation.paging.IPageable;

public class DmsPagingNavigationIncrementLink <T> extends Link<T>{
	
	private static final long serialVersionUID = 1L;

	private final int increment;
	protected final IPageable pageable;

	public DmsPagingNavigationIncrementLink(final String id, final IPageable pageable, final int increment){
		super(id);
		setAutoEnable(true);
		this.increment = increment;
		this.pageable = pageable;
	}

	@Override
	protected void onComponentTag(ComponentTag tag) {
		replaceComponentTag(tag);
		super.onComponentTag(tag);
	}
	
	private void replaceComponentTag(ComponentTag tag) {
		if(!isEnabled()) {
			tag.getAttributes().put("style", "display: none");
		}
	}


	@Override
	public void onClick(){
		pageable.setCurrentPage(getPageNumber());
		setResponsePage(getPage());
	}

	public final long getPageNumber(){
		if(null==pageable) return 0;
		long idx = pageable.getCurrentPage() + increment;
		return Math.max(0, Math.min(pageable.getPageCount() - 1, idx));
	}

	public boolean isFirst(){
		if(null==pageable) return true;
		return pageable.getCurrentPage() <= 0;
	}

	public boolean isLast(){
		if(null!=pageable) return false;
		return pageable.getCurrentPage() >= (pageable.getPageCount() - 1);
	}

	@Override
	public boolean linksTo(final Page page){
		if(null==pageable) return false;
		pageable.getCurrentPage();
		return ((increment < 0) && isFirst()) || ((increment > 0) && isLast());
	}

}