/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.components.pagging;

import java.io.Serializable;
import java.util.ArrayList;

import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.behavior.Behavior;
import org.apache.wicket.extensions.markup.html.repeater.data.table.DataTable;
import org.apache.wicket.markup.ComponentTag;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.CheckBox;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.link.AbstractLink;
import org.apache.wicket.markup.html.navigation.paging.IPageable;
import org.apache.wicket.markup.html.navigation.paging.IPageableItems;
import org.apache.wicket.markup.html.navigation.paging.IPagingLabelProvider;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.data.DataView;
import org.apache.wicket.model.PropertyModel;
import org.biffo.dms.web.helpers.ResourceText;


public class DmsPagingNavigator extends Panel{
	
	private static final long serialVersionUID = 1L;
	
	public class DmsPagingLabel implements Serializable { 
 
		private static final long serialVersionUID = 1L;
		
		private final IPageableItems _pageable;
		private long _rowCount;
		public DmsPagingLabel(final IPageable pageable, long rowCount) {
			_pageable = (IPageableItems) pageable;
			_rowCount = rowCount;
		}
		
		public void setRowCount(long rowCount) {
			_rowCount = rowCount;
		}
		
		public String getLabelText() {
			if(null==_pageable) return "";
			
			int itemsPerpage =(int) _pageable.getItemsPerPage();
			int rowCount = (int) _rowCount; 
			int currentPage = (int) getPageable().getCurrentPage();
			
			int x = (int) ((currentPage*_pageable.getItemsPerPage())+1);
			int y = ((currentPage+1)*(itemsPerpage));
			if(y>rowCount) {
				return (x + " "+ResourceText.getText("lblTo", DmsPagingNavigator.this)+" " + rowCount + " "+ResourceText.getText("lblFrom", DmsPagingNavigator.this)+" "+rowCount);
			}else {			
				return (x + " "+ResourceText.getText("lblTo", DmsPagingNavigator.this)+" " + ((currentPage+1)*(itemsPerpage)) + " "+ResourceText.getText("lblFrom", DmsPagingNavigator.this)+" "+rowCount);
			}
		}
		
	}
	

	private WebMarkupContainer wmc;
	private final IPageable pageable;
	private final DmsPagingLabel pagingLabel;
	private long _rowCount=0;
	
	private DmsPagingNavigationLink<?> alFirst, alLast;
	private DmsPagingNavigationIncrementLink<?> alPrev, alNext;
	private Label lblPaging, lblNumberOfRows, lblDynamicAdaption, lblTags, lblUsers;
	private DropDownChoice<String> ddcNumberOfRows;
	
	private CheckBox chkDynamicView;
	private CheckBox chkShowTags;
	private CheckBox chkShowUsers;
	
	private boolean showNbOfItems=false;
	private String _numberOfItems = "15";
	
	private boolean showDynamicViewControl=false;
	private boolean _dynamicEnabled=false;
	private boolean _changeNumberOfRows=true;
	
	private boolean _showUsers=false;
	private boolean _showTags=false;
	
	private boolean showUsersControl=false;
	private boolean showTagsControl=false;
	


	public DmsPagingNavigator(final String id, final IPageable pageable, boolean showNbOfItems, boolean showDynamicView, boolean showTagsControl, boolean showUsersControl){
		this(id, pageable, null, -1, showNbOfItems, showDynamicView, showTagsControl, showUsersControl);
	}

	
	@SuppressWarnings("rawtypes")
	public DmsPagingNavigator(final String id, final IPageable pageable, final IPagingLabelProvider labelProvider, long rowCount, boolean showNbOfItems, boolean showDynamicView,  boolean showTagsControl, boolean showUsersControl){
		super(id);
		this.pageable = pageable;
		this.showNbOfItems = showNbOfItems;
		this.showDynamicViewControl = showDynamicView;
		this.showTagsControl=showTagsControl;
		this.showUsersControl=showUsersControl;
		_rowCount = rowCount;
		
		
		wmc = new WebMarkupContainer("main");
		wmc.setOutputMarkupId(true);
		add(wmc);
		
		if(this.pageable!=null) {
			if(this.pageable instanceof DataTable) {
				_rowCount = ((DataTable)this.pageable).getRowCount();
			}else if(this.pageable instanceof DataView) {
				_rowCount = ((DataView)this.pageable).getRowCount();
			}
		}
		this.pagingLabel = new DmsPagingLabel(this.pageable, _rowCount);
		setOutputMarkupId(true);
		
	}

	public final IPageable getPageable() {
		return pageable;
	}

	@SuppressWarnings("rawtypes")
	@Override
	protected void onBeforeRender() {
		super.onBeforeRender();
		
		if(pageable!=null) {
		
			if(pageable instanceof DataTable) {
				_rowCount = ((DataTable)pageable).getRowCount();
			}else if(pageable instanceof DataView) {
				_rowCount = ((DataView)pageable).getRowCount();
			}
			pagingLabel.setRowCount(_rowCount);
			if(pageable.getPageCount()<=1) {
				alFirst.setVisible(false);
				alLast.setVisible(false);
				alPrev.setVisible(false);
				alNext.setVisible(false);
				lblPaging.setVisible(false);
			}else {
				alFirst.setVisible(true);
				alLast.setVisible(true);
				alPrev.setVisible(true);
				alNext.setVisible(true);
				lblPaging.setVisible(true);
			}
		}
		
		if(showNbOfItems) {
			ddcNumberOfRows.setVisible(true);
			lblNumberOfRows.setVisible(true);
		}
		if(showDynamicViewControl) {
			chkDynamicView.setVisible(true);
			lblDynamicAdaption.setVisible(true);
		}
		
		if(showUsersControl) {
			chkShowUsers.setVisible(true);
			lblUsers.setVisible(true);
		}
		
		if(showTagsControl) {
			chkShowTags.setVisible(true);
			lblTags.setVisible(true);
		}
		
		
		if(_dynamicEnabled) {
			ddcNumberOfRows.setEnabled(false);
		}

		
		
		
	}
	
 
	@Override
	protected void onInitialize(){
		super.onInitialize();
		
		
		alFirst = (DmsPagingNavigationLink<?>) newPagingNavigationLink("first", pageable, 0).add(new TitleAppender("PagingNavigator.first"));
		wmc.add(alFirst);
		alPrev = (DmsPagingNavigationIncrementLink<?>) newPagingNavigationIncrementLink("prev", pageable, -1).add(new TitleAppender("PagingNavigator.previous"));
		wmc.add(alPrev);
		alNext = (DmsPagingNavigationIncrementLink<?>) newPagingNavigationIncrementLink("next", pageable, 1).add(new TitleAppender("PagingNavigator.next"));
		wmc.add(alNext);
		alLast = (DmsPagingNavigationLink<?>) newPagingNavigationLink("last", pageable, -1).add(new TitleAppender("PagingNavigator.last"));
		wmc.add(alLast);
		lblPaging =new Label("text", new PropertyModel<String>(pagingLabel, "LabelText"));
		wmc.add(lblPaging);
		
		if(null==pageable) {
			alFirst.setVisible(false);
			alPrev.setVisible(false);
			alNext.setVisible(false);
			alLast.setVisible(false);
			lblPaging.setVisible(false);
		}
		
				
		ArrayList<String> numberOfRows = new ArrayList<String>();
		numberOfRows.add("15");
		numberOfRows.add("30");
		numberOfRows.add("45");
		numberOfRows.add("60");
		numberOfRows.add("75");
		
		ddcNumberOfRows = new DropDownChoice<String>("numberofrows", new PropertyModel<String>(this, "_numberOfItems"), numberOfRows); 
		ddcNumberOfRows.setOutputMarkupId(true);
		ddcNumberOfRows.add(new AjaxFormComponentUpdatingBehavior("onchange") {

			private static final long serialVersionUID = 1L;

			protected void onUpdate(AjaxRequestTarget target) {
				long itemsPerPage = Long.valueOf(_numberOfItems).longValue();
				OnNbOfItemsChanged(target, itemsPerPage);
			}
		});
		ddcNumberOfRows.setVisible(false);
		ddcNumberOfRows.setEnabled(_changeNumberOfRows);
		wmc.add(ddcNumberOfRows);
		lblNumberOfRows = new Label("txtnumberofrows", ResourceText.getText("lblNumberOfRows", this));
		lblNumberOfRows.setVisible(false);
		wmc.add(lblNumberOfRows);
		
		
		chkDynamicView = new CheckBox("dynamicview", new PropertyModel<Boolean>(this, "_dynamicEnabled"));
		chkDynamicView.setOutputMarkupId(true);
		chkDynamicView.add(new AjaxFormComponentUpdatingBehavior("onclick") {
 
			private static final long serialVersionUID = 1L;

			@Override
			protected void onUpdate(AjaxRequestTarget target) {
				OnDynamicViewChanged(target, _dynamicEnabled);
				if(_dynamicEnabled)	ddcNumberOfRows.setEnabled(false);	
				else ddcNumberOfRows.setEnabled(true);
				target.add(chkDynamicView);
				target.add(ddcNumberOfRows);
				
			}
			
		});
		chkDynamicView.setVisible(false);
		wmc.add(chkDynamicView);
		
		lblDynamicAdaption = new Label("textdynamic", ResourceText.getText("lblDynamicAdaption", this));
		lblDynamicAdaption.setVisible(false);
		wmc.add(lblDynamicAdaption);
		
		
		
		
		chkShowTags = new CheckBox("showtags", new PropertyModel<Boolean>(this, "_showTags"));
		chkShowTags.setOutputMarkupId(true);
		chkShowTags.add(new AjaxFormComponentUpdatingBehavior("onclick") {
 
			private static final long serialVersionUID = 1L;

			@Override
			protected void onUpdate(AjaxRequestTarget target) {
				OnShowTagsChanged(target, _showTags);
				target.add(chkShowTags);
			}
			
		});
		chkShowTags.setVisible(false);
		wmc.add(chkShowTags);
		 
		
		lblTags = new Label("texttags", ResourceText.getText("lblTags", this));
		lblTags.setVisible(false);
		wmc.add(lblTags);
		
	 
		
		chkShowUsers = new CheckBox("showusers", new PropertyModel<Boolean>(this, "_showUsers"));
		chkShowUsers.setOutputMarkupId(true);
		chkShowUsers.add(new AjaxFormComponentUpdatingBehavior("onclick") {
 
			private static final long serialVersionUID = 1L;

			@Override
			protected void onUpdate(AjaxRequestTarget target) {
				OnShowUsersChanged(target, _showUsers);
				target.add(chkShowUsers);
				
			}
			
		});
		chkShowUsers.setVisible(false);
		wmc.add(chkShowUsers);
 
		
		lblUsers = new Label("textusers", ResourceText.getText("lblUsers", this));
		lblUsers.setVisible(false);
		wmc.add(lblUsers);
		
 
		
		
		
	}
	
	protected void OnNbOfItemsChanged(AjaxRequestTarget target, long itemPerPage){}
	protected void OnDynamicViewChanged(AjaxRequestTarget target, boolean enabled){}
	protected void OnShowTagsChanged(AjaxRequestTarget target, boolean enabled){}
	protected void OnShowUsersChanged(AjaxRequestTarget target, boolean enabled){}

	protected AbstractLink newPagingNavigationIncrementLink(String id, IPageable pageable, int increment){
		return new DmsPagingNavigationIncrementLink<Void>(id, pageable, increment);
	}
	
	protected AbstractLink newPagingNavigationLink(String id, IPageable pageable, int pageNumber){
		return new DmsPagingNavigationLink<Void>(id, pageable, pageNumber);
	}

	public boolean isShowTags() {return _showTags;}
	public void setShowTags(boolean showTags) {_showTags = showTags;}

	public boolean isShowUsers() {return _showUsers;}
	public void setShowUsers(boolean showUsers) {_showUsers = showUsers;}
	
	public boolean isDynamicEnabled() {return _dynamicEnabled;}
	public void setDynamicEnabled(boolean dynamicEnabled) {this._dynamicEnabled = dynamicEnabled;}
	
	public boolean isChangeNumberOfRows() {return _changeNumberOfRows;}
	public void setChangeNumberOfRows(boolean changeNumberOfRows) {this._changeNumberOfRows = changeNumberOfRows;}
	
	public String getNumberOfItems() {return _numberOfItems;}
	public void setNumberOfItems(String numberOfItems) {this._numberOfItems = numberOfItems;}

	private final class TitleAppender extends Behavior{
		
		private static final long serialVersionUID = 1L;
		private final String resourceKey;

		public TitleAppender(String resourceKey){
			this.resourceKey = resourceKey;
		}

		@Override
		public void onComponentTag(Component component, ComponentTag tag){
			tag.put("title", DmsPagingNavigator.this.getString(resourceKey));
		}
	}
	
	
}
