/**
 * 
 */
class MenuItem{
	id=null;
	text=null;
	event=null;
	checkpattern=null;
	
	constructor(id, text, event, checkpattern){
		this.id = id;
		this.text = text;
		this.event = event;
		this.checkpattern = checkpattern;
	}
}

class CMenu{
	id=-1;
	
	#cMenuItems=new Array();
	#selectedText;
	
	#contextMenuClassName = "context-menu";
  	#contextMenuLinkClassName = "context-menu__link";
  	#contextMenuActive = "context-menu--active";
	
	menu;
	menuState = 0;

	constructor(id, callback){
		this.id = id;
		this.callback = callback;
	}
	

  	init() {
    	this.contextListener();
	    this.clickListener();
    	this.keyupListener();
    	this.resizeListener();
    	this.menu = document.getElementById(this.id);
  	}
  	
  	changeMenu(){
		for(var i=0;i<this.#cMenuItems.length;i++){
			var cMenuItem = this.#cMenuItems[i];
			var menuItem = document.getElementById(cMenuItem.id);
			if(menuItem){
				menuItem.innerHTML=cMenuItem.text + "&nbsp;<span style='font-weight: bold;'>"+this.#selectedText+"</span>";	
			}
			if(cMenuItem.checkpattern=="DATE"){
				var goodDate = /(0[1-9]|1[012])[- \/.](0[1-9]|[12][0-9]|3[01])[- \/.]((19|20)\d\d|\d\d)/;
				//var goodDate=cMenuItem.checkpattern;
				if(this.#selectedText.match(goodDate)){
					menuItem.setAttribute("class", "context-menu__link");
				}else{
					menuItem.setAttribute("class", "context-menu__link_disabled");
				}
			}
		}
	}
  
  	contextListener() {
		var that = this;
		document.addEventListener( "contextmenu", function(e) {
 			that.#selectedText=CMenu.getSelectionText();
			if ( that.#selectedText ) {
				e.preventDefault();
				that.changeMenu();
        		that.toggleMenuOn();
        		that.positionMenu(e);
			} else {
        		that.#selectedText = null;
        		that.toggleMenuOff();
			}
		});
	}
	
	keyupListener() {
		var that=this;
    	window.onkeyup = function(e) {
      		if ( e.keyCode === 27 ) {
        		that.toggleMenuOff();
      		}
    	}
  	}

  	resizeListener() {
		var that=this;
    	window.onresize = function(e) {
      		that.toggleMenuOff();
    	};
  	}
	
	
	addItem(id, text, event, checkpattern){
		
		
		// Create Items
		if(!CMenu.arrayContainsId(id, this.#cMenuItems)){
			var contextMenu = document.getElementById(this.id);
			var contextEntries = contextMenu.getElementsByTagName("ul")[0];
			var menuitem = document.createElement("li");
			menuitem.setAttribute("class", "context-menu__item");
			
			var menulink = document.createElement("a");
			menulink.setAttribute("id", id);
			menulink.setAttribute("class","context-menu__link");
			menulink.setAttribute("data-action", event);
			
			var menutext = document.createElement("span");
			menutext.innerHTML=text;
			menulink.appendChild(menutext);
			contextEntries.appendChild(menulink);
			var mi = new MenuItem(id, text, event, checkpattern);
			this.#cMenuItems.push(mi);
		}
	}
	
	clickListener() {
		var that = this;
    	document.addEventListener( "click", function(e) {
      		var clickeElIsLink = CMenu.clickInsideElement( e, that.#contextMenuLinkClassName );
	      	if ( clickeElIsLink ) {
    	    	e.preventDefault();
        		that.menuItemListener( clickeElIsLink );
        		that.toggleMenuOff();
      		} else {
	        	var button = e.which || e.button;
    	    	if ( button === 1 ) {
        	  		that.toggleMenuOff();
        		}
      		}
    	});
  	}
  	
  	menuItemListener(link) {
		// Hier Callback
    	console.log(this.#selectedText + " " + link.getAttribute("data-action"));
    	if(this.callback==null) return;
    	this.callback(this.id, link.getAttribute("data-action"), this.#selectedText);
    	
  	}
  	
  	toggleMenuOn() {
    	if (this.menuState !== 1 ) {
      		this.menuState = 1;
		    this.menu.classList.add( this.#contextMenuActive );
    	}
  	}
  	
  	toggleMenuOff() {
    	if ( this.menuState !== 0 ) {
      		this.menuState = 0;
      		this.menu.classList.remove( this.#contextMenuActive );
    	}
  	}
  	
  	positionMenu(e) {
    	
    	var clickCoords = CMenu.getPosition(e);
    	var clickCoordsX = clickCoords.x;
    	var clickCoordsY = clickCoords.y;

    	var menuWidth = this.menu.offsetWidth + 4;
    	var menuHeight = this.menu.offsetHeight + 4;

    	var windowWidth = window.innerWidth;
    	var windowHeight = window.innerHeight;

    	if ( (windowWidth - clickCoordsX) < menuWidth ) {
      		this.menu.style.left = windowWidth - menuWidth + "px";
    	} else {
      		this.menu.style.left = clickCoordsX + "px";
    	}

    	if ( (windowHeight - clickCoordsY) < menuHeight ) {
      		this.menu.style.top = windowHeight - menuHeight + "px";
    	} else {
      		this.menu.style.top = clickCoordsY + "px";
    	}
  	}
	
	
	static getSelectionText() {
    	var text = "";
    	var activeEl = document.activeElement;
    	var activeElTagName = activeEl ? activeEl.tagName.toLowerCase() : null;
    	if (
      		(activeElTagName == "textarea") || (activeElTagName == "input" &&
      		/^(?:text|search|password|tel|url)$/i.test(activeEl.type)) &&
      		(typeof activeEl.selectionStart == "number")
    	) 	{
        	text = activeEl.value.slice(activeEl.selectionStart, activeEl.selectionEnd);
    	} else if (window.getSelection) {
        	text = window.getSelection().toString();
    	}
    	return text;
	}
	
	static clickInsideElement( e, className ) {
    	var el = e.srcElement || e.target;
	    if ( el.classList.contains(className) ) {
      		return el;
    	} else {
      		while ( el = el.parentNode ) {
        		if ( el.classList && el.classList.contains(className) ) {
          			return el;
        		}
      		}
    	}

    	return false;
  	}
  
  	static getPosition(e) {
    	var posx = 0;
    	var posy = 0;

    	if (!e) var e = window.event;
    
    	if (e.pageX || e.pageY) {
      		posx = e.pageX;
      		posy = e.pageY;
    	} else if (e.clientX || e.clientY) {
      		posx = e.clientX + document.body.scrollLeft + document.documentElement.scrollLeft;
      		posy = e.clientY + document.body.scrollTop + document.documentElement.scrollTop;
    	}

    	return {
      		x: posx,
      		y: posy
    	}
  	}
  	
  	static arrayContainsId(id, array){
		for(var i=0;i<array.length;i++){
			if(array[i].id==id)return true;
		}
		return false;
	}
	
 
	
	
}
  
 