/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
*
*	Select2 Modified Component
*	This Component is based on:
*
*	Select2 4.0.13
*	https://select2.github.io
*
*	Released under the MIT license
*	https://github.com/select2/select2/blob/master/LICENSE.md
* 
*
**********************************************************************/
package org.biffo.dms.components.select2;


import org.apache.wicket.Component;
import org.apache.wicket.ajax.AbstractDefaultAjaxBehavior;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.ComponentTag;
import org.apache.wicket.markup.head.CssReferenceHeaderItem;
import org.apache.wicket.markup.head.IHeaderResponse;
import org.apache.wicket.markup.head.JavaScriptHeaderItem;
import org.apache.wicket.markup.head.JavaScriptReferenceHeaderItem;
import org.apache.wicket.markup.head.OnDomReadyHeaderItem;
import org.apache.wicket.markup.html.WebComponent;
import org.apache.wicket.model.IModel;
import org.apache.wicket.request.cycle.RequestCycle;
import org.apache.wicket.request.resource.CssResourceReference;
import org.apache.wicket.request.resource.JavaScriptResourceReference;
import org.apache.wicket.util.value.IValueMap;

public class Select2 extends WebComponent{

	private static final JavaScriptResourceReference SELECT2_JS = new JavaScriptResourceReference(Select2.class, "select2.min.js");
	private static final CssResourceReference SELECT2_CSS = new CssResourceReference(Select2.class, "select2.css");
 
	private static final long serialVersionUID = 1L;
	
	private final Select2Settings settings = new Select2Settings();
	public final Select2Settings getSettings(){return settings;}
	
	@Override
	public void renderHead(IHeaderResponse response) {

		response.render(JavaScriptHeaderItem.forScript("var cb"+getMarkupId()+";", "select2"+getMarkupId()));
		
		StringBuilder sb = new StringBuilder();
    	sb.append("$('."+getMarkupId()+"').select2(");
    	sb.append(settings.toString());
    	sb.append(");");
    	
    	sb.append("$('."+getMarkupId()+"').on('change', function(e){");
    	sb.append("var data = $('."+getMarkupId()+"').select2('data');");
    	sb.append("if(data!==undefined){");
    	sb.append("Wicket.Ajax.get({ u: cb"+getMarkupId()+", ep: {'event': 'change', 'data': JSON.stringify(data)}});");
    	sb.append("}});");
    	
		response.render(OnDomReadyHeaderItem.forScript(sb.toString()));
    	response.render(JavaScriptReferenceHeaderItem.forReference(SELECT2_JS));
    	response.render(CssReferenceHeaderItem.forReference(SELECT2_CSS));
    	super.renderHead(response);
	}
	
	@Override
	protected void onComponentTag(final ComponentTag tag) {
		checkComponentTag(tag, "select");
		replaceComponentTag(tag);
		super.onComponentTag(tag);
	}
	
	protected class Select2AjaxBehavior extends AbstractDefaultAjaxBehavior{
		 
		private static final long serialVersionUID = 1L;
		
		
        @Override
        public void renderHead(Component component, IHeaderResponse response){
            super.renderHead(component, response);
            String callbackUrl = getCallbackUrl().toString();
            String callbackVar = "cb"+Select2.this.getMarkupId();
            response.render(OnDomReadyHeaderItem.forScript(";"+callbackVar+" = '" + callbackUrl + "';"));
        }

		@Override
		protected void respond(final AjaxRequestTarget target){
			RequestCycle requestCycle = RequestCycle.get();
			String event = requestCycle.getRequest().getRequestParameters().getParameterValue("event").toString(null);
			if(event!=null) {
				if(event.equals("change")) {
					String value = requestCycle.getRequest().getRequestParameters().getParameterValue("data").toString(null);
					OnChange(target, event, value);
				}
			}
		}		
	}
 	
	private void replaceComponentTag(ComponentTag tag) {
		
		String tempStyle=null, tempName=null;
		final IValueMap tagAttributes = tag.getAttributes();
		if(tagAttributes.containsKey("style")) {
			tempStyle = tagAttributes.getString("style");
		}
		if(tagAttributes.containsKey("name")) {
			tempName = tagAttributes.getString("name");
		}

		tagAttributes.clear();
		tagAttributes.put("wicket:id", getMarkupId());
		tagAttributes.put("class", getMarkupId());
		if(null!=tempStyle)tagAttributes.put("style",tempStyle);
		if(null!=tempName)tagAttributes.put("name",tempName);
		
	}
	
	
	public Select2(String id) {
		this(id, null);
	}

	public Select2(final String id, final IModel<?> model) {
		super(id, model);
		setOutputMarkupId(true);
		add(new Select2AjaxBehavior());
	}
	
	
	
	
	public void clear(AjaxRequestTarget target) {
		settings.getData().clear();
	}
	
	public void destroy(AjaxRequestTarget target) {
		target.appendJavaScript("$('."+getMarkupId()+"').select2('destroy');");
	}

	public void open(AjaxRequestTarget target) {
		target.appendJavaScript("$('."+getMarkupId()+"').select2('open');");
	}
	
	public void close(AjaxRequestTarget target) {
		target.appendJavaScript("$('."+getMarkupId()+"').select2('close');");
	}
	
	public void setDisabled(boolean disabled) {settings.setDisabled(disabled);}
	public boolean isDisabled() {return settings.getDisabled();}
		
	protected void OnChange(AjaxRequestTarget target, String event, String value){}
	
	
}
