/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.components.select2;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.List;

 
import org.apache.wicket.ajax.json.JSONObject;
import org.apache.wicket.ajax.json.JsonFunction;

public class Select2Settings extends JSONObject implements Serializable{

	private static final long serialVersionUID = 1L;
	
	private Boolean allowClear; 

	private String amdLanguageBase;
	private Boolean closeOnSelect;
	private List<ISelectData> data;

	private String dataAdapter;
	private Boolean debug; 
	private String dir;
	private Boolean disabled=false;
	private String dropdownAdapter;
	private Boolean dropdownAutoWidth; 	
	private String dropdownCssClass;
	private String dropdownParent;
	private String escapeMarkup;
	private String language;
	private String matcher;
	private Integer maximumInputLength;
	private Integer maximumSelectionLength;
	private Integer minimumInputLength;
	private Integer minimumResultsForSearch;
	private Boolean multiple;
	private String placeholder;
	private String resultsAdapter;
	private String selectionAdapter;
	private String selectionCssClass;
	private Boolean selectOnClose;
	private String sorter;		
	private Boolean tags;
	private String templateResult;
	private String templateSelection;
	private String theme;
	private String tokenizer;
	private String tokenSeparators;
	private String width;
	private Boolean scrollAfterSelect;

	
	@Override
	public String toString() {
		try {
			
			if(null!=allowClear)put("allowClear", allowClear);
			if(null!=amdLanguageBase)put("amdLanguageBase", amdLanguageBase);
			if(null!=closeOnSelect)put("closeOnSelect", closeOnSelect);
			
			if(null!=data)put("data", data);
			
			if(null!=debug)put("debug", debug);
			if(null!=dir)put("dir", dir);
			if(null!=disabled)put("disabled", disabled);
 
			if(null!=dropdownAutoWidth)put("dropdownAutoWidth", dropdownAutoWidth);
			if(null!=dropdownCssClass)put("dropdownCssClass", dropdownCssClass);
			if(null!=language)put("language", language);
			
			if(null!=maximumInputLength)put("maximumInputLength", maximumInputLength);
			if(null!=maximumSelectionLength)put("maximumSelectionLength", maximumSelectionLength);
			if(null!=minimumInputLength)put("minimumInputLength", minimumInputLength);			
			
			if(null!=minimumResultsForSearch)put("minimumResultsForSearch", minimumResultsForSearch);
			if(null!=multiple)put("multiple", multiple);
			if(null!=placeholder)put("placeholder", placeholder);
			
			if(null!=selectionCssClass)put("selectionCssClass", selectionCssClass);
			if(null!=selectOnClose)put("selectOnClose", selectOnClose);
			if(null!=sorter)put("sorter", sorter);

			if(null!=tags)put("tags", tags);
			if(null!=theme)put("theme", theme);
			if(null!=tokenSeparators)put("tokenSeparators", tokenSeparators);

			if(null!=width)put("width", width);
			if(null!=scrollAfterSelect)put("scrollAfterSelect", scrollAfterSelect);
			
			if(null!=dataAdapter)put("dataAdapter", new JsonFunction(dataAdapter));
			if(null!=dropdownAdapter)put("dropdownAdapter", new JsonFunction(dropdownAdapter));
 			if(null!=dropdownParent)put("dropdownParent", new JsonFunction(dropdownParent));

 			if(null!=escapeMarkup)put("escapeMarkup", new JsonFunction(escapeMarkup));
			if(null!=matcher)put("matcher", new JsonFunction(matcher));
			if(null!=resultsAdapter)put("resultsAdapter", new JsonFunction(resultsAdapter));

 			if(null!=selectionAdapter)put("selectionAdapter", new JsonFunction(selectionAdapter));
			if(null!=templateResult)put("templateResult", new JsonFunction(templateResult));
			if(null!=templateSelection)put("templateSelection", new JsonFunction(templateSelection));
			if(null!=tokenizer)put("tokenizer", new JsonFunction(tokenizer));
 
			if(length()>0)return super.toString();
		}catch(Exception e) {
		}
		return null;
	}
	
	
 
	
	public List<ISelectData> getData() {return data;}
	public void setData(List<ISelectData> data) {
		if(this.data==null)this.data = new LinkedList<ISelectData>();
		this.data.addAll(data);
	}
	public void replaceData(List<ISelectData> data) {
		this.data = new LinkedList<ISelectData>();
		this.data.addAll(data);
	}
	
	public Boolean getAllowClear() {return allowClear;}
	public void setAllowClear(Boolean allowClear) {this.allowClear = allowClear;}

	public String getAmdLanguageBase() {return amdLanguageBase;}
	public void setAmdLanguageBase(String amdLanguageBase) {this.amdLanguageBase = amdLanguageBase;}

	public Boolean getCloseOnSelect() {return closeOnSelect;}
	public void setCloseOnSelect(Boolean closeOnSelect) {this.closeOnSelect = closeOnSelect;}

	public Boolean getDebug() {return debug;}
	public void setDebug(Boolean debug) {this.debug = debug;}

	public String getDir() {return dir;}
	public void setDir(String dir) {this.dir = dir;}

	public Boolean getDisabled() {return disabled;}
	public void setDisabled(Boolean disabled) {this.disabled = disabled;}

	public String getDataAdapter() {return dataAdapter;}
	public void setDataAdapter(String dataAdapter) {this.dataAdapter = dataAdapter;}

	public String getDropdownAdapter() {return dropdownAdapter;}
	public void setDropdownAdapter(String dropdownAdapter) {this.dropdownAdapter = dropdownAdapter;}

	public Boolean getDropdownAutoWidth() {return dropdownAutoWidth;}
	public void setDropdownAutoWidth(Boolean dropdownAutoWidth) {this.dropdownAutoWidth = dropdownAutoWidth;}

	public String getDropdownCssClass() {return dropdownCssClass;}
	public void setDropdownCssClass(String dropdownCssClass) {this.dropdownCssClass = dropdownCssClass;}

	public String getDropdownParent() {return dropdownParent;}
	public void setDropdownParent(String dropdownParent) {this.dropdownParent = dropdownParent;}

	public String getEscapeMarkup() {return escapeMarkup;}
	public void setEscapeMarkup(String escapeMarkup) {this.escapeMarkup = escapeMarkup;}

	public String getLanguage() {return language;}
	public void setLanguage(String language) {this.language = language;}

	public String getMatcher() {return matcher;}
	public void setMatcher(String matcher) {this.matcher = matcher;}

	public Integer getMaximumInputLength() {return maximumInputLength;}
	public void setMaximumInputLength(Integer maximumInputLength) {this.maximumInputLength = maximumInputLength;}

	public Integer getMaximumSelectionLength() {return maximumSelectionLength;}
	public void setMaximumSelectionLength(Integer maximumSelectionLength) {this.maximumSelectionLength = maximumSelectionLength;}

	public Integer getMinimumInputLength() {return minimumInputLength;}
	public void setMinimumInputLength(Integer minimumInputLength) {this.minimumInputLength = minimumInputLength;}

	public Integer getMinimumResultsForSearch() {return minimumResultsForSearch;}
	public void setMinimumResultsForSearch(Integer minimumResultsForSearch) {this.minimumResultsForSearch = minimumResultsForSearch;}

	public Boolean getMultiple() {return multiple;}
	public void setMultiple(Boolean multiple) {this.multiple = multiple;}

	public String getPlaceholder() {return placeholder;}
	public void setPlaceholder(String placeholder) {this.placeholder = placeholder;}

	public String getResultsAdapter() {return resultsAdapter;}
	public void setResultsAdapter(String resultsAdapter) {this.resultsAdapter = resultsAdapter;}

	public String getSelectionAdapter() {return selectionAdapter;}
	public void setSelectionAdapter(String selectionAdapter) {this.selectionAdapter = selectionAdapter;}
	
	public String getSelectionCssClass() {return selectionCssClass;}
	public void setSelectionCssClass(String selectionCssClass) {this.selectionCssClass = selectionCssClass;}

	public Boolean getSelectOnClose() {return selectOnClose;}
	public void setSelectOnClose(Boolean selectOnClose) {this.selectOnClose = selectOnClose;}

	public String getSorter() {return sorter;}
	public void setSorter(String sorter) {this.sorter = sorter;}
	
	public Boolean getTags() {return tags;}
	public void setTags(Boolean tags) {	this.tags = tags;}

	public String getTemplateResult() {return templateResult;}
	public void setTemplateResult(String templateResult) {this.templateResult = templateResult;}

	public String getTemplateSelection() {return templateSelection;}
	public void setTemplateSelection(String templateSelection) {this.templateSelection = templateSelection;}

	public String getTheme() {return theme;}
	public void setTheme(String theme) {this.theme = theme;}

	public String getTokenizer() {return tokenizer;}
	public void setTokenizer(String tokenizer) {this.tokenizer = tokenizer;}

	public String getTokenSeparators() {return tokenSeparators;}
	public void setTokenSeparators(String tokenSeparators) {this.tokenSeparators = tokenSeparators;}

	public String getWidth() {return width;}
	public void setWidth(String width) {this.width = width;}

	public Boolean getScrollAfterSelect() {return scrollAfterSelect;}
	public void setScrollAfterSelect(Boolean scrollAfterSelect) {this.scrollAfterSelect = scrollAfterSelect;}
	
}
