/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.components.tagbox;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.ajax.AjaxEventBehavior;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.ComponentTag;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.util.value.IValueMap;
import org.biffo.dms.entity.Tags.Tag;


public class TagBoxPanel extends Panel {

	
	
	private static final long serialVersionUID = 1L;
	
	private final Tag _tag;
	private boolean _selected;
	private boolean _withEvent=false;
	
	private String _tagClassName = "tag";

	
	
	public TagBoxPanel(String id, final Tag tag) {
		this(id, tag, false);
	}
	
	
	public TagBoxPanel(String id, final Tag tag, boolean withEvent) {
		super(id);
		_tag = tag;
		_withEvent = withEvent;
	}
	
	private void replaceComponentTag(ComponentTag tag) {
	
		final IValueMap tagAttributes = tag.getAttributes();
		tagAttributes.clear();
		tagAttributes.put("wicket:id", getMarkupId());
		tag.setName("div");

		if(_selected)tagAttributes.put("class", _tagClassName +" active");
		else tagAttributes.put("class", _tagClassName);
		tagAttributes.put("style", "border: 2px solid #"+_tag.getTagBgColor()+"; background-color: #"+_tag.getTagBgColor()+";color: #"+_tag.getTagFgColor()+";");
		tagAttributes.put("id", getMarkupId());
		if(_tag.getBeschreibung()!=null)tagAttributes.put("title", _tag.getBeschreibung());
		
	}
	
	@Override
	protected void onBeforeRender() {
		Label lbltagvalue = new Label("tagvalue", _tag.getTagValue());
		lbltagvalue.setOutputMarkupId(true);
		lbltagvalue.add(new AttributeModifier("class", _tagClassName+"value"));
		add(lbltagvalue);
		
		if(_withEvent) {
			add(new AjaxEventBehavior("onclick") {
				private static final long serialVersionUID = 1L;

				protected void onEvent(AjaxRequestTarget target) {
					if(_selected)_selected=false;
					else _selected=true;
					onSelect("click", _selected, target);
				}
			});
			add(new AjaxEventBehavior("dblclick") {
				private static final long serialVersionUID = 1L;

				protected void onEvent(AjaxRequestTarget target) {
					if(_selected)_selected=false;
					onSelect("dblclick", _selected, target);
				}
			});
			
		}
		super.onBeforeRender();
	}
	
	@Override
	protected void onComponentTag(ComponentTag tag) {
		replaceComponentTag(tag);
	}
	
	
	public Tag getTag() {return _tag;}
	public boolean isSelected() {return _selected;}
	public void setSelected(boolean selected) {_selected=selected;}
	public boolean isWithEvent() {return _withEvent;}
	public void setWithEvent(boolean _withEvent) {this._withEvent = _withEvent;}
	
	
	protected void onSelect(String event, boolean selection, AjaxRequestTarget target) {}

	public String getTagClassName() {
		return _tagClassName;
	}

	public void setTagClassName(String _tagClassName) {
		this._tagClassName = _tagClassName;
	}




}
