/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.components.upload;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.wicket.Component;
import org.apache.wicket.ajax.AbstractDefaultAjaxBehavior;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.attributes.CallbackParameter;
import org.apache.wicket.behavior.AttributeAppender;
import org.apache.wicket.markup.ComponentTag;
import org.apache.wicket.markup.head.IHeaderResponse;
import org.apache.wicket.markup.head.JavaScriptHeaderItem;
import org.apache.wicket.markup.head.OnDomReadyHeaderItem;
import org.apache.wicket.markup.html.WebComponent;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.FormComponent;
import org.apache.wicket.markup.html.form.FormComponentPanel;
import org.apache.wicket.markup.html.form.upload.FileUpload;
import org.apache.wicket.model.IModel;
import org.apache.wicket.protocol.http.IMultipartWebRequest;
import org.apache.wicket.request.IRequestParameters;
import org.apache.wicket.request.Request;
import org.apache.wicket.request.resource.JavaScriptResourceReference;
import org.apache.wicket.request.resource.ResourceReference;
import org.apache.wicket.util.convert.ConversionException;
import org.apache.wicket.util.string.Strings;
import org.apache.wicket.util.upload.FileItem;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;

public class UploadField extends FormComponentPanel<Collection<FileUpload>>{

	private static final long serialVersionUID = 1L;
	private static final String _className = "UploadField";
	private DmsWebSession _session = DmsWebSession.getSession();
	
 		
	public static final ResourceReference JS = new JavaScriptResourceReference(UploadField.class, "UploadField.min.js");

	private final WebComponent upload;
	private final Label label;
	
	private boolean labelVisible=true;

	private UploadFieldResponse _uploadFieldResponse=new UploadFieldResponse();
	private transient String[] inputArrayCache = null;
	private String _markupId, _markupVar, _uploadContainerId;

	public UploadField(String id, IModel<? extends Collection<FileUpload>> model) {
		this(id, "Select Files", model);
	}

	@SuppressWarnings("unchecked")
	public UploadField(String id, String labelText, IModel<? extends Collection<FileUpload>> model) {
		
		super(id, (IModel<Collection<FileUpload>>)model);

		_markupId = getMarkupId();
		_markupId = "muf"+_markupId.replaceAll(id, "");
		_markupVar = "mufVar"+_markupId.replaceAll(id, "");
		
		upload = new WebComponent("upload") {
 
			private static final long serialVersionUID = 1L;

			@Override
			protected void onComponentTag(ComponentTag tag) {
				tag.getAttributes().put("name", _markupId+"n");
				super.onComponentTag(tag);
			}
		};
		upload.setMarkupId(_markupId);
		add(upload);
		
		label = new Label("label", labelText) {
 
			private static final long serialVersionUID = 1L;

			@Override
			protected void onComponentTag(ComponentTag tag) {
				tag.getAttributes().put("for", _markupId);
				super.onComponentTag(tag);
			}
		};
		add(label);
		
		add(newAbstractDefaultAjaxBehavior());
		
	}
	
	private AbstractDefaultAjaxBehavior newAbstractDefaultAjaxBehavior() {
        
    	return new AbstractDefaultAjaxBehavior() {

    		private static final long serialVersionUID = 1L;

			@Override
            public void renderHead(Component component, IHeaderResponse response) {
                super.renderHead(component, response);
                CharSequence callbackScript = getCallbackFunction(
                        CallbackParameter.explicit("event"),
                        CallbackParameter.explicit("uploadfiles"),
                        CallbackParameter.explicit("id")
            	);
                
                if(null==_uploadContainerId) _uploadContainerId="NA";
                
                CharSequence script = _markupVar + " = new DmsUp('"+_markupId+"', " + callbackScript + ", '"+_uploadContainerId+"');";
                response.render(OnDomReadyHeaderItem.forScript(script));

                StringBuilder sb = new StringBuilder();
            	sb.append("$('#"+_markupId+"').on('change', function(e){");
        		sb.append(_markupVar + ".buildUploadList('BUILD');");
        		sb.append("});");
        		response.render(OnDomReadyHeaderItem.forScript(sb.toString()));
        		
        		script = _markupVar + ".addDND();";
        		response.render(OnDomReadyHeaderItem.forScript(script));
        		
            }


            @Override
            protected void respond(AjaxRequestTarget target) {
            	IRequestParameters requestParams = getComponent().getRequest().getRequestParameters();
            	String event = getStringValueFromRequestParam(requestParams,"event");
            	String uploadFiles = getStringValueFromRequestParam(requestParams, "uploadfiles");
            	String id = getStringValueFromRequestParam(requestParams, "id");
            	_uploadFieldResponse.setValues(event, uploadFiles, id);
                if(!Strings.isEmpty(event)) {
                	if(event.equals("CLEAR"))UploadField.this.OnClear(target);
                	else if(event.equals("BUILD"))UploadField.this.OnSelect(target, _uploadFieldResponse.getUploadFiles());
                	else if(event.equals("DROP"))UploadField.this.OnDrop(target, _uploadFieldResponse.getUploadFiles());
                }
            }
        };
    }
	
	private String getStringValueFromRequestParam(IRequestParameters requestParams, String name) {
    	try {
    		if(null!=requestParams.getParameterValue(name)) {
    			return requestParams.getParameterValue(name).toString(null);
    		}
    	}catch(Exception e) {
    		DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"getStringValueFromRequestParam",
					null,
					e
				);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
    	}
    	return null;
    }

	@Override
	protected void onComponentTag(ComponentTag tag){

		if(!isEnabled())tag.put("disabled","disabled");
		super.onComponentTag(tag);
		
	}

	@Override
	protected void onBeforeRender(){
		super.onBeforeRender();

		Form<?> form = findParent(Form.class);
		if (form == null){
			throw new IllegalStateException("Component " + getClass().getName() + " must have a " +
				Form.class.getName() + " component above in the hierarchy");
		}
		label.setVisible(labelVisible);
		if(!labelVisible) {
			label.add(new AttributeAppender("style", "visibility: hidden; width: 0%; height: 0%"));
			form.add(new AttributeAppender("style", "visibility: hidden; width: 0%; height: 0%"));
		}
	}

	@Override
	public boolean isMultiPart(){
		return true;
	}

	@Override
	public void renderHead(IHeaderResponse response){
		
		response.render(JavaScriptHeaderItem.forScript("var "+_markupVar+";", "DmsUploadField"+_markupVar));
		response.render(JavaScriptHeaderItem.forReference(JS));
	}

	@Override
	public String[] getInputAsArray(){

		if (inputArrayCache == null){
			ArrayList<String> names = null;

			final Request request = getRequest();
			if (request instanceof IMultipartWebRequest){

				final Map<String, List<FileItem>> itemNameToItem = ((IMultipartWebRequest)request).getFiles();
				for (Entry<String, List<FileItem>> entry : itemNameToItem.entrySet()){
					final String name = entry.getKey();
					final List<FileItem> fileItems = entry.getValue();
					if (!Strings.isEmpty(name) && name.startsWith(_markupId+"n") && !fileItems.isEmpty() &&
						!Strings.isEmpty(fileItems.get(0).getName())){
						names = (names != null) ? names : new ArrayList<String>();
						names.add(name);
					}
				}
			}

			if (names != null){
				inputArrayCache = names.toArray(new String[names.size()]);
			}
		}
		return inputArrayCache;

	}


	@Override
	protected Collection<FileUpload> convertValue(String[] value) throws ConversionException{
		Collection<FileUpload> uploads = null;
		final String[] filenames = getInputAsArray();

		if (filenames != null){
			
			final IMultipartWebRequest request = (IMultipartWebRequest)getRequest();

			uploads = new ArrayList<FileUpload>(filenames.length);

			for (String filename : filenames){
				List<FileItem> fileItems = request.getFile(filename);
				for (FileItem fileItem : fileItems){
					uploads.add(new FileUpload(fileItem));
				}
			}
		}

		return uploads;

	}


	@Override
	public void updateModel(){
		FormComponent.updateCollectionModel(this);
	}

	
	@Override
	protected void onDetach(){
		if (forceCloseStreamsOnDetach()){

			Collection<FileUpload> uploads = getConvertedInput();
			if (uploads != null){
				for (FileUpload upload : uploads){
					upload.closeStreams();
				}
			}
			
			inputArrayCache = null;
			Collection<FileUpload> modelObject = getModelObject();
			if (modelObject != null){
				modelObject.clear();
			}
		}
		super.onDetach();
	}
	
	public List<UploadFieldFile> getUploadFiles(){
		return _uploadFieldResponse.getUploadFiles();
	}

	
	public void clearList(AjaxRequestTarget target) {
		target.appendJavaScript(_markupVar + ".clearFileList();");
	}
 
	protected boolean forceCloseStreamsOnDetach(){
		return true;
	}
	
	public void setUploadContainerId(String uploadContainerId) {
		_uploadContainerId = uploadContainerId;
	}
	
	public void setLabelVisible(boolean labelVisible) {
		this.labelVisible=labelVisible;
		
		
		
		
	}
	
	
	public void showUploadDialog(AjaxRequestTarget target) {
		String submit = "document.getElementById('"+upload.getMarkupId()+"').click();";
		target.appendJavaScript(submit);
	}

	protected void OnSelect(AjaxRequestTarget target, List<UploadFieldFile> uploadFiles ){}
	protected void OnDrop(AjaxRequestTarget target, List<UploadFieldFile> uploadFiles ){}
	protected void OnClear(AjaxRequestTarget target){}
	

}