/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.components.upload;

import java.io.Serializable;

import org.apache.wicket.ajax.json.JSONObject;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;

public class UploadFieldFile implements Serializable {

	private static final long serialVersionUID = 1L;
	private static final String _className = "UploadFieldFile";
	private DmsWebSession _session = DmsWebSession.getSession();
	

	private int id; 
	private String name;
	private int size;
	private long lastmodified;
	private String type;
	private String webkitRelativePath;
	
	
	public UploadFieldFile() {
		super();
	}
	
	public UploadFieldFile(JSONObject jsonObject) {
		try {
			if(jsonObject.has("id") && !jsonObject.isNull("id"))id = jsonObject.getInt("id");
			if(jsonObject.has("name") && !jsonObject.isNull("name"))name = jsonObject.getString("name");
			if(jsonObject.has("size") && !jsonObject.isNull("size"))size = jsonObject.getInt("size");
			if(jsonObject.has("lastmodified") && !jsonObject.isNull("lastmodified"))lastmodified = jsonObject.getLong("lastmodified");
			if(jsonObject.has("type") && !jsonObject.isNull("type"))type = jsonObject.getString("type");
			if(jsonObject.has("webkitRelativePath") && !jsonObject.isNull("webkitRelativePath"))webkitRelativePath = jsonObject.getString("webkitRelativePath");
		}catch(Exception e) {
	  		DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"UploadFieldFile",
					null,
					e
				);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
	}
	
	public int getId() {return id;}
	public void setId(int id) {this.id = id;}

	public String getName() {return name;}
	public void setName(String name) {this.name = name;}

	public int getSize() {return size;}
	public void setSize(int size) {this.size = size;}

	public long getLastmodified() {return lastmodified;}
	public void setLastmodified(long lastmodified) {this.lastmodified = lastmodified;}

	public String getType() {return type;}
	public void setType(String type) {this.type = type;}

	public String getWebkitRelativePath() {return webkitRelativePath;}
	public void setWebkitRelativePath(String webkitRelativePath) {this.webkitRelativePath = webkitRelativePath;}
 
	@Override
	public String toString() {
		return "DmsUploadFieldFile [id=" + id + ", name=" + name + ", size=" + size + ", lastmodified=" + lastmodified
				+ ", type=" + type + ", webkitRelativePath=" + webkitRelativePath + "]";
	}
}