/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.components.upload;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.List;

import org.apache.wicket.ajax.json.JSONArray;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;

public class UploadFieldResponse implements Serializable {
	
	private static final long serialVersionUID = 1L;
	private static final String _className = "UploadFieldResponse";
	private DmsWebSession _session = DmsWebSession.getSession();
	

	
	private String event;
	private String id;
	private List<UploadFieldFile> uploadFiles = new LinkedList<UploadFieldFile>();
 	
    public int getNumUploadFiles() {
    	if(uploadFiles!=null)return uploadFiles.size();
    	return -1;
    }
 
	
	public UploadFieldResponse() {
		super();
	}
	
    static public UploadFieldResponse of(String event, String uploadfiles, String id) {
        return new UploadFieldResponse(event, uploadfiles, id);
    }
    
    public void setValues(String event, String uploadfiles, String id ) {
    	this.event = event;
		this.id = id;
		this.uploadFiles = new LinkedList<UploadFieldFile>();
		try {
			if(uploadfiles!=null && !uploadfiles.isEmpty()) {
				JSONArray tempArray = new JSONArray(uploadfiles);
				for(int i=0;i<tempArray.length();i++) {
					uploadFiles.add(new UploadFieldFile(tempArray.getJSONObject(i)));
				}
			}
		}catch(Exception e) {
	  		DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"setValues",
					null,
					e
				);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
    }
    
	
	
	private UploadFieldResponse(String event, String uploadfiles, String id ) {
		this.event = event;
		this.id = id;
		this.uploadFiles = new LinkedList<UploadFieldFile>();
		
		try {
			JSONArray tempArray = new JSONArray(uploadfiles);
			for(int i=0;i<tempArray.length();i++) {
				uploadFiles.add(new UploadFieldFile(tempArray.getJSONObject(i)));
			}
		}catch(Exception e) {
	  		DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"UploadFieldResponse",
					null,
					e
				);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
	}
	
 

	public String getEvent() {return event;}
	public void setEvent(String event) {this.event = event;}

	public String getId() {return id;}
	public void setId(String id) {this.id = id;}

	public List<UploadFieldFile> getUploadFiles() {return uploadFiles;}
	public void setUploadFiles(List<UploadFieldFile> uploadFiles) {this.uploadFiles = uploadFiles;}
 
	

	@Override
	public String toString() {
		return "DmsUploadFieldResponse [event=" + event + ", id=" + id + ", uploadFiles=" + uploadFiles + "]";
	}
	
	

}
