/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.components.upload;

import java.util.ArrayList;
import java.util.List;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.form.AjaxSubmitLink;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.upload.FileUpload;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.util.lang.Bytes;
import org.biffo.dms.dataprovider.config.DPImporters;
import org.biffo.dms.entity.Processing.ProcessType;
import org.biffo.dms.entity.Variables.Impl.ImportServerImpl;
import org.biffo.dms.processQueue.tasks.DocumentUploadTask;
 
import org.biffo.dms.web.helpers.processing.ProcessingPanel;
import org.biffo.dms.web.logs.normal.LogsPage;
import org.biffo.dms.web.server.importer.ImporterPage;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;

public class UploadPanel extends Panel{

	
	private static final long serialVersionUID = 1L;
    private static final String toolbarCSS = "uploadtoolbarbutton";
    private final int _idImportServer;
    
	private Form<?> _form;
	private AjaxSubmitLink _alDummySubmit;
	private UploadField _uploadField;
	private ArrayList<FileUpload> uploads = new ArrayList<>();
	private ProcessingPanel _processingPanel;
        
    private Toolbar _toolBar;
    private ToolbarButton _btLogfile, _btInfo, _btDescription;
    private DPImporters _qtdp = new DPImporters();
    
    private int processId=-1;
	
	public UploadPanel(String id, int idImportServer) {
		super(id);
		_idImportServer = idImportServer;
		
		WebMarkupContainer wmc = new WebMarkupContainer("uploadcontainer");
		wmc.setOutputMarkupId(true);
		add(wmc);
		
		_form = new Form<Void>("form");
		_form.setOutputMarkupId(true);
		wmc.add(_form);
		
		_uploadField =  new UploadField("fileInput", new PropertyModel<>(this, "uploads")) {
			 
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnSelect(AjaxRequestTarget target, List<UploadFieldFile> uploadFiles) {
				super.OnSelect(target, uploadFiles);
				String submit = "document.getElementById('"+_alDummySubmit.getMarkupId()+"').click();";
				target.appendJavaScript(submit);
			}

			@Override
			protected void OnDrop(AjaxRequestTarget target, List<UploadFieldFile> uploadFiles) {
				super.OnDrop(target, uploadFiles);
				String submit = "document.getElementById('"+_alDummySubmit.getMarkupId()+"').click();";
				target.appendJavaScript(submit);
			}

			
			@Override
			protected void OnClear(AjaxRequestTarget target) {
				super.OnClear(target);
			}
		};
		_uploadField.setUploadContainerId(wmc.getMarkupId());
		_uploadField.setOutputMarkupId(true);
		
		_form.add(_uploadField);
		_form.setMaxSize(Bytes.MAX);
		_form.setFileMaxSize(Bytes.MAX);

		addProcessingPanel();
		addToolBar();
		
		_alDummySubmit = new AjaxSubmitLink("submit", _form) {

			private static final long serialVersionUID = 1L;

			@Override
			protected void onSubmit(AjaxRequestTarget target, Form<?> form) {
				
				try {
					processId=-1;
					_btLogfile.setVisible(false);
					_processingPanel.start(target, new DocumentUploadTask(uploads,_idImportServer));
					target.add(_btLogfile);
				} catch (Exception e) {
				}
			}
		};
		_alDummySubmit.setOutputMarkupId(true);
		add(_alDummySubmit);
		
		_btLogfile.setVisible(false);
		
	}
	
	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				if(event.equals("Info")) {
					ImportServerImpl importServer = _qtdp.getImportServerById(_idImportServer);
					ImporterPage page = new ImporterPage(UploadPanel.this.getPage().getPageReference(), importServer);
					setResponsePage(page);
				}else if(event.equals("Logfile")) {
					LogsPage page = new LogsPage(UploadPanel.this.getPage().getPageReference(), -1, processId);
					setResponsePage(page);
				}
			}
		};
		
		ImportServerImpl importServer = _qtdp.getImportServerById(_idImportServer);
		
		_btInfo = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
		_btInfo.setUtf8Code("xf09b");
		_btInfo.setEventName("Info");
		_btInfo.setAlternate(importServer.getDescription());
		_btInfo.setDefaultFormProcessing(false);
		_btInfo.setOutputMarkupId(true);
		_btInfo.setCssClass(toolbarCSS);
		_toolBar.add(_btInfo);
		
		_btLogfile = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
		_btLogfile.setUtf8Code("xf071");
		_btLogfile.setEventName("Logfile");
		_btLogfile.setDefaultFormProcessing(false);
		_btLogfile.setOutputMarkupId(true);
		_btLogfile.setCssClass(toolbarCSS);
		_toolBar.add(_btLogfile);
		
		_btDescription = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED_TEXT).setToolbarState(ToolbarState.STATE_VIEW);
		_btDescription.setUtf8Code(importServer.getProcessName());
		_btDescription.setDefaultFormProcessing(false);
		_btDescription.setOutputMarkupId(true);
		_btDescription.setCssClass("uploadtoolbartext");
		_toolBar.add(_btDescription);
		
		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		_form.add(_toolBar);
	}
	
	 
 	
 	private void addProcessingPanel() {
		
 		_processingPanel = new ProcessingPanel("modalprocess") {
			private static final long serialVersionUID = 1L;
			
			@Override
			protected void onFinished(AjaxRequestTarget target, ProcessType processType, int documentSourceId, int documentTargetId, int processId) {
				super.onFinished(target, processType, documentSourceId, documentTargetId, processId);
				UploadPanel.this.processId=processId;
				if(processId!=-1)_btLogfile.setVisible(true);
				UploadPanel.this.OnUploadFinish(target, processId);
				target.add(_btLogfile);
			}
		};
		_processingPanel.setOutputMarkupId(true);
		add(_processingPanel);

 	}
 	
 	@Override
 	public boolean isVisible() {
 		return super.isVisible();
 	}

 	protected void OnUploadFinish(AjaxRequestTarget target, int processId){}

}
