/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.components.userbox;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.ajax.AjaxEventBehavior;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.ComponentTag;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.util.value.IValueMap;
import org.biffo.dms.entity.user.Users.User;
 
public class UserBoxPanel extends Panel {

	
	
	private static final long serialVersionUID = 1L;
	
	private final User _user;
	private boolean _selected;
	private boolean _withEvent=false;
	
	private String userClassName = "user";

	
	
	public UserBoxPanel(String id, final User user) {
		this(id, user, false);
	}
	
	
	public UserBoxPanel(String id, final User user, boolean withEvent) {
		super(id);
		_user = user;
		_withEvent = withEvent;
	}
	
	private void replaceComponentTag(ComponentTag user) {
	
		final IValueMap userAttributes = user.getAttributes();
		userAttributes.clear();
		userAttributes.put("wicket:id", getMarkupId());
		user.setName("div");

		 
		
		if(_selected)userAttributes.put("class", userClassName +" active");
		else userAttributes.put("class", userClassName);
		userAttributes.put("id", getMarkupId());
		if(_user.getBeschreibung()!=null)userAttributes.put("title", _user.getBeschreibung());
		
	}
	
	@Override
	protected void onBeforeRender() {
		Label lbluservalue = new Label("uservalue", _user.getBenutzerName());
		lbluservalue.setOutputMarkupId(true);
		lbluservalue.add(new AttributeModifier("class", userClassName+"value"));
		add(lbluservalue);
		
		if(_withEvent) {
			add(new AjaxEventBehavior("onclick") {
				private static final long serialVersionUID = 1L;

				protected void onEvent(AjaxRequestTarget target) {
					if(_selected)_selected=false;
					else _selected=true;
					onSelect("click", _selected, target);
				}
			});
			add(new AjaxEventBehavior("dblclick") {
				private static final long serialVersionUID = 1L;

				protected void onEvent(AjaxRequestTarget target) {
					if(_selected)_selected=false;
					onSelect("dblclick", _selected, target);
				}
			});
			
		}
		super.onBeforeRender();
	}
	
	@Override
	protected void onComponentTag(ComponentTag user) {
		replaceComponentTag(user);
	}
	
	
	public User getUser() {return _user;}
	public boolean isSelected() {return _selected;}
	public void setSelected(boolean selected) {_selected=selected;}
	public boolean isWithEvent() {return _withEvent;}
	public void setWithEvent(boolean _withEvent) {this._withEvent = _withEvent;}
	
	
	protected void onSelect(String event, boolean selection, AjaxRequestTarget target) {}

	public String getUserClassName() {
		return userClassName;
	}

	public void setUserClassName(String userClassName) {
		this.userClassName = userClassName;
	}




}
