/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.dataprovider;


 
import java.util.Iterator;

import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.filter.IFilterStateLocator;
import org.apache.wicket.extensions.markup.html.repeater.util.SortParam;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.util.string.Strings;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Fragments.Fragment;
import org.biffo.dms.entity.Fragments.FragmentAggregationClass;
import org.biffo.dms.entity.Fragments.FragmentFilterClass;
import org.biffo.dms.entity.Relation.RelationType;
import org.biffo.dms.entity.user.Users.UsersAccessType;

public class DPFragmentsAggregated extends SortableDataProvider<Fragment, String> implements IFilterStateLocator<FragmentFilterClass> {

	private static final String _className = "DPFragmentsAggregated";
	
	private static final long serialVersionUID = 1L;
	private final String _defaultSortColumn = "fragmentvalue";

	FragmentAggregationClass _aggregationClass = new FragmentAggregationClass();
	private FragmentFilterClass _filterClass = new FragmentFilterClass();
	private long _size=0;
	private String _userCondition="";
	private String _linkedWhereCondition="";
	
	private DmsWebSession _session =  DmsWebSession.getSession();
	
	public DPFragmentsAggregated() {
		setSort(_defaultSortColumn, SortOrder.DESCENDING);
		_aggregationClass.addObjectForSelect("Fragmentname");
		_aggregationClass.addObjectForSelect("Fragmentvalue");
		_aggregationClass.addSumObjectForSelect("idDocument");
		_aggregationClass.addSumObjectForSelect("idDocument_list");
		
		if(_session.getUserAccessType()==UsersAccessType.USER) {
			_userCondition = "dms_fragments.iddocument IN (SELECT dms_relations.Idrelation1 FROM dms_relations WHERE RelationType="+RelationType.DOCUMENT_TO_USER.getCode()+" AND Idrelation2="+_session.getCurrentUserId()+")";
		}else {
			_userCondition =" 1=1 ";
		}
	}
 
	 
	
	
	@Override
	public void detach() {
		super.detach();
	}
	
	@Override
	public Iterator<? extends Fragment> iterator(long first, long count) {
		try {
			prepareUserFilterCondition();
			return StaticLocators.getFragments().getEntities(first, count, buildDBSortCondition(getSort(),_defaultSortColumn), _filterClass, _aggregationClass).listIterator();	
		} catch (Exception e) {
			DmsException dmsException = new DmsException(
				1000, 
				"E", 
				_className, 
				"iterator",
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
		return null;
	}

	@Override
	public IModel<Fragment> model(Fragment entity) {
		return Model.of(entity);
	}

	@Override
	public long size() {
		try{
			prepareUserFilterCondition();
			_size = StaticLocators.getFragments().getNumberOfEntities(_filterClass, _aggregationClass);
		}catch(Exception e){
			DmsException dmsException = new DmsException(
				1000, 
				"E", 
				_className, 
				"size",
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
			_size=0;
		}	
		return _size;
	}


	@Override
	public FragmentFilterClass getFilterState() {
		return _filterClass;
	}
	
	private String buildDBSortCondition(SortParam<?> sort, String defaultColumn){
		String strSortParam=" order by "+defaultColumn+" ASC";
    	if(null!=sort){
    		
    		String sortProperty = sort.getProperty().toString();
			strSortParam = " ORDER BY "+sortProperty;
			if(sort.isAscending()) strSortParam = strSortParam + " ASC";
			else strSortParam = strSortParam + " DESC";
		}
    	return strSortParam;
	}
	
	private void prepareUserFilterCondition() {
		if(!Strings.isEmpty(_userCondition) && !Strings.isEmpty(_linkedWhereCondition)) {
			_filterClass.setLinkedWhere(_linkedWhereCondition + " AND "+ _userCondition);
		}else if(!Strings.isEmpty(_userCondition)) {
			_filterClass.setLinkedWhere(_userCondition);
		}
	}


	@Override
	public void setFilterState(FragmentFilterClass filterClass) {
		_filterClass = filterClass;
		if(!Strings.isEmpty(_filterClass.getLinkedWhere())) {
			_linkedWhereCondition = _filterClass.getLinkedWhere();
		}else {
			_linkedWhereCondition = "";
		}
	}
	
	
}
