/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.dataprovider;



import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.filter.IFilterStateLocator;
import org.apache.wicket.extensions.markup.html.repeater.util.SortParam;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Logentries.LogEntry;
import org.biffo.dms.entity.Logentries.LogEntryFilterClass;
 
 


public class DPLogEntries extends SortableDataProvider<LogEntry, String> implements IFilterStateLocator<LogEntryFilterClass> {
 
	private static final long serialVersionUID = -1L;
	private LogEntryFilterClass _filterClass = new LogEntryFilterClass();
	private final String defaultSortColumn = "id";
	
	private long _size=0;

	
	@Override
	public void detach() {
		super.detach();
	}
	
	public DPLogEntries(){
		setSort(defaultSortColumn, SortOrder.DESCENDING);
	}

	@Override
	public Iterator<? extends LogEntry> iterator(long first, long count) {
		try {
			return StaticLocators.getLogEntries().getEntities(first, count, buildDBSortCondition(getSort(),defaultSortColumn), _filterClass).listIterator();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	
	

	@Override
	public long size() {
		try{
			if(null==_filterClass) {
				_size = StaticLocators.getLogEntries().getNumberOfEntities(null);
			}else {
				_size = StaticLocators.getLogEntries().getNumberOfEntities(_filterClass);
			}
		}catch(Exception e){
			_size=0;
		}	
		return _size;
	}


	@Override
	public IModel<LogEntry> model(LogEntry entity) {
		return Model.of(entity);
	}


	@Override
	public LogEntryFilterClass getFilterState() {
		return _filterClass;
	}


	@Override
	public void setFilterState(LogEntryFilterClass filterClass) {
		_filterClass = filterClass;
	}
	
	public void clearAll() {
		_filterClass = new LogEntryFilterClass();
	}
	
	
	private String buildDBSortCondition(SortParam<?> sort, String defaultColumn){
		String strSortParam=" order by "+defaultColumn+" ASC";
    	if(null!=sort){
    		
    		String sortProperty = sort.getProperty().toString();
    		if(sortProperty.toLowerCase().equals("score")) sortProperty = "id";
			strSortParam = " ORDER BY "+sortProperty;
			if(sort.isAscending()) strSortParam = strSortParam + " ASC";
			else strSortParam = strSortParam + " DESC";
		}
    	return strSortParam;
	}
	
	@SuppressWarnings("unused")
	private void sortList(List<LogEntry> lstEntities, final String sortKey, final boolean isAscending){
		try{
			if(sortKey!=null){
				Collections.sort(lstEntities, new Comparator<Object>() {
					@SuppressWarnings({ "unchecked", "rawtypes" })
					@Override
					public int compare(Object c1, Object c2) {
						int result=1;
						PropertyModel<Comparable<?>> model1= new PropertyModel<Comparable<?>>(c1, sortKey);
						PropertyModel<Comparable<?>> model2= new PropertyModel<Comparable<?>>(c2, sortKey);

						if(model1.getObject() == null && model2.getObject() == null) result = 0;
						else if(model1.getObject() == null) result = 1;
						else if(model2.getObject() == null) result = -1;
						else result = ((Comparable)model1.getObject()).compareTo(model2.getObject());
						result = isAscending ? result : -result;
						return result;
					}
				});
			}
		}catch(Exception e){
		}
	}
	
	
	
	
	
}
