/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.dataprovider;


 
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.filter.IFilterStateLocator;
import org.apache.wicket.extensions.markup.html.repeater.util.SortParam;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Relation.Relation;
import org.biffo.dms.entity.Relation.RelationFilterClass;
 

 

public class DPRelations extends SortableDataProvider<Relation, String> implements IFilterStateLocator<RelationFilterClass> {

	private static final String _className = "DPRelations";
	private DmsWebSession _session = DmsWebSession.getSession();

	private static final long serialVersionUID = 1L;
	private final String _defaultSortColumn = "id";

	private ArrayList<Relation> lstRelations = new ArrayList<Relation>();
	private RelationFilterClass _filterClass = new RelationFilterClass();

	
	public DPRelations() {
		setSort(_defaultSortColumn, SortOrder.DESCENDING);
	}
	@Override
	public void detach() {
		super.detach();
	}
	
	@Override
	public Iterator<? extends Relation> iterator(long first, long count) {
		try {
			sortList(lstRelations,_defaultSortColumn, getSort().isAscending() );
			return lstRelations.subList((int)first, ((int)first+(int)count)).iterator();	
		} catch (Exception e) {
			DmsException dmsException = new DmsException(
				1000, 
				"E", 
				_className, 
				"iterator",
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
		return null;
	}

	@Override
	public IModel<Relation> model(Relation entity) {
		return Model.of(entity);
	}

	@Override
	public long size() {
		refreshEntities();
		return lstRelations.size();
	}

	
	private void refreshEntities() {
		try {
			lstRelations.clear();
			lstRelations.addAll(StaticLocators.getRelations().getEntities(-1, -1, "", _filterClass));
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
				1000, 
				"E", 
				_className, 
				"refreshEntities",
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
	}

	@Override
	public RelationFilterClass getFilterState() {
		return _filterClass;
	}
	
	@SuppressWarnings("unused")
	private String buildDBSortCondition(SortParam<?> sort, String defaultColumn){
		String strSortParam=" order by "+defaultColumn+" ASC";
    	if(null!=sort){
    		
    		String sortProperty = sort.getProperty().toString();
    		
			strSortParam = " ORDER BY "+sortProperty;
			if(sort.isAscending()) strSortParam = strSortParam + " ASC";
			else strSortParam = strSortParam + " DESC";
		}
    	return strSortParam;
	}
	
 
	private void sortList(List<Relation> lstEntities, final String sortKey, final boolean isAscending){
		try{
			if(sortKey!=null){
				Collections.sort(lstEntities, new Comparator<Object>() {
					@SuppressWarnings({ "unchecked", "rawtypes" })
					@Override
					public int compare(Object c1, Object c2) {
						int result=1;
						PropertyModel<Comparable<?>> model1= new PropertyModel<Comparable<?>>(c1, sortKey);
						PropertyModel<Comparable<?>> model2= new PropertyModel<Comparable<?>>(c2, sortKey);

						if(model1.getObject() == null && model2.getObject() == null) result = 0;
						else if(model1.getObject() == null) result = 1;
						else if(model2.getObject() == null) result = -1;
						else result = ((Comparable)model1.getObject()).compareTo(model2.getObject());
						result = isAscending ? result : -result;
						return result;
					}
				});
			}
		}catch(Exception e){
			DmsException dmsException = new DmsException(
				1000, 
				"E", 
				_className, 
				"sortList",
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
	} 
 
	public ArrayList<Relation> getRelationById1(int id) {
		ArrayList<Relation> rc = new ArrayList<Relation>();
		for(int i=0;i<lstRelations.size();i++) {
			if(lstRelations.get(i).getIdrelation1()==id) {
				rc.add(lstRelations.get(i));
			}
		}
		return rc;
	}
	public ArrayList<Relation> getRelationById2(int id) {
		ArrayList<Relation> rc = new ArrayList<Relation>();
		for(int i=0;i<lstRelations.size();i++) {
			if(lstRelations.get(i).getIdrelation2()==id) {
				rc.add(lstRelations.get(i));
			}
		}
		return rc;
	}


	@Override
	public void setFilterState(RelationFilterClass filterClass) {
		_filterClass = filterClass;		
	}
	
	
}
