/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.dataprovider;


 
import java.util.Iterator;

import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.filter.IFilterStateLocator;
import org.apache.wicket.extensions.markup.html.repeater.util.SortParam;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Variables.Variable;
import org.biffo.dms.entity.Variables.VariableFilterClass;
 

public class DPVariables extends SortableDataProvider<Variable, String> implements IFilterStateLocator<VariableFilterClass> {

 
	private static final String _className = "DPVariables";
	private static final long serialVersionUID = 1L;
	private final String _defaultSortColumn = "id";
	private DmsWebSession _session = DmsWebSession.getSession();

	private VariableFilterClass _filterClass = new VariableFilterClass();
	private long _size=0;
	
	public DPVariables() {
		setSort(_defaultSortColumn, SortOrder.DESCENDING);
	}
	
	@Override
	public void detach() {
		super.detach();
	}
	
	@Override
	public Iterator<? extends Variable> iterator(long first, long count) {
		try {
			return StaticLocators.getVariables().getEntities(first, count, buildDBSortCondition(getSort(),_defaultSortColumn), _filterClass).listIterator();	
		} catch (Exception e) {
			DmsException dmsException = new DmsException(
				1000, 
				"E", 
				_className, 
				"iterator",
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
		return null;
	}

	@Override
	public IModel<Variable> model(Variable entity) {
		return Model.of(entity);
	}

	@Override
	public long size() {
		try{
			if(true) {
				_size = StaticLocators.getVariables().getNumberOfEntities(_filterClass);
			}
		}catch(Exception e){
			DmsException dmsException = new DmsException(
				1000, 
				"E", 
				_className, 
				"size",
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
			_size=0;
		}	
		return _size;
	}


	@Override
	public VariableFilterClass getFilterState() {
		return _filterClass;
	}
	
	private String buildDBSortCondition(SortParam<?> sort, String defaultColumn){
		String strSortParam=" order by "+defaultColumn+" ASC";
    	if(null!=sort){
    		
    		String sortProperty = sort.getProperty().toString();
    		
			strSortParam = " ORDER BY "+sortProperty;
			if(sort.isAscending()) strSortParam = strSortParam + " ASC";
			else strSortParam = strSortParam + " DESC";
		}
    	return strSortParam;
	}
	

	@Override
	public void setFilterState(VariableFilterClass filterClass) {
		_filterClass = filterClass;		
	}
	
	
  
	
	
}
