/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.dataprovider.config;

import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Variables.Impl.FTPServerImpl;
import org.biffo.dms.helpers.ImportProcessHelpers;
 
 

public class DPFtpServers extends SortableDataProvider<FTPServerImpl, String>{

	
	private static final String _className = "DPFtpServers";
	private static final long serialVersionUID = 1L;

	private DmsWebSession _session = DmsWebSession.getSession();
	private List<FTPServerImpl> lstServers = new LinkedList<FTPServerImpl>();

	private String _defaultSortColumn = "serverName";
	
	public DPFtpServers() {
		setSort(_defaultSortColumn, SortOrder.DESCENDING);
	}
	
	
	@Override
	public Iterator<? extends FTPServerImpl> iterator(long first, long count) {
		sortList(lstServers,_defaultSortColumn, getSort().isAscending() );
		return lstServers.subList((int)first, ((int)first+(int)count)).iterator();	
	}

	@Override
	public IModel<FTPServerImpl> model(FTPServerImpl model) {
		return Model.of(model);
	}

	@Override
	public long size() {
		if(lstServers.size()==0)refreshFTPServers();
		return lstServers.size();
	}
	
	public void refresh() {
		refreshFTPServers();
	}
	
	
	
	private void sortList(List<FTPServerImpl> lstEntities, final String sortKey, final boolean isAscending){
		try{
			if(sortKey!=null){
				Collections.sort(lstEntities, new Comparator<Object>() {
					@SuppressWarnings({ "unchecked", "rawtypes" })
					@Override
					public int compare(Object c1, Object c2) {
						int result=1;
						PropertyModel<Comparable<?>> model1= new PropertyModel<Comparable<?>>(c1, sortKey);
						PropertyModel<Comparable<?>> model2= new PropertyModel<Comparable<?>>(c2, sortKey);

						if(model1.getObject() == null && model2.getObject() == null) result = 0;
						else if(model1.getObject() == null) result = 1;
						else if(model2.getObject() == null) result = -1;
						else result = ((Comparable)model1.getObject()).compareTo(model2.getObject());
						result = isAscending ? result : -result;
						return result;
					}
				});
			}
		}catch(Exception e){
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"sortList",
					null,
					null,
					e
				);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
	}
	
	@SuppressWarnings("unchecked")
	private void refreshFTPServers() {
		try {
			lstServers.clear();
			lstServers.addAll((Collection<? extends FTPServerImpl>) ImportProcessHelpers.getServerFTPProcesses());
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"refreshFTPServers",
					null,
					null,
					e
				);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
			
		}
	}
	
	 
	
	public List<FTPServerImpl> getFtpServers(){
		if(lstServers.isEmpty()) {
			refreshFTPServers();
		}
		return lstServers;
	}

}
