/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.dataprovider.users;



import java.util.Iterator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.filter.IFilterStateLocator;
import org.apache.wicket.extensions.markup.html.repeater.util.SortParam;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.user.Tenant.Tenant;
import org.biffo.dms.entity.user.Tenant.TenantFilterClass;
 
public class DPTenants extends SortableDataProvider<Tenant, String> implements IFilterStateLocator<TenantFilterClass> {
 
	private static final String _className = "DPTenants";
	private DmsWebSession _session = DmsWebSession.getSession();
	private static final long serialVersionUID = -1L;
	private TenantFilterClass _filterClass = new TenantFilterClass();
	private final String defaultSortColumn = "id";
	
	private long _size=0;

	
	@Override
	public void detach() {
		super.detach();
	}
	
	public DPTenants(){
		setSort(defaultSortColumn, SortOrder.DESCENDING);
	}

	@Override
	public Iterator<? extends Tenant> iterator(long first, long count) {
		try {
			return StaticLocators.getTenants().getEntities(first, count, buildDBSortCondition(getSort(),defaultSortColumn), _filterClass).listIterator();
		} catch (Exception e) {
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"iterator",
					null,
					e
				);
				StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
		return null;
	}

	
	

	@Override
	public long size() {
		try{
			if(null==_filterClass) {
				_size = StaticLocators.getTenants().getNumberOfEntities(null);
			}else {
				_size = StaticLocators.getTenants().getNumberOfEntities(_filterClass);
			}
 
		}catch(Exception e){
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"iterator",
					null,
					e
				);
				StaticLocators.getMessageHandler().createMessage(dmsException, _session);
			_size=0;
		}	
		return _size;
	}


	@Override
	public IModel<Tenant> model(Tenant entity) {
		return Model.of(entity);
	}


	@Override
	public TenantFilterClass getFilterState() {
		return _filterClass;
	}


	@Override
	public void setFilterState(TenantFilterClass filterClass) {
		_filterClass = filterClass;
	}
	
	
	private String buildDBSortCondition(SortParam<?> sort, String defaultColumn){
		String strSortParam=" order by "+defaultColumn+" ASC";
    	if(null!=sort){
    		
    		String sortProperty = sort.getProperty().toString();
    		if(sortProperty.toLowerCase().equals("score")) sortProperty = "id";
    		
    		
			strSortParam = " ORDER BY "+sortProperty;
			if(sort.isAscending()) strSortParam = strSortParam + " ASC";
			else strSortParam = strSortParam + " DESC";
		}
    	return strSortParam;
	}
	
}
