/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.dataprovider.users;



import java.util.Date;
import java.util.Iterator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.filter.IFilterStateLocator;
import org.apache.wicket.extensions.markup.html.repeater.util.SortParam;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Helpers.EntityUUID;
import org.biffo.dms.entity.Helpers.SaltedMD5;
import org.biffo.dms.entity.user.Usergroups.UserGroup;
import org.biffo.dms.entity.user.Usergroups.UserGroupFilterClass;
import org.biffo.dms.entity.user.Userrelation.UserRelation;
import org.biffo.dms.entity.user.Userrelation.UserRelationType;
import org.biffo.dms.entity.user.Users.User;
import org.biffo.dms.entity.user.Users.UserFilterClass;
 
public class DPUsers extends SortableDataProvider<User, String> implements IFilterStateLocator<UserFilterClass> {
 
	private static final String _className = "DPUsers";
	private DmsWebSession _session = DmsWebSession.getSession();
	private static final long serialVersionUID = -1L;
	private UserFilterClass _filterClass = new UserFilterClass();
	private final String defaultSortColumn = "id";
	
	private long _size=0;

	
	@Override
	public void detach() {
		super.detach();
	}
	
	public DPUsers(){
		setSort(defaultSortColumn, SortOrder.DESCENDING);
	}

	@Override
	public Iterator<? extends User> iterator(long first, long count) {
		try {
			return StaticLocators.getUsers().getEntities(first, count, buildDBSortCondition(getSort(),defaultSortColumn), _filterClass).listIterator();
		} catch (Exception e) {
			DmsException dmsException = new DmsException(
				1000, 
				"E", 
				_className, 
				"iterator",
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
		return null;
	}

	
	

	@Override
	public long size() {
		try{
			if(null==_filterClass) {
				_size = StaticLocators.getUsers().getNumberOfEntities(null);
			}else {
				_size = StaticLocators.getUsers().getNumberOfEntities(_filterClass);
			}
 
		}catch(Exception e){
			DmsException dmsException = new DmsException(
				1000, 
				"E", 
				_className, 
				"size",
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);			
			_size=0;
		}	
		return _size;
	}


	@Override
	public IModel<User> model(User entity) {
		return Model.of(entity);
	}


	@Override
	public UserFilterClass getFilterState() {
		return _filterClass;
	}


	@Override
	public void setFilterState(UserFilterClass filterClass) {
		_filterClass = filterClass;
	}
	
	
	private String buildDBSortCondition(SortParam<?> sort, String defaultColumn){
		String strSortParam=" order by "+defaultColumn+" ASC";
    	if(null!=sort){
    		
    		String sortProperty = sort.getProperty().toString();
    		if(sortProperty.toLowerCase().equals("score")) sortProperty = "id";
    		
			strSortParam = " ORDER BY "+sortProperty;
			if(sort.isAscending()) strSortParam = strSortParam + " ASC";
			else strSortParam = strSortParam + " DESC";
		}
    	return strSortParam;
	}

	public void pasteData(String data, String userName) throws Exception {
		String[] strArr = data.split("\n");
		for(int i=0;i<strArr.length;i++) {
			String temp = strArr[i];
			
			try {
				String[] tempArr = temp.split(";");
				User user = new User();
				UserGroup ug=null;
				String benutzerName = tempArr[0].trim();
				if(benutzerName!=null && benutzerName.length()>0) {
					user.setBenutzerName(benutzerName);
					user.setBenutzerEmail(user.getBenutzerName());
					user.setBenutzerPassword(SaltedMD5.generateStorngPasswordHash(user.getBenutzerName()));
					user.setGesperrt(true);
				}
				if(tempArr.length>=2) {
					String groupName = tempArr[1].trim();
					if(groupName!=null && !groupName.isEmpty()) {
						ug=getUserGroup(groupName, userName, true);
					}
				}
				
				user.setErstellungsDatum(new Date());
				user.setErstelltVon(userName);
				user.setAenderungsDatum(user.getErstellungsDatum());
				user.setGeaendertVon(user.getErstelltVon());
				user.setUUID(EntityUUID.createUserUUID(user));
				
				int userId = StaticLocators.getUsers().getEntityId(user);
				if(userId==-1)StaticLocators.getUsers().addEntity(user);
				if(userId==-1 && user.getId()!=-1 && ug!=null && ug.getId()!=-1) {
					UserRelation userRelation = new UserRelation();
					userRelation.setErstelltVon(userName);
					userRelation.setErstellungsDatum(new Date());
					userRelation.setGeaendertVon(userRelation.getErstelltVon());
					userRelation.setAenderungsDatum(userRelation.getErstellungsDatum());
					userRelation.setIdrelation1(ug.getId());
					userRelation.setIdrelation2(user.getId());
					userRelation.setUserRelationType(UserRelationType.GROUP_TO_USER);
					StaticLocators.getUserRelations().addEntity(userRelation);
				}
				
			}catch(Exception e) {
				DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"pasteData",
					null,
					e
				);
				StaticLocators.getMessageHandler().createMessage(dmsException, _session);
			}
			
		}
	} 
	
	private UserGroup getUserGroup(String groupName, String userName, boolean withCreation) {
		try {
			UserGroupFilterClass ugfc = new UserGroupFilterClass();
			ugfc.setName(groupName);
			long numberOfEntities = StaticLocators.getUserGroups().getNumberOfEntities(ugfc);
			if(numberOfEntities==1) {
				return StaticLocators.getUserGroups().getEntities(0, 1, "", ugfc).get(0);
			}else if(numberOfEntities==0 && withCreation) {
				UserGroup ug = new UserGroup();
				ug.setName(groupName);
				ug.setBeschreibung(groupName);
				ug.setErstellungsDatum(new Date());
				ug.setErstelltVon(userName);
				ug.setAenderungsDatum(ug.getErstellungsDatum());
				ug.setGeaendertVon(ug.getErstelltVon());
				ug.setUUID(EntityUUID.createUserGroupUUID(ug));	
				StaticLocators.getUserGroups().addEntity(ug);
				return ug;
			}
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
				1000, 
				"E", 
				_className, 
				"getUserGroup",
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
		return null;
	}
	
	
		
	
	
	
	
	
}
