/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.helpers;

import java.sql.Connection;
import java.sql.DriverManager;

import org.apache.wicket.Application;
import org.apache.wicket.ThreadContext;

public class CheckDBConnection3 extends Thread {

	private static Application _application;
	
	private String dbURL;
	private boolean connectionWorking=false;
	private long timeout=10;
	
	public CheckDBConnection3(){
		try {
			Class.forName("org.postgresql.Driver");
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		_application = Application.get();
	}
	
	
	
	public CheckDBConnection3(String dbURL){
		try {
			Class.forName("org.postgresql.Driver");
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		this.dbURL = dbURL;
		_application = Application.get();
	}
	
 
	 
	public synchronized void setDbURL(String dbURL) {
		if(this.dbURL!=null && dbURL!=null) {
			connectionWorking=false;
		}
		this.dbURL = dbURL;
	} 
	
	public synchronized String getDbUrl() {
		return dbURL;
	}
	
	@Override
	public void run(){
		try {
			if(!Application.exists()){
				ThreadContext.setApplication(_application);
			}
			while (!Thread.currentThread().isInterrupted()) {
				try {
					if(isWorking()) {
						if(!connectionWorking) {
							OnDBConnectionValid();
							connectionWorking=true;
							timeout=10000;
						}
					}else{
						
						OnDBConnectionInvalid(null);
						connectionWorking=false;
						timeout=1000;
					};
	            } catch (Exception et) {
	            	et.printStackTrace();
	            }
				Thread.sleep(timeout);
			}
			ThreadContext.detach();
			OnDBConnectionInvalid(null);
		}catch(Exception e){
			OnDBConnectionInvalid(e);
		} finally {
			OnDBConnectionInvalid(null);
			ThreadContext.detach();
		}

	}
	
	private boolean isWorking(){
		try{
			if(this.dbURL==null || this.dbURL.isEmpty()) return false;
			Connection dbConn = DriverManager.getConnection(this.dbURL+"&connectTimeout=5"); 
			if(dbConn==null) return false;
			if(dbConn.isClosed()) return false;
			dbConn.prepareStatement("SELECT 1").executeQuery();
			dbConn.close();
			return true;
		}catch(Exception e){
		}
		return false;
	}
	
	protected void OnDBConnectionValid() {};
	protected void OnDBConnectionInvalid(Exception e) {};
	 
	
}
