/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.helpers;


 

import org.apache.wicket.Application;
import org.apache.wicket.ThreadContext;
import de.kleinekuh.os.OsConnection;
 

 


public class CheckESConnection3 extends Thread {

	private static Application _application;
	
	private String esURL;
	private boolean connectionWorking=false;
	private long timeout=1000;
	
	public CheckESConnection3(){
		_application = Application.get();
	}
	
	
	public CheckESConnection3(String esURL){
		this.esURL = esURL;
		_application = Application.get();
	}
	
	public synchronized void setEsURL(String esURL) {
		if(this.esURL!=null && esURL!=null) {
			connectionWorking=false;
		}
		this.esURL = esURL;
	} 
	public synchronized String getEsURL() {
		return this.esURL;
	} 
	
	@Override
	public void run(){
		try {
			if(!Application.exists()){
				ThreadContext.setApplication(_application);
			}
			while (!Thread.currentThread().isInterrupted()) {
				try {
					if(isWorking()) {
						if(!connectionWorking) {
							OnEsConnectionValid();
							connectionWorking=true;
							timeout=10000;
						}
					}else{
						OnEsConnectionInvalid(null);
						connectionWorking=false;
						timeout=1000;
						//System.out.println("ES not woring "+timeout);
					}
	                
	            } catch (Exception e) {
	            } 
				Thread.sleep(timeout);
			}
			OnEsConnectionInvalid(null);
			ThreadContext.detach();
		}catch(Exception e){
			OnEsConnectionInvalid(e);
		} finally {
			OnEsConnectionInvalid(null);
			ThreadContext.detach();
		}
	}
	
	private boolean isWorking(){
		try{
			if(this.esURL==null || this.esURL.isEmpty()) return false;
			OsConnection esConn = new OsConnection(this.esURL);
			if(!esConn.test()) return false;
			esConn.close();
			return true;
		}catch(Exception e){
		}
		return false;
	}
	
	protected void OnEsConnectionValid() {};
	protected void OnEsConnectionInvalid(Exception e) {};
	 
	
}
