/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.helpers;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.io.Serializable;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.SocketAddress;
import java.nio.file.Files;
import java.nio.file.Paths;

import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Variables.Variable;
import org.biffo.dms.entity.Variables.VariableFilterClass;
import org.biffo.dms.entity.Variables.VariableGroupType;


public class ConfigHelpers implements Serializable{
 
	private static final long serialVersionUID = 1L;

	
	public static boolean checkTesseractExecutable(String executableToCheck) {
		try {
			StringBuilder sbError = new StringBuilder();
			ProcessBuilder pb = new ProcessBuilder(executableToCheck, "--version");
			Process p = pb.start();
			BufferedReader reader = new BufferedReader(new InputStreamReader(p.getInputStream()));
			String line;
			while ((line = reader.readLine()) != null) {
				sbError.append(line).append("\n");
			}

			int exitCode = p.waitFor();
			if(exitCode!=0) {
				throw new Exception(line);
			}
			return true;
		}catch(Exception e) {
		}
		return false;
	}
	
	public static boolean checkFileSystemFolderExists(String folderToCheck) {
		try {
			File fIn = new File(folderToCheck);
			if(fIn.exists() && fIn.isDirectory()) return true;
		}catch(Exception e) {
		}
		return false;
	}
	
	public static boolean checkFileSystemFileExists(String fileToCheck) {
		try {
			File fIn = new File(fileToCheck);
			if(fIn.exists() && fIn.isFile()) return true;
		}catch(Exception e) {
		}
		return false;
	}
	
	public static boolean isWritable(String folderToCheck) {
		File file = new File(folderToCheck);
	    if (file.exists() && file.isDirectory()) {
	    	try {
	    		File fX = new File(folderToCheck+"/xxx.txt");
	    		fX.createNewFile();
	    		fX.delete();
	    		return true;
	    	}catch(Exception e) {
	    	}
	    }
	    return false;
	}
	
	public static boolean createFolder(String folderToCreate) {
		try {
			if(!checkFileSystemFolderExists(folderToCreate)) {
				Files.createDirectory(Paths.get(folderToCreate));
			}
			return true;
		}catch(Exception e) {
			e.printStackTrace();
		}
		return false;
	}
	
	public static boolean checkServerNameExists(int id,  String serverName, VariableGroupType groupType) {
		try {
			
			VariableFilterClass vfc = new VariableFilterClass();
			vfc.setVariableGroupType(groupType);
			vfc.setVariableGroup(serverName);
			if(groupType==VariableGroupType.FOLDERS)vfc.setVariableName("folder");
			else if(groupType==VariableGroupType.API)vfc.setVariableName("classname");
			else vfc.setVariableName("port");
			Variable variable = StaticLocators.getVariables().getEntity(vfc);
			if(null==variable) return false;
			else if(variable.getId()==id) return false;
		}catch(Exception e) {
		}
		return true;
	}
	
	public static boolean checkServerPortExists(int id, int port, VariableGroupType groupType) {
		try {
			VariableFilterClass vfc = new VariableFilterClass();
			vfc.setVariableGroupType(groupType);
			vfc.setVariableName("port");
			vfc.setVariableValue(""+port);
			Variable variable = StaticLocators.getVariables().getEntity(vfc);
			if(null==variable) return false;
			else if(variable.getId()==id) return false;
		}catch(Exception e) {
		}
		return true;
	}
	
	public static boolean checkUploadFolderExists(int id, int folderId, String folder, VariableGroupType groupType) {
		try {
			VariableFilterClass vfc = new VariableFilterClass();
			vfc.setVariableGroupType(groupType);
			vfc.setVariableName("folder");
			vfc.setVariableValue(folder);
			Variable variable = StaticLocators.getVariables().getEntity(vfc);
			if(null==variable) return false;
			else if(variable.getId()==folderId) return false;
		}catch(Exception e) {
		}
		return true;
	}
	
	public static boolean portUsed(String host, int port) {
		SocketAddress sockaddr = new InetSocketAddress(host, port);
		Socket socket = new Socket();
		try {
             socket.connect(sockaddr, 100);
             socket.close();
             return true;
		}catch (Exception e) {
		}
		return false;
	}
	
	
	
	
	
}
