/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.helpers;

import java.util.Date;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;

import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Helpers.EntityUUID;
import org.biffo.dms.entity.Relation.Relation;
import org.biffo.dms.entity.Relation.RelationFilterClass;
import org.biffo.dms.entity.Relation.RelationStatusType;
import org.biffo.dms.entity.Relation.RelationType;
import org.biffo.dms.entity.Tasks.Task;
import org.biffo.dms.entity.Tasks.TaskFilterClass;
import org.biffo.dms.entity.Tasks.TaskStatusType;
import org.biffo.dms.entity.Variables.Impl.IVariablesImpl;
import org.biffo.dms.entity.user.Userrelation.UserRelation;
import org.biffo.dms.entity.user.Userrelation.UserRelationFilterClass;
import org.biffo.dms.entity.user.Userrelation.UserRelationType;
import org.biffo.dms.entity.user.Users.User;

public class DmsDBHelpersUser {

	private static final String _className = "DmsDBHelpersUser";
	
	private static boolean documentToUserRelationExists(HashSet<User> users, int idrelation2) {
		for(User user : users) {
			if(user.getId()==idrelation2) return true;
		}
		return false;
	}
	
	private static boolean userExistsOnDocument(Integer documentId, Integer userId, int tennant) throws Exception {
		RelationFilterClass rfc = new RelationFilterClass();
		rfc.setIdTenant(tennant);
		rfc.setIdrelation1(documentId);
		rfc.setIdrelation2(userId);
		rfc.setRelationType(RelationType.DOCUMENT_TO_USER);
		if(StaticLocators.getRelations().getNumberOfEntities(rfc)>0) return true;
		return false;
	}
	
	
	
	
	public static void addUserToDocument(Integer documentId, Integer idUserToAdd, String user, int tennant) throws Exception {
		if(userExistsOnDocument(documentId, idUserToAdd, tennant)) return;

		Relation relation = new Relation();
		relation.setIdrelation1(documentId);
		relation.setIdrelation2(idUserToAdd);
		relation.setErstelltAm(new Date());
		relation.setErstelltVon(user);
		relation.setGeaendertAm(relation.getErstelltAm());
		relation.setGeaendertVon(relation.getErstelltVon());
		relation.setIdTenant(tennant);
		relation.setRelationType(RelationType.DOCUMENT_TO_USER);
		relation.setRelationStatus(RelationStatusType.ACTIVE);
		StaticLocators.getRelations().addEntity(relation);
		StaticLocators.getOpenSearchProcessing().updateDocument(StaticLocators.getDocuments().getEntityById(documentId));
	}
	
	public static void addUserTaskToDocument(Integer documentId, Integer idUserToAdd, String user, int tennant) throws Exception {
		
		Task task = new Task();
		task.setErstelltAm(null);
		task.setErstelltVon(user);
		task.setIdTenant(tennant);
		task.setIdUser(idUserToAdd);
		task.setObjectStatusType(TaskStatusType.INBOX, true);
		
		task.setTaskname("Inbox_"+StaticLocators.getDateTimeStamp());
		task.setErstelltAm(new Date());
		task.setErstelltVon(user);
		task.setGeaendertAm(task.getErstelltAm());
		task.setGeaendertVon(task.getErstelltVon());
		EntityUUID.createTaskUUID(task);
		int taskId = StaticLocators.getTasks().addEntity(task);
		if(taskId!=-1) {
			Relation relation = new Relation();
			relation.setRelationType(RelationType.DOCUMENT_TO_TASK);
			relation.setIdrelation1(documentId);
			relation.setIdrelation2(taskId);
			relation.setErstelltAm(new Date());
			relation.setErstelltVon(user);
			relation.setGeaendertAm(relation.getErstelltAm());
			relation.setGeaendertVon(relation.getErstelltVon());
			relation.setIdTenant(tennant);
			relation.setRelationStatus(RelationStatusType.ACTIVE);
			StaticLocators.getRelations().addEntity(relation);
		}
	}
	
	
	
	
	
	public static void removeDocumentRelationToTask(Integer documentId, String user, int tennantId) throws Exception {
		RelationFilterClass rfc = new RelationFilterClass();
		rfc.setRelationType(RelationType.DOCUMENT_TO_TASK);
		rfc.setIdrelation1(documentId);
		rfc.setIdTenant(tennantId);
		
		List<Relation> relationsToDelete = StaticLocators.getRelations().getEntities(-1, -1, "", rfc);
		for(int i=0;i<relationsToDelete.size();i++) {
			
			try {
				StaticLocators.getTasks().deleteEntityById(relationsToDelete.get(i).getIdrelation2());
				StaticLocators.getRelations().deleteEntityById(relationsToDelete.get(i).getId());
			}catch(Exception e) {
				DmsException dmsException = new DmsException(
						1000, 
						"E", 
						_className, 
						"removeDocumentRelationToTask",
						null,
						null,
						e
					);
				StaticLocators.getMessageHandler().createMessage(dmsException, DmsWebSession.getSession());
			}
		}
	}
	
	public static void deleteUsersFromDocuments(HashSet<Integer> documentIds, HashSet<User> users, String user, int tennant) throws Exception {
		try {
			for(Integer documentId : documentIds) {
				RelationFilterClass rfc = new RelationFilterClass();
				rfc.setIdTenant(tennant);
				rfc.setIdrelation1(documentId);
				rfc.setRelationType(RelationType.DOCUMENT_TO_USER);
				
				List<Relation> relationsToDelete = StaticLocators.getRelations().getEntities(-1, -1, "", rfc);
				if(relationsToDelete.size()>0) {
					for(Relation relationToDelete : relationsToDelete) {
						if(documentToUserRelationExists(users, relationToDelete.getIdrelation2())) {
							StaticLocators.getRelations().deleteEntityById(relationToDelete.getId());	
						}
					}
				}
				
				
				rfc = new RelationFilterClass();
				rfc.setIdTenant(tennant);
				rfc.setIdrelation1(documentId);
				rfc.setRelationType(RelationType.DOCUMENT_TO_TASK);
				
				relationsToDelete = StaticLocators.getRelations().getEntities(-1, -1, "", rfc);
				if(relationsToDelete.size()>0) {
					for(Relation relationToDelete : relationsToDelete) {
						
						if(documentToUserRelationExists(users, relationToDelete.getIdrelation2())){
							StaticLocators.getTasks().deleteEntityById(relationToDelete.getIdrelation2());
							StaticLocators.getRelations().deleteEntityById(relationToDelete.getId());
						}
						
					}
				}
				StaticLocators.getOpenSearchProcessing().updateDocument(StaticLocators.getDocuments().getEntityById(documentId));
			}
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"deleteUsersFromDocuments",
					null,
					null,
					e
				);
			StaticLocators.getMessageHandler().createMessage(dmsException, DmsWebSession.getSession());
		}
	}
	
	
	
	
	public static void addUsersToDocuments(HashSet<Integer> documentIds, HashSet<User> users, String user, int tennant, boolean withTask) throws Exception {
		try {
			for(Integer documentId : documentIds) {
				for(User userToAdd : users) {
					addUserToDocument(
						documentId, 
						userToAdd.getId(), 
						user,
						tennant
					);
					if(withTask) {
						addUserTaskToDocument(
							documentId, 
							userToAdd.getId(), 
							user,
							tennant
						);
					}
				}
			}
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"addUsersToDocuments",
					null,
					null,
					e
				);
			StaticLocators.getMessageHandler().createMessage(dmsException, DmsWebSession.getSession());
		}
	}
	
	public static void replaceUsersFromDocuments(HashSet<Integer> documentIds, HashSet<User> users, String user, int tennant, boolean withTask) throws Exception {
		try {
			for(Integer documentId : documentIds) {
				RelationFilterClass rfc = new RelationFilterClass();
				rfc.setIdTenant(tennant);
				rfc.setIdrelation1(documentId);
				rfc.setRelationType(RelationType.DOCUMENT_TO_USER);
				
				List<Relation> relationsToDelete = StaticLocators.getRelations().getEntities(-1, -1, "", rfc);
				if(relationsToDelete.size()>0) {
					for(Relation relationToDelete : relationsToDelete) {
						if(!documentToUserRelationExists(users, relationToDelete.getIdrelation2())) {
							StaticLocators.getRelations().deleteEntityById(relationToDelete.getId());
						}
					}
				}
				rfc = new RelationFilterClass();
				rfc.setIdTenant(tennant);
				rfc.setIdrelation1(documentId);
				rfc.setRelationType(RelationType.DOCUMENT_TO_TASK);
				
				relationsToDelete = StaticLocators.getRelations().getEntities(-1, -1, "", rfc);
				if(relationsToDelete.size()>0) {
					for(Relation relationToDelete : relationsToDelete) {
						
						if(!documentToUserRelationExists(users, relationToDelete.getIdrelation2())){
							StaticLocators.getTasks().deleteEntityById(relationToDelete.getIdrelation2());
							StaticLocators.getRelations().deleteEntityById(relationToDelete.getId());
						}
					}
				}
				
				for(User userToAdd : users) {
					addUserToDocument(documentId, userToAdd.getId(), user, tennant);
					if(withTask) {
						addUserTaskToDocument(documentId, userToAdd.getId(), user, tennant);
					}
				}
				
			}
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"replaceUsersFromDocuments",
					null,
					null,
					e
				);
			StaticLocators.getMessageHandler().createMessage(dmsException, DmsWebSession.getSession());
		}
	}
	
	public static boolean isUserLinkedToServer(User user) {
		List<IVariablesImpl> serverImpls = new LinkedList<IVariablesImpl>();
		DmsDBHelpersUser.retrieveServerProcessesFromServerUser(serverImpls, user, RelationType.FTP_USER_TO_FTP_SERVER);
		DmsDBHelpersUser.retrieveServerProcessesFromServerUser(serverImpls, user, RelationType.SFTP_USER_TO_SFTP_SERVER);
		DmsDBHelpersUser.retrieveServerProcessesFromServerUser(serverImpls, user, RelationType.SMTP_USER_TO_SMTP_SERVER);
		if(serverImpls.size()>0) return true;
		return false;
	}
	
	
	public static boolean retrieveServerProcessesFromServerUser(List<IVariablesImpl> serverList, User user, RelationType relationType) {
		boolean found=false;
		try {
			List<IVariablesImpl> servers = new LinkedList<>();
			servers = ImportProcessHelpers.getServerProcesses();
			RelationFilterClass rfc = new RelationFilterClass();
			rfc.setIdrelation1(user.getId());
			rfc.setRelationType(relationType);
			List<Relation> entities = StaticLocators.getRelations().getEntities(-1, -1, "", rfc);
			for(int i=0;i<entities.size();i++) {
				for(int j=0;j<servers.size();j++) {
					if(servers.get(j).getId()==entities.get(i).getIdrelation2()) {
						serverList.add(servers.get(j));
						found=true;
					}
				}
			}
			return found;
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"getImportProcessFromServerUser",
					null,
					null,
					e
			);
			StaticLocators.getMessageHandler().createMessage(dmsException, DmsWebSession.getSession());
		}
		return found;
	}
	
	
	public static void deleteUser(User user, String userName, int tennant) throws Exception {
	
		try {
			deleteTasksForUser(user);
			deleteUserRelations(user, UserRelationType.GROUP_TO_USER, true);
			deleteUserRelations(user, UserRelationType.USER_TO_TENANT, false);
			deleteUserRelations(user, UserRelationType.USER_TO_USERLOG, false);
			
			deleteRelation(user, RelationType.DOCUMENT_TO_USER, true);
			deleteRelation(user, RelationType.FTP_USER_TO_FTP_SERVER, false);
			deleteRelation(user, RelationType.SFTP_USER_TO_SFTP_SERVER, false);
			deleteRelation(user, RelationType.SMTP_USER_TO_SMTP_SERVER, false);
			deleteRelation(user, RelationType.USER_TO_IMPORT_PROCESS, false);
			
			StaticLocators.getUsers().deleteEntityById(user.getId());
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"deleteUser",
					null,
					null,
					e
				);
				StaticLocators.getMessageHandler().createMessage(dmsException, DmsWebSession.getSession());
		}
	
	}
		
	public static void deleteUserRelations(User user, UserRelationType userRelationType, boolean Relation2) {
		try {
			UserRelationFilterClass urfc = new UserRelationFilterClass();
			urfc.setUserRelationType(userRelationType);
			if(Relation2)urfc.setIdrelation2(user.getId());
			else urfc.setIdrelation1(user.getId());
			List<UserRelation> entitiesToDelete = StaticLocators.getUserRelations().getEntities(-1, -1, "", urfc);
			for(UserRelation entity : entitiesToDelete) {
				StaticLocators.getUserRelations().deleteEntityById(entity.getId());
			}
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
				1000, 
				"E", 
				_className, 
				"deleteUserRelations",
				null,
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsException, DmsWebSession.getSession());
		}
	}
		
	public static void deleteRelation(User user, RelationType relationType, boolean Relation2) {
		try {
			RelationFilterClass rfc = new RelationFilterClass();
			rfc.setRelationType(relationType);
			if(Relation2)rfc.setIdrelation2(user.getId());
			else rfc.setIdrelation1(user.getId());
			List<Relation> entitiesToDelete = StaticLocators.getRelations().getEntities(-1, -1, "", rfc);
			for(Relation entity : entitiesToDelete) {
				StaticLocators.getRelations().deleteEntityById(entity.getId());
				if(relationType==RelationType.DOCUMENT_TO_USER) {
					StaticLocators.getOpenSearchProcessing().updateDocument(StaticLocators.getDocuments().getEntityById(entity.getIdrelation1()));
				}
			}
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
				1000, 
				"E", 
				_className, 
				"deleteRelation",
				null,
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsException, DmsWebSession.getSession());
		}
	}
		
		
	public static void deleteTasksForUser(User user) {
		try {
			TaskFilterClass tfc = new TaskFilterClass();
			tfc.setIdUser(user.getId());
			
			List<Task> lstTasks = StaticLocators.getTasks().getEntities(-1, -1, "", tfc);
			for(int i=0;i<lstTasks.size();i++) {
				int idTask = lstTasks.get(i).getId();
				RelationFilterClass rfc = new RelationFilterClass();
				rfc.setRelationType(RelationType.DOCUMENT_TO_TASK);
				rfc.setIdrelation2(idTask);
				
				List<Relation> lstRelations = StaticLocators.getRelations().getEntities(-1, -1, "", rfc);
				for(Relation relation : lstRelations) {
					StaticLocators.getRelations().deleteEntityById(relation.getId());
				}
				StaticLocators.getTasks().deleteEntityById(idTask);
			}
			
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"deleteTasksForUser",
					null,
					null,
					e
				);
				StaticLocators.getMessageHandler().createMessage(dmsException, DmsWebSession.getSession());
		}
	}	
	
	
	
}
