/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.helpers;

import java.util.LinkedList;
import java.util.List;

import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Relation.Relation;
import org.biffo.dms.entity.Relation.RelationFilterClass;
import org.biffo.dms.entity.Relation.RelationStatusType;
import org.biffo.dms.entity.Relation.RelationType;
import org.biffo.dms.entity.Variables.Variable;
import org.biffo.dms.entity.Variables.VariableFilterClass;
import org.biffo.dms.entity.Variables.VariableGroupType;
import org.biffo.dms.entity.Variables.Impl.ApiModuleImpl;
import org.biffo.dms.entity.Variables.Impl.FTPServerImpl;
import org.biffo.dms.entity.Variables.Impl.FolderServerImpl;
import org.biffo.dms.entity.Variables.Impl.IVariablesImpl;
import org.biffo.dms.entity.Variables.Impl.ImportServerImpl;
import org.biffo.dms.entity.Variables.Impl.SFTPServerImpl;
import org.biffo.dms.entity.Variables.Impl.SmtpServerImpl;

public class ImportProcessHelpers {

	public static List<IVariablesImpl> getServerProcesses() throws Exception {
		List<IVariablesImpl> lstServers = new LinkedList<IVariablesImpl>();
		lstServers.addAll(getServerFTPProcesses());
		lstServers.addAll(getServerSFTPProcesses());
		lstServers.addAll(getServerSMTPProcesses());
		lstServers.addAll(getServerFolderProcesses());
		return lstServers;
	}
	
	
	public static List<IVariablesImpl> getServerFTPProcesses() throws Exception {
		VariableFilterClass vfc = new VariableFilterClass();
		vfc.setVariableGroupType(VariableGroupType.FTP);
		vfc.setVariableName("port");
		List<Variable> entities = StaticLocators.getVariables().getEntities(-1, -1, "", vfc);
		List<IVariablesImpl> lstServers = new LinkedList<IVariablesImpl>();
		for(Variable entity:entities) {
			FTPServerImpl serverImpl = new FTPServerImpl();
			serverImpl.setVariable(entity);
			
			vfc.setVariableGroup(entity.getVariableGroup());
			vfc.setVariableName(null);
			List<Variable> entities2 = StaticLocators.getVariables().getEntities(-1, -1, "", vfc);
			for(Variable entity2:entities2) {
				serverImpl.setVariable(entity2);
			}
			lstServers.add(serverImpl);
		}
		return lstServers;
	}
	
	
	public static List<IVariablesImpl> getServerSMTPProcesses() throws Exception {
		VariableFilterClass vfc = new VariableFilterClass();
		vfc.setVariableGroupType(VariableGroupType.SMTP);
		vfc.setVariableName("port");
		List<Variable> entities = StaticLocators.getVariables().getEntities(-1, -1, "", vfc);
		List<IVariablesImpl> lstServers = new LinkedList<IVariablesImpl>();
		for(Variable entity:entities) {
			SmtpServerImpl serverImpl = new SmtpServerImpl();
			serverImpl.setVariable(entity);
			
			vfc.setVariableGroup(entity.getVariableGroup());
			vfc.setVariableName(null);
			List<Variable> entities2 = StaticLocators.getVariables().getEntities(-1, -1, "", vfc);
			for(Variable entity2:entities2) {
				serverImpl.setVariable(entity2);
			}
			lstServers.add(serverImpl);
		}
		return lstServers;
	}
	
	public static List<IVariablesImpl> getServerSFTPProcesses() throws Exception {
		VariableFilterClass vfc = new VariableFilterClass();
		vfc.setVariableGroupType(VariableGroupType.SFTP);
		vfc.setVariableName("port");
		List<Variable> entities = StaticLocators.getVariables().getEntities(-1, -1, "", vfc);
		List<IVariablesImpl> lstServers = new LinkedList<IVariablesImpl>();

		for(Variable entity:entities) {
			SFTPServerImpl serverImpl = new SFTPServerImpl();
			serverImpl.setVariable(entity);
			
			vfc.setVariableGroup(entity.getVariableGroup());
			vfc.setVariableName(null);
			List<Variable> entities2 = StaticLocators.getVariables().getEntities(-1, -1, "", vfc);
			for(Variable entity2:entities2) {
				serverImpl.setVariable(entity2);
			}
			lstServers.add(serverImpl);
		}
		return lstServers;
	}
	
	public static List<IVariablesImpl> getServerFolderProcesses() throws Exception {
		VariableFilterClass vfc = new VariableFilterClass();
		vfc.setVariableGroupType(VariableGroupType.FOLDERS);
		vfc.setVariableName("folder");
		List<Variable> entities = StaticLocators.getVariables().getEntities(-1, -1, "", vfc);
		List<IVariablesImpl> lstServers = new LinkedList<IVariablesImpl>();
		for(Variable entity:entities) {
			FolderServerImpl serverImpl = new FolderServerImpl();
			serverImpl.setVariable(entity);
			
			vfc.setVariableGroup(entity.getVariableGroup());
			vfc.setVariableName(null);
			List<Variable> entities2 = StaticLocators.getVariables().getEntities(-1, -1, "", vfc);
			for(Variable entity2:entities2) {
				serverImpl.setVariable(entity2);
			}
			lstServers.add(serverImpl);
		}
		return lstServers;
	}
	
	public static List<IVariablesImpl> getModuleProcesses() throws Exception {
		VariableFilterClass vfc = new VariableFilterClass();
		vfc.setVariableGroupType(VariableGroupType.API);
		vfc.setVariableName("classname");
		List<Variable> entities = StaticLocators.getVariables().getEntities(-1, -1, "", vfc);
		List<IVariablesImpl> lstServers = new LinkedList<IVariablesImpl>();
		for(Variable entity:entities) {
			ApiModuleImpl apiImpl = new ApiModuleImpl();
			apiImpl.setVariable(entity);
			
			vfc.setVariableGroup(entity.getVariableGroup());
			vfc.setVariableName(null);
			List<Variable> entities2 = StaticLocators.getVariables().getEntities(-1, -1, "", vfc);
			for(Variable entity2:entities2) {
				apiImpl.setVariable(entity2);
			}
			lstServers.add(apiImpl);
		}
		return lstServers;
	}
	
	public static List<IVariablesImpl> getImportProcesses() throws Exception {

		VariableFilterClass vfc = new VariableFilterClass();
		vfc.setVariableGroupType(VariableGroupType.IMPORT_PROCESS);
		vfc.setVariableName("processname");
		List<Variable> entities = StaticLocators.getVariables().getEntities(-1, -1, "", vfc);
		List<IVariablesImpl> lstServers = new LinkedList<IVariablesImpl>();
		for(Variable entity:entities) {
			ImportServerImpl importProcess = new ImportServerImpl();
			importProcess.setVariable(entity);
			vfc.setVariableGroup(entity.getVariableGroup());
			vfc.setVariableName(null);
			List<Variable> entities2 = StaticLocators.getVariables().getEntities(-1, -1, "", vfc);
			for(Variable entity2:entities2) {
				importProcess.setVariable(entity2);
			}
			lstServers.add(importProcess);
		}
		return lstServers;
	}
	
	
	
	public static int[] getAssignedTagsToImportProcess(int idImportProcess){
		int rc[] = null;
		try {
			RelationFilterClass rfc = new RelationFilterClass();
			rfc.setRelationType(RelationType.TAG_TO_IMPORT_PROCESS);
			rfc.setIdrelation2(idImportProcess);
			rfc.setRelationStatus(RelationStatusType.ACTIVE);
			List<Relation> entities = StaticLocators.getRelations().getEntities(-1, -1, "", rfc);
			if(entities!=null && entities.size()>0) {
				rc = new int[entities.size()];
				for(int i=0;i<entities.size();i++)rc[i] = entities.get(i).getIdrelation1();
			}
			return rc;
		}catch(Exception e) {
		}
		return null;
	}
	
	public static int[] getAssignedUsersToImportProcess(int idImportProcess, int idUser){
		try {
			List<Integer> lstRC = new LinkedList<Integer>();
			RelationFilterClass rfc = new RelationFilterClass();
			rfc.setRelationType(RelationType.USER_TO_IMPORT_PROCESS);
			rfc.setIdrelation2(idImportProcess);
			rfc.setRelationStatus(RelationStatusType.ACTIVE);
			List<Relation> entities = StaticLocators.getRelations().getEntities(-1, -1, "", rfc);
			for(Relation entity : entities) {
				lstRC.add(entity.getIdrelation1());
			}
			
			if(idUser!=-1 && !lstRC.contains(idUser)) lstRC.add(idUser);
			
			int[] array = new int[lstRC.size()];
			for(int i = 0; i < lstRC.size(); i++) array[i] = lstRC.get(i);
			return array;
		}catch(Exception e) {
		}
		return null;
	}
	
	public static int[] getAssignedUserGroupsToImportProcess(int idImportProcess){
		
		int rc[] = null;
		try {
			List<Integer> lstRC = new LinkedList<Integer>();
			RelationFilterClass rfc = new RelationFilterClass();
			rfc.setRelationType(RelationType.USERGROUP_TO_IMPORT_PROCESS);
			rfc.setIdrelation2(idImportProcess);
			rfc.setRelationStatus(RelationStatusType.ACTIVE);
			List<Relation> entities = StaticLocators.getRelations().getEntities(-1, -1, "", rfc);
			for(Relation entity : entities) {
				lstRC.add(entity.getIdrelation1());
			}
			
			if(entities!=null && entities.size()>0) {
				rc = new int[entities.size()];
				for(int i=0;i<entities.size();i++)rc[i] = entities.get(i).getIdrelation1();
			}
			return rc;
		}catch(Exception e) {
		}
		return null;
	}
	
	public static int[] getAssignedModulesToImportProcess(int idImportProcess){
		int rc[] = null;
		try {
			RelationFilterClass rfc = new RelationFilterClass();
			rfc.setRelationType(RelationType.MODULE_TO_IMPORT_PROCESS);
			rfc.setIdrelation2(idImportProcess);
			rfc.setRelationStatus(RelationStatusType.ACTIVE);
			List<Relation> entities = StaticLocators.getRelations().getEntities(-1, -1, "", rfc);
			if(entities!=null && entities.size()>0) {
				rc = new int[entities.size()];
				for(int i=0;i<entities.size();i++)rc[i] = entities.get(i).getIdrelation1();
			}
			return rc;
		}catch(Exception e) {
		}
		return null;
	}
	
	
	
}
