/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.helpers;

import java.io.Serializable;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import org.biffo.dms.DmsException;
import org.biffo.dms.DmsProcessingMapper;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Helpers.EntityUUID;
import org.biffo.dms.entity.Tags.Tag;
import org.biffo.dms.entity.Variables.Impl.ApiModuleImpl;

public class ModuleTagHelper implements Serializable {


	private final static String tagsDelim=";";
	private final static String tagDelim="\\|";
	
	private static final long serialVersionUID = 1L;

	
	public static List<Tag> getTagListFromModule(ApiModuleImpl apiModule, DmsWebSession session){
		if(apiModule==null) return null;
		if(null==apiModule.getTags()) return null;
		
		List<Tag> lstTags = new LinkedList<>();
		String tagsArr[] = apiModule.getTags().split(tagsDelim);
		for(String stag : tagsArr) {
			String tagArr[] = stag.split(tagDelim);
			if(null!=tagArr && tagArr.length==3) {
				Tag tag = new Tag();
				tag.setTagValue(tagArr[0]);
				tag.setTagFgColor(tagArr[1].replaceAll("#", ""));
				tag.setTagBgColor(tagArr[2].replaceAll("#", ""));
				tag.setTagName(tag.getTagValue());
				tag.setTagGroup("NA");
				tag.setIdTenant(session.getCurrentTenantId());
				EntityUUID.createTagUUID(tag);
				try {
					int tagId= StaticLocators.getTags().getEntityId(tag);
					if(tagId==-1) {
						tag.setGeaendertAm(new Date());
						tag.setGeaendertVon(session.getCurrentUser());
						tag.setErstelltAm(tag.getGeaendertAm());
						tag.setErstelltVon(tag.getGeaendertVon());
						tagId = StaticLocators.getTags().addEntity(tag);
					}
					lstTags.add(StaticLocators.getTags().getEntityById(tagId));
				} catch (Exception e) {
					DmsException dmsException = new DmsException(
						1000, 
						"E", 
						"ModuleTagHelper", 
						"getTagListFromModule",
						null,
						null,
						e
					);
					StaticLocators.getMessageHandler().createMessage(dmsException, session);
				}
			}
		}
		return lstTags;
	}
	
	public static List<Tag> getTagListFromModule(ApiModuleImpl apiModule, DmsProcessingMapper dmp){
		if(apiModule==null) return null;
		if(null==apiModule.getTags()) return null;
		
		List<Tag> lstTags = new LinkedList<>();
		String tagsArr[] = apiModule.getTags().split(tagsDelim);
		for(String stag : tagsArr) {
			String tagArr[] = stag.split(tagDelim);
			if(null!=tagArr && tagArr.length==3) {
				Tag tag = new Tag();
				tag.setTagValue(tagArr[0]);
				tag.setTagFgColor(tagArr[1].replaceAll("#", ""));
				tag.setTagBgColor(tagArr[2].replaceAll("#", ""));
				tag.setTagName(tag.getTagValue());
				tag.setTagGroup("NA");
				EntityUUID.createTagUUID(tag);
				try {
					int tagId= dmp.getTagsDB().getEntityId(tag);
					if(tagId!=-1) {
						lstTags.add(dmp.getTagsDB().getEntityById(tagId));
					}
					
				} catch (Exception e) {
					DmsException dmsException = new DmsException(
						1000, 
						"E", 
						"ModuleTagHelper", 
						"getTagListFromModule",
						null,
						null,
						e
					);
					StaticLocators.getMessageHandler().createMessage(dmsException, null);
				}
			}
		}
		return lstTags;
	}
	
	
	
	
}
