/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.helpers;

import java.io.Serializable;

import org.biffo.dms.Types.SearchStatusType;
import org.biffo.dms.entity.Document.DocumentOSFilterClass;

public class SearchConditionBuilder implements Serializable{

	private static final long serialVersionUID = 1L;

	public static String buildCondition(DocumentOSFilterClass dsfc, SearchConditionBuilderType...builderTypes) {

		if(null==builderTypes || builderTypes.length==0) return null;
		String condition = "";
		int x=0;
		
		for(int i=0;i<builderTypes.length;i++) {
			SearchConditionBuilderType builderType = builderTypes[i];
			switch(builderType){
				case DATE:
					if((dsfc.getSearchStatusType(SearchStatusType.DFROM_SET) || dsfc.getSearchStatusType(SearchStatusType.DTO_SET))){
						if(dsfc.getDateSearchWhere()!=null) {
							if(x>0) condition = condition + " AND ";
							condition = condition + dsfc.getDateSearchWhere();
							x++;
						}else {
							if(x>0) condition = condition + " AND ";
							condition = condition + " 1=0 ";
							x++;
						}
					}
					break;
				case SINGLEFIELD:
					
					if(dsfc.getSearchStatusType(SearchStatusType.SINGLEFIELD_SET)){
						if(dsfc.getSingelFieldSearchWhere()!=null) {
							if(x>0) condition = condition + " AND ";
							condition = condition + dsfc.getSingelFieldSearchWhere();
							x++;
						}
					}
					break;
				case TAGS:
					
					if(dsfc.getSearchStatusType(SearchStatusType.TAG_SET)){
						if(dsfc.getTagsSearchWhere()!=null) {
							if(x>0) condition = condition + " AND ";
							condition = condition + dsfc.getTagsSearchWhere();
							x++;
						}
					}
					break;
			}
			
		}
		if(x>0) {
			return condition;
		}else {
			return null;
		}
	}
	
}
