/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.helpers;

import java.io.Serializable;
import java.util.LinkedList;

public class SearchHelper implements Serializable{

 
	private static final long serialVersionUID = 1L;
	
	private int _idDocument;
	private String _searchString;
	private LinkedList<String> _foundPages = new LinkedList<String>();
	
	public SearchHelper() {
	}
	
	public SearchHelper(int idDocument, String searchString,LinkedList<String> foundPages ) {
		_idDocument = idDocument;
		_searchString = searchString;
		if(null!=foundPages)_foundPages.addAll(foundPages);
	}

	public int getIdDocument() {return _idDocument;}
	public void setIdDocument(int idDocument) {this._idDocument = idDocument;}

	public String getSearchString() {return _searchString;}
	public void setSearchString(String searchString) {this._searchString = searchString;}

	public LinkedList<String> getFoundPages() {return _foundPages;}
	public void setFoundPages(LinkedList<String> foundPages) {
		_foundPages.clear();
		if(null!=foundPages)_foundPages.addAll(foundPages);
	}
	
	public String getFilterArray() {
		if(_foundPages!=null && _foundPages.size()>0) {
			String strArr[] = _foundPages.toArray(new String[_foundPages.size()]);
			StringHelpers.sortNumericStringArray(strArr, strArr.length);
			return "["+String.join(",", strArr)+"]";
		}
		return null;
	}
	
	
	
	

	@Override
	public String toString() {
		return "SearchHelper [_idDocument=" + _idDocument + ", _searchString=" + _searchString + ", _foundPages="
				+ _foundPages + "]";
	}

}
