/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.helpers;

public enum TesseractOemType {
	_0(0, "Legacy engine only."),
	_1(1, "Neural nets LSTM engine only."),
	_2(2, "Legacy + LSTM engines."),
	_3(3, "Default, based on what is available."),
	;

	private final int _code;
    private final String _description;

    TesseractOemType(int code, String description){
    	_code = code;
    	_description = description;
    }

    public String getDescription(){
    	return _description;
    }
	public int getCode(){
		return _code;
	}

	public static TesseractOemType getTesseractOemTypeByCode(int code) {
		for(TesseractOemType vt : TesseractOemType.values()) {
			if(vt.getCode()==code) return vt;
		}
		return null;
	}

	public static TesseractOemType getTesseractOemTypeByName(String name) {
		if(null==name || name.isEmpty()) return null;
		String temp = name.toLowerCase();
		for(TesseractOemType vt : TesseractOemType.values()) {
			if(temp.equals(vt.name().toLowerCase())) return vt;
		}
		return null;

	}
}
