/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.helpers;

public enum TesseractPsmType {
	_0(0, "Orientation and script detection (OSD) only."),
	_1(1, "Automatic page segmentation with OSD."),
	_2(2, "Automatic page segmentation, but no OSD, or OCR. (not implemented)"),
	_3(3, "Fully automatic page segmentation, but no OSD. (Default)"),
	_4(4, "Assume a single column of text of variable sizes."),
	_5(5, "Assume a single uniform block of vertically aligned text."),
	_6(6, "Assume a single uniform block of text."),
	_7(7, "Treat the image as a single text line."),
	_8(8, "Treat the image as a single word."),
	_9(9, "Treat the image as a single word in a circle."),
	_10(10, "Treat the image as a single character."),
	_11(11, "Sparse text. Find as much text as possible in no particular order."),
	_12(12, "Sparse text with OSD."),
	_13(13, "Raw line. Treat the image as a single text line, bypassing hacks that are Tesseract-specific."),
	;
	
	
	private final int _code;
    private final String _description;

    TesseractPsmType(int code, String description){
    	_code = code;
    	_description = description;
    }

    public String getDescription(){
    	return _description;
    }
	public int getCode(){
		return _code;
	}

	public static TesseractPsmType getTesseractPsmTypeByCode(int code) {
		for(TesseractPsmType vt : TesseractPsmType.values()) {
			if(vt.getCode()==code) return vt;
		}
		return null;
	}

	public static TesseractPsmType getTesseractPsmTypeByName(String name) {
		if(null==name || name.isEmpty()) return null;
		String temp = name.toLowerCase();
		for(TesseractPsmType vt : TesseractPsmType.values()) {
			if(temp.equals(vt.name().toLowerCase())) return vt;
		}
		return null;

	}
}
