/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.lock;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.biffo.dms.Types.EntityType;
import org.biffo.dms.lock.Lock.LockType;



public class Locks implements Serializable {

	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private List<Lock> locks = new ArrayList<Lock>();
	
	public Locks(){
	}
	
	public synchronized void aquireLock(int entityId, int userId, EntityType entityType) throws Exception {
		aquireLock(entityId, userId, entityType, LockType.VIEW);
	}
	
	
	public synchronized void aquireLock(int entityId, int userId, EntityType entityType, LockType lockType) throws Exception {
		
		Lock lock = new Lock();
		lock.setEntityId(entityId);
		lock.setEntityType(entityType);
		lock.setUserId(userId);
		lock.setLockType(lockType);
		if(isLocked(lock)) {
			int lockedByUserId = getLock(lock).getUserId();
			throw new Exception("Entity allready locked by "+lockedByUserId);
		}
		lock.setAquiredTime(new Date());
		add(lock);
		//lock.setAquiredTime(new Date());
		//locks.add(lock);
	}
	
	public synchronized void removeLock(int entityId, EntityType entityType) throws Exception {
		int pos=-1;
		for(int i=0;i<locks.size();i++){
			Lock xLock = locks.get(i);
			if(entityId== xLock.getEntityId() && entityType ==xLock.getEntityType() ){
				pos=i;
				break;
			};
		}
		if(pos!=-1){
			locks.remove(pos);
		}
	}
	
	

	public synchronized void add(Lock lock) {
		locks.add(lock);
	}
	
	public synchronized void addLocks(List<Lock> locks) {
		this.locks.addAll(locks);
	}

	public synchronized boolean isLocked(Lock lock) {
		return null==getLock(lock) ? false : true;
	}
	
	public synchronized Lock getLock(Lock lock) {
		for(Lock xLock : locks){
			if(lock.getEntityId()== xLock.getEntityId() && lock.getEntityType() ==xLock.getEntityType() ) return xLock;
		}
		return null;
	}
	
	public synchronized void remove(Lock lock){
		if(isLocked(lock)){
			int pos = -1;
			for(int i=0;i<locks.size();i++){
				Lock xLock = locks.get(i);
				if(lock.getEntityId()== xLock.getEntityId() && lock.getEntityType() ==xLock.getEntityType() ){
					pos=i;
					break;
				};
			}
			if(pos!=-1){
				locks.remove(pos);
			}
		}
	}
	
	public synchronized void removeLocks(List<Lock> locks){
		for(Lock lock :locks)remove(lock);
	}
	
	
	public synchronized void removeAllUserLocks(int userId){
		
		List<Lock> lockTemp = new LinkedList<>();
		for(Lock xLock :  locks){
			if(userId== xLock.getUserId()){
				lockTemp.add(xLock);
			}
		}
		if(lockTemp.size()>0)locks.removeAll(lockTemp);
		
	}
	
	public synchronized Iterator<Lock> getLocksPage(int start, int size) {
		int min = Math.min(size, locks.size());
		return new ArrayList<Lock>(locks.subList(start, min)).iterator();
	}

	public synchronized List<Lock> getLocks() {
		return new ArrayList<Lock>(locks.subList(0, locks.size()));
	}

	

	public synchronized long countLocks() {
		return locks.size();
	}
}
