/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.processQueue;

import java.io.Serializable;

public class ProcessBridge implements Serializable {


	private static final long serialVersionUID = 1L;

	private String taskName;
	private String primaryTaskName;
    private volatile boolean stop = false;
    private volatile boolean cancel = false;
    private volatile boolean refresh = false;
    private volatile int progress = 0;
    private volatile Exception processException=null;
    private String message;
    private String status;

    public ProcessBridge() {
    }
    
    public  synchronized String getTaskName() {return taskName;}
    public synchronized  void setTaskName(String taskName) {this.taskName = taskName;}
    
    public  synchronized String getPrimaryTaskName() {return primaryTaskName;}
    public synchronized  void setPrimaryTaskName(String primaryTaskName) {this.primaryTaskName = primaryTaskName;}
    
    public  synchronized boolean isStop() {return stop;}
    public synchronized void setStop(boolean stop) {this.stop = stop;}
    
    public  synchronized boolean isCancel() {return cancel;}
    public void setCancel(boolean cancel) {this.cancel = cancel;}
    
    public  synchronized int getProgress() {return progress;}
    public synchronized void setProgress(int progress) {this.progress = progress;}
    
    public synchronized boolean isFinished() {return isCancel() || progress >= 100;}
    
	public synchronized String getStatus() {return status;}
	public synchronized void setStatus(String status) {this.status = status;}
    
    public synchronized  String getMessage() {return message;}
    public synchronized  void setMessage(String message) {this.message = message;}
    
    public synchronized  Exception getProcessException() {return processException;}
    public synchronized  void setProcessException(Exception processException) {this.processException = processException;}
    
    public synchronized  boolean isRefresh() {return refresh;};
    public synchronized  void setRefresh(boolean refresh) {this.refresh = refresh;};
    
    
}

