/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.processQueue;

import org.apache.wicket.Application;
import org.apache.wicket.Session;
import org.apache.wicket.ThreadContext;

public class ProcessRunnable  implements Runnable {
	
	private final IProcessQueueTask task;
	private final ProcessBridge bridge;
	private final Application application;
	private final Session session;
	
	public ProcessRunnable(IProcessQueueTask task, ProcessBridge bridge) {
		this.task = task;
		this.bridge = bridge;
		this.application = Application.get();
		this.session = Session.exists() ? Session.get() : null;
	}
	
	@Override
	public void run() {
		try {
			ThreadContext.setApplication(application);
			ThreadContext.setSession(session);
			task.doIt(bridge);
		} finally {
			ThreadContext.detach();
		}
	}

}
