/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.processQueue.folderServer;


 

import org.apache.wicket.Application;
import org.apache.wicket.ThreadContext;
import org.biffo.dms.DmsException;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Processing.ProcessType;
import org.biffo.dms.entity.Variables.Impl.FolderServerImpl;
import org.biffo.dms.processQueue.IProcessQueueTask;
import org.biffo.dms.processQueue.ProcessBridge;
 
 

public class FolderServerTask implements IProcessQueueTask {

	private static final long serialVersionUID = 1L;
	
	private Application _application;
	private ProcessBridge _bridge;
	private ProcessType _processType = ProcessType.IMPORT_FOLDER;
	private String _taskName;
	private String _message;
	private String _status;
	
	 
	
	public FolderServerTask(FolderServerImpl serverImpl) {
		_application = Application.get();
		_taskName=serverImpl.getServerName();
		_status = "Start "+_processType.getDescription();
	}
	
	public void refresh() {
		
	}
	
	
	@Override
	public void doIt(ProcessBridge processBridge) {

		_bridge = processBridge;
		_bridge.setTaskName(_taskName);
		_bridge.setStop(false);
		
		try {
			
			if(!Application.exists())ThreadContext.setApplication(_application);
		
			
			createMessage(new DmsException(2000, "I", 
				"FolderServerTask", "doIt", _taskName+" started", null,	null),
				true
			);
			
			boolean running = true;
			while(running) {
				
				if(_bridge.isStop()) {
					createMessage(new DmsException(2000, "I", 
						"FolderServerTask", "doIt", _taskName+" stopped", null, null),
						true
					);
					_bridge.setProgress(100);
					_bridge.setStop(true);
					running=false;
				}
				
				if(_bridge.isCancel()) {
					_bridge.setStop(true);
				}
				
				stopWait(1);
				
			}
			_bridge.setProgress(100);
			_bridge.setStop(true);
			
		}catch(Exception e) {
			createMessage(new DmsException(2000, "E", 
				"FolderServerTask", "doIt", _taskName+" error", null, e),
				false
			);
			_bridge.setProcessException(e);
		}
	}
	
	private synchronized void createMessage(DmsException dmsInfo, boolean withBridgeStatus) {
		StaticLocators.getMessageHandler().createMessage(dmsInfo, null);
		if(null!=_bridge && withBridgeStatus) _bridge.setMessage(dmsInfo.getMessage());
	}
	
	private synchronized void stopWait(int ms) {
		try {
			Thread.sleep(ms);
		}catch(Exception e) {}
	}
	

	@Override
	public synchronized String getMessage() {return _message;}
	@Override
	public synchronized String getTaskName() {return _taskName;}
	@Override
	public synchronized ProcessBridge getProcessBridge() {return _bridge;}
	@Override
	public synchronized ProcessType getProcessType() {return _processType;}
	@Override
	public synchronized int getSourceDocumentId() {return -1;}
	@Override
	public synchronized int getTargetDocumentId() {return -1;}
	@Override
	public synchronized int getIdProcess() {return -1;}
	@Override
	public synchronized String getStatus() {return _status;}
	
	
	
 
	
	
	

}
