/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.processQueue.importServer;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.nio.file.FileSystems;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardWatchEventKinds;
import java.nio.file.WatchEvent;
import java.nio.file.WatchKey;
import java.nio.file.WatchService;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.apache.wicket.Application;
import org.apache.wicket.ThreadContext;
 

public class CheckFolderThread extends Thread implements Serializable{
 
	
	private static final long serialVersionUID = 1L;
	private Application application;
	private WatchService watcher;
	private final String folderToWatch;
	private boolean stopped=false;
	
	public CheckFolderThread(final Application application, String folderToWatch){
		this.application = application;
		this.folderToWatch = folderToWatch;
	}
	
	@Override
	public void run(){
		try {
			if(!Application.exists())ThreadContext.setApplication(application);
			ExecutorService executor = Executors.newFixedThreadPool(2);
			
			watcher = FileSystems.getDefault().newWatchService();
			Path p = Paths.get(folderToWatch);
			WatchKey key = p.register(watcher, StandardWatchEventKinds.ENTRY_CREATE);
			
			while (!Thread.currentThread().isInterrupted() && !stopped) {
				try {
		            try {
		                key = watcher.take();
		            } catch (InterruptedException x) {
		            	stopped=true;
		                break;
		            }
					for(WatchEvent<?> event : key.pollEvents()){
						if(event.kind() == StandardWatchEventKinds.ENTRY_CREATE){
							Path result = (Path)event.context();
							CheckFileAvailable di = new CheckFileAvailable(this, folderToWatch, result.getFileName().toString());
							executor.submit(di);
						}
					}
					key.reset();
				} catch (Exception et) {
	            	System.out.println("et "+et.getMessage());
	            }
				Thread.sleep(1000);
			}
			watcher.close();
			executor.shutdownNow();
			ThreadContext.detach();
		}catch(Exception e){
			System.out.println("run "+e.toString());
		} finally {
			ThreadContext.detach();
			try {
				watcher.close();
			} catch (IOException e) {
				System.out.println("finally "+e.toString());
			}
		}
	}
	
	protected void OnFileReceived(String fileName) {};

	
	private class CheckFileAvailable implements Runnable, Serializable {
		 
		private static final long serialVersionUID = 1L;

		private final CheckFolderThread checkFolderThread;
		private final String folder;
		private final String fileName;
		
		
		private CheckFileAvailable(CheckFolderThread checkFolderThread, String folder, String fileName) {
			this.checkFolderThread = checkFolderThread;
			this.fileName = fileName;
			this.folder = folder;
		}
		
		@Override
		public void run() {
			File fIn = new File(folder+"/"+ fileName );
			try {
				while(!isCompletelyWritten(fIn)) {
					Thread.sleep(1000);
				}
				checkFolderThread.OnFileReceived(fileName);
			}catch(Exception e) {
			}
		}
		
	}
	
	private synchronized boolean isCompletelyWritten(File file) throws InterruptedException{
	    Long fileSizeBefore = file.length();
	    Thread.sleep(2000);
	    Long fileSizeAfter = file.length();
	    if (fileSizeBefore.equals(fileSizeAfter)) {
	        return true;
	    }
	    return false;
	}
	
	public synchronized Application getCheckFolderApplication() {
		//System.out.println(Application.get());
		//System.out.println(ThreadContext.getApplication());
		return application;
	}
	
}
	

 
