/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.resource;

import java.security.Key;
import java.text.SimpleDateFormat;
import java.util.Base64;

import javax.crypto.Cipher;
import javax.crypto.spec.SecretKeySpec;

import org.biffo.dms.DmsWebSession;
import org.biffo.dms.entity.Document.Document;

public class DmsCrypt{
	
	private static final SimpleDateFormat _sdfFolder = new SimpleDateFormat("yyyyMMdd");

	
	public static String getEncodedResource(DmsWebSession session, Document document)  {
		return getEncodedResource(session, _sdfFolder.format(document.getErstelltAm()), document.getDocName(), document.getTitle());
	}
	 
	
	public static String getEncodedResource(DmsWebSession session, String folder, String image) {
		try {
			return encrypt(folder+"<|>"+image, session.getId());
		}catch(Exception e) {
			return null;
		}
	}
	
	public static String getEncodedResource(DmsWebSession session, String folder, String image, String title) {
		try {
			return encrypt(folder+"<|>"+image+"<|>"+title, session.getId());
		}catch(Exception e) {
			return null;
		}
	}
	
	public static String[] getDecodedResource(DmsWebSession session, String strIn)   {
		try {
			String url = decrypt(strIn, session.getId());
			return url.split("\\<\\|\\>");
		}catch(Exception e) {
			return null;
		}
	}
	
	
	public static String encrypt(String strIn, String key) throws Exception{
		Key aesKey = new SecretKeySpec(key.getBytes(), "AES");
		Cipher cipher = Cipher.getInstance("AES");
		cipher.init(Cipher.ENCRYPT_MODE, aesKey);
		byte[] encrypted = cipher.doFinal(strIn.getBytes());
		Base64.Encoder encoder = Base64.getEncoder();
		String encryptedString = encoder.encodeToString(encrypted);
		return encryptedString;
	}
	
	public static String decrypt(String strIn, String key) throws Exception{
		Key aesKey = new SecretKeySpec(key.getBytes(), "AES");
		Base64.Decoder decoder = Base64.getDecoder();
		Cipher cipher = Cipher.getInstance("AES");
		cipher.init(Cipher.DECRYPT_MODE, aesKey);
		String decrypted = new String(cipher.doFinal(decoder.decode(strIn)));
		return decrypted;
	}
	 
}
