/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.resource;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;

import org.apache.wicket.request.Response;
import org.apache.wicket.request.http.WebResponse;
import org.apache.wicket.request.resource.DynamicImageResource;
import org.apache.wicket.request.resource.IResource;
import org.apache.wicket.request.resource.PackageResourceReference;
import org.apache.wicket.request.resource.ResourceReference;
import org.biffo.dms.DmsWebSession;

public class DmsImageResource extends ResourceReference{


	private static final long serialVersionUID = 1L;
	private String dmsMainFolder;
	private String rootFolder;
	private static PackageResourceReference naImageResourcs = new PackageResourceReference(DmsImageResource.class, "na.jpg");

	public DmsImageResource(String name, String dmsMainFolder) {
		super(name);
		this.dmsMainFolder = dmsMainFolder;
		rootFolder = name;
	}
	
	

	@Override
	public IResource getResource() {

		return new DynamicImageResource() {
			
			private static final long serialVersionUID = 1L;
			
			@Override
			protected void configureCache(final ResourceResponse data, final Attributes attributes) {
				
				Response response = attributes.getResponse();
				if (response instanceof WebResponse){
				    WebResponse webResponse = (WebResponse)response;
				    webResponse.disableCaching();
				} 
			}
			

			@Override
	        protected byte[] getImageData(Attributes attributes) {
	        	
				DmsWebSession session = (DmsWebSession)DmsWebSession.getSession();
				if(session == null || !session.isSignedIn()) {
					return writeNA();
				}
				
				String temp = attributes.getRequest().getUrl().toString();
				temp = temp.replaceFirst(rootFolder+"\\/", "");
				if(temp==null || temp.length()==0) {
					return writeNA();
				}
				
				byte[] fileContent=null;
	        	String strArr[]=null;
				try {
					strArr = DmsCrypt.getDecodedResource(session, temp);
					if(strArr==null || strArr.length<2) return writeNA();
				} catch (Exception e1) {
					return writeNA();				}

				File fi = new File(dmsMainFolder+"/"+strArr[0]+"/"+strArr[1]+"_thumb.png");
				try {
					fileContent = Files.readAllBytes(fi.toPath());
				} catch (IOException e) {
					return writeNA();
				}
	        	return fileContent;
	        }
	    };
	}
	
	private static byte[] writeNA() {
		try {
			InputStream inputStream = naImageResourcs.getResource().getResourceStream().getInputStream();
			int length = inputStream.available();
			byte[] bArr = new byte[length];
			inputStream.read(bArr, 0, length);
			return bArr;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
	
	

}
