/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.resource;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;

import javax.servlet.http.HttpServletResponse;

import org.apache.wicket.request.Response;
import org.apache.wicket.request.http.WebResponse;
import org.apache.wicket.request.resource.AbstractResource;
import org.apache.wicket.request.resource.ContentDisposition;
import org.apache.wicket.request.resource.IResource;
import org.apache.wicket.request.resource.PackageResourceReference;
import org.apache.wicket.request.resource.ResourceReference;
import org.apache.wicket.util.time.Time;
import org.biffo.dms.DmsWebSession;

public class DmsPdfResource extends ResourceReference{


	private static final long serialVersionUID = 1L;
	private String dmsMainFolder;
	private String rootFolder;
	private static PackageResourceReference naPdfResource = new PackageResourceReference(DmsPdfResource.class, "na.pdf");
	
	public DmsPdfResource(String name, String dmsMainFolder) {
		super(name);
		this.dmsMainFolder = dmsMainFolder;
		rootFolder = name;
	}
		
		
		
	@Override
	public IResource getResource() {
		
		return new DynamicPdfResource() {
				
				@Override
				protected void configureCache(final ResourceResponse data, final Attributes attributes) {
					
					Response response = attributes.getResponse();
					if (response instanceof WebResponse){
					    WebResponse webResponse = (WebResponse)response;
					    webResponse.disableCaching();
					} 
				}
				
		    	private static final long serialVersionUID = 1L;

				@Override
				protected byte[] getPdfData(Attributes attributes) {
					
					DmsWebSession session = (DmsWebSession)DmsWebSession.getSession();
					if(session == null || !session.isSignedIn()) {
						//return null;
						return writeNA();
					}

					String temp = attributes.getRequest().getUrl().toString();
					temp = temp.replaceFirst(rootFolder+"\\/", "");
					if(temp==null || temp.length()==0) {
						return writeNA();
					}
					
					String strArr[];
					try {
						strArr = DmsCrypt.getDecodedResource(session, temp);
						if(strArr==null || strArr.length<2) return writeNA();						
					} catch (Exception e) {
						return writeNA();
					}
		        	
		        	
					
					
		        	File fi = new File(dmsMainFolder+"/"+strArr[0]+"/"+strArr[1]+".pdf");
		        	byte[] fileContent=null;
					try {
						fileContent = Files.readAllBytes(fi.toPath());
					} catch (IOException e) {
						return writeNA();
					}
		        	return fileContent;
				}
		    };
		}
	
	
		private static byte[] writeNA() {
			try {
				InputStream inputStream = naPdfResource.getResource().getResourceStream().getInputStream();
				int length = inputStream.available();
				byte[] bArr = new byte[length];
				inputStream.read(bArr, 0, length);
				return bArr;
			} catch (Exception e) {
				e.printStackTrace();
			}
			return null;
		}
	
	
	};
 

	
	
	
	abstract class DynamicPdfResource extends AbstractResource{
		private static final long serialVersionUID = 1L;
		private Time lastModifiedTime;

		public DynamicPdfResource() {}
	
		protected synchronized void setLastModifiedTime(Time time){
			lastModifiedTime = time;
		}
		

		protected abstract byte[] getPdfData(Attributes attributes);

		protected void configureResponse(final ResourceResponse response, final Attributes attributes) {}

		@Override
		protected ResourceResponse newResourceResponse(final Attributes attributes){
		
			final ResourceResponse response = new ResourceResponse();

			if (lastModifiedTime != null){
				response.setLastModified(lastModifiedTime);
			}else{
				response.setLastModified(Time.now());
			}

			
			String temp = attributes.getRequest().getUrl().toString();
			if(temp!=null && temp.length()>0) {
				temp = temp.replaceFirst("pdf/", "");
				DmsWebSession session = (DmsWebSession)DmsWebSession.getSession();
				String strArr[]=null;
				try {
					strArr = DmsCrypt.getDecodedResource(session, temp);
				} catch (Exception e) {
				}
				if(strArr!=null && strArr.length==2) {
					response.setFileName(strArr[1]);	
				}else if(strArr!=null && strArr.length==3) {
					response.setFileName(strArr[2]);	
				}
			}
			
			
			if (response.dataNeedsToBeWritten(attributes)){
				response.setContentDisposition(ContentDisposition.INLINE);
				final byte[] imageData = getPdfData(attributes);
				if (imageData == null){
					response.setError(HttpServletResponse.SC_NOT_FOUND);
				}else{
					response.setContentType("application/pdf");
					response.setWriteCallback(new WriteCallback(){
						@Override
						public void writeData(final Attributes attributes){
							attributes.getResponse().write(imageData);
						}
					});
					configureResponse(response, attributes);
				}
			}

			return response;
		}
	}
	


