/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.rest;


import javax.servlet.http.HttpServletRequest;

import org.apache.wicket.protocol.http.servlet.ServletWebRequest;

import org.apache.wicket.request.http.WebRequest;
import org.apache.wicket.request.http.WebResponse;
import org.apache.wicket.util.crypt.Base64;
import org.apache.wicket.util.string.Strings;

import com.google.gson.Gson;


public class RestHelpers {
	
	public static String getUserNameFromRequest(WebRequest webRequest){
		String rc = null;
		try{
			String authorization = webRequest.getHeader("Authorization");
			if (!Strings.isEmpty(authorization) && authorization.startsWith("Basic")){ 
				String base64Credentials = authorization.substring("Basic".length()).trim();
				String credentials = new String(Base64.decodeBase64(base64Credentials));
				final String[] values = credentials.split(":",4);
				if(null!=values && values.length>1) rc = values[0];
			}
		}catch(Exception e){
		}
		return rc;
	}

	public static String getPasswordFromRequest(WebRequest webRequest){
		String rc = null;
		try{
			String authorization = webRequest.getHeader("Authorization");
			if (!Strings.isEmpty(authorization) && authorization.startsWith("Basic")){ 
				String base64Credentials = authorization.substring("Basic".length()).trim();
				String credentials = new String(Base64.decodeBase64(base64Credentials));
				final String[] values = credentials.split(":",4);
				if(null!=values && values.length>1) rc = values[1];
			}
		}catch(Exception e){
		}
		return rc;
	}
	
	public static String getSignatureFromRequest(WebRequest webRequest){
		String rc = null;
		try{
			rc = webRequest.getHeader("Signature");
		}catch(Exception e){
		}
		return rc;
	}
	
	
	public static String getClientIpAddr(ServletWebRequest request) {  
		
		HttpServletRequest httpReq = (HttpServletRequest) request.getContainerRequest();
		
	    String ip = httpReq.getHeader("X-Forwarded-For");  
	    if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {  
	        ip = httpReq.getHeader("Proxy-Client-IP");  
	    }  
	    if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {  
	        ip = httpReq.getHeader("WL-Proxy-Client-IP");  
	    }  
	    if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {  
	        ip = httpReq.getHeader("HTTP_X_FORWARDED_FOR");  
	    }  
	    if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {  
	        ip = httpReq.getHeader("HTTP_X_FORWARDED");  
	    }  
	    if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {  
	        ip = httpReq.getHeader("HTTP_X_CLUSTER_CLIENT_IP");  
	    }  
	    if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {  
	        ip = httpReq.getHeader("HTTP_CLIENT_IP");  
	    }  
	    if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {  
	        ip = httpReq.getHeader("HTTP_FORWARDED_FOR");  
	    }  
	    if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {  
	        ip = httpReq.getHeader("HTTP_FORWARDED");  
	    }  
	    if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {  
	        ip = httpReq.getHeader("HTTP_VIA");  
	    }  
	    if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {  
	        ip = httpReq.getHeader("REMOTE_ADDR");  
	    }  
	    if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {  
	        ip = httpReq.getRemoteAddr();  
	    }  
	    return ip;  
	}

	
	public static void pushException(WebResponse response, Exception exception){

		if(null!=exception.getCause() && exception.getCause() instanceof RestResourceException){
			RestResourceException urre = (RestResourceException) exception.getCause();
			RestStatus status = new RestStatus();
			status.setCode(urre.getErrorCode());
			status.setMessage(urre.getMessage());
			response.setContentType("application/json");
			response.write(new Gson().toJson(status));
			response.setStatus(urre.getHttpErrorCode());
			response.addHeader("WWW-Authenticate", "Basic realm=\"fake\"");
			response.flush();
		}else if(exception instanceof RestResourceException){
			RestResourceException urre = (RestResourceException) exception;
			RestStatus status = new RestStatus();
			status.setCode(urre.getErrorCode());
			status.setMessage(urre.getMessage());
			response.setContentType("application/json");
			response.write(new Gson().toJson(status));
			response.setStatus(urre.getHttpErrorCode());
			response.addHeader("WWW-Authenticate", "Basic realm=\"fake\"");
			response.flush();
		}else if(exception instanceof IllegalArgumentException){
			IllegalArgumentException urre = (IllegalArgumentException) exception;
			RestStatus status = new RestStatus();
			status.setCode(98);
			status.setMessage(urre.getMessage());
			response.setContentType("application/json");
			response.write(new Gson().toJson(status));
			response.setStatus(500);
			response.addHeader("WWW-Authenticate", "Basic realm=\"fake\"");
			response.flush();
		}else{
			RestStatus status = new RestStatus();
			status.setCode(99);
			status.setMessage(exception.getMessage());
			response.setContentType("application/json");
			response.write(new Gson().toJson(status));
			response.setStatus(500);
			response.addHeader("WWW-Authenticate", "Basic realm=\"fake\"");
			response.flush();
		}
	}

	
}
