/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.session.parameters;

import java.io.Serializable;
import java.util.BitSet;
import java.util.HashMap;

import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.user.Users.ISessionParameters;
import org.biffo.dms.entity.user.Users.User;
import org.biffo.dms.user.UserRestrictionType;

public class DmsWebSessionParameters implements ISessionParameters, Serializable{

 
	private static final long serialVersionUID = 1L;

	private transient final int userId;
	private BitSet _restrictionTypes=new BitSet();
	private HashMap<SessionParameterTypes,IDmsWebSessionParameters> lstParameters = new HashMap<SessionParameterTypes, IDmsWebSessionParameters>();
	
	public DmsWebSessionParameters(int userId) {
		this.userId = userId;
	}

	public DmsWebSessionParameters(int userId, DmsWebSessionParameters other) {
		this.userId = userId;
		if(null!=other) {
			for(SessionParameterTypes type : SessionParameterTypes.values()) {
				lstParameters.put(type, other.getParametersByType(type));
			}
		}
	}

	public void reinitAll() {
		for(SessionParameterTypes sessionParameterType : SessionParameterTypes.values()) {
			try {
				Object object = sessionParameterType.getParameterClass().getDeclaredConstructor().newInstance();
				lstParameters.put(sessionParameterType, (IDmsWebSessionParameters) object);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		storeParameters();
		
	}
	
	
	public IDmsWebSessionParameters getParametersByType(SessionParameterTypes sessionParameterType) {
		if(lstParameters.containsKey(sessionParameterType)==false || null==lstParameters.get(sessionParameterType)) {
			try {
				Object object = sessionParameterType.getParameterClass().getDeclaredConstructor().newInstance();
				lstParameters.put(sessionParameterType, (IDmsWebSessionParameters) object);
			} catch (Exception e) {
				//e.printStackTrace();
			}
		}
		return lstParameters.get(sessionParameterType);
	}
	
	public void setParametersByType(SessionParameterTypes sessionParameterType, IDmsWebSessionParameters parameters) {
		try {
			lstParameters.put(sessionParameterType, parameters);
			storeParameters();
		}catch(Exception e) {
			e.printStackTrace();
		}
	}
	
	public void storeParameters() {
		try {
			User user = StaticLocators.getUsers().getEntityById(userId);
			if(null!=user) {
				user.setSessionParameters(this);
				StaticLocators.getUsers().updateEntity(user, false);
			}
		}catch(Exception e) {
			e.printStackTrace();
		}
	}
	
	public boolean hasRestrictionTypes(UserRestrictionType... statusTypes){
		for(UserRestrictionType mst : statusTypes){
			if(getRestrictionType(mst)) return true;
		}
		return false;
	}

	public boolean getRestrictionType( UserRestrictionType restriction){return _restrictionTypes.get(restriction.getCode());}
	public void replaceRestrictionType( UserRestrictionType replace, UserRestrictionType with){
		if(hasRestrictionTypes(replace)){
			setRestrictionType(replace, false);
			setRestrictionType(with, true);
		}
	}

	public void setRestrictionType( UserRestrictionType restriction, Boolean b){
		_restrictionTypes.set(restriction.getCode(), b);
	}
	
	public void setUsersRestrictions(User user) {
		
		for(UserRestrictionType type : UserRestrictionType.values()) {
			setRestrictionType(type, true);
		}
	}
 
	
	
}
