/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.session.parameters;

import java.io.Serializable;

import org.apache.wicket.extensions.markup.html.repeater.util.SortParam;
import org.biffo.dms.web.documents.DocumentsPage;
import org.biffo.dms.web.documents.DocumentsPage.Style;

import de.kleinekuh.entity.helpers.filterbuilder.IFilterClass;

public class DocumentsListParameters implements IDmsWebSessionParameters, Serializable{

	private static final long serialVersionUID = 1L;

	private transient IFilterClass _filterObject;
	private transient String parameterName;
	private transient SortParam<String> sortParam;
	private transient long currentPageNumber=0;
	
	private transient long currentPageNumberN=0;
	private transient long currentPageNumberS=0;
	
	private DocumentsPage.Style displayStyle=Style.TAB;

	private boolean searchVisible=false;
	private boolean tagsVisible=false;
	private boolean usersVisible=false;
	private boolean uploadVisible=false;
	private boolean modulesVisible=false;
	
	private boolean dynamicArrangeS=false;
	private boolean dynamicArrangeN=false;
	
	private boolean showTags=true;
	private boolean showUsers=false;
	
	private long numberOfItemsS=15;
	private long numberOfItemsN=15;
	
	
	
	public DocumentsListParameters() {}
	
	
	@Override
	public String getParameterName() {return parameterName;}
	@Override
	public void setParameterName(String parameterName) {this.parameterName = parameterName;}

	@Override
	public SortParam<String> getSortParam() {return sortParam;}
	@Override
	public void setSortParam(SortParam<String> sortParam) {this.sortParam = sortParam;}

	@Override
	public long getCurrentPageNumber() {return currentPageNumber;}
	@Override
	public void setCurrentPageNumber(long pageNumber) {this.currentPageNumber = pageNumber;}

	@Override
	public IFilterClass getFilterObject() {return  _filterObject;}
	@Override
	public void setFilterObject(IFilterClass filterObject) {_filterObject = filterObject;}
	
	public boolean isSearchVisible() {return searchVisible;}
	public void setSearchVisible(boolean searchVisible) {this.searchVisible = searchVisible;}

	public boolean isTagsVisible() {return tagsVisible;}
	public void setTagsVisible(boolean tagsVisible) {this.tagsVisible = tagsVisible;}

	public boolean isUsersVisible() {return usersVisible;}
	public void setUsersVisible(boolean usersVisible) {this.usersVisible = usersVisible;}
	
	public long getCurrentPageNumberS() {return currentPageNumberS;}
	public void setCurrentPageNumberS(long pageNumber) {this.currentPageNumberS = pageNumber;}

	public long getCurrentPageNumberN() {return currentPageNumberN;}
	public void setCurrentPageNumberN(long pageNumber) {this.currentPageNumberN = pageNumber;}

	public DocumentsPage.Style getCurrentDisplayStyle() {return displayStyle;}
	public void setCurrentDisplayStyle(DocumentsPage.Style displayStyle) {this.displayStyle = displayStyle;}
	
	public boolean isDynamicArrangeS() {return dynamicArrangeS;}
	public void setDynamicArrangeS(boolean dynamicArrangeS) {this.dynamicArrangeS = dynamicArrangeS;}

	public boolean isDynamicArrangeN() {return dynamicArrangeN;}
	public void setDynamicArrangeN(boolean dynamicArrangeN) {this.dynamicArrangeN = dynamicArrangeN;}

	public long getNumberOfItemsS() {return numberOfItemsS;}
	public void setNumberOfItemsS(long numberOfItemsS) {this.numberOfItemsS = numberOfItemsS;}

	public long getNumberOfItemsN() {return numberOfItemsN;}
	public void setNumberOfItemsN(long numberOfItemsN) {this.numberOfItemsN = numberOfItemsN;}

	public boolean isUploadVisible() {return uploadVisible;}
	public void setUploadVisible(boolean uploadVisible) {this.uploadVisible = uploadVisible;}
	
	public boolean isModulesVisible() {return modulesVisible;}
	public void setModulesVisible(boolean modulesVisible) {this.modulesVisible = modulesVisible;}
	
	
	
	public boolean isShowTags() {return showTags;}
	public void setShowTags(boolean showTags) {this.showTags = showTags;}
	
	public boolean isShowUsers() {return showUsers;}
	public void setShowUsers(boolean showUsers) {this.showUsers = showUsers;}
	
 
	
	
	
}
