/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.administration;

import java.util.ArrayList;
import java.util.List;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.form.AjaxSubmitLink;
import org.apache.wicket.extensions.ajax.markup.html.tabs.AjaxTabbedPanel;
import org.apache.wicket.extensions.markup.html.tabs.AbstractTab;
import org.apache.wicket.extensions.markup.html.tabs.ITab;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.panel.Panel;
import org.biffo.dms.web.helpers.ResourceText;
import org.biffo.dms.web.server.folder.FolderServerPanel;
import org.biffo.dms.web.server.folder.FolderServersPanelViewList;
import org.biffo.dms.web.server.ftp.FTPServerPanel;
import org.biffo.dms.web.server.ftp.FTPServersPanelViewList;
import org.biffo.dms.web.server.importer.ImportServerPanel;
import org.biffo.dms.web.server.importer.ImportServerPanelViewList;
import org.biffo.dms.web.server.sftp.SFTPServerPanel;
import org.biffo.dms.web.server.sftp.SFTPServersPanelViewList;
import org.biffo.dms.web.server.smtp.SMTPServerPanel;
import org.biffo.dms.web.server.smtp.SMTPServersPanelViewList;
import org.biffo.dms.web.users.UserPanel;
import org.biffo.dms.web.users.UsersPanelViewList;

public class AdministrationTabedPanel extends Panel {
 
	
	
	public enum TabContent{
		USERS(0, "Benutzer", UsersPanelViewList.class, UserPanel.class),
		FTP(1, "FTP-Server", FTPServersPanelViewList.class, FTPServerPanel.class),
		SFTP(2, "SFTP-Server", SFTPServersPanelViewList.class, SFTPServerPanel.class),
		SMTP(3, "SMTP-Server", SMTPServersPanelViewList.class, SMTPServerPanel.class),
		FOLDER(4, "Folder-Server", FolderServersPanelViewList.class, FolderServerPanel.class),
		IMPORTERS(5, "Importers", ImportServerPanelViewList.class, ImportServerPanel.class),
		;
		
		private final Class<?> _viewClass;
		private final Class<?> _newClass;
		private final int _pos;
		private final String _description;

		
		TabContent(int pos, String description, Class<?> viewClazz, Class<?> newClazz){
			_pos = pos;
			_description = description;
			_viewClass = viewClazz;
			_newClass = newClazz;
		}
		
		public int getPos() {return _pos;}
		public String getDescription() {return _description;}
		public Class<?> getWebPanelViewClass(){return _viewClass;}
	    public Class<?> getWebPanelEditClass(){return _newClass;}
		
		
	}
	
	
	
	
	private static final long serialVersionUID = 1L;

	private final AjaxTabbedPanel<ITab> _usersPanel;
	private final List<ITab> _lstPanels = new ArrayList<>();
	private final Form<?> _form;
	
	private TabContent _actualTabContent;
	private boolean _add=false;
	private Object _origin;
	
	public AdministrationTabedPanel(String id) {
		this(id, null, false, null);
	}
	
	
	
	public AdministrationTabedPanel(String id, TabContent actualTabContent, boolean add,  Object origin) {
		super(id);

		_origin = origin;
		_actualTabContent = actualTabContent;
		_add = add;
		
		_form = new Form<Void>("form");
		addPanels();
		
		_usersPanel = new AjaxTabbedPanel<ITab>("tabs", _lstPanels){
			
			private static final long serialVersionUID = 1L;
			
		    @Override
		    protected WebMarkupContainer newLink(String linkId, final int index) {
		    	
		    	return new AjaxSubmitLink(linkId, _form) {
		    		private static final long serialVersionUID = 1L;
		    		
		    		@Override
		    		protected void onSubmit(AjaxRequestTarget target, Form<?> form) {
		    			setSelectedTab(index);
		    			if (target != null) {
		    				target.add(_usersPanel);
		    			}
		    			onAjaxUpdate(target);
		    		}
		 
		    		@Override
		    		protected void onError(AjaxRequestTarget target, Form<?> form) {
		    		}
		        };
		    }
		};
		_usersPanel.setOutputMarkupId(true);
		_usersPanel.add(AttributeModifier.replace("class", "adminpanel"));
		_form.add(_usersPanel);
		
		if(_add) _usersPanel.setSelectedTab(_actualTabContent._pos);
		add(_form);
	}
	
	
	private void addPanels(){
		for(TabContent tabContent : TabContent.values()) {
			
			_lstPanels.add(new AbstractTab(ResourceText.getModelText("tabTxt"+tabContent, this)){
				private static final long serialVersionUID = 1L;
				@Override
	            public Panel getPanel(String panelId){
					try {
						Panel panel = null;
						if(_add && tabContent.equals(_actualTabContent)) {
							panel = (Panel) tabContent.getWebPanelEditClass().getDeclaredConstructor(String.class, int.class, Object.class).newInstance(panelId, -1, _origin);
							panel.setOutputMarkupId(true);
						}else{
							panel = (Panel) tabContent.getWebPanelViewClass().getDeclaredConstructor(String.class).newInstance(panelId);
							panel.setOutputMarkupId(true);
						}
						return panel;
					}catch(Exception e) {
					}
					return null;
	            }
	        });
		}
	}
}
