/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.components.search.singelfield;

import java.io.Serializable;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.apache.wicket.Component;
import org.apache.wicket.ajax.AbstractDefaultAjaxBehavior;
import org.apache.wicket.ajax.AjaxEventBehavior;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.attributes.AjaxCallListener;
import org.apache.wicket.ajax.attributes.AjaxRequestAttributes;
import org.apache.wicket.ajax.attributes.IAjaxCallListener;
import org.apache.wicket.ajax.form.OnChangeAjaxBehavior;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.extensions.ajax.markup.html.autocomplete.AbstractAutoCompleteRenderer;
import org.apache.wicket.extensions.ajax.markup.html.autocomplete.AutoCompleteSettings;
import org.apache.wicket.extensions.ajax.markup.html.autocomplete.AutoCompleteTextField;
import org.apache.wicket.markup.head.IHeaderResponse;
import org.apache.wicket.markup.head.JavaScriptHeaderItem;
import org.apache.wicket.markup.head.OnDomReadyHeaderItem;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.Response;
import org.apache.wicket.request.cycle.RequestCycle;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.dataprovider.os.DPSearchDocumentsIds;
import org.biffo.dms.dataprovider.os.DPSearchDocumentsSingelField;
import org.biffo.dms.entity.Document.DocumentOSFilterClass;
import org.biffo.dms.os.SearchResult;
import org.biffo.dms.os.SearchResults;

public class SingleFieldSearchInput extends Panel {

	private static final long serialVersionUID = 1L;
	private static final String _className = "SingleFieldSearchInput";
	private static final List<SearchResult> emptyList = Collections.emptyList();
	private DmsWebSession _session = DmsWebSession.getSession();
	
	private Form<?> _singelfieldsearchform;
	private AutoCompleteTextField<SearchResult> _searchinput;
	private String _callbackVar;
	private boolean autocomplete=true;
	
	private Model<SearchResult> _selection = new Model<SearchResult>();
	
	
	private DPSearchDocumentsSingelField _dpSearchDocuments = new DPSearchDocumentsSingelField();
	private DPSearchDocumentsIds _dpSearchDocumentIds = new DPSearchDocumentsIds();
 
	
	private final DocumentOSFilterClass _dsfc; 
	
	private int _maxEntries = 30;
	private AjaxLink<String> _clearLink;
	private WebMarkupContainer _wmcSearchinputresult;
	private Label _lblSearchinputresulttext;
	private boolean _withResultText=false;
	private boolean isDoccontentOnly=false;
	
	private ResulttextHelper rsh = new ResulttextHelper();
	
	
	class ResulttextHelper implements Serializable{

		@SuppressWarnings("unused")
		private String resultText;
		private int foundPages=-1;

		private static final long serialVersionUID = 1L;

		public ResulttextHelper() {
		}
		
		public void setFoundPages(int foundPages) {
			this.foundPages = foundPages;
		}
		
		public String getResultText() {
			if(foundPages==-1) return null;
			if(foundPages==1) return foundPages + " Seite gefunden.";
			return foundPages + " Seiten gefunden";
		}
		public void setResultText(String resultText) {
			if(resultText==null) foundPages=-1;
			this.resultText=resultText;
		}
		
	}
	
	
	@Override
	protected void onBeforeRender() {
		
		if(_dsfc.getSearchString()!=null && !_dsfc.getSearchString().isEmpty()) {
			_searchinput.setDefaultModelObject(_dsfc.getSearchString());	
		}
		
		_wmcSearchinputresult.setVisible(_withResultText);
		super.onBeforeRender();
	}
	

	
	@Override
	public void renderHead(IHeaderResponse response) {
		super.renderHead(response);
		response.render(JavaScriptHeaderItem.forScript("var cb"+getMarkupId()+";", "SingleFieldSearchInput"+getMarkupId()));
	}
	
	public SingleFieldSearchInput(String id) {
		super(id);
		_singelfieldsearchform = new Form<Void>("singelfieldsearchform");
		_singelfieldsearchform.setOutputMarkupId(true);
		_dsfc = new DocumentOSFilterClass();
		add(_singelfieldsearchform);
		addAutoComplete();
		addButtons();
		addResulttext();
	}
	
	public SingleFieldSearchInput(String id, final DocumentOSFilterClass dsfc) {
		this(id, dsfc, false);
	}
	
	
	public SingleFieldSearchInput(String id, final DocumentOSFilterClass dsfc, boolean doccontentOnly) {
		super(id);
		_singelfieldsearchform = new Form<Void>("singelfieldsearchform");
		_singelfieldsearchform.setOutputMarkupId(true);
		_dsfc = dsfc;
		isDoccontentOnly = doccontentOnly;
		if(_dsfc.getSearchString()!=null && !_dsfc.getSearchString().isEmpty()) {
			refreshSearchQuery("Init", _dsfc.getSearchString(), true);
		}
		add(_singelfieldsearchform);
		addAutoComplete();
		addButtons();
		addResulttext();
	}
	
	private void addResulttext() {
		_wmcSearchinputresult = new WebMarkupContainer("searchinputresult");
		_wmcSearchinputresult.setOutputMarkupId(true);
		_singelfieldsearchform.add(_wmcSearchinputresult);

		_lblSearchinputresulttext = new Label("searchinputresulttext", new PropertyModel<String>(rsh, "resultText"));
		_lblSearchinputresulttext.setOutputMarkupId(true);
		_wmcSearchinputresult.add(_lblSearchinputresulttext);
		
	}
	
	
	private void addAutoComplete(){
		
		AbstractAutoCompleteRenderer<SearchResult> autoCompleteRenderer = new AbstractAutoCompleteRenderer<SearchResult>() {

			private static final long serialVersionUID = 1L;
			
			@Override
			protected String getTextValue(SearchResult searchResult) {
				return searchResult.getFoundTerm();
			}

			@Override
			protected void renderChoice(SearchResult searchResult, Response response, String arg2) {
				response.write(searchResult.getFoundTerm());
			}
			
			@Override
			protected CharSequence getOnSelectJavaScriptExpression(SearchResult searchResult) {
				StringBuilder js = new StringBuilder();
				js.append("Wicket.Ajax.get({u: "+_callbackVar+", ep: {'event': 'selection', 'query':'"+searchResult.getSearchedTerm()+"', 'found':'"+searchResult.getFoundTerm()+"'}});");
				js.append("input;");
				return js.toString();
			}  
			
		 
			
		};
		
		
		AutoCompleteSettings acs = new AutoCompleteSettings();
		acs.setCssClassName("searchboxform");
		acs.setThrottleDelay(300);
		
		_searchinput = new AutoCompleteTextField<SearchResult>("searchinput", _selection, null, autoCompleteRenderer, acs) {

			private static final long serialVersionUID = 1L;

			@Override
			protected Iterator<SearchResult> getChoices(String input) {
				_dsfc.setFilterObject(null);
				_dsfc.setLinkedWhere(null);
				if(input==null || input.isEmpty() || input.trim().isEmpty() || input.length()<3) {
					return emptyList.iterator();
				}
				if(!autocomplete) return emptyList.iterator();
				
				_dsfc.setSize(100);
				if(input.length()>5)_dsfc.setSize(1000);
				
				SearchResults searchResults = _dpSearchDocuments.getSearchResults(_dsfc);
				if(searchResults.size()==0) {
					return emptyList.iterator();
				}else if(searchResults.size()<_maxEntries) {
					return searchResults.iterator();
				}else {
					return searchResults.subList(0, _maxEntries).iterator();
				}
						
			}
			
		};
		
		AbstractDefaultAjaxBehavior autoCompleteBehavior= new AbstractDefaultAjaxBehavior(){

			private static final long serialVersionUID = 1L;
			@Override
			public void renderHead(Component component, IHeaderResponse response){
				super.renderHead(component, response);
	            String callbackUrl = getCallbackUrl().toString();
	            _callbackVar = "cb"+SingleFieldSearchInput.this.getMarkupId();
	            response.render(OnDomReadyHeaderItem.forScript(""+_callbackVar+" = '" + callbackUrl + "';"));
	            
			}
			
			@Override
			protected void respond(AjaxRequestTarget target) {
				RequestCycle requestCycle = RequestCycle.get();
				String event = requestCycle.getRequest().getRequestParameters().getParameterValue("event").toString(null);
				if(event!=null) {
					String found = requestCycle.getRequest().getRequestParameters().getParameterValue("found").toString(null);
					refreshSearchQuery(event, found, false);
					if(_withResultText)target.add(_lblSearchinputresulttext);
					//target.appendJavaScript("$('#"+_searchinput.getMarkupId()+"').blur();");
					if(event.equals("selection")) {
						OnSelectionFilterClick(target);
					}else if(event.equals("enter")) {
						OnEnterFilterClick(target);
					}
				}
			}
			
			
		};
		
		OnChangeAjaxBehavior onChange = new OnChangeAjaxBehavior() {

			private static final long serialVersionUID = 1L;

			protected void onUpdate(final AjaxRequestTarget target){
				try {
					_dsfc.setSearchString(_searchinput.getDefaultModelObjectAsString());
				}catch(Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "onChange", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
			}
		};
		
		
		AjaxEventBehavior onEnter = new AjaxEventBehavior("onkeypress") {
	 
			private static final long serialVersionUID = 1L;

			@Override
			protected void updateAjaxAttributes(AjaxRequestAttributes attributes) {
	            IAjaxCallListener listener = new AjaxCallListener(){

					private static final long serialVersionUID = 1L;

					@Override
	                public CharSequence getPrecondition(Component component) {
						StringBuilder sb = new StringBuilder();
						sb.append("var keycode=Wicket.Event.keyCode(attrs.event);");
						sb.append("if(keycode===13){");
						sb.append("attrs.event.preventDefault();");
						sb.append("return true;");
						sb.append("}else{");
						sb.append("return false;");
						sb.append("}");
						return sb.toString();
	                }
					
	            };
	            attributes.getAjaxCallListeners().add(listener);
	            
	            attributes.getDynamicExtraParameters()
	            	.add("var eventKeycode = Wicket.Event.keyCode(attrs.event);" +
                     "return {keycode: eventKeycode};");
				
				attributes.setAllowDefault(true);
				super.updateAjaxAttributes(attributes);
			}
			
			@Override
	        protected void onEvent(AjaxRequestTarget target) {
				String temp = ("Wicket.Ajax.get({u: "+_callbackVar+", ep: {'event': 'enter', 'query':'"+_dsfc.getSearchString()+"', 'found':'"+_dsfc.getSearchString()+"'}});");
				target.appendJavaScript(temp);
	        }
	    };
 		_searchinput.setOutputMarkupId(true);
		_searchinput.add(onChange);
		_searchinput.add(autoCompleteBehavior);
		_searchinput.add(onEnter);
		_singelfieldsearchform.add(_searchinput);
	}
	
	private void refreshSearchQuery(String event, String searchIn, boolean setAutoComplete) {
		try {
    		_dsfc.setFilterObject(null);
			_dsfc.setLinkedWhere(null);
			_dsfc.setSingelFieldSearchWhere(null);
			_dsfc.setSearchString(searchIn);
			_dsfc.setSize(0);
			HashMap<String,LinkedList<String>> hmX =_dpSearchDocumentIds.getDocumentIds(_dsfc, true, isDoccontentOnly);
			String strArr[]= hmX.keySet().toArray(new String[hmX.size()]);
			String condition = String.join(",", strArr);
			if(condition!=null && !condition.isEmpty()) {
				_dsfc.setLinkedWhere(" id in ("+condition+")");
			}
			rsh.setFoundPages(_dsfc.getTotalNumberOfDocuments());
			_dsfc.setFilterObject(hmX);
    	}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", _className, "refreshSearchQuery", null, e);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
    	}
	}
	
	
	
	
	private void addButtons() {
		
		_clearLink = new AjaxLink<String>("clearbtn") {

			private static final long serialVersionUID = 1L;

			@Override
			public void onClick(AjaxRequestTarget target) {
				clear(target);
				OnClearFilterClick(target);
			}
		};
		_clearLink.setOutputMarkupId(true);
		_singelfieldsearchform.add(_clearLink);
 
		
	}
	
	public void clear(AjaxRequestTarget target) {
		_dsfc.setSearchString(null);
		_dsfc.setFilterObject(null);
		_dsfc.setSingelFieldSearchWhere(null);
		
		_selection = new Model<SearchResult>();
		_searchinput.setDefaultModel(_selection);
		
		rsh.setResultText(null);
		target.add(_searchinput);
		if(_withResultText)target.add(_lblSearchinputresulttext);
	}
	
	public boolean isAutoComplete() {return autocomplete;}
	public void setAutoComplete(boolean autocomplete) {this.autocomplete = autocomplete;}
	
	
	protected void OnSelectionFilterClick(AjaxRequestTarget target){}
	protected void OnEnterFilterClick(AjaxRequestTarget target){}
	protected void OnClearFilterClick(AjaxRequestTarget target){}
	
	public void clearSearch(AjaxRequestTarget target) {
		clear(target);
	}

	public boolean isWithResultText() {return _withResultText;}
	public void setWithResultText(boolean withResultText) {_withResultText = withResultText;}
 
	
}
