/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.config;

import java.util.ArrayList;
import java.util.List;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.form.AjaxSubmitLink;
import org.apache.wicket.extensions.ajax.markup.html.tabs.AjaxTabbedPanel;
import org.apache.wicket.extensions.markup.html.tabs.AbstractTab;
import org.apache.wicket.extensions.markup.html.tabs.ITab;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.Model;
import org.biffo.dms.DmsWebApplication;
import org.biffo.dms.web.variables.VariablePanel;
import org.biffo.dms.web.variables.VariablesPanelViewList;
 

public class ConfigTabedPanel extends Panel {
 	
	
	public enum TabContent{
		POSTGRESSDB(0,"PostgreSQL", null, DBConfigPanel.class),
		OPENSEARCH(1,"Opensearch", null, OSConfigPanel.class),
		DMSCONFIG(2,"DmsConfig", null, DmsConfigPanel.class),
		TESSERACT(3,"Tesseract-OCR", null, TesseractConfigPanel.class),
		VARIABLES(4, "Variables", VariablesPanelViewList.class, VariablePanel.class)
		;
		
		private final Class<?> _viewClass;
		private final Class<?> _newClass;
		private final int _pos;
		private final String _description;

		
		TabContent(int pos, String description, Class<?> viewClazz, Class<?> newClazz){
			_pos = pos;
			_description = description;
			_viewClass = viewClazz;
			_newClass = newClazz;
		}
		
		public int getPos() {return _pos;}
		public String getDescription() {return _description;}
		public Class<?> getWebPanelViewClass(){return _viewClass;}
	    public Class<?> getWebPanelEditClass(){return _newClass;}
		
	}
	
	
	
	
	private static final long serialVersionUID = 1L;

	private final AjaxTabbedPanel<ITab> _tabedPanel;
	private final List<ITab> _lstPanels = new ArrayList<>();
	private final Form<?> _form;
	
	private TabContent _actualTabContent;
	private boolean _add=false;
	private Object _origin;
	
	public ConfigTabedPanel(String id) {
		this(id, null, false, null);
	}
	
	
	
	public ConfigTabedPanel(String id, TabContent actualTabContent, boolean add,  Object origin) {
		super(id);

		_origin = origin;
		_actualTabContent = actualTabContent;
		_add = add;
		
		_form = new Form<Void>("form");
		addPanels();
		
		_tabedPanel = new AjaxTabbedPanel<ITab>("tabs", _lstPanels){
			
			private static final long serialVersionUID = 1L;
			
		    @Override
		    protected WebMarkupContainer newLink(String linkId, final int index) {
		    	
		    	return new AjaxSubmitLink(linkId, _form) {
		    		private static final long serialVersionUID = 1L;
		    		
		    		@Override
		    		protected void onSubmit(AjaxRequestTarget target, Form<?> form) {
		    			setSelectedTab(index);
		    			if (target != null) {
		    				target.add(_tabedPanel);
		    			}
		    			onAjaxUpdate(target);
		    		}
		 
		    		@Override
		    		protected void onError(AjaxRequestTarget target, Form<?> form) {
		    		}
		        };
		    }
		};
		_tabedPanel.setOutputMarkupId(true);
		_tabedPanel.add(AttributeModifier.replace("class", "adminpanel"));
		_form.add(_tabedPanel);
		
		if(_add) _tabedPanel.setSelectedTab(_actualTabContent._pos);
		add(_form);
	}
	
	
	private void addPanels(){
		
		boolean noconfig = DmsWebApplication.getApplication().isNoConfig();
		
		for(TabContent tabContent : TabContent.values()) {
			
			AbstractTab tabToAdd = new AbstractTab(new Model<String>(tabContent.getDescription())){
				private static final long serialVersionUID = 1L;
				@Override
	            public Panel getPanel(String panelId){
					try {
						Panel panel = null;
						if(tabContent.getWebPanelViewClass()==null || (_add && tabContent.equals(_actualTabContent))) {
							panel = (Panel) tabContent.getWebPanelEditClass().getDeclaredConstructor(String.class, int.class, Object.class).newInstance(panelId, -1, _origin);
						}else {
							panel = (Panel) tabContent.getWebPanelViewClass().getDeclaredConstructor(String.class).newInstance(panelId);
							panel.setOutputMarkupId(true);
						}
						
						/*Panel panel = (Panel) tabContent.getWebPanelEditClass().getDeclaredConstructor(String.class, int.class, Object.class).newInstance(panelId, -1, _origin);
						panel.setOutputMarkupId(true);
						/*if(_add && tabContent.equals(_actualTabContent)) {
							panel = (Panel) tabContent.getWebPanelEditClass().getDeclaredConstructor(String.class, int.class, Object.class).newInstance(panelId, -1, _origin);
							panel.setOutputMarkupId(true);
						}else{
							panel = (Panel) tabContent.getWebPanelViewClass().getDeclaredConstructor(String.class).newInstance(panelId);
							panel.setOutputMarkupId(true);
						}*/
						
						panel.setOutputMarkupId(true);
						return panel;
					}catch(Exception e) {
						e.printStackTrace();
					}
					return null;
	            }
	        };
	        if(noconfig) {
	        	if(tabContent.getPos()==0)_lstPanels.add(tabToAdd);
	        }else {
	        	_lstPanels.add(tabToAdd);
	        }
		}
	}
	
	
}
