/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.config;

 

import java.util.List;

import org.apache.wicket.Page;
import org.apache.wicket.PageReference;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.PasswordTextField;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.util.string.Strings;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebApplication;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.auth.SignInPage;
import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.infobox.InfoboxPanel.INFOBOX_STATUS;
import org.biffo.dms.config.WebAppConfig;
import org.biffo.dms.config.WebAppConfigParams;
import org.biffo.dms.setup.DmsDefaultData;
import org.biffo.dms.setup.DmsSetup;
import org.biffo.dms.setup.DmsSetup.DMS_Tables;
import org.biffo.dms.setup.DmsTable;
import org.biffo.dms.web.helpers.ErrorLabel;
import org.biffo.dms.web.helpers.PanelTitel;
import org.biffo.dms.web.helpers.ResourceText;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;


public class DBConfigPanel extends Panel{

	private static final long serialVersionUID = 1L;
	private static final String toolbarCSS = "dataformToolbar";
		
	private WebAppConfig _wac;
	private WebAppConfigParams _wacp;
	private Label _lblTitel;
	
	private TextField<String> _dburl,  _dbuser;
	private PasswordTextField _dbpassword;
	private ErrorLabel _dbpassword_info, _dbuser_info, _dburl_info;
	
 	private Form<?> _form;
	private Toolbar _toolBar;
	private ToolbarButton _btEdit, _btCancel, _btSave, _btOK, _btCheck, _btReinit;

	private Object _origin;
	private PanelTitel _panelTitel = new PanelTitel();
	
	@SuppressWarnings("unused")
	private DmsWebSession _session = DmsWebSession.getSession();
	private DmsSetup _dmsSetup = new DmsSetup();
	

	private WebMarkupContainer wmcInfo;
	private InfoboxPanel infobox;
	private DBConfigTablesPanel _dctp;
	
	public DBConfigPanel(String id, int dbConfigID,  Object origin) {
		super(id);
		_origin = origin;

		_wac = DmsWebApplication.getApplication().getWebAppConfig();
		
		
		_form = new Form<Void>("form");
		_form.setOutputMarkupId(true);
		add(_form);
		
        addTextFields();
        addCheckBoxen();
        
        addDropDownBoxen();
        addAdditionalControls();
        
        addToolBar();
        addInfobox();
        
        refreshFormObjects();
        
        
        if(!_wac.Exists()) {
        	 _toolBar.setViewState(ToolbarState.STATE_CHANGE);
        	setFormComponentsEditable(true);
        }
        else setFormComponentsEditable(false);
		setOutputMarkupId(true);
		
		_panelTitel.setTitel(ResourceText.getText("titEditConfig", this));
        
	}
	
	
	
	
	private void addTextFields(){
		
		
		_dburl= new TextField<String>("dburl");
		_dburl.setOutputMarkupId(true);
		_form.add(_dburl); 
		
		_dbpassword= new PasswordTextField("dbpassword");
		_dbpassword.setOutputMarkupId(true);
		_dbpassword.setResetPassword(false);
		_form.add(_dbpassword); 

		
		_dbuser= new TextField<String>("dbuser");
		_dbuser.setOutputMarkupId(true);
		_form.add(_dbuser); 

		_dbpassword_info= new ErrorLabel("dbpassword_info");
		_form.add(_dbpassword_info);
		
		_dbuser_info= new ErrorLabel("dbuser_info");
		_form.add(_dbuser_info);
		
		_dburl_info= new ErrorLabel("dburl_info");
		_form.add(_dburl_info);

	}
	

	
	
	
	private void addCheckBoxen(){}
 
	private void addDropDownBoxen(){}
	
	private void addAdditionalControls(){
		_lblTitel = new Label("paneltitel", new PropertyModel<String>(this._panelTitel, "Titel"));
		_lblTitel.setOutputMarkupId(true);
		add(_lblTitel);
		
		_dctp = new DBConfigTablesPanel("dbtables");
		_dctp.setOutputMarkupId(true);
		add(_dctp);
	}
	
	@SuppressWarnings("unused")
	private void closeForm(AjaxRequestTarget target){
		if(_origin instanceof Panel){
			target.add(DBConfigPanel.this.replaceWith((Panel)_origin).getParent());
		}else if(_origin instanceof Page){
		}else if(_origin instanceof PageReference){
			PageReference pg = (PageReference)_origin;
			setResponsePage(pg.getPage());
		}
	}
	
	
	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};

		_btEdit = new ToolbarButton().setButtonType(ToolbarButtonType.EDIT).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btEdit.setAlternate(ResourceText.getText("tbAltEdit", this));
		_toolBar.add(_btEdit);
		
		_btCancel = new ToolbarButton().setButtonType(ToolbarButtonType.CANCEL).setToolbarState(ToolbarState.STATE_NEW | ToolbarState.STATE_CHANGE).setCssClass(toolbarCSS);
		_btCancel.setAlternate(ResourceText.getText("tbAltCancel", this));
		_toolBar.add(_btCancel);
		
		_btOK = new ToolbarButton(_form).setButtonType(ToolbarButtonType.OK).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btOK.setAlternate(ResourceText.getText("tbAltOk", this));
		//_toolBar.add(_btOK);
		
		_btSave = new ToolbarButton(_form).setButtonType(ToolbarButtonType.SAVE).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btSave.setAlternate(ResourceText.getText("tbAltSave", this));
		_toolBar.add(_btSave);
		
		
		_btCheck = new ToolbarButton(_form).setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW | ToolbarState.STATE_CHANGE |  ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT);
		_btCheck.setUtf8Code("xf232");
		_btCheck.setAlternate(ResourceText.getText("tbAltCheck", this));
		_btCheck.setEventName("check");
		_btCheck.setCssClass(toolbarCSS);
		_toolBar.add(_btCheck);
		
		_btReinit = new ToolbarButton(_form).setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW| ToolbarState.WITH_EVENT);
		_btReinit.setUtf8Code("xe88a");
		_btReinit.setAlternate(ResourceText.getText("tbAltReinit", this));
		_btReinit.setEventName("reinit");
		_btReinit.setCssClass(toolbarCSS);
		_toolBar.add(_btReinit);
		
				
		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		
		_form.add(_toolBar);
	}
	
	private void processEvent(String event, AjaxRequestTarget target) {
		if(event.equals("Save")) save(target, true);
		else if(event.equals("Ok")) ok(target);
		else if(event.equals("Cancel")) cancel(target);
		else if(event.equals("Edit")) edit(target);
		else if(event.equals("reinit")) {
			_session.invalidateNow();
			setResponsePage(SignInPage.class);
		}else if(event.equals("check")) {
			clearError(target);
			if(checkShowError(target)) {
				infobox.refresh(target, ResourceText.getText("errConfigurationInvalid", this), INFOBOX_STATUS.error);
			}else{
				
				boolean tableError=false;
				try {
					_dmsSetup.checkTables(DMS_Tables.USER);
					_dmsSetup.checkTables(DMS_Tables.DATA);
				}catch(Exception e) {
					tableError=true;
				}
				if(tableError) {
					infobox.refresh(target, ResourceText.getText("errConfigurationTableInvalid", this), INFOBOX_STATUS.error);
				}else {
					infobox.refresh(target, ResourceText.getText("infConfigurationOk", this), INFOBOX_STATUS.success);
				}
			};
			infobox.setVisible(true);
			target.add(wmcInfo);
		}
		target.add(_form);
		target.add(_toolBar);
	}
	
	private void cancel(AjaxRequestTarget target){
		 try {
			 clearError(target);
			 refreshFormObjects();
			 if(_toolBar.getViewState()==ToolbarState.STATE_VIEW || _toolBar.getViewState()==ToolbarState.STATE_NEW ) {
			 }else {
				 try {
					 //refreshFormObjects();
					 _toolBar.setViewState(ToolbarState.STATE_VIEW);
					 setFormComponentsEditable(false);
					 target.add(_dctp);
					 target.add(_form);
					 target.add(_toolBar);
				 }catch(Exception e2) {
					 infobox.refresh(target, ResourceText.getText("errChangeCancel", this)+".\n"+e2.getMessage(), INFOBOX_STATUS.error);
					 infobox.setVisible(true);
					 target.add(wmcInfo);					 
				 }
			 } 
		 }catch(Exception e) {
			infobox.refresh(target, ResourceText.getText("errChangeCancel", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);			 
		 }

	}
	
	
	private void edit(AjaxRequestTarget target){
		 try {
			 _toolBar.setViewState(ToolbarState.STATE_CHANGE);
			 setFormComponentsEditable(true);
	 	}catch(Exception e) {
			infobox.refresh(target, ResourceText.getText("errChangeEditMode", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);	 		
	 	}
	}	
	
	
 
	private void ok(AjaxRequestTarget target){
		if(save(target, false)) {
			_toolBar.setViewState(ToolbarState.STATE_VIEW);
			setFormComponentsEditable(false);
		}
	} 
	

	private void clearError(AjaxRequestTarget target) {
		_dburl_info.setVisible(false);
		_dbuser_info.setVisible(false);
		_dbpassword_info.setVisible(false);
		
		target.add(_dburl_info);
		target.add(_dbuser_info);		
		target.add(_dbpassword_info);
	}
	
	
	
	private boolean checkShowError(AjaxRequestTarget target) {
		
		
		boolean error=false;
		clearError(target);
		
		if(_wacp==null) return false;
	 
		_dburl_info.setVisible(false);
		if(Strings.isEmpty(_wacp.getDBUrl())){
			_dburl_info.setErrorText(ResourceText.getText("errTextJDBCUrlEmpty", this));
			_dburl_info.setVisible(true);
			error = true;
		}else {
			_dburl_info.setVisible(false);
		}
		if(Strings.isEmpty(_wacp.getDBUser())){
			_dbuser_info.setErrorText(ResourceText.getText("errTextDBUserEmpty", this));
			_dbuser_info.setVisible(true);
			error = true;
		}else {
			_dbuser_info.setVisible(false);
		}
		if(Strings.isEmpty(_wacp.getDBPassword())){
			_dbpassword_info.setErrorText(ResourceText.getText("errTextDBPasswordEmpty", this));
			_dbpassword_info.setVisible(true);
			error = true;
		}else {
			_dbpassword_info.setVisible(false);
		}

		if(!_dburl_info.isVisible()) {
			try {
				_dmsSetup.checkDBJDBC(_wacp.getDBUrl());
			}catch(DmsException e) {
				_dburl_info.setErrorText(ResourceText.getText(""+e.getCode(), this));
				_dburl_info.setVisible(true);
				error = true;
			}
		}
		
		if(!error) {
			try {
				_dmsSetup.checkDMSDBConnection(_wacp.getDBUser(), _wacp.getDBPassword());
			}catch(DmsException e) {
				_dburl_info.setErrorText(ResourceText.getText(""+e.getCode(), this));
				_dburl_info.setVisible(true);
				error=true;
			}
		}
		if(!error) {
			_dctp.setConnectionParamsUrl(_wacp.getDBUrl(), _wacp.getDBUser(), _wacp.getDBPassword());
			target.add(_dctp);
		}else {
			_dctp.setNodata(target);
		}
		
		//target.add(_dctp);
		target.add(_dburl_info);
		target.add(_dbuser_info);		
		target.add(_dbpassword_info);
		
		return error;
		
	}

	
	private boolean createTables(DMS_Tables tablesType) throws Exception{
		boolean error=false;
		try {
			_dmsSetup.checkTables(tablesType);
		}catch(DmsException e) {
			error=true;
		}
		int tablesToCreate=0;
		if(error) {
			List<DmsTable> lstTables = _dmsSetup.getTables(tablesType);
			for(int i=0;i<lstTables.size();i++) {
				if(!lstTables.get(i).isExists()) {
					lstTables.get(i).setCreate(true);
					tablesToCreate++;
				}
			}
		}
		if(tablesToCreate>0) {
			try {
				_dmsSetup.createTables(tablesType);
				error=false;
			}catch(Exception e) {
				e.printStackTrace();
			}
			return true;
		}
		return false;
	}
	
	
	private boolean save(AjaxRequestTarget target, boolean withMessage) {
		try { 
			if(checkShowError(target)) {
				infobox.refresh(target, ResourceText.getText("errConfigurationInvalid", this), INFOBOX_STATUS.error);
				infobox.setVisible(true);
				target.add(wmcInfo);
				return false;
			}else {
				
				if(!_wac.Exists()) {
					_wac.initConfig();
				}
				
				//-->if(!DmsWebApplication.getApplication().getWebAppConfig().Exists())DmsWebApplication.getApplication().getWebAppConfig().initConfig();
				
			
				if(createTables(DMS_Tables.DATA)) {
					try {
						DmsDefaultData dmsDefaultData = new DmsDefaultData(_dmsSetup.getDatabase());
						dmsDefaultData.writeDBData();
						dmsDefaultData.writeDefaultDmsConfig();
						dmsDefaultData.writeDefaultImporter();
						dmsDefaultData.writeOpenSearchData();
						dmsDefaultData.writeTesseractData();
					}catch(Exception e) {
						throw e;
					}
					
					
				};
				if(createTables(DMS_Tables.USER)) {
					try {
						DmsDefaultData dmsDefaultData = new DmsDefaultData(_dmsSetup.getDatabase());
						dmsDefaultData.createDefaultUser();
					}catch(Exception e) {
						throw e;
					}
					
				};
				 
				try {
					_dctp.setConnectionParamsUrl(_wacp.getDBUrl(), _wacp.getDBUser(), _wacp.getDBPassword());
				}catch(Exception e) {
					//tableError=true;
				}
				_wac.setConfig(_wacp);

			}
			infobox.refresh(target, ResourceText.getText("infConfigurationSaved", this), INFOBOX_STATUS.success);
			infobox.setVisible(true);
			target.add(wmcInfo);
		}catch(Exception e) {
			infobox.refresh(target, ResourceText.getText("errConfigurationSaved", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);
		}
		return true;
	}
	
 
	
	
	private void setFormComponentsEditable(boolean editable){
		try{
			_dburl.setEnabled(editable);
			_dbpassword.setEnabled(editable);
			_dbuser.setEnabled(editable);
		}catch(Exception e){
			//e.printStackTrace();
		}
	}
	
 
 
	private void addInfobox() {
		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		
		if(!_wac.Exists()) {
			infobox = new InfoboxPanel("infobox", ResourceText.getText("errConfigurationNotExists", this), INFOBOX_STATUS.error);
			infobox.setVisible(true);
		}else {
			infobox = new InfoboxPanel("infobox");
			infobox.setVisible(false);
		}
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
	}
	
	
	private void refreshFormObjects(){
		
		_wacp = _wac.getConfig();
		if(null==_wacp) {
			_wacp = new WebAppConfigParams();
		}
		
		_dburl.setDefaultModel(new PropertyModel<String>(_wacp, "DBUrl"));
		_dbuser.setDefaultModel(new PropertyModel<String>(_wacp, "DBUser"));
		_dbpassword.setDefaultModel( new PropertyModel<String>(_wacp, "DBPassword"));
		_dctp.setConnectionParamsUrl(_wacp.getDBUrl(), _wacp.getDBUser(), _wacp.getDBPassword());
 
	}
	
}
