/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.config;

import java.util.ArrayList;
import java.util.List;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.DefaultDataTable;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.biffo.dms.dataprovider.config.DPDmsTables;
import org.biffo.dms.setup.DmsTable;
import org.biffo.dms.web.helpers.ResourceText;


public class DBConfigTablesPanel  extends Panel{

	@SuppressWarnings("unused")
	private static final String _className = "DBConfigTablesPanel";
	private static final long serialVersionUID = 1L;

	private DPDmsTables _qtdp = new DPDmsTables();
	private DefaultDataTable<?, ?> _ddt;
	private Form<?> _form;
	

	private WebMarkupContainer _wmcNoData;
	private Label _lblNoData;
	
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public DBConfigTablesPanel(String id) {
		super(id);
		
		initParameters();
		setOutputMarkupId(true);
		_form = new Form("dataform");
		_form.setOutputMarkupId(true);
		add(_form);
		
		List<IColumn<?,?>> columns = new ArrayList<IColumn<?,?>>();
		columns.add(new PropertyColumn<DmsTable, String>(null, "TableName"));
		columns.add(new PropertyColumn<DmsTable, String>(null, "NumberOfRows"));
		columns.add(new AbstractColumn<DmsTable, String>(new Model<String>("")){
			 
			private static final long serialVersionUID = 1L;

			@Override
		    public void populateItem(Item<ICellPopulator<DmsTable>> item, String componentId, IModel<DmsTable> rowModel){
				buildStatusCell(item, componentId, rowModel.getObject().isExists());
			}
		});
		
		
		
		int size = (int) _qtdp.size();
		if(size==0)size=1;
		_ddt = new DefaultDataTable("datatable", columns, _qtdp, size){

			private static final long serialVersionUID = 1L;

			@Override
			protected void onPageChanged() {
				super.onPageChanged();
			}
		};
		_ddt.setOutputMarkupId(true);
		_form.add(_ddt);
 
		addNoData();
		
	}
	
	public void setConnectionParamsUrl(String dburl, String username, String password) {
		try {
			_qtdp.setConnectionParamsUrl(dburl, username, password);
			_qtdp.refresh();
			int size = (int) _qtdp.size();
			if(size>0)_ddt.setItemsPerPage(size);
			checkNodata(null);
		}catch(Exception e) {
			//e.printStackTrace();
		}
	};
	
	
	private void initParameters() {
		
	}
	
	@Override
	protected void onBeforeRender() {
		checkNodata(null);
		super.onBeforeRender();
	}
	
	private void checkNodata(AjaxRequestTarget target) {
		long numberOfDocuments = _qtdp.size();
		if(numberOfDocuments==0) {
			_lblNoData.setVisible(true);
			_ddt.setVisible(false);
		}else {
			_lblNoData.setVisible(false);
			_ddt.setVisible(true);
		}
		
		if(null!=target) {
			target.add(_wmcNoData);
			target.add(_ddt);
		}
	}
	
	public void setNodata(AjaxRequestTarget target) {
		_lblNoData.setVisible(true);
		_ddt.setVisible(false);
		if(null!=target) {
			target.add(_wmcNoData);
			target.add(_ddt);
		}
	}
 
	
	private void addNoData() {
		_wmcNoData = new WebMarkupContainer("nodata");
		_wmcNoData.setOutputMarkupId(true);
		add(_wmcNoData);
		
		String temp = ResourceText.getText("lblNoData", this);
		temp = "No Tables exists!";
		_lblNoData = new Label("lblnodata",temp);
		_lblNoData.setVisible(false);
		_lblNoData.setEscapeModelStrings(false);
		_lblNoData.setOutputMarkupId(true);
		_wmcNoData.add(_lblNoData);
		
	}
	
	private void buildStatusCell(Item<ICellPopulator<DmsTable>> item, String componentId, boolean exists) {
		
		Label lbl = null;
		
		if(exists) {
			lbl = new Label(componentId, "<span class=\"statusType info\">&#xe2e6;</span>");
		}else {
			lbl = new Label(componentId, "<span class=\"statusType error\">&#xe000;</span>");
		}
		lbl.setEscapeModelStrings(false);
		lbl.setOutputMarkupId(true);
		item.add(lbl);
	}
	
}
