/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.config;


import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

import org.apache.wicket.Page;
import org.apache.wicket.PageReference;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.PasswordTextField;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.util.string.Strings;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebApplication;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.infobox.InfoboxPanel.INFOBOX_STATUS;
import org.biffo.dms.entity.Helpers.EntityUUID;
import org.biffo.dms.entity.Variables.Variable;
import org.biffo.dms.entity.Variables.Impl.OpenSearchVariablesImpl;
import org.biffo.dms.setup.DmsSetup;
import org.biffo.dms.web.helpers.ErrorLabel;
import org.biffo.dms.web.helpers.PanelTitel;
import org.biffo.dms.web.helpers.ResourceText;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;

public class OSConfigPanel extends Panel{

	private static final long serialVersionUID = 1L;
	private static final String toolbarCSS = "dataformToolbar";
	
	private Label _lblTitel;
	
	private TextField<String> _osurl, _osuser, _osindex;
	private PasswordTextField _ospassword;
	private ErrorLabel _ospassword_info, _osuser_info, _osurl_info, _osindex_info;
	
 	private Form<?> _form;
	private Toolbar _toolBar;
	private ToolbarButton _btEdit, _btCancel, _btSave, _btOK, _btCheck;

	private Object _origin;
	private PanelTitel _panelTitel = new PanelTitel();
	
	private DmsWebSession _session = DmsWebSession.getSession();
	private DmsSetup _dmsSetup = new DmsSetup();
 
	private WebMarkupContainer wmcInfo;
	private InfoboxPanel infobox;
	
	private OpenSearchVariablesImpl _configVariables = new OpenSearchVariablesImpl();
	private OpenSearchVariablesImpl _configVariables_Backup = new OpenSearchVariablesImpl();
	
	public OSConfigPanel(String id, int dbConfigID,  Object origin) {
		super(id);

		_origin = origin;
		readConfig();
		 
		
		_form = new Form<Void>("form");
		_form.setOutputMarkupId(true);
		add(_form);
		
        addTextFields();
        addCheckBoxen();
        
        addDropDownBoxen();
        addAdditionalControls();
        
        addToolBar();
        addInfobox();
        
        refreshFormObjects();
		setFormComponentsEditable(false);
		setOutputMarkupId(true);
		
		_panelTitel.setTitel(ResourceText.getText("titEditConfig", this));
        
	}
	
	
	private void addTextFields(){
		
		
		_osurl= new TextField<String>("osurl");
		_osurl.setOutputMarkupId(true);
		_form.add(_osurl); 
		
		_osindex= new TextField<String>("osindex");
		_osindex.setOutputMarkupId(true);
		_form.add(_osindex); 

		_osuser= new TextField<String>("osuser");
		_osuser.setOutputMarkupId(true);
		_form.add(_osuser); 
		
		_ospassword= new PasswordTextField("ospassword");
		_ospassword.setOutputMarkupId(true);
		_ospassword.setResetPassword(false);
		_form.add(_ospassword); 
		

		_ospassword_info= new ErrorLabel("ospassword_info");
		_form.add(_ospassword_info);
		
		_osuser_info= new ErrorLabel("osuser_info");
		_form.add(_osuser_info);
		
		_osurl_info= new ErrorLabel("osurl_info");
		_form.add(_osurl_info);
		
		_osindex_info= new ErrorLabel("osindex_info");
		_form.add(_osindex_info);

	}
	

	
	
	
	private void addCheckBoxen(){}
 
	private void addDropDownBoxen(){
	}
	
	private void addAdditionalControls(){
		_lblTitel = new Label("paneltitel", new PropertyModel<String>(this._panelTitel, "Titel"));
		_lblTitel.setOutputMarkupId(true);
		add(_lblTitel);
	}
	
	@SuppressWarnings("unused")
	private void closeForm(AjaxRequestTarget target){
		if(_origin instanceof Panel){
			target.add(OSConfigPanel.this.replaceWith((Panel)_origin).getParent());
		}else if(_origin instanceof Page){
		}else if(_origin instanceof PageReference){
			PageReference pg = (PageReference)_origin;
			setResponsePage(pg.getPage());
		}
	}
	
	
	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};

		_btEdit = new ToolbarButton().setButtonType(ToolbarButtonType.EDIT).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btEdit.setAlternate(ResourceText.getText("tbAltEdit", this));
		_toolBar.add(_btEdit);
		
		_btCancel = new ToolbarButton().setButtonType(ToolbarButtonType.CANCEL).setToolbarState(ToolbarState.STATE_NEW | ToolbarState.STATE_CHANGE).setCssClass(toolbarCSS);
		_btCancel.setAlternate(ResourceText.getText("tbAltCancel", this));
		_toolBar.add(_btCancel);
		
		_btOK = new ToolbarButton(_form).setButtonType(ToolbarButtonType.OK).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btOK.setAlternate(ResourceText.getText("tbAltOk", this));
		//_toolBar.add(_btOK);
		
		_btSave = new ToolbarButton(_form).setButtonType(ToolbarButtonType.SAVE).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btSave.setAlternate(ResourceText.getText("tbAltSave", this));
		_toolBar.add(_btSave);
		
		
		_btCheck = new ToolbarButton(_form).setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW | ToolbarState.STATE_CHANGE |  ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT);
		_btCheck.setUtf8Code("xf232");
		_btCheck.setAlternate(ResourceText.getText("tbAltCheck", this));
		_btCheck.setEventName("check");
		_btCheck.setCssClass(toolbarCSS);
		_toolBar.add(_btCheck);
		
		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		
		_form.add(_toolBar);
	}
	
	private void processEvent(String event, AjaxRequestTarget target) {
		if(event.equals("Save")) save(target, true);
		else if(event.equals("Ok")) ok(target);
		else if(event.equals("Cancel")) cancel(target);
		else if(event.equals("Edit")) edit(target);
		else if(event.equals("check")) {
			if(checkShowError(_configVariables, target)) {
				infobox.refresh(target, ResourceText.getText("errConfigurationInvalid", this), INFOBOX_STATUS.error);
				infobox.setVisible(true);
				target.add(wmcInfo);
			}else{
				infobox.refresh(target, ResourceText.getText("infConfigurationOk", this), INFOBOX_STATUS.success);
				infobox.setVisible(true);
				target.add(wmcInfo);
			}
		}
		target.add(_form);
		target.add(_toolBar);
	}
	
	private void cancel(AjaxRequestTarget target){
		 try {
			 refreshFormObjects();
			 if(_toolBar.getViewState()==ToolbarState.STATE_VIEW || _toolBar.getViewState()==ToolbarState.STATE_NEW ) {
				 //closeForm(target);
			 }else {
				 try {
					 //_configVariables = _configVariables_Backup; 
					 //refreshFormObjects();
					 _toolBar.setViewState(ToolbarState.STATE_VIEW);
					 setFormComponentsEditable(false);
					 target.add(_form);
					 target.add(_toolBar);
				 }catch(Exception e2) {
					 infobox.refresh(target, ResourceText.getText("errChangeCancel", this)+".\n"+e2.getMessage(), INFOBOX_STATUS.error);
					 infobox.setVisible(true);
					 target.add(wmcInfo);					 
				 }
			 } 
		 }catch(Exception e) {
			infobox.refresh(target, ResourceText.getText("errChangeCancel", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);			 
		 }

	}
	
	
	private void edit(AjaxRequestTarget target){
		 try {
			 _toolBar.setViewState(ToolbarState.STATE_CHANGE);
			 setFormComponentsEditable(true);
	 	}catch(Exception e) {
			infobox.refresh(target, ResourceText.getText("errChangeEditMode", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);	 		
	 	}
	}	
	
	
 
	private void ok(AjaxRequestTarget target){
		if(save(target, false)) {
			_toolBar.setViewState(ToolbarState.STATE_VIEW);
			setFormComponentsEditable(false);
			//cancel(target);
		}
	} 
	
	
	
	
	private boolean checkShowError(OpenSearchVariablesImpl config, AjaxRequestTarget target) {
		
		
		boolean error=false;
		_osurl_info.setVisible(false);
		if(Strings.isEmpty(config.getUrl())){
			_osurl_info.setErrorText(ResourceText.getText("errTextOSUrlEmpty", this));
			_osurl_info.setVisible(true);
			error = true;
		}else {
			try {
				if(!error)_dmsSetup.checkSearchJDBC(config.getUrl());
			}catch(DmsException e) {
				_osurl_info.setErrorText( e.getMessage());
				_osurl_info.setVisible(true);
				error=true;
			}
			try {
				if(!error)_dmsSetup.checkSocket();
			}catch(DmsException e) {
				_osurl_info.setErrorText( e.getMessage());
				_osurl_info.setVisible(true);
				error=true;
			}

		}
		
		_osindex_info.setVisible(false);
		if(Strings.isEmpty(config.getIndexname())){
			_osindex_info.setErrorText(ResourceText.getText("errTextOsIndexEmpty", this));
			_osindex_info.setVisible(true);
			error = true;
		}
		
		target.add(_osurl_info);
		target.add(_osindex_info);		
		
		return error;
		
	}
	
 
	private boolean save(AjaxRequestTarget target, boolean withMessage) {
		try {
			
			if(checkShowError(_configVariables, target)) {
				infobox.refresh(target, ResourceText.getText("errConfigurationInvalid", this), INFOBOX_STATUS.error);
				infobox.setVisible(true);
				target.add(wmcInfo);
				return false;
			}else {
				HashMap<String, Variable> hmVariables = _configVariables.getVariables();
				Set<String> keys = hmVariables.keySet();
				Date aenderungsDatum = new Date();
				for(String key: keys) {
					Variable var = hmVariables.get(key);
					var.setGeaendertAm(aenderungsDatum);
					var.setGeaendertVon(_session.getCurrentUser());
					var.setUUID(EntityUUID.createVariableUUID(var));
					StaticLocators.getVariables().updateEntity(var, true);
				}
				
				try {
					_dmsSetup.createOpenSearchIndex(_configVariables.getIndexname());
				}catch(DmsException e) {
					if(e.getCode()==1027) {
						infobox.refresh(target, ResourceText.getText("infConfigurationSavedIndexExists", this), INFOBOX_STATUS.warning);
						infobox.setVisible(true);
						target.add(wmcInfo);
						withMessage = false;
					}else {
						throw e;
					}
				}
				
				try {
					DmsWebApplication.getApplication().reinitConfig();
				}catch(Exception e) {
					
					/*infobox.refresh(target, ResourceText.getText("infConfigurationSavedIndexExists", this), INFOBOX_STATUS.warning);
					infobox.setVisible(true);
					target.add(wmcInfo);
					withMessage = false;
					*/
					throw e;
				}
				
				 
				
				
				DmsException dmsInfo = new DmsException(2000, "I", 
					this.getClass().getSimpleName(), 
					"save",
					ResourceText.getText("dmsInfoConfigurationChanged", this, this.getClass().getSimpleName()),
					null,
					_configVariables
				);
				StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
				_toolBar.setViewState(ToolbarState.STATE_CHANGE);
				refreshFormObjects();
			}
			if(withMessage) {
				infobox.refresh(target, ResourceText.getText("infConfigurationSaved", this), INFOBOX_STATUS.success);
				infobox.setVisible(true);
				target.add(wmcInfo);
				target.add(_lblTitel);
			}
			return true;
		}catch(Exception e) {
			infobox.refresh(target, ResourceText.getText("errConfigurationSaved", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);			 
		}
		return false;
		
	} 
	
	/*
	private void refreshFormObjects(){
		_dburl.setDefaultModel(new PropertyModel<String>(_configVariables, "Url"));
		_dbpassword.setDefaultModel(new PropertyModel<String>(_configVariables, "Password")); 
		_dbuser.setDefaultModel(new PropertyModel<String>(_configVariables, "User"));
	}*/
	
	private void setFormComponentsEditable(boolean editable){
		try{
			 _osurl.setEnabled(editable); 
			 _osindex.setEnabled(editable);
			 _ospassword.setEnabled(false);
			 _osuser.setEnabled(false);
			 
		}catch(Exception e){
			//e.printStackTrace();
		}
	}
	
 
 
	private void addInfobox() {
		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		
		infobox = new InfoboxPanel("infobox");
		infobox.setVisible(false);
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
	}
	
	
	private void readConfig() {
		try {
			_configVariables = new OpenSearchVariablesImpl();
			List<Variable> entities = StaticLocators.getVariables().getEntities(-1, -1, "", OpenSearchVariablesImpl.getDefaultVariableFilterClass());
			for(Variable entity:entities) {
				_configVariables.setVariable(entity);
				_configVariables_Backup.setVariable(entity);
			}
		}catch(Exception e) {
			e.printStackTrace();
		}
	}
	
	private void refreshFormObjects(){
		_osurl.setDefaultModel(new PropertyModel<String>(_configVariables, "Url"));
		_osindex.setDefaultModel(new PropertyModel<String>(_configVariables, "Indexname"));
		 
		_ospassword.setDefaultModel(new PropertyModel<String>(_configVariables, "Password"));
		_osuser.setDefaultModel(new PropertyModel<String>(_configVariables, "User"));
 
	}
	
	
	
	
}
