/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.documents;

import org.apache.wicket.Component;
import org.apache.wicket.ajax.AbstractDefaultAjaxBehavior;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.head.IHeaderResponse;
import org.apache.wicket.markup.head.JavaScriptReferenceHeaderItem;
import org.apache.wicket.markup.head.OnDomReadyHeaderItem;
import org.apache.wicket.request.cycle.RequestCycle;

public class DocumentsListAjaxBehavior extends AbstractDefaultAjaxBehavior{
 
	private static final long serialVersionUID = 1L;
	
	private final String containerMarkup;
	
	public DocumentsListAjaxBehavior(String containerMarkup) {
		super();
		this.containerMarkup = containerMarkup;
	}
	

	@Override
    public void renderHead(Component component, IHeaderResponse response){
		super.renderHead(component, response);
		
		 
        String callbackUrl = getCallbackUrl().toString();
        String callbackVar = "cb"+component.getMarkupId();
        response.render(OnDomReadyHeaderItem.forScript(callbackVar+" = '" + callbackUrl + "';"));
		
		StringBuilder sb = new StringBuilder();
    	sb.append("window.addEventListener('load', function(event) {dslpResize();}, true);");
    	sb.append("window.addEventListener('resize', function(event) {dslpResize();}, true);");
    	
    	sb.append("function dslpResize() {");
    	sb.append("var box = document.getElementById('"+containerMarkup+"');");
    	sb.append("if(box==null) return;");
    	sb.append("const height = box.offsetHeight;");
    	sb.append("const width = box.offsetWidth;");
		sb.append("Wicket.Ajax.get({ u: cb"+component.getMarkupId()+", ep: {'event': 'resize', 'width': width, 'height': height }});");
    	sb.append("};");
    	
    	
    	sb.append("function dslpClick(event, id) {\n");
    	sb.append("if(event.target.id.includes('imag')) {");
    	sb.append("event.stopPropagation();");
    	sb.append("return;");
    	sb.append("}");
    	sb.append("Wicket.Ajax.get({ u: cb"+component.getMarkupId()+", ep: {'event': 'click', 'value': id }});");
    	sb.append("};\n");
    	
    	sb.append("function dslpDblClick(event, id) {");
    	sb.append("Wicket.Ajax.get({ u: cb"+component.getMarkupId()+", ep: {'event': 'dblclick', 'value': id }});");
    	sb.append("};");

    	sb.append("function dslpDblClickFragment(event, idDocument, idFragment) {");
    	sb.append("Wicket.Ajax.get({ u: cb"+component.getMarkupId()+", ep: {'event': 'dblclickfragment', 'value': id, 'fragment':idFragment }});");
    	sb.append("};");
  	 
 		response.render(JavaScriptReferenceHeaderItem.forScript(sb.toString(), component.getMarkupId()));
		
		
       
    }
	
	@Override
	protected void respond(final AjaxRequestTarget target){
		RequestCycle requestCycle = RequestCycle.get();
		String event = requestCycle.getRequest().getRequestParameters().getParameterValue("event").toString(null);
		if(event!=null) {
			if(event.equals("click")) {
				int documentId  = Integer.valueOf(requestCycle.getRequest().getRequestParameters().getParameterValue("value").toString("-1"));
				OnDocumentClick(documentId, target);
			}else if(event.equals("dblclick")){
				int documentId  = Integer.valueOf(requestCycle.getRequest().getRequestParameters().getParameterValue("value").toString("-1"));
				OnDocumentDblClick(documentId, target);
			}else if(event.equals("resize")){
				int width  = Integer.valueOf(requestCycle.getRequest().getRequestParameters().getParameterValue("width").toString("1"));
				int height  = Integer.valueOf(requestCycle.getRequest().getRequestParameters().getParameterValue("height").toString("1"));
				OnResizeView(target, height, width);
			}else if(event.equals("dblclickfragment")) {
				int documentId  = Integer.valueOf(requestCycle.getRequest().getRequestParameters().getParameterValue("value").toString("-1"));
				int fragmentId  = Integer.valueOf(requestCycle.getRequest().getRequestParameters().getParameterValue("fragment").toString("-1"));
				OnFragmentDblClick(documentId, fragmentId, target);
			}
		}
	}
	
	protected void OnDocumentClick(int documentId, AjaxRequestTarget target){}
	protected void OnDocumentDblClick(int documentId, AjaxRequestTarget target){}
	protected void OnFragmentDblClick(int documentId, int fragmentId, AjaxRequestTarget target){}
	protected void OnResizeView(AjaxRequestTarget target, int height, int width){}

	

}
