/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.documents;
 

import org.apache.wicket.Page;
import org.apache.wicket.PageReference;
import org.biffo.dms.DmsWebApplicationPages;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.helpers.SearchHelper;
import org.biffo.dms.DmsWebApplicationPages.ApplicationPage;
import org.biffo.dms.session.parameters.DocumentsListParameters;
import org.biffo.dms.session.parameters.SessionParameterTypes;
import org.biffo.dms.web.documents.page.normal.DocumentsNormalListPanel;
import org.biffo.dms.web.documents.page.small.DocumentsSmallListPanel;
import org.biffo.dms.web.documents.page.timeline.DocumentsTimeLineListPanel;
import org.biffo.dms.web.documents.page.view.DocumentPanel;
import org.biffo.dms.web.template.DmsTemplate;

public class DocumentsPage extends DmsTemplate {
	
	public enum Style{
		TAB,
		LIST,
		CALENDAR
	}
 
	private DocumentsListViewType getViewType(ApplicationPage appPage) {
		if(appPage==ApplicationPage.DELETED) return DocumentsListViewType.DELETED;
		else if(appPage==ApplicationPage.ARCHIVE) return DocumentsListViewType.ARCHIVED;
		return DocumentsListViewType.NORMAL;
	}
	
	
	private static final long serialVersionUID = 1L;
	private PageReference _pagefrom;
	
	public DocumentsPage() {
		this(DmsWebApplicationPages.ApplicationPage.DOCUMENTS);
	}
	
	public DocumentsPage(ApplicationPage appPage){
		
		super(appPage);
		DocumentsListViewType listViewType = getViewType(appPage);
		switch (listViewType) {
			case NORMAL:
				if(getDisplayStyle()==Style.LIST) {
					replace(new DocumentsNormalListPanel(CONTENT_ID));
				}else if(getDisplayStyle()==Style.TAB) {
					replace(new DocumentsSmallListPanel(CONTENT_ID, DocumentsListViewType.NORMAL));	
				}else {
					replace(new DocumentsTimeLineListPanel(CONTENT_ID));	
				}
				break;
			case ARCHIVED:
				replace(new DocumentsSmallListPanel(CONTENT_ID, DocumentsListViewType.ARCHIVED));
				break;
			case DELETED:
				replace(new DocumentsSmallListPanel(CONTENT_ID, DocumentsListViewType.DELETED));
				break;
			case INBOX:
				break;
		}
	}
	
	
	public DocumentsPage(ApplicationPage appPage, PageReference pagefrom, Style viewStyle){
		super(appPage);
 
		if(appPage ==ApplicationPage.HOME) {
			replace(new DocumentsNormalListPanel(CONTENT_ID));
		}else{
	 		if(viewStyle==Style.TAB) {
	 			replace(new DocumentsSmallListPanel(CONTENT_ID, getViewType(appPage)));
	 		}else if(viewStyle==Style.LIST) {
	 			replace(new DocumentsNormalListPanel(CONTENT_ID));
	 		}else {
				replace(new DocumentsTimeLineListPanel(CONTENT_ID));	
			}
		}
		_pagefrom = pagefrom;
	}

 
	
	public DocumentsPage(ApplicationPage appPage, PageReference pagefrom, int documentId){
		this(appPage, pagefrom, documentId, DocumentStateType.VIEW);
	}
	
	
	public DocumentsPage(ApplicationPage appPage, PageReference pagefrom, int documentId, DocumentStateType viewState){
		super(appPage);
		replace(new DocumentPanel(CONTENT_ID, documentId, pagefrom, viewState ));
		_pagefrom = pagefrom;
	} 
	
	public DocumentsPage(ApplicationPage appPage, PageReference pagefrom, int documentId, DocumentStateType viewState, SearchHelper searchHelper){
		super(appPage);
		replace(new DocumentPanel(CONTENT_ID, documentId, pagefrom, viewState, searchHelper ));
		_pagefrom = pagefrom;
	} 
	
	@Override
	protected void onConfigure() {
		super.onConfigure();
	    //if(!NamesWebSession.get().isSignedIn()) NamesWebApplication.getApplication().restartResponseAtSignInPage();
	}
	
	public Page getPageFrom(){
		if(null!=_pagefrom) return _pagefrom.getPage();
		return null;
	}
	
	private DocumentsPage.Style getDisplayStyle() {
		DocumentsListParameters params = (DocumentsListParameters) DmsWebSession.getSession().getSessionParameters().getParametersByType(SessionParameterTypes.DOCUMENTS);
		return params.getCurrentDisplayStyle();
	}
	

	
}
