/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.documents.page.normal;

 
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.markup.repeater.data.DataView;
import org.apache.wicket.util.string.Strings;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebApplicationPages;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.DmsWebApplicationPages.ApplicationPage;

import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.infobox.InfoboxPanel.INFOBOX_STATUS;
import org.biffo.dms.components.pagging.DmsPagingNavigator;
import org.biffo.dms.dataprovider.DPDocuments;
import org.biffo.dms.dataprovider.os.DPSearchDocumentsHighlighted;
import org.biffo.dms.entity.Document.Document;
import org.biffo.dms.entity.Document.DocumentFilterClass;
import org.biffo.dms.entity.Document.DocumentOSFilterClass;
import org.biffo.dms.entity.Document.DocumentStatusType;
import org.biffo.dms.entity.Processing.ProcessType;
import org.biffo.dms.entity.Tags.Tag;
import org.biffo.dms.entity.user.Users.User;
import org.biffo.dms.helpers.DmsDBHelpersTag;
import org.biffo.dms.helpers.DmsDBHelpersUser;
import org.biffo.dms.helpers.SearchHelper;
 
import org.biffo.dms.session.parameters.DocumentsListParameters;
import org.biffo.dms.session.parameters.SessionParameterTypes;
import org.biffo.dms.web.components.search.searchbox.SearchboxPanel;
import org.biffo.dms.web.components.tags.TaggingPanel;
import org.biffo.dms.web.components.users.UsersSelectionPanel;
import org.biffo.dms.web.documents.DocumentStateType;
import org.biffo.dms.web.documents.DocumentsListAjaxBehavior;
import org.biffo.dms.web.documents.DocumentsPage;
import org.biffo.dms.web.documents.DocumentsProcessingDialogs;
import org.biffo.dms.web.documents.DocumentsPage.Style;
import org.biffo.dms.web.helpers.ResourceText;
import org.biffo.dms.web.uploads.UploadPanelViewList;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;
 

 
 


public class DocumentsNormalListPanel extends Panel{

	private static final String _className = "DocumentsNormalListPanel";
	private static final long serialVersionUID = 1L;
	
	private ApplicationPage _appPage = DmsWebApplicationPages.ApplicationPage.DOCUMENTS;
	
	private static final String toolbarCSS = "viewListToolbar";
	
	private DataView<Document> _dvDocuments;
	private final ArrayList<Document> _tempDocumentList=new ArrayList<Document>();	
	
	private DocumentsListParameters _thisParameters;
	private DPDocuments _dpDocuments = new DPDocuments();
	private DocumentOSFilterClass _dsfc = new DocumentOSFilterClass();
	private DocumentFilterClass _dfcDocuments = new DocumentFilterClass();

	
	private WebMarkupContainer _wmcDocuments,  _wmcNoData;
	private HashSet<Integer> _selectionList = new HashSet<Integer>();
	private Toolbar _toolBar;
	private ToolbarButton _btAll, _btNone, _btUSERs, _btRefresh, _btSortName, _btSortDate,_btSEARCH, _btTAGs, _btUPLOAD, _btListView, _btCalendarView, _btDelete;

	private DmsWebSession _session =  DmsWebSession.getSession();
	
	private TaggingPanel _tsbp;
	private UsersSelectionPanel _usbp;
	private WebMarkupContainer _wmcLeft;
	
	private WebMarkupContainer wmcInfo;
	private InfoboxPanel infobox;
	private DmsPagingNavigator _dpn;
	private Label _lblNoData;
	
	private SearchboxPanel _sbbp;
	private UploadPanelViewList _upvl;
	
	private boolean _uploadExists=true;
	private DocumentsProcessingDialogs _dpd;
	
	private SessionParameterTypes _sessionParameterTypes;
	
	public DocumentsNormalListPanel(String id) {
		this(id, SessionParameterTypes.DOCUMENTS);
	}
	
	
	public DocumentsNormalListPanel(String id, SessionParameterTypes sessionParameterTypes) {
		super(id);
		_sessionParameterTypes = sessionParameterTypes;
		
		initParameters();
		addInfobox();
		addNoData();
		addProcessingDialogs();

		_wmcDocuments = new WebMarkupContainer("wmcdocuments") {

			private static final long serialVersionUID = 1L;
			
			@Override
			protected void onBeforeRender() {
				StringBuilder sb = new StringBuilder();
				sb.append("innercontent ");
				boolean visible=false;
				if(_thisParameters.isSearchVisible()) {
					_sbbp.setVisible(true);
					visible=true;
				}else {
					_sbbp.setVisible(false);
				}
				if(_thisParameters.isTagsVisible()) {
					_tsbp.setVisible(true);
					visible=true;
				}else {
					_tsbp.setVisible(false);
				}
				if(_thisParameters.isUploadVisible()) {
					if(_upvl.getNumberOfImporters()>0) {
						_upvl.setVisible(true);
						visible=true;
					}
				}else {
					_upvl.setVisible(false);
				}
				if(_thisParameters.isUsersVisible()) {
					_usbp.setVisible(true);
					visible=true;
				}else {
					_usbp.setVisible(false);
				}
				if(visible)sb.append("innercontent_tags_active ");
				
				super.onBeforeRender();
				_wmcDocuments.add(AttributeModifier.replace("class",sb.toString().trim()));		
			}
			
		};
		_wmcDocuments.setOutputMarkupId(true);
		
		_wmcLeft = new WebMarkupContainer("left");
		_wmcLeft.setOutputMarkupId(true);
		add(_wmcLeft);
		
		
 
		
		_dvDocuments = new DataView<Document>("documents", _dpDocuments, _thisParameters.getNumberOfItemsN() ) {

			private static final long serialVersionUID = 1L;

			@Override
			protected void populateItem(Item<Document> item) {
				
				DocumentNormalPanel dp = new DocumentNormalPanel( "document", item.getModel(), item.getIndex()) {

					private static final long serialVersionUID = 1L;
					
					@Override
					protected void onBeforeRender() {
						for(int i=0;i<_tempDocumentList.size();i++) {
							if(_tempDocumentList.get(i).getId()==item.getModel().getObject().getId()) {
								this.setDocumentSearch(_tempDocumentList.get(i));
								break;
							}
						}
						
						if(_selectionList.contains(item.getModel().getObject().getId())) {
							setSelected(true);	
						}else {
							setSelected(false);
						}
						super.onBeforeRender();
					}
					
					@Override
					protected void OnDeleteClick(Document document, AjaxRequestTarget target) {
						_selectionList.clear();
						_selectionList.add(document.getId());
						_dpd.processDelete(target, _selectionList);
					}

					
					@Override
					protected void OnTagDrop(Tag tag, int documentId, AjaxRequestTarget target) {
						componentSingelRefresh(target, documentId);
					}
					@Override
					protected void OnUserDrop(User user, int documentId, AjaxRequestTarget target) {
						componentSingelRefresh(target, documentId);
					}

					
				};
				dp.setOutputMarkupId(true);
				dp.setShowTags(_thisParameters.isShowTags());
				dp.setShowUsers(_thisParameters.isShowUsers());
				item.setOutputMarkupId(true);
				item.add(dp);
			}
			
			@Override
			protected void onAfterRender() {
				_thisParameters.setCurrentPageNumberN(_dvDocuments.getCurrentPage());
				_thisParameters.setSortParam(_dpDocuments.getSort());
				super.onAfterRender();
			}
			
			@Override
			protected void onBeforeRender() {
				if(!Strings.isEmpty(_dsfc.getSearchString()) && _sbbp.hasResults()){
					buildTempDocumentList();
				}else {
					_tempDocumentList.clear();
				}
				super.onBeforeRender();
			} 
			
		};
		_dvDocuments.setOutputMarkupId(true);
		_wmcDocuments.add(_dvDocuments);
		add(_wmcDocuments);

		addToolBar();
		addUploadPanel();
		addSearchPanel();
		addTaggingPanel();
		addUsersPanel();
		addBehavior();
		
		
		_dpn = new DmsPagingNavigator("navigation", _dvDocuments, true, false, true, true) {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnNbOfItemsChanged(AjaxRequestTarget target, long itemPerPage) {
				_thisParameters.setNumberOfItemsN(itemPerPage);
				_dvDocuments.setItemsPerPage(itemPerPage);
				storeParameters();
				target.add(_wmcDocuments);
				target.add(_dpn);
			}
			@Override
			protected void OnShowTagsChanged(AjaxRequestTarget target, boolean enabled) {
				_thisParameters.setShowTags(enabled);
				storeParameters();
				target.add(_wmcDocuments);
				target.add(_dpn);
			}
			@Override
			protected void OnShowUsersChanged(AjaxRequestTarget target, boolean enabled) {
				_thisParameters.setShowUsers(enabled);
				storeParameters();
				target.add(_wmcDocuments);
				target.add(_dpn);
			}
			
		};
		_dpn.setOutputMarkupId(true);
		_dpn.setNumberOfItems(String.valueOf(_thisParameters.getNumberOfItemsN()));
		_dpn.setDynamicEnabled(false);
		_dpn.setShowTags(_thisParameters.isShowTags());
		_dpn.setShowUsers(_thisParameters.isShowUsers());
		
		add(_dpn);
		
		if(_thisParameters.getCurrentPageNumberN()!=-1) {
			_dvDocuments.setCurrentPage(_thisParameters.getCurrentPageNumberN());
		}
		
		
		if(_sbbp.getCondition()!=null) {
			_dfcDocuments.setLinkedWhere(" id in ("+_sbbp.getCondition()+")");
			_dpDocuments.setFilterState(_dfcDocuments);
			//_dpDocuments.getFilterState().setLinkedWhere(" id in ("+_sbbp.getCondition()+")");
		}
		
		if(_dpDocuments.size()==0) {
			checkNodata(null);
		}
		
	}
	
	private void addSearchPanel() {
		
		_sbbp = new SearchboxPanel("searchbox", _dsfc) {
			private static final long serialVersionUID = 1L;
			@Override
			protected void OnButtonClear(AjaxRequestTarget target) {
				super.OnButtonClear(target);
				_session.clearSearchParameters();
				_tempDocumentList.clear();
				_dfcDocuments.setLinkedWhere(null);
				_dpDocuments.setFilterState(_dfcDocuments);
				//_dpDocuments.getFilterState().setLinkedWhere(null);
				_dpDocuments.size();
				_thisParameters.setFilterObject(_dsfc);
				storeParameters();
				target.add(_wmcDocuments);
				target.add(_dpn);
			};
			@Override
			protected void OnButtonSearch(AjaxRequestTarget target) {
				super.OnButtonSearch(target);
				if(hasResults()) {
					String condition = getCondition();
					_dfcDocuments.setLinkedWhere(" id in ("+condition+")");
					_dpDocuments.setFilterState(_dfcDocuments);
					//_dpDocuments.getFilterState().setLinkedWhere(" id in ("+condition+")");
					_dpDocuments.size();
					_thisParameters.setFilterObject(_dsfc);
					storeParameters();
					target.add(_wmcDocuments);
					target.add(_dpn);
				}
			}
			@Override
			protected void OnQueryChanged(AjaxRequestTarget target) {
				super.OnQueryChanged(target);
				if(hasResults()) {
					String condition = getCondition();
					_dfcDocuments.setLinkedWhere(" id in ("+condition+")");
					_dpDocuments.setFilterState(_dfcDocuments);
					//_dpDocuments.getFilterState().setLinkedWhere(" id in ("+condition+")");
					_dpDocuments.size();
					_thisParameters.setFilterObject(_dsfc);
					storeParameters();
					target.add(_wmcDocuments);
					target.add(_dpn);
				}
			}
			
			@Override
			protected void OnSearchNoData(AjaxRequestTarget target) {
				super.OnSearchNoData(target);
				infobox.refresh(target, ResourceText.getText("lblNoSearchResults", this), INFOBOX_STATUS.warning);
				infobox.setVisible(true);
				target.add(wmcInfo);
			}
			
		};
		_sbbp.setOutputMarkupId(true);
		_wmcLeft.add(_sbbp);
	}
	
	private void addUploadPanel() {
		_upvl = new UploadPanelViewList("uploadpanel") {

			private static final long serialVersionUID = 1L;

			@Override
			protected void OnUploadPanelViewListFinished(AjaxRequestTarget target, int processId, int  uploads, int dublicates) {

				if(dublicates>0) {
					infobox.refresh(target, ResourceText.getText("warUploadDublicate", this, uploads, dublicates), INFOBOX_STATUS.warning);
				}else{
					infobox.refresh(target, ResourceText.getText("infUploadOK", this, uploads), INFOBOX_STATUS.success);
				};
				infobox.setVisible(true);

				_dpDocuments.size();
				target.add(wmcInfo);
				target.add(_dpn);
				target.add(_wmcDocuments);
				
			}
		};
		_upvl.setOutputMarkupId(true);
		_wmcLeft.add(_upvl);
		if(_upvl.getNumberOfImporters()==0) {
			_upvl.setVisible(false);
		}
	}
	
	 
	private void addUsersPanel() {
		_usbp = new UsersSelectionPanel("users") {
 
			private static final long serialVersionUID = 1L;
			
			@Override
			protected void onAdd(AjaxRequestTarget target, HashSet<User> userslist) {
				super.onAdd(target, userslist);
				try {
					DmsDBHelpersUser.addUsersToDocuments(
							filterArchivedDocuments(_selectionList),
							userslist,
							_session.getCurrentUser(),
							_session.getCurrentTenantId(),
							false
					);
					_selectionList = new HashSet<Integer>();
					_dpDocuments.size();
					target.add(_wmcDocuments);
					target.add(_wmcLeft);
				}catch(Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "onAdd", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);

					
				}
			}
			
			@Override
			protected void onReplace(AjaxRequestTarget target, HashSet<User> userlist) {
				super.onReplace(target, userlist);
				try {
					DmsDBHelpersUser.replaceUsersFromDocuments(
						filterArchivedDocuments(_selectionList),
						userlist,
						_session.getCurrentUser(),
						_session.getCurrentTenantId(),
						true
					);
					_selectionList = new HashSet<Integer>();
					_dpDocuments.size();
					target.add(_wmcDocuments);
					target.add(_wmcLeft);
				}catch(Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "onReplace", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);

				}
				
			}
			
			@Override
			protected void onDelete(AjaxRequestTarget target, HashSet<User> userlist) {
				super.onDelete(target, userlist);
				try {
					DmsDBHelpersUser.deleteUsersFromDocuments(
						filterArchivedDocuments(_selectionList),
						userlist,
						_session.getCurrentUser(),
						_session.getCurrentTenantId()
					);
					_selectionList = new HashSet<Integer>();
					_dpDocuments.size();
					target.add(_wmcDocuments);
					target.add(_wmcLeft);
				}catch(Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "onDelete", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);

				}
			}
			
		};
		_usbp.setOutputMarkupId(true);
		_wmcLeft.add(_usbp);
 
	}
	
	private void addTaggingPanel() {
		_tsbp = new TaggingPanel("tags") {
			
			private static final long serialVersionUID = 1L;

			@Override
			protected void onAdd(AjaxRequestTarget target, HashSet<Tag> taglist) {
				super.onAdd(target, taglist);
				try {
					DmsDBHelpersTag.addTagsToDocuments(filterArchivedDocuments(_selectionList), taglist, _session.getCurrentUser(), _session.getCurrentTenantId());
					_selectionList = new HashSet<Integer>();
					_dpDocuments.size();
					target.add(_wmcDocuments);
					target.add(_wmcLeft);
				}catch(Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "onAdd", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);

				}
			}
			
			@Override
			protected void onReplace(AjaxRequestTarget target, HashSet<Tag> taglist) {
				super.onReplace(target, taglist);
				try {
					DmsDBHelpersTag.replaceTagsFromDocuments(filterArchivedDocuments(_selectionList), taglist, _session.getCurrentUser(), _session.getCurrentTenantId());
					_selectionList = new HashSet<Integer>();
					_dpDocuments.size();
					target.add(_wmcDocuments);
					target.add(_wmcLeft);
				}catch(Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "onReplace", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);

				}
			}

			@Override
			protected void onDelete(AjaxRequestTarget target, HashSet<Tag> taglist) {
				super.onDelete(target, taglist);
				try {
					DmsDBHelpersTag.deleteTagsFromDocuments(filterArchivedDocuments(_selectionList), taglist, _session.getCurrentUser(), _session.getCurrentTenantId());
					_selectionList = new HashSet<Integer>();
					_dpDocuments.size();
					target.add(_wmcDocuments);
					target.add(_wmcLeft);
				}catch(Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "onDelete", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);

				}
			}
			
		};
		_tsbp.setOutputMarkupId(true);
		_wmcLeft.add(_tsbp);
	}
	
	
	

	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};
		
		_btAll= new ToolbarButton().setButtonType(ToolbarButtonType.ALL).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btAll.setAlternate(ResourceText.getText("tbAll", this));
		_toolBar.add(_btAll);
		
		
		_btNone= new ToolbarButton().setButtonType(ToolbarButtonType.NONE).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btNone.setAlternate(ResourceText.getText("tbNone", this));
		_toolBar.add(_btNone);
		
		_btDelete=new ToolbarButton().setButtonType(ToolbarButtonType.DELETE).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btDelete.setAlternate(ResourceText.getText("tbDelete", this));
		_toolBar.add(_btDelete);
		
		_btRefresh = new ToolbarButton().setButtonType(ToolbarButtonType.REFRESH).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btRefresh.setAlternate(ResourceText.getText("btRefresh", this));
		_toolBar.add(_btRefresh);

		_btListView = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT);
		_btListView.setUtf8Code("xe9b0");
		_btListView.setEventName("ChangeListView");
		_btListView.setAlternate(ResourceText.getText("btListView", this));
		_btListView.setCssClass(toolbarCSS);
		_toolBar.add(_btListView);
		
		_btCalendarView = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btCalendarView.setUtf8Code("xebcc");
		_btCalendarView.setEventName("ChangeCalendarView");
		_btCalendarView.setAlternate(ResourceText.getText("btCalendarView", this));
		_btCalendarView.setCssClass(toolbarCSS);
		_toolBar.add(_btCalendarView);
		
		_btSEARCH = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT);
		_btSEARCH.setUtf8Code("xe8b6");
		_btSEARCH.setEventName("SEARCH");
		_btSEARCH.setAlternate(ResourceText.getText("btSEARCH", this));
		_btSEARCH.setCssClass(toolbarCSS);
		_toolBar.add(_btSEARCH);
		
		_btTAGs = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btTAGs.setUtf8Code("xf05b");
		_btTAGs.setEventName("TAGS");
		_btTAGs.setAlternate(ResourceText.getText("btTAGs", this));
		_toolBar.add(_btTAGs);
		
		_btUSERs = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btUSERs.setUtf8Code("xe7ef");
		_btUSERs.setEventName("USERS");
		_btUSERs.setAlternate(ResourceText.getText("btUSERs", this));
		_toolBar.add(_btUSERs);
		
		_btUPLOAD = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btUPLOAD.setUtf8Code("xf09b");
		_btUPLOAD.setEventName("UPLOAD");
		_btUPLOAD.setAlternate(ResourceText.getText("btUPLOAD", this));
		if(_uploadExists)_toolBar.add(_btUPLOAD);
		
		_btSortName = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT);
		_btSortName.setUtf8Code("xe053");
		_btSortName.setEventName("SortName");
		_btSortName.setCssClass(toolbarCSS);
		_btSortName.setAlternate(ResourceText.getText("btSortName", this));
		_toolBar.add(_btSortName);
		
		
		_btSortDate = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT);
		_btSortDate.setUtf8Code("xe164");
		_btSortDate.setEventName("SortDate");
		_btSortDate.setCssClass(toolbarCSS);
		_btSortDate.setAlternate(ResourceText.getText("btSortDate", this));
		_toolBar.add(_btSortDate);
		
		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		
		add(_toolBar);
		
	}
	
	private void processEvent(String event, AjaxRequestTarget target) {
		if(_sbbp.isVisible())_sbbp.closeAll(target);
		if(event.equals("All")) {
			fselectAll(true);
			target.add(_wmcDocuments);
		}else if(event.equals("None")) {
			fselectAll(false);
			target.add(_wmcDocuments);
		}else if(event.equals("Refresh")) {
			if(_sbbp.isVisible()) {
				refreshFromSearchBox(target);
			}else {
				_dpDocuments.size();
				target.add(_dpn);
				target.add(_wmcDocuments);
			}
		}else if(event.equals("SEARCH")) {
			showHideModules(_sbbp, target);
		}else if(event.equals("TAGS")) {
			showHideModules(_tsbp, target);
		}else if(event.equals("UPLOAD")) {
			showHideModules(_upvl, target);
		}else if(event.equals("USERS")) {
			showHideModules(_usbp, target);
		}else if(event.equals("SortName")) {
			if(_dpDocuments.getSort()==null || !_dpDocuments.getSort().isAscending()) {
				_dpDocuments.setSort("DocName", SortOrder.ASCENDING);
			}else if(_dpDocuments.getSort().isAscending()) {
				_dpDocuments.setSort("DocName", SortOrder.DESCENDING);
			}
			_dpDocuments.size();
			target.add(_wmcDocuments);
		}else if(event.equals("SortDate")) {
			if(_dpDocuments.getSort()==null || !_dpDocuments.getSort().isAscending()) {
				_dpDocuments.setSort("documentdatum", SortOrder.ASCENDING);
			}else if(_dpDocuments.getSort().isAscending()) {
				_dpDocuments.setSort("documentdatum", SortOrder.DESCENDING);
			}
			_dpDocuments.size();
			target.add(_wmcDocuments);
		}else if(event.equals("Delete")) {
			if(_selectionList.size()>0) {
				_dpd.processDelete(target, _selectionList);
			}
		}else if(event.equals("ChangeListView")) {
			DocumentsPage page = new DocumentsPage(_appPage, getPage().getPageReference(), Style.TAB);
			setResponsePage(page);
			storeParameters();
		}else if(event.equals("ChangeCalendarView")) {
			DocumentsPage page = new DocumentsPage(_appPage, getPage().getPageReference(), Style.CALENDAR);
			setResponsePage(page);
			storeParameters();
		}
					
	}
	
	private void fselectAll(boolean select){
		long itemsPerPage = _dvDocuments.getItemsPerPage();
		if(_dvDocuments.size()<itemsPerPage)itemsPerPage=_dvDocuments.size();
		long currentPage = _dvDocuments.getCurrentPage();
		Iterator<?> iter =_dvDocuments.getDataProvider().iterator(itemsPerPage*currentPage, itemsPerPage);
		while(iter.hasNext()) {
			Document ql =(Document)iter.next();
			if(select){
				if(!_selectionList.contains(ql.getId())) _selectionList.add(ql.getId());
			}else{
				if(_selectionList.contains(ql.getId())) _selectionList.remove(ql.getId());
			}
		}
	}
	
	public HashSet<Integer> getSelectionIds(){
		return _selectionList;
	}
	
	private void addInfobox() {
		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		
		infobox = new InfoboxPanel("infobox");
		infobox.setVisible(false);
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
	}
	
	private void addNoData() {
		_wmcNoData = new WebMarkupContainer("nodata");
		_wmcNoData.setOutputMarkupId(true);
		add(_wmcNoData);
		_lblNoData = new Label("lblnodata",ResourceText.getText("lblNoData", this));
		_lblNoData.setVisible(false);
		_lblNoData.setOutputMarkupId(true);
		_wmcNoData.add(_lblNoData);
		
	}
	
	private void initParameters() {
		_thisParameters = (DocumentsListParameters) _session.getSessionParameters().getParametersByType(_sessionParameterTypes);
		_thisParameters.setCurrentDisplayStyle(Style.LIST);
		
		if(_thisParameters.getSortParam()!=null) _dpDocuments.setSort(_thisParameters.getSortParam());
		_dpDocuments.getFilterState().setObjectStatusType(DocumentStatusType.DELETED, false);
		_dpDocuments.getFilterState().setObjectStatusType(DocumentStatusType.ARCHIVED, false);

		
		if(DmsDBHelpersTag.getDefaultPDFImporterID()==-1) {
			_uploadExists=false;
			_thisParameters.setUploadVisible(false);
		};
		
		if(_thisParameters.getFilterObject()!=null && _thisParameters.isSearchVisible()) _dsfc = (DocumentOSFilterClass) _thisParameters.getFilterObject();
		_dsfc.setObjectStatusType(DocumentStatusType.DELETED, false);
		_dsfc.setObjectStatusType(DocumentStatusType.ARCHIVED, false);
		
		
	}
	
	private void storeParameters() {
		_session.getSessionParameters().setParametersByType(_sessionParameterTypes, _thisParameters);
	}
	
	
	

	
	private void addBehavior() {
		DocumentsListAjaxBehavior dsab = new DocumentsListAjaxBehavior(_wmcDocuments.getMarkupId()) {
 
			private static final long serialVersionUID = 1L;
			@Override
			protected void OnDocumentClick(int documentId, AjaxRequestTarget target) {
				setSelectUnselect(Integer.valueOf(documentId));
				componentSingelRefresh(target, documentId);
			}
			
			@Override
			protected void OnDocumentDblClick(int documentId, AjaxRequestTarget target) {
				DocumentsPage page = null;
				SearchHelper sh=_sbbp.getSearchHelper(""+documentId);
				if(sh!= null && sh.getSearchString()!=null && !sh.getSearchString().isEmpty())page = new DocumentsPage(_appPage, getPage().getPageReference(), documentId, DocumentStateType.VIEW, sh);
				else page = new DocumentsPage(_appPage, getPage().getPageReference(), documentId);
				setResponsePage(page);
				
			}
			
		};
		add(dsab);
	}
	
	private void setSelectUnselect(int documentId) {
		if(_selectionList.contains(documentId)) {
			_selectionList.remove(documentId);
		}else {
			_selectionList.add(documentId);
		}
	}
 
	
	
	
	private HashSet<Integer> filterArchivedDocuments(HashSet<Integer> selectionList){
		HashSet<Integer> selectionListFilter = new HashSet<Integer>();
		for(Integer id : selectionList) {
			try {
				Document document = StaticLocators.getDocuments().getEntityById(id);
				if(!document.hasObjectStatusTypes(DocumentStatusType.ARCHIVED)) {
					selectionListFilter.add(id);
				}
			}catch(Exception e) {
				DmsException dmsException = new DmsException(1000, "E", _className, "filterArchivedDocuments", null, e);
				StaticLocators.getMessageHandler().createMessage(dmsException, _session);

			}
		}
		return selectionListFilter;
	}
	
	private void componentSingelRefresh(AjaxRequestTarget target, int documentId) {
		int indexToRefresh = -1;
		
		Item<?> documentItem = null;
		Iterator<Component> components = _dvDocuments.iterator();
		while(components.hasNext()) {
			Component comp = components.next();
			documentItem = (Item<?>) comp;
			if(documentItem.getDefaultModelObject() instanceof Document) {
				Document doc = (Document) documentItem.getDefaultModelObject();
				if(doc.getId()==documentId) {
					indexToRefresh= documentItem.getIndex();
					break;
				}
			}
		}
		if(indexToRefresh!=-1)target.add(_dvDocuments.get(indexToRefresh));
	}

	
	private void buildTempDocumentList() {
		try {
			_tempDocumentList.clear();
			Iterator<?> iter =_dvDocuments.getDataProvider().iterator(_thisParameters.getNumberOfItemsN()*_dvDocuments.getCurrentPage(), _thisParameters.getNumberOfItemsN());
			int resultSize =0;
			List<Integer> ids = new LinkedList<Integer>();
			while(iter.hasNext()) {
				Document selDoc =(Document)iter.next();
				resultSize = resultSize+selDoc.getNumberOfPages();
				ids.add(selDoc.getId());
			}
			DocumentOSFilterClass dosfc = new DocumentOSFilterClass();
			dosfc.getIDList().addAll(ids);
			if(resultSize>1000)dosfc.setSize(1000);
			else dosfc.setSize(resultSize);
			dosfc.setSearchString(_dsfc.getSearchString());
			DPSearchDocumentsHighlighted dpsd = new DPSearchDocumentsHighlighted();
			_tempDocumentList.addAll(dpsd.getHightlightedDocuments(dosfc));
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", _className, "buildTempDocumentList", null, e);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);

		}
	}
	
	private void checkNodata(AjaxRequestTarget target) {
		long numberOfDocuments = 0;
		try {
			DocumentFilterClass dfc = new DocumentFilterClass();
			dfc.setObjectStatusType(DocumentStatusType.ARCHIVED, false);
			dfc.setObjectStatusType(DocumentStatusType.DELETED, false);
			numberOfDocuments = StaticLocators.getDocuments().getNumberOfEntities(dfc);			
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", _className, "checkNodata", null, e);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);

		}
		
		if(numberOfDocuments==0) {
			_toolBar.setVisible(false);
			_wmcDocuments.setVisible(false);
			_wmcLeft.setVisible(false);
			_dpn.setVisible(false);
			_lblNoData.setVisible(true);
		}else {
			_toolBar.setVisible(true);
			_wmcDocuments.setVisible(true);
			_dpn.setVisible(true);
			_lblNoData.setVisible(false);			
		}
		
		if(null!=target) {
			target.add(_toolBar);
			target.add(_wmcDocuments);
			target.add(_wmcLeft);
			target.add(_wmcNoData);
		}
	}
	
	private void addProcessingDialogs() {
		_dpd = new DocumentsProcessingDialogs("processingdialogs") {

			private static final long serialVersionUID = 1L;
			
			@Override
			protected void OnProcessError(AjaxRequestTarget target, ProcessType processType, String message) {
				infobox.refresh(target, message, INFOBOX_STATUS.error);
				infobox.setVisible(true);
				fselectAll(false);
				_selectionList = new HashSet<Integer>();
				_dpDocuments.size();
				target.add(wmcInfo);
				target.add(_dpn);
				target.add(_wmcDocuments);
				checkNodata(target);
			}
			
			@Override
			protected void OnProcessFinished(AjaxRequestTarget target, ProcessType processType, String message, int sourceDocumentId, int targetDocumentId) {
				infobox.refresh(target, message, INFOBOX_STATUS.success);
				infobox.setVisible(true);
				fselectAll(false);
				_selectionList = new HashSet<Integer>();
				_dpDocuments.size();
				target.add(wmcInfo);
				target.add(_dpn);
				target.add(_wmcDocuments);
				checkNodata(target);
			}
			
		};
		_dpd.setOutputMarkupId(true);
		add(_dpd);
		
	}
	
	private void refreshFromSearchBox(AjaxRequestTarget target) {
		_sbbp.refresh(target);
		if(_sbbp.hasResults()) {
			_dfcDocuments.setLinkedWhere(" id in ("+_sbbp.getCondition()+")");
			_dpDocuments.setFilterState(_dfcDocuments);
			_dpDocuments.size();
			_thisParameters.setFilterObject(_dsfc);
			storeParameters();
			target.add(_dpn);
			target.add(_wmcDocuments);
		}
	}
	
	
	private void showHideModules(Panel panel, AjaxRequestTarget target) {
		
		boolean setVisible = !panel.isVisible();
		boolean hideLeft=true;
		if(panel instanceof TaggingPanel) {
			_thisParameters.setTagsVisible(setVisible);
		}else if(panel instanceof UsersSelectionPanel) {
			_thisParameters.setUsersVisible(setVisible);
		}else if(panel instanceof SearchboxPanel) {
			_thisParameters.setSearchVisible(setVisible);
		}else if(panel instanceof UploadPanelViewList) {
			_thisParameters.setUploadVisible(setVisible);
		}
		panel.setVisible(setVisible);
		if(_tsbp.isVisible())hideLeft=false;
		if(_usbp.isVisible())hideLeft=false;
		if(_sbbp.isVisible())hideLeft=false;
		if(_upvl.isVisible())hideLeft=false;

		
		if(hideLeft) {
			target.appendJavaScript("document.getElementById(\""+_wmcDocuments.getMarkupId()+"\").classList.remove('innercontent_tags_active');dslpResize();");
		}else {
			target.appendJavaScript("document.getElementById(\""+_wmcDocuments.getMarkupId()+"\").classList.add('innercontent_tags_active');dslpResize();");
		}
		storeParameters();
		target.add(_wmcLeft);
		
	}
}
