/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.documents.page.small;


import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.markup.repeater.data.DataView;
import org.apache.wicket.util.string.Strings;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebApplicationPages;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.DmsWebApplicationPages.ApplicationPage;
import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.infobox.InfoboxPanel.INFOBOX_STATUS;
import org.biffo.dms.components.pagging.DmsPagingNavigator;
import org.biffo.dms.dataprovider.DPDocuments;
import org.biffo.dms.dataprovider.os.DPSearchDocumentsHighlighted;
import org.biffo.dms.entity.Document.Document;
import org.biffo.dms.entity.Document.DocumentFilterClass;
import org.biffo.dms.entity.Document.DocumentOSFilterClass;
import org.biffo.dms.entity.Document.DocumentStatusType;
import org.biffo.dms.entity.Processing.ProcessType;
import org.biffo.dms.entity.Tags.Tag;
import org.biffo.dms.entity.Variables.Impl.ApiModuleImpl;
import org.biffo.dms.entity.user.Users.User;
import org.biffo.dms.helpers.DmsDBHelpersTag;
import org.biffo.dms.helpers.DmsDBHelpersUser;
import org.biffo.dms.helpers.SearchHelper;
import org.biffo.dms.processQueue.tasks.ModuleApplyTask;
import org.biffo.dms.session.parameters.DocumentsListParameters;
import org.biffo.dms.session.parameters.SessionParameterTypes;
import org.biffo.dms.user.UserRestrictionType;
import org.biffo.dms.web.components.modules.ModulesSelectionPanel;
import org.biffo.dms.web.components.search.searchbox.SearchboxPanel;
import org.biffo.dms.web.components.tags.TaggingPanel;
import org.biffo.dms.web.components.users.UsersSelectionPanel;
import org.biffo.dms.web.documents.DocumentStateType;
import org.biffo.dms.web.documents.DocumentsPage;
import org.biffo.dms.web.documents.DocumentsProcessingDialogs;
import org.biffo.dms.web.documents.DocumentsListAjaxBehavior;
import org.biffo.dms.web.documents.DocumentsPage.Style;
import org.biffo.dms.web.helpers.ResourceText;
import org.biffo.dms.web.helpers.processing.ProcessingPanel;
import org.biffo.dms.web.uploads.UploadPanelViewList;
import org.biffo.dms.web.documents.DocumentsListViewType;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;



public class DocumentsSmallListPanel extends Panel{

	private static final long serialVersionUID = 1L;
	private static final String _className = "DocumentsSmallListPanel";
	private static final String toolbarCSS = "viewListToolbar";
		
	private DataView<Document> _dvDocuments;
	private DPDocuments _dpDocuments = new DPDocuments();
	private DocumentOSFilterClass _dsfc = new DocumentOSFilterClass();
	private DocumentFilterClass _dfcDocuments = new DocumentFilterClass();
	
	private final ArrayList<Document> _tempDocumentList=new ArrayList<Document>();
	
	private ApplicationPage _appPage = DmsWebApplicationPages.ApplicationPage.DOCUMENTS;
	
	private WebMarkupContainer _wmcDocuments, _wmcNoData;
	private HashSet<Integer> _selectionList = new HashSet<Integer>();
	private Toolbar _toolBar;
	private ToolbarButton _btAll, _btNone, _btRefresh, _btSortName, _btSortDate,_btSEARCH; 
	private ToolbarButton _btTAGs, _btUSERs, _btUPLOAD, _btListView, _btCalendarView, _btMerge, _btDelete;
	private ToolbarButton _btRestore, _btMODULES;
	
	private DmsWebSession _session =  DmsWebSession.getSession();
	private DocumentsListParameters _thisParameters;
	
	private TaggingPanel _tsbp;
	private UsersSelectionPanel _usbp;
	private SearchboxPanel _sbbp;
	private UploadPanelViewList _upvl;
	private ModulesSelectionPanel _msbp;
	
	private WebMarkupContainer _wmcLeft;
	private WebMarkupContainer wmcInfo;

	private InfoboxPanel infobox;
	private DmsPagingNavigator _dpn;
	private Label _lblNoData;


	
	private boolean _uploadExists=true;
	private DocumentsProcessingDialogs _dpd;
	private ProcessingPanel _prp;
	
	private String _documentKategorie;
	
	private DocumentsListViewType _viewType=DocumentsListViewType.NORMAL;
 	
	public DocumentsSmallListPanel(String id) {
		this(id, DocumentsListViewType.NORMAL);
	}
	
	
	public DocumentsSmallListPanel(String id, DocumentsListViewType viewType) {
		super(id);

		_viewType = viewType;
		
		
		
		initParameters();
		addInfobox();
		addNoData();
		addProcessingDialogs();
		
		_wmcDocuments = new WebMarkupContainer("wmcdocuments") {

			private static final long serialVersionUID = 1L;
			
			@Override
			protected void onBeforeRender() {
				StringBuilder sb = new StringBuilder();
				sb.append("innercontent ");
				boolean visible=false;
				if(_thisParameters.isSearchVisible()) {
					_sbbp.setVisible(true);
					visible=true;
				}else {
					_sbbp.setVisible(false);
				}
				if(_thisParameters.isTagsVisible()) {
					_tsbp.setVisible(true);
					visible=true;
				}else {
					_tsbp.setVisible(false);
				}
				if(_thisParameters.isUploadVisible()) {
					if(_upvl.getNumberOfImporters()>0) {
						_upvl.setVisible(true);
						visible=true;
					}
				}else {
					_upvl.setVisible(false);
				}
				if(_thisParameters.isUsersVisible()) {
					_usbp.setVisible(true);
					visible=true;
				}else {
					_usbp.setVisible(false);
				}
				if(_thisParameters.isModulesVisible()) {
					_msbp.setVisible(true);
					visible=true;
				}else {
					_msbp.setVisible(false);
				}
				
				if(visible)sb.append("innercontent_tags_active ");
				
				super.onBeforeRender();
				_wmcDocuments.add(AttributeModifier.replace("class",sb.toString().trim()));			
			}
			
		};
		_wmcDocuments.setOutputMarkupId(true);
		_wmcLeft = new WebMarkupContainer("left");
		_wmcLeft.setOutputMarkupId(true);
		add(_wmcLeft);
		
		_dvDocuments = new DataView<Document>("documents", _dpDocuments, _thisParameters.getNumberOfItemsS() ) {

			private static final long serialVersionUID = 1L;

			@Override
			protected void populateItem(Item<Document> item) {
				
				DocumentSmallPanel dp = new DocumentSmallPanel( "document", item.getModel(), item.getIndex()) {

					private static final long serialVersionUID = 1L;
					
					
					@Override
					protected void onBeforeRender() {
						for(int i=0;i<_tempDocumentList.size();i++) {
							if(_tempDocumentList.get(i).getId()==item.getModel().getObject().getId()) {
								this.setDocumentSearch(_tempDocumentList.get(i));
								break;
							}
						}
						
						if(_selectionList.contains(item.getModel().getObject().getId())) {
							setSelected(true);	
						}else {
							setSelected(false);
						}
						super.onBeforeRender();
					}
						
					@Override
					protected void OnDeleteClick(Document document, AjaxRequestTarget target) {
						_selectionList.clear();
						_selectionList.add(document.getId());
						if(_viewType==DocumentsListViewType.NORMAL) {
							_dpd.processDelete(target, _selectionList);
						}else if(_viewType==DocumentsListViewType.DELETED) {
							_dpd.processFullDelete(target, _selectionList);
						}else if(_viewType==DocumentsListViewType.ARCHIVED) {
							_dpd.processDelete(target, _selectionList);
						}
					}
					
					@Override
					protected void OnRestoreClick(Document document, AjaxRequestTarget target) {
						_selectionList.clear();
						_selectionList.add(document.getId());
						_dpd.processUndelete(target, _selectionList);
					}
					
					@Override
					protected void OnTagDrop(Tag tag, int documentId, AjaxRequestTarget target) {
						componentSingelRefresh(target, documentId);
					}
					
					@Override
					protected void OnUserDrop(User user, int documentId, AjaxRequestTarget target) {
						componentSingelRefresh(target, documentId);
					}
					
					@Override
					protected void OnModuleDrop(ApiModuleImpl apiModulImpl, int documentId, AjaxRequestTarget target) {
						//super.OnModuleDrop(apiModulImpl, documentId, target);
						DocumentsSmallListPanel.this._prp.start(target, new ModuleApplyTask(apiModulImpl.getId(), documentId));
					}
					
					
				};
				dp.setOutputMarkupId(true);
				dp.setShowTags(_thisParameters.isShowTags());
				dp.setShowUsers(_thisParameters.isShowUsers());
				item.setOutputMarkupId(true);
				item.add(dp);
			}
			
			
			@Override
			protected void onAfterRender() {
				_thisParameters.setCurrentPageNumberS(_dvDocuments.getCurrentPage());
				_thisParameters.setSortParam(_dpDocuments.getSort());
				super.onAfterRender();
			}
			
			@Override
			protected void onBeforeRender() {
				if(!Strings.isEmpty(_dsfc.getSearchString()) && _sbbp.hasResults()){
					buildTempDocumentList();
				}
				super.onBeforeRender();
			}
			
			
		};
		_dvDocuments.setOutputMarkupId(true);
		_wmcDocuments.add(_dvDocuments);
		add(_wmcDocuments);
		
		addToolBar();
		addUploadPanel();
		addSearchPanel();
		addTaggingPanel();
		addUsersPanel();
		addModulesPanel();
		
		addProcessingPanel();
		
		addBehavior();
		
		_dpn = new DmsPagingNavigator("navigation", _dvDocuments, true, true, true, true) {
 
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnNbOfItemsChanged(AjaxRequestTarget target, long itemPerPage) {
				_thisParameters.setNumberOfItemsS(itemPerPage);
				_dvDocuments.setItemsPerPage(itemPerPage);
				storeParameters();
				target.add(_wmcDocuments);
				target.add(_dpn);
			}
			@Override
			protected void OnDynamicViewChanged(AjaxRequestTarget target, boolean enabled) {
				_thisParameters.setDynamicArrangeS(enabled);
				if(!enabled) {
					_dvDocuments.setItemsPerPage(_thisParameters.getNumberOfItemsS());
					target.add(_wmcDocuments);
					target.add(_dpn);
				}else {
					target.appendJavaScript("dslpResize();");
				}
				storeParameters();
			}
			
			@Override
			protected void OnShowTagsChanged(AjaxRequestTarget target, boolean enabled) {
				_thisParameters.setShowTags(enabled);
				storeParameters();
				target.add(_wmcDocuments);
				target.add(_dpn);
			}
			@Override
			protected void OnShowUsersChanged(AjaxRequestTarget target, boolean enabled) {
				_thisParameters.setShowUsers(enabled);
				storeParameters();
				target.add(_wmcDocuments);
				target.add(_dpn);
			}
			
			
		};
		_dpn.setOutputMarkupId(true);
		_dpn.setNumberOfItems(String.valueOf(_thisParameters.getNumberOfItemsS()));
		_dpn.setDynamicEnabled(_thisParameters.isDynamicArrangeS());
		
		_dpn.setShowTags(_thisParameters.isShowTags());
		_dpn.setShowUsers(_thisParameters.isShowUsers());
		
		
		add(_dpn);
		
		if(_thisParameters.getCurrentPageNumberS()!=-1) {
			_dvDocuments.setCurrentPage(_thisParameters.getCurrentPageNumberS());
		}
		
		if(_sbbp.getCondition()!=null) {
			_dfcDocuments.setLinkedWhere(" id in ("+_sbbp.getCondition()+")");
			_dpDocuments.setFilterState(_dfcDocuments);
			//_dpDocuments.getFilterState().setLinkedWhere(" id in ("+_sbbp.getCondition()+")");
		}
		
		if(_dpDocuments.size()==0) {
			checkNodata(null);
		}
		
		applyUserRestrictions();
	}
	
	private void addModulesPanel() {
		_msbp = new ModulesSelectionPanel("modulepanel") {

			private static final long serialVersionUID = 1L;

			@Override
			protected void onAdd(AjaxRequestTarget target, HashSet<ApiModuleImpl> moduleslist) {
				List<Integer> selectedModules = new LinkedList<Integer>();
				List<Integer> selectedDocuments = new LinkedList<Integer>();
				for(ApiModuleImpl apiModule : moduleslist) selectedModules.add(apiModule.getId());
				for(Integer documentId:_selectionList) selectedDocuments.add(documentId);
				_prp.start(target, new ModuleApplyTask(selectedModules, selectedDocuments));
			};
		};
		_msbp.setOutputMarkupId(true);
		_wmcLeft.add(_msbp);
	}
	
	
	private void addSearchPanel() {
		
		_sbbp = new SearchboxPanel("searchbox", _dsfc) {
			private static final long serialVersionUID = 1L;
			@Override
			protected void OnButtonClear(AjaxRequestTarget target) {
				super.OnButtonClear(target);
				_session.clearSearchParameters();
				_tempDocumentList.clear();
				_dfcDocuments.setLinkedWhere(null);
				_dpDocuments.setFilterState(_dfcDocuments);
				//_dpDocuments.getFilterState().setLinkedWhere(null);
				_dpDocuments.size();
				_thisParameters.setFilterObject(_dsfc);
				storeParameters();
				target.add(_dpn);
				target.add(_wmcDocuments);
			};
			@Override
			protected void OnButtonSearch(AjaxRequestTarget target) {
				super.OnButtonSearch(target);
				if(hasResults()) {
					String condition = getCondition();
					_dfcDocuments.setLinkedWhere(" id in ("+condition+")");
					//_dpDocuments.getFilterState().setLinkedWhere(" id in ("+condition+")");
					_dpDocuments.setFilterState(_dfcDocuments);
					_dpDocuments.size();
					_thisParameters.setFilterObject(_dsfc);
					storeParameters();
					target.add(_dpn);
					target.add(_wmcDocuments);
				}
			}
			@Override
			protected void OnQueryChanged(AjaxRequestTarget target) {
				super.OnQueryChanged(target);
				if(hasResults()) {
					String condition = getCondition();
					_dfcDocuments = _dpDocuments.getFilterState();
					_dfcDocuments.setLinkedWhere(" id in ("+condition+")");
					_dpDocuments.setFilterState(_dfcDocuments);
					//_dpDocuments.getFilterState().setLinkedWhere(" id in ("+condition+")");
					_dpDocuments.size();
					_thisParameters.setFilterObject(_dsfc);
					storeParameters();
					target.add(_dpn);
					target.add(_wmcDocuments);
				}
				
			}
			
			@Override
			protected void OnSearchNoData(AjaxRequestTarget target) {
				super.OnSearchNoData(target);
				infobox.refresh(target, ResourceText.getText("lblNoSearchResults", this), INFOBOX_STATUS.warning);
				infobox.setVisible(true);
				target.add(wmcInfo);
			}
			
		};
		_sbbp.setOutputMarkupId(true);
		_wmcLeft.add(_sbbp);
	}

	private void addUploadPanel() {
		_upvl = new UploadPanelViewList("uploadpanel") {

			private static final long serialVersionUID = 1L;

			@Override
			protected void OnUploadPanelViewListFinished(AjaxRequestTarget target, int processId, int  uploads, int dublicates) {

				if(dublicates>0) {
					infobox.refresh(target, ResourceText.getText("warUploadDublicate", this, uploads, dublicates), INFOBOX_STATUS.warning);
				}else{
					infobox.refresh(target, ResourceText.getText("infUploadOK", this, uploads), INFOBOX_STATUS.success);
				};
				
				infobox.setVisible(true);

				_dpDocuments.size();
				target.add(wmcInfo);
				target.add(_dpn);
				target.add(_wmcDocuments);
				
			}
		};
		_upvl.setOutputMarkupId(true);
		_wmcLeft.add(_upvl);
		if(_upvl.getNumberOfImporters()==0 || !_session.getSessionParameters().getRestrictionType(UserRestrictionType.ALLOW_DEFAULT_UPLOAD)) {
			_upvl.setVisible(false);
		}
		
	}
	
	
	private void addUsersPanel() {
		_usbp = new UsersSelectionPanel("users") {
 
			private static final long serialVersionUID = 1L;
			
			@Override
			protected void onAdd(AjaxRequestTarget target, HashSet<User> userslist) {
				super.onAdd(target, userslist);
				try {
					DmsDBHelpersUser.addUsersToDocuments(
							filterArchivedDocuments(_selectionList),
							userslist,
							_session.getCurrentUser(),
							_session.getCurrentTenantId(),
							false
					);
					_selectionList = new HashSet<Integer>();
					_dpDocuments.size();
					target.add(_wmcDocuments);
					target.add(_wmcLeft);
				}catch(Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "onAdd", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
			}
			
			@Override
			protected void onReplace(AjaxRequestTarget target, HashSet<User> userlist) {
				super.onReplace(target, userlist);
				try {
					DmsDBHelpersUser.replaceUsersFromDocuments(
						filterArchivedDocuments(_selectionList),
						userlist,
						_session.getCurrentUser(),
						_session.getCurrentTenantId(),
						true
					);
					_selectionList = new HashSet<Integer>();
					_dpDocuments.size();
					target.add(_wmcDocuments);
					target.add(_wmcLeft);
				}catch(Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "onReplace", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
				
				
			}
			
			@Override
			protected void onDelete(AjaxRequestTarget target, HashSet<User> userlist) {
				super.onDelete(target, userlist);
				try {
					DmsDBHelpersUser.deleteUsersFromDocuments(
						filterArchivedDocuments(_selectionList),
						userlist,
						_session.getCurrentUser(),
						_session.getCurrentTenantId()
					);
					_selectionList = new HashSet<Integer>();
					_dpDocuments.size();
					target.add(_wmcDocuments);
					target.add(_wmcLeft);
				}catch(Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "onDelete", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
			}
			
		};
		_usbp.setOutputMarkupId(true);
		_wmcLeft.add(_usbp);
 
	}
	
	
	private void addTaggingPanel() {
		_tsbp = new TaggingPanel("tags") {
			
			private static final long serialVersionUID = 1L;

			@Override
			protected void onAdd(AjaxRequestTarget target, HashSet<Tag> taglist) {
				super.onAdd(target, taglist);
				try {
					DmsDBHelpersTag.addTagsToDocuments(filterArchivedDocuments(_selectionList), taglist, _session.getCurrentUser(), _session.getCurrentTenantId());
					_selectionList = new HashSet<Integer>();
					_dpDocuments.size();
					target.add(_wmcDocuments);
					target.add(_wmcLeft);
				}catch(Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "onAdd", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
			}
			
			@Override
			protected void onReplace(AjaxRequestTarget target, HashSet<Tag> taglist) {
				super.onReplace(target, taglist);
				try {
					DmsDBHelpersTag.replaceTagsFromDocuments(filterArchivedDocuments(_selectionList), taglist, _session.getCurrentUser(), _session.getCurrentTenantId());
					_selectionList = new HashSet<Integer>();
					_dpDocuments.size();
					target.add(_wmcDocuments);
					target.add(_wmcLeft);
				}catch(Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "onReplace", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
			}

			@Override
			protected void onDelete(AjaxRequestTarget target, HashSet<Tag> taglist) {
				super.onDelete(target, taglist);
				try {
					DmsDBHelpersTag.deleteTagsFromDocuments(filterArchivedDocuments(_selectionList), taglist, _session.getCurrentUser(), _session.getCurrentTenantId());
					_selectionList = new HashSet<Integer>();
					_dpDocuments.size();
					target.add(_wmcDocuments);
					target.add(_wmcLeft);
				}catch(Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "onDelete", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
			}
		};
		_tsbp.setOutputMarkupId(true);
		_wmcLeft.add(_tsbp);
	}
	
	private void addProcessingPanel() {
		_prp = new ProcessingPanel("processingpanel") {
			 
			private static final long serialVersionUID = 1L;

			@Override
			protected void onCancel(AjaxRequestTarget target, int processId) {
				//super.onCancel(target, processId);
			}
			
			@Override
			protected void onFinished(AjaxRequestTarget target, ProcessType processType, int documentSourceId, int documentTargetId, int processId) {
				infobox.refresh(target, ResourceText.getText("infoDocumentModule", this), INFOBOX_STATUS.success);
				infobox.setVisible(true);
				_selectionList = new HashSet<Integer>();
				_dpDocuments.size();
				target.add(wmcInfo);
				target.add(_dpn);
				target.add(_wmcDocuments);
			}
			
			@Override
			protected void onError(AjaxRequestTarget target, ProcessType processType, int processId, Exception e) {
				infobox.refresh(target, ResourceText.getText("errDocumentModule", this), INFOBOX_STATUS.error);
				infobox.setVisible(true);
				_selectionList = new HashSet<Integer>();
				_dpDocuments.size();
				target.add(wmcInfo);
				target.add(_dpn);
				target.add(_wmcDocuments);
			}
		};
		_prp.setOutputMarkupId(true);
		add(_prp);
	}
	
	

	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};
		_btRestore = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT);
		_btRestore.setUtf8Code("xe760");
		_btRestore.setEventName("Restore");
		_btRestore.setCssClass(toolbarCSS);
		_btRestore.setAlternate(ResourceText.getText("tbRestore", this));
		if(_viewType==DocumentsListViewType.DELETED)_toolBar.add(_btRestore);
		
		_btAll = new ToolbarButton().setButtonType(ToolbarButtonType.ALL).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btAll.setAlternate(ResourceText.getText("tbAll", this));
		_toolBar.add(_btAll);
		
		_btNone = new ToolbarButton().setButtonType(ToolbarButtonType.NONE).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btNone.setAlternate(ResourceText.getText("tbNone", this));
		_toolBar.add(_btNone);
		
		_btDelete  = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		if(_viewType==DocumentsListViewType.DELETED) {
			_btDelete.setAlternate(ResourceText.getText("tbFullDelete", this));
			_btDelete.setUtf8Code("xe92b");
		}else {
			_btDelete.setAlternate(ResourceText.getText("tbDelete", this));
			_btDelete.setUtf8Code("xe872");
		}
		_btDelete.setEventName("Delete");
		_btDelete.setToolbarState(ToolbarState.STATE_VIEW);
		_btDelete.setDefaultFormProcessing(false);
		_btDelete.setCssClass(toolbarCSS);
		_btDelete.setOutputMarkupId(true);
		_toolBar.add(_btDelete);
		
		_btRefresh = new ToolbarButton().setButtonType(ToolbarButtonType.REFRESH).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btRefresh.setAlternate(ResourceText.getText("btRefresh", this));
		_toolBar.add(_btRefresh);
		
		_btListView = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btListView.setUtf8Code("xe8ef");
		_btListView.setEventName("ChangeListView");
		_btListView.setAlternate(ResourceText.getText("btListView", this));
		if(_viewType==DocumentsListViewType.NORMAL)_toolBar.add(_btListView);

		_btCalendarView = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btCalendarView.setUtf8Code("xebcc");
		_btCalendarView.setEventName("ChangeCalendarView");
		_btCalendarView.setAlternate(ResourceText.getText("btCalendarView", this));
		if(_viewType==DocumentsListViewType.NORMAL)_toolBar.add(_btCalendarView);
		
		_btSEARCH = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btSEARCH.setUtf8Code("xe8b6");
		_btSEARCH.setEventName("SEARCH");
		_btSEARCH.setAlternate(ResourceText.getText("btSEARCH", this));
		_toolBar.add(_btSEARCH);
		
		_btTAGs = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btTAGs.setUtf8Code("xf05b");
		_btTAGs.setEventName("TAGS");
		_btTAGs.setAlternate(ResourceText.getText("btTAGs", this));
		if(_viewType==DocumentsListViewType.NORMAL)_toolBar.add(_btTAGs);
		
		
		_btUSERs = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btUSERs.setUtf8Code("xe7ef");
		_btUSERs.setEventName("USERS");
		_btUSERs.setAlternate(ResourceText.getText("btUSERs", this));
		if(_viewType==DocumentsListViewType.NORMAL)_toolBar.add(_btUSERs);
		
		_btUPLOAD = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btUPLOAD.setUtf8Code("xf09b");
		_btUPLOAD.setEventName("UPLOAD");
		_btUPLOAD.setAlternate(ResourceText.getText("btUPLOAD", this));
		if(_viewType==DocumentsListViewType.NORMAL && _uploadExists)_toolBar.add(_btUPLOAD);
		
		_btMODULES = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btMODULES.setUtf8Code("xe8f0");
		_btMODULES.setEventName("MODULES");
		_btMODULES.setAlternate(ResourceText.getText("btMODULES", this));
		if(_viewType==DocumentsListViewType.NORMAL)_toolBar.add(_btMODULES);
		
		
		_btSortName = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btSortName.setUtf8Code("xe053");
		_btSortName.setEventName("SortName");
		_btSortName.setAlternate(ResourceText.getText("btSortName", this));
		_toolBar.add(_btSortName);

		_btSortDate = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btSortDate.setUtf8Code("xe164");
		_btSortDate.setEventName("SortDate");
		_btSortDate.setAlternate(ResourceText.getText("btSortDate", this));
		_toolBar.add(_btSortDate);

		_btMerge = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btMerge.setUtf8Code("xe14d");
		_btMerge.setEventName("Merge");
		_btMerge.setAlternate(ResourceText.getText("btMerge", this));
		if(_viewType==DocumentsListViewType.NORMAL) _toolBar.add(_btMerge);
		
		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		add(_toolBar);
		
	}
	
	

	
	
	
	private void processEvent(String event, AjaxRequestTarget target) {
		
		
		if(_sbbp.isVisible())_sbbp.closeAll(target);
		
		if(event.equals("All")) {
			fselectAll(true);
			target.add(_wmcDocuments);
		}else if(event.equals("None")) {
			fselectAll(false);
			target.add(_wmcDocuments);
		}else if(event.equals("Refresh")) {
			if(_sbbp.isVisible()) {
				refreshFromSearchBox(target);
			}else {
				_dpDocuments.size();
				target.add(_dpn);
				target.add(_wmcDocuments);
			}
		}else if(event.equals("SEARCH")) {
			showHideModules(_sbbp, target);
		}else if(event.equals("TAGS")) {
			showHideModules(_tsbp, target);
		}else if(event.equals("UPLOAD")) {
			showHideModules(_upvl, target);
		}else if(event.equals("USERS")) {
			showHideModules(_usbp, target);
		}else if(event.equals("MODULES")) {
			showHideModules(_msbp, target);
		}else if(event.equals("SortName")) {
			if(_dpDocuments.getSort()==null || !_dpDocuments.getSort().isAscending()) {
				_dpDocuments.setSort("title", SortOrder.ASCENDING);
			}else if(_dpDocuments.getSort().isAscending()) {
				_dpDocuments.setSort("title", SortOrder.DESCENDING);
			}
			_dpDocuments.size();
			target.add(_wmcDocuments);
		}else if(event.equals("SortDate")) {
			if(_dpDocuments.getSort()==null || !_dpDocuments.getSort().isAscending()) {
				_dpDocuments.setSort("documentdatum", SortOrder.ASCENDING);
			}else if(_dpDocuments.getSort().isAscending()) {
				_dpDocuments.setSort("documentdatum", SortOrder.DESCENDING);
			}
			_dpDocuments.size();
			target.add(_wmcDocuments);
		}else if(event.equals("Delete")) {
			if(_selectionList.size()>0) {
				if(_viewType==DocumentsListViewType.NORMAL) {
					_dpd.processDelete(target, _selectionList);
				}else if(_viewType==DocumentsListViewType.DELETED) {
					_dpd.processFullDelete(target, _selectionList);
				}else if(_viewType==DocumentsListViewType.ARCHIVED) {
					_dpd.processDelete(target, _selectionList);
				}
			}
		}else if(event.equals("Merge")) {
			if(_selectionList.size()>1) {
				_dpd.processMerge(target, _selectionList);
			}
		}else if(event.equals("ChangeListView")) {
			DocumentsPage page = new DocumentsPage(_appPage,getPage().getPageReference(), Style.LIST);
			setResponsePage(page);
			storeParameters();
		}else if(event.equals("ChangeCalendarView")) {
			DocumentsPage page = new DocumentsPage(_appPage,getPage().getPageReference(), Style.CALENDAR);
			setResponsePage(page);
			storeParameters();
		}else if(event.equals("Restore")) {
			if(_selectionList.size()>0) {
				_dpd.processUndelete(target, _selectionList);
			}
		}
	}
	
	private void fselectAll(boolean select){
		long itemsPerPage = _dvDocuments.getItemsPerPage();
		if(_dvDocuments.size()<itemsPerPage)itemsPerPage=_dvDocuments.size();
		long currentPage = _dvDocuments.getCurrentPage();
		Iterator<?> iter =_dvDocuments.getDataProvider().iterator(itemsPerPage*currentPage, itemsPerPage);
		while(iter.hasNext()) {
			Document ql =(Document)iter.next();
			if(select){
				if(!_selectionList.contains(ql.getId())) _selectionList.add(ql.getId());
			}else{
				if(_selectionList.contains(ql.getId())) _selectionList.remove(ql.getId());
			}
		}
	}
	
	public HashSet<Integer> getSelectionIds(){
		return _selectionList;
	}
	
	private void addInfobox() {
		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		
		infobox = new InfoboxPanel("infobox");
		infobox.setVisible(false);
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
	}
	
	private void addNoData() {
		_wmcNoData = new WebMarkupContainer("nodata");
		_wmcNoData.setOutputMarkupId(true);
		add(_wmcNoData);
		
		String temp = ResourceText.getText("lblNoData", this);
		temp = temp.replace("T1", _documentKategorie);
		_lblNoData = new Label("lblnodata",temp);
		_lblNoData.setVisible(false);
		_lblNoData.setEscapeModelStrings(false);
		_lblNoData.setOutputMarkupId(true);
		_wmcNoData.add(_lblNoData);
		
	}
	
	private void initParameters() {
		
		_dpDocuments.getFilterState().setObjectStatusType(DocumentStatusType.DELETED, false);
		_dpDocuments.getFilterState().setObjectStatusType(DocumentStatusType.ARCHIVED, false);
		_dsfc.setObjectStatusType(DocumentStatusType.DELETED, false);
		_dsfc.setObjectStatusType(DocumentStatusType.ARCHIVED, false);
		
		 		
		switch (_viewType) {
			case ARCHIVED:
				_thisParameters = (DocumentsListParameters) _session.getSessionParameters().getParametersByType(SessionParameterTypes.DOCUMENTS_ARCHIVE);
				if(_thisParameters.getFilterObject()!=null && _thisParameters.isSearchVisible()) _dsfc = (DocumentOSFilterClass) _thisParameters.getFilterObject();
				if(_thisParameters.getSortParam()!=null) _dpDocuments.setSort(_thisParameters.getSortParam());

				_dpDocuments.getFilterState().setObjectStatusType(DocumentStatusType.ARCHIVED, true);
				_dsfc.setObjectStatusType(DocumentStatusType.ARCHIVED, true);
				_appPage = DmsWebApplicationPages.ApplicationPage.ARCHIVE;
				_documentKategorie=ResourceText.getText("lblArchive", this);
				break;
			case DELETED:
				_thisParameters = (DocumentsListParameters) _session.getSessionParameters().getParametersByType(SessionParameterTypes.DOCUMENTS_DELETED);
				if(_thisParameters.getFilterObject()!=null && _thisParameters.isSearchVisible()) _dsfc = (DocumentOSFilterClass) _thisParameters.getFilterObject();
				if(_thisParameters.getSortParam()!=null) _dpDocuments.setSort(_thisParameters.getSortParam());

				_dpDocuments.getFilterState().setObjectStatusType(DocumentStatusType.DELETED, true);
				_dpDocuments.getFilterState().setObjectStatusType(DocumentStatusType.ARCHIVED, null);
				_dsfc.setObjectStatusType(DocumentStatusType.DELETED, true);
				_dsfc.setObjectStatusType(DocumentStatusType.ARCHIVED, null);
				_appPage = DmsWebApplicationPages.ApplicationPage.DELETED;
				_documentKategorie=ResourceText.getText("lblDeleted", this);
				break;
			case NORMAL:
				_thisParameters = (DocumentsListParameters) _session.getSessionParameters().getParametersByType(SessionParameterTypes.DOCUMENTS);
				if(_thisParameters.getFilterObject()!=null && _thisParameters.isSearchVisible()) _dsfc = (DocumentOSFilterClass) _thisParameters.getFilterObject();
				if(_thisParameters.getSortParam()!=null) _dpDocuments.setSort(_thisParameters.getSortParam());
				_documentKategorie=ResourceText.getText("lblDocuments", this);
				break;
			case INBOX:
				_thisParameters = (DocumentsListParameters) _session.getSessionParameters().getParametersByType(SessionParameterTypes.DOCUMENTS_INBOX);
				break;
		}
		_thisParameters.setCurrentDisplayStyle(Style.TAB);
		
		if(DmsDBHelpersTag.getDefaultPDFImporterID()==-1 || !_session.getSessionParameters().getRestrictionType(UserRestrictionType.ALLOW_DEFAULT_UPLOAD)) {
			_uploadExists=false;
			_thisParameters.setUploadVisible(false);
		};
		
		
		
	}
	
	private void storeParameters() {
		if(_viewType==DocumentsListViewType.NORMAL) {
			_session.getSessionParameters().setParametersByType(SessionParameterTypes.DOCUMENTS, _thisParameters);
		}else if(_viewType==DocumentsListViewType.DELETED) {
			_session.getSessionParameters().setParametersByType(SessionParameterTypes.DOCUMENTS_DELETED, _thisParameters);
		}else if(_viewType==DocumentsListViewType.ARCHIVED) {
			_session.getSessionParameters().setParametersByType(SessionParameterTypes.DOCUMENTS_ARCHIVE, _thisParameters);
		}
	}
	
	
	private void addBehavior() {
		DocumentsListAjaxBehavior dsab = new DocumentsListAjaxBehavior(_wmcDocuments.getMarkupId()) {
 
			private static final long serialVersionUID = 1L;
			@Override
			protected void OnDocumentClick(int documentId, AjaxRequestTarget target) {
				setSelectUnselect(Integer.valueOf(documentId));
				componentSingelRefresh(target, documentId);
			}
			
			@Override
			protected void OnDocumentDblClick(int documentId, AjaxRequestTarget target) {
				DocumentsPage page = null;
				SearchHelper sh= _sbbp.getSearchHelper(""+documentId);
				if(sh!= null && sh.getSearchString()!=null && !sh.getSearchString().isEmpty()) {
					page = new DocumentsPage(_appPage, getPage().getPageReference(), documentId, DocumentStateType.VIEW, sh);
				}else {
					page = new DocumentsPage(_appPage, getPage().getPageReference(), documentId);
				}
				setResponsePage(page);
			}
			
			@Override
			protected void OnResizeView(AjaxRequestTarget target, int height, int width) {
				if(_thisParameters.isDynamicArrangeS()) {
					resizeDvDocuments(target, height, width);
				}
			}
			
		};
		add(dsab);
	}
	
	
	private void setSelectUnselect(int documentId) {
		if(_selectionList.contains(documentId)) {
			_selectionList.remove(documentId);
		}else {
			_selectionList.add(documentId);
		}
	}
	
	private HashSet<Integer> filterArchivedDocuments(HashSet<Integer> selectionList){
		HashSet<Integer> selectionListFilter = new HashSet<Integer>();
		for(Integer id : selectionList) {
			try {
				Document document = StaticLocators.getDocuments().getEntityById(id);
				if(!document.hasObjectStatusTypes(DocumentStatusType.ARCHIVED)) {
					selectionListFilter.add(id);
				}
			}catch(Exception e) {
				DmsException dmsException = new DmsException(1000, "E", _className, "filterArchivedDocuments", null, e);
				StaticLocators.getMessageHandler().createMessage(dmsException, _session);
			}
		}
		return selectionListFilter;
	}
	
	private void componentSingelRefresh(AjaxRequestTarget target, int documentId) {
		int indexToRefresh = -1;
		
		Item<?> documentItem = null;
		Iterator<Component> components = _dvDocuments.iterator();
		while(components.hasNext()) {
			Component comp = components.next();
			documentItem = (Item<?>) comp;
			if(documentItem.getDefaultModelObject() instanceof Document) {
				Document doc = (Document) documentItem.getDefaultModelObject();
				if(doc.getId()==documentId) {
					indexToRefresh= documentItem.getIndex();
					break;
				}
			}
		}
		if(indexToRefresh!=-1)target.add(_dvDocuments.get(indexToRefresh));
	}
	
	
	private void addProcessingDialogs() {
		_dpd = new DocumentsProcessingDialogs("processingdialogs") {

			private static final long serialVersionUID = 1L;
			
			@Override
			protected void OnProcessError(AjaxRequestTarget target, ProcessType processType, String message) {
				infobox.refresh(target, message, INFOBOX_STATUS.error);
				infobox.setVisible(true);
				fselectAll(false);
				_selectionList = new HashSet<Integer>();
				_dpDocuments.size();
				target.add(wmcInfo);
				target.add(_dpn);
				target.add(_wmcDocuments);
				checkNodata(target);
			}
			
			@Override
			protected void OnProcessFinished(AjaxRequestTarget target, ProcessType processType, String message, int sourceDocumentId, int targetDocumentId) {
				infobox.refresh(target, message, INFOBOX_STATUS.success);
				infobox.setVisible(true);
				fselectAll(false);
				_selectionList = new HashSet<Integer>();
				_dpDocuments.size();
				target.add(wmcInfo);
				target.add(_dpn);
				target.add(_wmcDocuments);
				checkNodata(target);
			}
			
		};
		_dpd.setOutputMarkupId(true);
		add(_dpd);
		
	}
	
	private void buildTempDocumentList() {
		try {
			_tempDocumentList.clear();
			Iterator<?> iter =_dvDocuments.getDataProvider().iterator(_thisParameters.getNumberOfItemsS()*_dvDocuments.getCurrentPage(), _thisParameters.getNumberOfItemsS());
			List<Integer> ids = new LinkedList<Integer>();
			while(iter.hasNext()) {
				ids.add(((Document)iter.next()).getId());
			}
			DocumentOSFilterClass dosfc = new DocumentOSFilterClass();
			dosfc.getIDList().addAll(ids);
			dosfc.setSize(ids.size());
			dosfc.setSearchString(_dsfc.getSearchString());
			DPSearchDocumentsHighlighted dpsd = new DPSearchDocumentsHighlighted();
			_tempDocumentList.addAll(dpsd.getHightlightedDocuments(dosfc));
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", _className, "buildTempDocumentList", null, e);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
	}
	
	
	
	private void resizeDvDocuments(AjaxRequestTarget target, int height, int width) {
		
		height = height+2;
		width = width+2;
		
		if(height<240 || width<232) return;
		int itemsY = (int) Math.floor(height/(272d));
		int itemsX = (int) Math.floor(width/(232d));
		int nbOfItems = itemsX*itemsY;
		
		if(nbOfItems!=_dvDocuments.getItemsPerPage() && nbOfItems>=1) {
			_dvDocuments.setItemsPerPage(nbOfItems);
			target.add(_wmcDocuments);
			target.add(_dpn);
		}
	}
	
	private void checkNodata(AjaxRequestTarget target) {
		long numberOfDocuments = _dpDocuments.size();
		if(numberOfDocuments==0) {
			_toolBar.setVisible(false);
			_wmcDocuments.setVisible(false);
			_wmcLeft.setVisible(false);
			_dpn.setVisible(false);
			_lblNoData.setVisible(true);
		}else {
			_toolBar.setVisible(true);
			_wmcDocuments.setVisible(true);
			_dpn.setVisible(true);
			_lblNoData.setVisible(false);			
		}
		
		if(null!=target) {
			target.add(_toolBar);
			target.add(_wmcDocuments);
			target.add(_wmcLeft);
			target.add(_wmcNoData);
		}
	}
	
	private void refreshFromSearchBox(AjaxRequestTarget target) {
		_sbbp.refresh(target);
		if(_sbbp.hasResults()) {
			_dfcDocuments.setLinkedWhere(" id in ("+_sbbp.getCondition()+")");
			_dpDocuments.setFilterState(_dfcDocuments);
			_dpDocuments.size();
			_thisParameters.setFilterObject(_dsfc);
			storeParameters();
			target.add(_dpn);
			target.add(_wmcDocuments);
		}
	}
	
	
	private void showHideModules(Panel panel, AjaxRequestTarget target) {
		
		boolean setVisible = !panel.isVisible();
		boolean hideLeft=true;
		if(panel instanceof TaggingPanel) {
			_thisParameters.setTagsVisible(setVisible);
		}else if(panel instanceof UsersSelectionPanel) {
			_thisParameters.setUsersVisible(setVisible);
		}else if(panel instanceof SearchboxPanel) {
			_thisParameters.setSearchVisible(setVisible);
		}else if(panel instanceof UploadPanelViewList) {
			_thisParameters.setUploadVisible(setVisible);
		}else if(panel instanceof ModulesSelectionPanel) {
			_thisParameters.setModulesVisible(setVisible);
		}
		panel.setVisible(setVisible);
		if(_tsbp.isVisible())hideLeft=false;
		if(_usbp.isVisible())hideLeft=false;
		if(_sbbp.isVisible())hideLeft=false;
		if(_upvl.isVisible())hideLeft=false;
		if(_msbp.isVisible())hideLeft=false;
		
		if(hideLeft) {
			target.appendJavaScript("document.getElementById(\""+_wmcDocuments.getMarkupId()+"\").classList.remove('innercontent_tags_active');dslpResize();");
		}else {
			target.appendJavaScript("document.getElementById(\""+_wmcDocuments.getMarkupId()+"\").classList.add('innercontent_tags_active');dslpResize();");
		}
		storeParameters();
		target.add(_wmcLeft);
		
	}
	
	private void applyUserRestrictions() {
		
		 
		if(_session.getSessionParameters().getRestrictionType(UserRestrictionType.ALLOW_USER_SELECTION)) {
			_btUSERs.setVisible(true);
		}else {
			_btUSERs.setVisible(false);
			_usbp.setVisible(false);
		}

		if(_session.getSessionParameters().getRestrictionType(UserRestrictionType.ALLOW_TAG_SELECTION)) {
			_btTAGs.setVisible(true);
		}else {
			_btTAGs.setVisible(false);
			_tsbp.setVisible(false);
		}
		
		if(_session.getSessionParameters().getRestrictionType(UserRestrictionType.ALLOW_DEFAULT_UPLOAD)) {
			_btUPLOAD.setVisible(true);
		}else {
			_btUPLOAD.setVisible(false);
			_upvl.setVisible(false);
		}
		
		if(_session.getSessionParameters().getRestrictionType(UserRestrictionType.ALLOW_MODULE_SELECTION)) {
			_btMODULES.setVisible(true);
		}else {
			_btMODULES.setVisible(false);
			_msbp.setVisible(false);
		}
	 
		
		if(_session.getSessionParameters().getRestrictionType(UserRestrictionType.ALLOW_MERGE_DOCUMENT))_btMerge.setVisible(true);
		else _btMerge.setVisible(false);

		if(_session.getSessionParameters().getRestrictionType(UserRestrictionType.ALLOW_DELETE_DOCUMENT))_btDelete.setVisible(true);
		else _btDelete.setVisible(false);

	}
	
 
	
	
}
