/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.documents.page.view;
 
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.PropertyModel;
import org.biffo.dms.models.DocumentModel;
 
 

 

public class ContentPanel extends Panel{
 
	private static final long serialVersionUID = 1L;
 

	private Form<?> _form;
	 
 
	private TextField<String> _nbofpages, _pagesize, _size;
	private TextArea<String> _content;
	
	private final DocumentModel _documentModel;
	private final ContentPanelHelper _cph = new ContentPanelHelper();
	
	public ContentPanel(String id, final DocumentModel documentModel) {
		super(id);
		_documentModel = documentModel;
		
		_form = new Form<Void>("form");
		_form.setOutputMarkupId(true);
		add(_form);
		addDefaultTextFields();
		refreshFormObjects();
		setFormComponentsEditable(false);
	}
 
	private void addDefaultTextFields() {
	
		_nbofpages = new TextField<String>("nbofpages");
		_nbofpages.setOutputMarkupId(true);
		_form.add(_nbofpages);
	
		_pagesize = new TextField<String>("pagesize");
		_pagesize.setOutputMarkupId(true);
		_form.add(_pagesize); 
    
		_size = new TextField<String>("size");
		_size.setOutputMarkupId(true);
		_form.add(_size); 

		
		_content= new TextArea<String>("content");
		_content.setOutputMarkupId(true);
		_form.add(_content);
	}

	public void refreshFormObjects(){

		_nbofpages.setDefaultModel(new PropertyModel<String>(_documentModel, "NumberOfPages"));
		if(_documentModel.getObject().getSize()!=-1) {
			_cph.setSize( _documentModel.getObject().getSize() + " KB");
		}
		_size.setDefaultModel(new PropertyModel<String>(_cph, "Size"));
		
		if(_documentModel.getObject().getWidth()!=-1 && _documentModel.getObject().getHeight()!=-1) {
			_cph.setPageSize(_documentModel.getObject().getHeight() + " mm x "+_documentModel.getObject().getWidth() +" mm");
		}
		_pagesize.setDefaultModel(new PropertyModel<String>(_cph, "PageSize"));
		_content.setDefaultModel(new PropertyModel<String>(_documentModel, "FirstLines"));
			 
	}

	private void setFormComponentsEditable(boolean editable){
		try{
			_nbofpages.setEnabled(editable);
			_pagesize.setEnabled(editable);
			_size.setEnabled(editable);
			_content.setEnabled(editable);
		}catch(Exception e){
		}
	}


}
