/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.documents.page.view;

import java.util.ArrayList;
import java.util.List;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.DefaultDataTable;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.biffo.dms.dataprovider.DPFragments;
import org.biffo.dms.entity.Fragments.Fragment;
import org.biffo.dms.entity.Fragments.FragmentFilterClass;
import org.biffo.dms.models.DocumentModel;
import org.biffo.dms.web.helpers.ResourceText;

public class FragmentsPanel extends Panel{
	 
		
	private static final long serialVersionUID = 1L;
	
	private DPFragments _qtdp = new DPFragments();
	private DefaultDataTable<?, ?> _ddt;
	private Form<?> _form;
	
	private final DocumentModel _documentModel;
	
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public FragmentsPanel(String id, final DocumentModel documentModel) {
		super(id);
		_documentModel = documentModel;
		int documentId = _documentModel.getId();
		try {
			FragmentFilterClass ffc = new FragmentFilterClass();
			ffc.setIdDocument(documentId);
			_qtdp.setFilterState(ffc);
		}catch(Exception e) {
		}
		List<IColumn<?,?>> columns = new ArrayList<IColumn<?,?>>();
		
		columns.add(new PropertyColumn<Fragment, String>(ResourceText.getModelText("colFragmentName", this), "Fragmentname","Fragmentname"));
		columns.add(new AbstractColumn<Fragment, String>(ResourceText.getModelText("colFragmentValue", this)){

			private static final long serialVersionUID = 1L;

			@Override
			public void populateItem(Item<ICellPopulator<Fragment>> item, String componentId, IModel<Fragment> rowModel) {
				
				FragmentPanel fp = new FragmentPanel(componentId, rowModel.getObject()) {

					private static final long serialVersionUID = 1L;

					@Override
					protected void OnFragmentViewClicked(AjaxRequestTarget target, Fragment fragment) {
						super.OnFragmentViewClicked(target, fragment);
						FragmentsPanel.this.OnFragmentViewClicked(target, rowModel.getObject());
					}
				};
				
				fp.setOutputMarkupId(true);
				item.add(fp);

			}
		});
		columns.add(new PropertyColumn<Fragment, String>(ResourceText.getModelText("colFragmentProbability", this), "Probability","Probability"));
		columns.add(new AbstractColumn<Fragment, String>(ResourceText.getModelText("colFragmentPage", this), "Page"){
			 
			private static final long serialVersionUID = 1L;

			@Override
		    public void populateItem(Item<ICellPopulator<Fragment>> item, String componentId, IModel<Fragment> rowModel){
				String label = "";
				if(rowModel.getObject().getPage()!=-1)label = ""+rowModel.getObject().getPage();
				Label lbl = new Label(componentId, label);
				item.add(lbl);
			}
		});
		columns.add(new AbstractColumn<Fragment, String>(new Model<String>("")){
			 
			private static final long serialVersionUID = 1L;

			@Override
		    public void populateItem(Item<ICellPopulator<Fragment>> item, String componentId, IModel<Fragment> rowModel){
				buildStatusCell(item, componentId, rowModel.getObject().isValid());
			}
		});
		
		int numberOfItems = (int) _qtdp.size();
		if(numberOfItems<=0)numberOfItems=1;
		
		_ddt = new DefaultDataTable("datatable", columns, _qtdp, numberOfItems){
 
			private static final long serialVersionUID = 1L;

			@Override
			protected void onPageChanged() {
				super.onPageChanged();
			}
		};
		
		
		_form = new Form<Void>("dataform");
		_form.add(_ddt);
		_form.setOutputMarkupId(true);
		add(_form);
	}
	
	private void buildStatusCell(Item<ICellPopulator<Fragment>> item, String componentId, boolean valid) {
		
		Label lbl = null;
		if(valid) {
			lbl = new Label(componentId, "<span class=\"statusType info\">&#xe2e6;</span>");
		}else {
			lbl = new Label(componentId, "<span class=\"statusType error\">&#xe000;</span>");
		}

		lbl.setEscapeModelStrings(false);
		lbl.setOutputMarkupId(true);
		item.add(lbl);
	}
	
	
	protected void OnFragmentViewClicked(AjaxRequestTarget target, Fragment fragment){}
	
	
}
