/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.documents.page.view;

 
import java.util.Date;

import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.PropertyModel;
import org.biffo.dms.models.DocumentModel;

public class MetadataPanel extends Panel{
 
	private static final long serialVersionUID = 1L;

	private final ContentPanelHelper _cph = new ContentPanelHelper();
	
	private TextField<String> _filename, _author, _creator, _producer, _id, _version;
	private TextField<String> _creationDate, _modDate,  _inputtype;
	
	private TextField<String> _geaendertVon, _aenderungsDatum, _erstelltVon, _erstellungsDatum;
	
	
	private DocumentModel _documentModel;
	
	
	public MetadataPanel(String id, final DocumentModel documentModel) {
		super(id);
		_documentModel = documentModel;
		addDefaultTextFields();
		refreshFormObjects();
		setFormComponentsEditable(false);
	}
 
	private void addDefaultTextFields() {
	
		

		_filename = new TextField<String>("filename");
		_filename.setOutputMarkupId(true);
		add(_filename);

		_id = new TextField<String>("id");
		_id.setOutputMarkupId(true);
		add(_id); 

		_version = new TextField<String>("version");
		_version.setOutputMarkupId(true);
		add(_version); 

		
		_author = new TextField<String>("author");
		_author.setOutputMarkupId(true);
		add(_author); 
		
		_creator = new TextField<String>("creator");
		_creator.setOutputMarkupId(true);
		add(_creator);
		
		_producer = new TextField<String>("producer");
		_producer.setOutputMarkupId(true);
		add(_producer);
		
		_creationDate = new TextField<String>("creationdate");
		_creationDate.setOutputMarkupId(true);
		add(_creationDate);
		
		_modDate = new TextField<String>("moddate");
		_modDate.setOutputMarkupId(true);
		add(_modDate);
		
		_inputtype = new TextField<String>("inputtype");
		_inputtype.setOutputMarkupId(true);
		add(_inputtype);
		
		_geaendertVon = new TextField<String>("geaendertvon");
		_geaendertVon.setOutputMarkupId(true);
		add(_geaendertVon);
		
		_aenderungsDatum = new TextField<String>("aenderungsdatum");
		_aenderungsDatum.setOutputMarkupId(true);
		add(_aenderungsDatum);
		
		_erstelltVon = new TextField<String>("erstelltvon");
		_erstelltVon.setOutputMarkupId(true);
		add(_erstelltVon);
		
		_erstellungsDatum = new TextField<String>("erstellungsdatum");
		_erstellungsDatum.setOutputMarkupId(true);
		add(_erstellungsDatum);
 
	}

	public void refreshFormObjects(){

		_cph.setCreationDate(new PropertyModel<Date>(_documentModel, "CreationDate").getObject());
		_cph.setErstelltAm(new PropertyModel<Date>(_documentModel, "ErstelltAm").getObject());
		_cph.setGeaendertAm(new PropertyModel<Date>(_documentModel, "GeaendertAm").getObject());
		_cph.setModificationDate(new PropertyModel<Date>(_documentModel, "ModificationDate").getObject());
		
		
		_filename.setDefaultModel(new PropertyModel<String>(_documentModel, "FileName"));
		_id.setDefaultModel(new PropertyModel<String>(_documentModel, "Id"));
		_version.setDefaultModel(new PropertyModel<String>(_documentModel, "Version"));
		_author.setDefaultModel(new PropertyModel<String>(_documentModel, "Author"));
		_creator.setDefaultModel(new PropertyModel<String>(_documentModel, "Creator"));
		_producer.setDefaultModel(new PropertyModel<String>(_documentModel, "Producer"));
		_creationDate.setDefaultModel(new PropertyModel<String>(_cph, "CreationDate"));
		_modDate.setDefaultModel(new PropertyModel<String>(_cph, "ModificationDate"));
		_inputtype.setDefaultModel(new PropertyModel<String>(_documentModel, "Inputtype"));
		_geaendertVon.setDefaultModel(new PropertyModel<String>(_documentModel, "GeaendertVon"));
		_aenderungsDatum.setDefaultModel(new PropertyModel<String>(_cph, "GeaendertAm"));
		_erstelltVon.setDefaultModel(new PropertyModel<String>(_documentModel, "ErstelltVon"));
		_erstellungsDatum.setDefaultModel(new PropertyModel<String>(_cph, "ErstelltAm"));
			 
	}

	private void setFormComponentsEditable(boolean editable){
		try{
			_filename.setEnabled(editable);
			_id.setEnabled(editable);
			_version.setEnabled(editable);
			_author.setEnabled(editable);
			_creator.setEnabled(editable);
			_producer.setEnabled(editable);
			_creationDate.setEnabled(editable);
			_modDate.setEnabled(editable);
			_inputtype.setEnabled(editable);
			_geaendertVon.setEnabled(editable);
			_aenderungsDatum.setEnabled(editable);
			_erstelltVon.setEnabled(editable);
			_erstellungsDatum.setEnabled(editable);
			
		}catch(Exception e){
		}
	}
	
}
