/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.fragments;

import java.text.SimpleDateFormat;


import org.apache.wicket.AttributeModifier;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.markup.repeater.data.DataView;
import org.apache.wicket.model.IModel;
import org.apache.wicket.request.cycle.RequestCycle;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.dataprovider.DPDocuments;
import org.biffo.dms.entity.Document.Document;
import org.biffo.dms.entity.Document.DocumentFilterClass;
import org.biffo.dms.processQueue.tasks.ProcessHelpers;
import org.biffo.dms.resource.DmsCrypt;
import org.biffo.dms.web.helpers.ResourceText;

public class FragmentDocuments  extends Panel{

	private static final long serialVersionUID = 1L;
		
	private DPDocuments _dpDocuments = new DPDocuments();
	private DataView<Document> _dvDocuments;
	private final String contextPath = RequestCycle.get().getRequest().getContextPath();
	private DmsWebSession _session =  DmsWebSession.getSession();
	private SimpleDateFormat _sdf = new SimpleDateFormat(_session.getLocaleParams().getDateFormat());


	
	public FragmentDocuments(String id, int fragmentId, String documentIDs) {
		super(id);

		DocumentFilterClass dfc = new DocumentFilterClass();
		dfc.setLinkedWhere(" id in ("+documentIDs+")");
		_dpDocuments.setFilterState(dfc);
		_dvDocuments = new DataView<Document>("documents", _dpDocuments, 10 ) {

			private static final long serialVersionUID = 1L;

			@Override
			protected void populateItem(Item<Document> item) {
				FragmentDocumentPanel fdp = new FragmentDocumentPanel("document", item.getModel(), item.getIndex()) {
					private static final long serialVersionUID = 1L;
					
				};
				fdp.setOutputMarkupId(true);
				item.setOutputMarkupId(true);
				item.add(fdp);
			}
			
			
			@Override
			protected void onAfterRender() {
				super.onAfterRender();
			}
			
			@Override
			protected void onBeforeRender() {
				super.onBeforeRender();
			}
			
			
		};
		_dvDocuments.setOutputMarkupId(true);
		add(_dvDocuments);
		
	}
	
	public class FragmentDocumentPanel extends Panel {

		private static final long serialVersionUID = 1L;

		private WebMarkupContainer wmcDvImage;
		
		private final Document document;
		
		public FragmentDocumentPanel(final String id, final IModel<Document> model, final int index) {
            super(id);
    		this.document = model.getObject();
    		setOutputMarkupId(true);
    		
    		wmcDvImage = new WebMarkupContainer("dvimage");
    		wmcDvImage.setOutputMarkupId(true);
    		
    		
    		String encodedResource = DmsCrypt.getEncodedResource(DmsWebSession.getSession(),  document);
    		wmcDvImage.add(AttributeModifier.replace("style", "background-image: url('"+contextPath+"/png/" + encodedResource+"')"));
    		wmcDvImage.add(AttributeModifier.append("ondblclick", "dslpDblClick(event,'"+document.getId()+"');"));
    		try {
    			String docName = ProcessHelpers.getDocumentTitle(document);
    			String altTitle= ResourceText.getText("imgTitle", this, docName, document.getErstelltVon(), _sdf.format(document.getErstelltAm()));
        		wmcDvImage.add(AttributeModifier.append("title", altTitle));
    		}catch(Exception e) {
    		}
    		
    		add(wmcDvImage);
        }
	}
	
	
}
