/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.helpers.processing;


import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.AjaxSelfUpdatingTimerBehavior;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.extensions.ajax.markup.html.modal.ModalWindow;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.util.time.Duration;
import org.biffo.dms.processQueue.IProcessQueueTask;

abstract public class AbstractModalProcessPanel extends Panel{

	private static final long serialVersionUID = 1L;
	private AjaxSelfUpdatingTimerBehavior updatingBehavior;
	
	private Label processingstatus, processingmessage;
	private IProcessQueueTask processTask;
	private String status="status";
	private String message="message";

	public AbstractModalProcessPanel(ModalWindow modalWindow) {
		
		super(modalWindow.getContentId());

		processingstatus = new Label("processingstatus", new PropertyModel<String>(this, "status"));
		processingstatus.setOutputMarkupId(true);
		processingstatus.setEscapeModelStrings(false);
		add(processingstatus);

		
		processingmessage = new Label("processingmessage", new PropertyModel<String>(this, "message"));
		processingmessage.setOutputMarkupId(true);
		processingmessage.setEscapeModelStrings(false);
		add(processingmessage);
		
		add(new AjaxLink<Void>("cancel"){

			private static final long serialVersionUID = 1L;

			@Override
			public void onClick(AjaxRequestTarget target){
				setStatus(null);
				setMessage(null);
				AbstractModalProcessPanel.this.onModalCancel(target);
			}
		});
		setOutputMarkupId(true);
	}
	
	public void setProcessTask(IProcessQueueTask processTask) {
		
		this.processTask = processTask;
		this.status = processTask.getStatus();
		this.message = processTask.getMessage();
	}
	
	
	public void start(AjaxRequestTarget target){
		
		updatingBehavior= new AjaxSelfUpdatingTimerBehavior(Duration.seconds(1)){

			private static final long serialVersionUID = 1L;

			@Override
			protected void onPostProcessTarget(AjaxRequestTarget target){
				setStatus(processTask.getProcessBridge().getStatus());
				setMessage(processTask.getProcessBridge().getMessage());
				if (processTask.getProcessBridge().isFinished()){
					stop(target);
					AbstractModalProcessPanel.this.onModalFinished(target);
					setStatus(null);
					setMessage(null);
				}else if (processTask.getProcessBridge().isCancel()){
					stop(target);
					AbstractModalProcessPanel.this.onModalFinished(target);
					setStatus(null);
					setMessage(null);
				}
				target.add(processingmessage);
			}
			
		};
		processingstatus.add(updatingBehavior);
	}

	public abstract void onModalCancel(AjaxRequestTarget target);
	public abstract void onModalFinished(AjaxRequestTarget target);

	public String getStatus() {return status;}
	public void setStatus(String status) {this.status = status;}
	
	public String getMessage() {return message;}
	public void setMessage(String message) {this.message = message;}


}
