/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.helpers.processing;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.panel.Panel;
import org.biffo.dms.DmsWebApplication;
import org.biffo.dms.components.custommodal.CustomModalWindow;
import org.biffo.dms.entity.Processing.ProcessType;
import org.biffo.dms.processQueue.IProcessQueueTask;
 

public class ProcessingPanel extends Panel{

	
	private static final long serialVersionUID = 1L;
	private CustomModalWindow _modalProcess = new CustomModalWindow("modalprocess");
    private AbstractModalProcessPanel _ampp;
    private IProcessQueueTask task;
 
	public ProcessingPanel(String id) {
		super(id);
		addModalProcessPanel();
	}
	
	private void addModalProcessPanel() {
		
		_modalProcess.setOutputMarkupId(true);
		_modalProcess.setAutoSize(false);
		_modalProcess.setHeightUnit("px");
		_modalProcess.setInitialHeight(105);
		_modalProcess.setInitialWidth(550);
		_modalProcess.setResizable(false);
		
		_ampp = new AbstractModalProcessPanel(_modalProcess) {

			private static final long serialVersionUID = 1L;

			@Override
			public void onModalCancel(AjaxRequestTarget target) {
				_modalProcess.close(target);
				ProcessingPanel.this.onCancel(target, task.getIdProcess());
				DmsWebApplication.getProcessQueue().remove(task.getTaskName());
			}

			@Override
			public void onModalFinished(AjaxRequestTarget target) {
				_modalProcess.close(target);
				ProcessingPanel.this.onFinished(target, task.getProcessType(), task.getSourceDocumentId(), task.getTargetDocumentId(), task.getIdProcess());
				DmsWebApplication.getProcessQueue().remove(task.getTaskName());
			}
			
		};
		_ampp.setOutputMarkupId(true);
		_modalProcess.setContent(_ampp);
		add(_modalProcess);
	}
	
	
	public void start(AjaxRequestTarget target, IProcessQueueTask task) {
		this.task = task;
		_ampp.setProcessTask(this.task);
		DmsWebApplication.getProcessQueue().add(this.task);
		
		_ampp.start(target);
		_modalProcess.show(target);
	}
	
	
	protected void onCancel(AjaxRequestTarget target, int processId) {};
	protected void onFinished(AjaxRequestTarget target, ProcessType processType, int documentSourceId, int documentTargetId, int processId) {};
	protected void onError(AjaxRequestTarget target, ProcessType processType, int processId, Exception e) {};

}
