/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.htmlhelpers;


import org.apache.wicket.AttributeModifier;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.markup.html.form.CheckBox;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.biffo.dms.entity.Variables.Variable;
import org.biffo.dms.entity.Variables.Impl.ImportServerImpl;
import org.biffo.dms.entity.user.Users.User;


public abstract class CheckBoxColumn<T> extends AbstractColumn<T,T> {

	private static final long serialVersionUID = 1L;

	public CheckBoxColumn(IModel<String> displayModel) {
		super(displayModel);
	}
	
	
	public void populateItem(Item<ICellPopulator<T>> cellItem, String componentId, IModel<T> rowModel) {
		
		boolean active=true;
		if(rowModel.getObject() instanceof ImportServerImpl) {
			ImportServerImpl serverImpl = (ImportServerImpl) rowModel.getObject();
			if(serverImpl!=null) {
				Variable var = serverImpl.getVariableByKey("processname");
				if(null!=var && var.isChangeable()!=null) {
					active = var.isChangeable();
				}
			}
		}else if(rowModel.getObject() instanceof User) {
			User user = (User) rowModel.getObject();
			if(null!=user && user.isChangeable()!=null) {
				active = user.isChangeable();
			}
		}   
			
		cellItem.add(new CheckPanel(componentId, newCheckBoxModel(rowModel), active));
		cellItem.add(new AttributeModifier("class", new Model<String>("checkboxcolumn")));
	}
	
	protected CheckBox newCheckBox(String id, IModel<Boolean> checkModel) {
		return new CheckBox("check", checkModel);
	}
	
	protected abstract IModel<Boolean> newCheckBoxModel(IModel<T> rowModel);
	
	private class CheckPanel extends Panel {

		private static final long serialVersionUID = 1L;

		public CheckPanel(String id, IModel<Boolean> checkModel, boolean active) {
			super(id);
			CheckBox cb = newCheckBox(id, checkModel);
			cb.setOutputMarkupId(true);
			cb.add(new AjaxFormComponentUpdatingBehavior("onclick") {

				private static final long serialVersionUID = -1L;

				@Override
	            protected void onUpdate(AjaxRequestTarget target) {
	                target.add(cb);
	            }
	        });
			cb.setEnabled(active);
			add(cb);
		}
	}

	

}
