/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.htmlhelpers;


 
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.link.PopupSettings;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.biffo.dms.entity.Document.Document;
import org.biffo.dms.entity.Tags.Tag;
import org.biffo.dms.entity.Variables.Variable;
import org.biffo.dms.entity.Variables.Impl.ApiModuleImpl;
import org.biffo.dms.entity.Variables.Impl.FTPServerImpl;
import org.biffo.dms.entity.Variables.Impl.FolderServerImpl;
import org.biffo.dms.entity.Variables.Impl.ImportServerImpl;
import org.biffo.dms.entity.Variables.Impl.SFTPServerImpl;
import org.biffo.dms.entity.Variables.Impl.SmtpServerImpl;
import org.biffo.dms.entity.user.Usergroups.UserGroup;
import org.biffo.dms.entity.user.Users.User;
 
 

@SuppressWarnings({ "unchecked", "rawtypes","serial" })
public abstract class LinkColumn extends PropertyColumn {

	
	private static final long serialVersionUID = 1L;
		PopupSettings popupSettings;
	    IModel labelModel;
	 
	    public LinkColumn(IModel displayModel, String sortProperty, String propertyExpression, PopupSettings popupSettings) {
	        this(displayModel, sortProperty, propertyExpression);
	        this.popupSettings = popupSettings;
	    }
	 
	    public LinkColumn(IModel displayModel, IModel labelModel) {
	        super(displayModel, null);
	        this.labelModel = labelModel;
	    }
	 
	     
		public LinkColumn(IModel displayModel, String sortProperty, String propertyExpression) {
	        super(displayModel, sortProperty, propertyExpression);
	    }
	 
	    public LinkColumn(IModel displayModel, String propertyExpressions) {
	        super(displayModel, propertyExpressions);
	    }
	 
	    @Override
	    public void populateItem(Item item, String componentId, IModel model) {
	        item.add(new LinkPanel(item, componentId, model));
	    }
 
	    public abstract void onClick(Item item, String componentId, IModel model);
	 
	 
	    public class LinkPanel extends Panel {
	 
	        public LinkPanel(final Item item, final String componentId, final IModel model) {
	            super(componentId);
	 
	            Link link = new Link("link") {
	                @Override
	                public void onClick() {
	                    LinkColumn.this.onClick(item, componentId, model);
	                }
	            };
	            link.setPopupSettings(popupSettings);
	            add(link);
	            
	            Object modelObject = model.getObject();
	            
	            if(modelObject instanceof Document){
	            	Document qj = (Document)modelObject;
	            	link.add(new Label("label", qj.getFileName()));
	            }else if(modelObject instanceof Tag){
	            	Tag qj = (Tag)modelObject;
	            	link.add(new Label("label", qj.getTagName()));
	            }else if(modelObject instanceof User){
	            	User qj = (User)modelObject;
	            	link.add(new Label("label", qj.getBenutzerName()));
	            }else if(modelObject instanceof UserGroup){
	            	UserGroup qj = (UserGroup)modelObject;
	            	link.add(new Label("label", qj.getName()));
	            }else if(modelObject instanceof Variable){
	            	Variable qj = (Variable)modelObject;
	            	link.add(new Label("label", qj.getVariableName()));
	            }else if(modelObject instanceof FTPServerImpl){
	            	FTPServerImpl qj = (FTPServerImpl)modelObject;
	            	link.add(new Label("label", qj.getServerName()));
	            }else if(modelObject instanceof SmtpServerImpl){
		           	SmtpServerImpl qj = (SmtpServerImpl)modelObject;
		           	link.add(new Label("label", qj.getServerName()));
	            }else if(modelObject instanceof SFTPServerImpl){
			      	SFTPServerImpl qj = (SFTPServerImpl)modelObject;
			      	link.add(new Label("label", qj.getServerName()));
	            	
	            }else if(modelObject instanceof ImportServerImpl){
	            	ImportServerImpl qj = (ImportServerImpl)modelObject;
	            	link.add(new Label("label", qj.getProcessName()));
	            }else if(modelObject instanceof FolderServerImpl){
	            	FolderServerImpl qj = (FolderServerImpl)modelObject;
	            	link.add(new Label("label", qj.getServerName()));
	            	
	            }else if(modelObject instanceof ApiModuleImpl){
	            	ApiModuleImpl qj = (ApiModuleImpl)modelObject;
	            	link.add(new Label("label", qj.getServerName()));
	            	
	            }else if(modelObject instanceof Document){
	            	Document qj = (Document)modelObject;
	            	link.add(new Label("label", qj.getId()));
	            	
	            }
	        }
	    }

}
