/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.locks;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.DefaultDataTable;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.model.AbstractCheckBoxModel;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.biffo.dms.DmsWebApplication;
import org.biffo.dms.components.custommodal.CustomModalWindow;
import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.infobox.InfoboxPanel.INFOBOX_STATUS;
import org.biffo.dms.components.pagging.DmsPagingNavigator;
import org.biffo.dms.lock.DPLocks;
import org.biffo.dms.lock.Lock;
import org.biffo.dms.web.htmlhelpers.CheckBoxColumn;
import org.biffo.dms.web.modals.AbstractModalDefaultPanel;
import org.biffo.dms.web.modals.AbstractModalDefaultPanel.ModalType;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;
 


public class LocksPanelViewList extends Panel{

	private static final long serialVersionUID = 1L;
	private static final String toolbarCSS = "viewListToolbar";
	//private StandardListParameters _thisParameters;
	
	private static final SimpleDateFormat _sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
	//private final static SimpleDateFormat _sdf2 = new SimpleDateFormat("dd.MM.yyyy HH:mm");
	
	private DPLocks _qtdp = new DPLocks();
	private DefaultDataTable<?, ?> _ddt;
	private Set<Lock> _selectionList = new HashSet<Lock>();
	private Toolbar _toolBar;
	private Form<?> _form;
	
	//private DmsWebSession _session = DmsWebSession.getSession();

	private WebMarkupContainer wmcInfo;
	private InfoboxPanel infobox;
	private DmsPagingNavigator _dpn;
	
	private CustomModalWindow modalDefault = new CustomModalWindow("modalDefault");
	
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public LocksPanelViewList(String id) {
		super(id);
		
		setOutputMarkupId(true);
		addInfobox();
		addModalDefault();
		
		_form = new Form("dataform");
		_form.setOutputMarkupId(true);
		add(_form);
		
		
		List<IColumn<?,?>> columns = new ArrayList<IColumn<?,?>>();
		columns.add(new CheckBoxColumn<Lock>(Model.of("")) {

			private static final long serialVersionUID = 1L;

			@Override
			protected IModel<Boolean> newCheckBoxModel(final IModel<Lock> rowModel) {
				return new AbstractCheckBoxModel() {

					private static final long serialVersionUID = 1L;
					@Override
					public void unselect() {
						_selectionList.remove(rowModel.getObject());
					}
					@Override
					public void select() {
						_selectionList.add(rowModel.getObject());
					}
					@Override
					public boolean isSelected() {
						return _selectionList.contains(rowModel.getObject());
					}
					@Override
					public void detach() {
						rowModel.detach();
					}
				};
			}
		});
		
		
		columns.add(new PropertyColumn<Lock, String>(new Model<String>("UserId"), "UserId","UserId"));
		columns.add(new PropertyColumn<Lock, String>(new Model<String>("EntityId"), "EntityId","EntityId"));
		columns.add(new PropertyColumn<Lock, String>(new Model<String>("EntityType"), "EntityType","EntityType"));
		//columns.add(new PropertyColumn<Lock, String>(new Model<String>("AquiredTime"), "AquiredTime","AquiredTime"));
		
		columns.add(new AbstractColumn<Lock, String>(new Model<String>("Gesperrt am"), "AquiredTime"){
			private static final long serialVersionUID = 1L;
			@Override
			public void populateItem(Item<ICellPopulator<Lock>> item, String componentId, IModel<Lock> rowModel){
				String label = "NA";
				try{
					label = _sdf.format(rowModel.getObject().getAquiredTime());
				}catch(Exception e){}
				Label lbl = new Label(componentId, label);
				item.add(lbl);
			}
		});
		
		
		_ddt = new DefaultDataTable("datatable", columns, _qtdp, 10){

			private static final long serialVersionUID = 1L;

			@Override
			protected void onPageChanged() {
				super.onPageChanged();
 
			}
		};
		_ddt.setOutputMarkupId(true);
		addToolBar(); 
		_form.add(_ddt);
		
		_dpn = new DmsPagingNavigator("navigation", _ddt, false, false, false, false);
		_dpn.setOutputMarkupId(true);
		add(_dpn);
		
	}
	
	
	 
	
	
	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};
		_toolBar.add(new ToolbarButton().setButtonType(ToolbarButtonType.ALL).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS));
		_toolBar.add(new ToolbarButton().setButtonType(ToolbarButtonType.NONE).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS));
		_toolBar.add(new ToolbarButton().setButtonType(ToolbarButtonType.DELETE).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS));
		_toolBar.add(new ToolbarButton().setButtonType(ToolbarButtonType.REFRESH).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS));

		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		add(_toolBar);
		
	}

	
	public Set<Lock> getSelection(){
		return _selectionList;
	}
	
	private void fselectAll(boolean select){
		long itemsPerPage = _ddt.getItemsPerPage();
		if(_qtdp.size()<itemsPerPage)itemsPerPage=_qtdp.size();
		long currentPage = _ddt.getCurrentPage();
		Iterator<?> iter =_ddt.getDataProvider().iterator(itemsPerPage*currentPage, itemsPerPage);
		while(iter.hasNext()) {
			Lock ql =(Lock)iter.next();
 
			if(select){
				_selectionList.add(ql);
			}else{
				 _selectionList.remove(ql);
			} 
		}
	}
	
	private void processEvent(String event, AjaxRequestTarget target) {
		if(event.equals("All")) {
			fselectAll(true);
			target.add(_ddt);
		}else if(event.equals("None")) {
			fselectAll(false);
			target.add(_ddt);
		}else if(event.equals("Delete")){
			if(_selectionList.size()>0)modalDefault.show(target);
		}else if(event.equals("Refresh")) {
			_qtdp.size();
			target.add(_dpn);
			target.add(_ddt);
		}
	}
	
	private void deleteEntities(AjaxRequestTarget target){
		try{
			
			for(Lock entityToDelete : _selectionList) {
				DmsWebApplication.getLocks().remove(entityToDelete);
			}
			
			infobox.refresh(target, "Sperren gelöscht.", INFOBOX_STATUS.success);
			infobox.setVisible(true);
			target.add(wmcInfo);
			
		}catch(Exception e){
			infobox.refresh(target, "Sperren wurden nicht gelöscht.\n"+e.getMessage(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);
		}
	}
	
	private void addInfobox() {
		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		
		infobox = new InfoboxPanel("infobox");
		infobox.setVisible(false);
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
	} 
	
	
	private void addModalDefault() {

		modalDefault.setAutoSize(false);
		modalDefault.setHeightUnit("px");
		modalDefault.setInitialHeight(80);
		modalDefault.setInitialWidth(400);
		modalDefault.setResizable(false);
		modalDefault.showUnloadConfirmation(false);
    	
    	AbstractModalDefaultPanel _amdp = new AbstractModalDefaultPanel(modalDefault) {
 
			private static final long serialVersionUID = 1L;

			@Override
			public void onOk(AjaxRequestTarget target) {
				deleteEntities(target);
				modalDefault.close(target);
				_ddt.size();
				target.add(_dpn);
				target.add(_ddt);
			}
			
			@Override
			public void onCancel(AjaxRequestTarget target) {
				modalDefault.close(target);
				fselectAll(false);
				target.add(_dpn);
				target.add(_ddt);
			}
		};
		_amdp.setModalType(ModalType.DELETE);
		_amdp.setModalText("Markierte Sperren löschen?");
		modalDefault.setContent(_amdp);
    	add(modalDefault);
    	
    }
	
	/*
	
	private void addModalDelete() {

		modalDelete.setAutoSize(false);
    	modalDelete.setHeightUnit("px");
    	modalDelete.setInitialHeight(70);
    	modalDelete.setInitialWidth(400);
    	modalDelete.setResizable(false);
    	modalDelete.setContent(new AbstractModalDeletePanel(modalDelete, "Markierte Sperren löschen?"){

    		private static final long serialVersionUID = 1L;

			@Override
			public void onOk(AjaxRequestTarget target) {
				deleteEntities(target);
				modalDelete.close(target);
				deleteEntities(target);
				_ddt.size();
				target.add(_dpn);
				target.add(_ddt);
			}

			@Override
			public void onCancel(AjaxRequestTarget target) {
				modalDelete.close(target);
				fselectAll(false);
				target.add(_dpn);
				target.add(_ddt);
			}

    	});
    	add(modalDelete);
    	
    }*/
	
	
	/*private void initParameters() {
		
		Lock lock = new Lock();
		lock.setAquiredTime(new Date());
		lock.setEntityId(1);
		lock.setEntityType(EntityType.DOCUMENT);
		lock.setUserId(88);
		
		DmsWebApplication.getLocks().add(lock);
		
	}*/

}
