/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.menu;




import java.lang.reflect.Constructor;

import org.apache.wicket.Page;
import org.apache.wicket.behavior.AttributeAppender;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.DmsWebApplicationPages.ApplicationPage;
import org.biffo.dms.entity.Tasks.TaskException;
import org.biffo.dms.entity.Tasks.TaskFilterClass;
import org.biffo.dms.entity.Tasks.TaskStatusType;
import org.biffo.dms.web.home.HomePage;

public class MenuEntry extends Panel{
	
	private static final String _className = "MenuEntry";
	private static final long serialVersionUID = 1L;
	private ApplicationPage _applicationPage;
	
	private Link<?> _navigationLink;
	private Label _navigationdescription, _navigationbadge, _navigationimage;
	
	private DmsWebSession _session = DmsWebSession.getSession();
	 
	public MenuEntry (String id, final IModel<?> model){
        super(id, model);
        
        if(model.getObject() instanceof ApplicationPage){
        	
        	_applicationPage = (ApplicationPage) model.getObject();
        	_navigationLink = new Link<Void>("navigationlink") {
 
				private static final long serialVersionUID = 1L;

				@Override
                public void onClick() {
    				if(_applicationPage==ApplicationPage.LOGOUT) {
    					DmsWebSession.getSession().invalidate();
    					setResponsePage(HomePage.class);
    				}else {
    					navigationLinkClick(_applicationPage);
    				}
                }
            };
            
            _navigationdescription = new Label("navigationdescription", _applicationPage.getDescription());
            _navigationLink.add(_navigationdescription);

            _navigationbadge = new Label("navigationbadge");
            _navigationLink.add(_navigationbadge);
            if(_applicationPage==ApplicationPage.INBOX) {
            	
            	int numberOfTasks = 0;
            	try {
            		numberOfTasks=getNumberOfInboxTasks(_session.getCurrentUserId());
            	}catch(Exception e) {
            	}
            	if(numberOfTasks>0) _navigationbadge.setDefaultModel(Model.of(""+numberOfTasks));
            	else _navigationbadge.setVisible(false);
            }else {
            	_navigationbadge.setVisible(false);
            } 
            
            _navigationimage = new Label("navigationimage", _applicationPage.getImage());
            _navigationimage.setEscapeModelStrings(false);
            _navigationimage.add(new AttributeAppender("class", "toolbar-icons"));
            _navigationLink.add(_navigationimage);
            add(_navigationLink);;
        }
	}
	
	public void setActive(){
		_navigationLink.add(new AttributeAppender("class", "mainmenu_current"));
	}
	
	private void navigationLinkClick(ApplicationPage applicationPage){
		
		if(null!=applicationPage.getWebPageClass()){
			Object responsePage;
			try {
				
				if(checkConstructor(applicationPage.getWebPageClass())) {
					responsePage = applicationPage.getWebPageClass().getDeclaredConstructor(ApplicationPage.class).newInstance(applicationPage);
				}else {
					responsePage = applicationPage.getWebPageClass().getConstructor().newInstance();
				}
				this.setResponsePage((Page)responsePage);
			} catch (Exception e) {
				e.printStackTrace();
				DmsException dmsException = new DmsException(1000, "E", 
					_className,
					"navigationLinkClick",
					null,
					e
				);
				StaticLocators.getMessageHandler().createMessage(dmsException, _session);
			}
		}
		
	}
	
	private boolean checkConstructor(Class<?> moduleClass) {
		for(Constructor<?> constructor : moduleClass.getDeclaredConstructors()){
			if(constructor.getParameterCount()==1 && constructor.getParameterTypes()[0] == ApplicationPage.class){
				return true;
			}
		}
		return false;
	}
	
	
	
	private static int getNumberOfInboxTasks(int userId) {
		TaskFilterClass tfc = new TaskFilterClass();
		tfc.setIdUser(userId);
		tfc.setObjectStatusType(TaskStatusType.INBOX, true);
		tfc.setObjectStatusType(TaskStatusType.VIEWED, false);
		
		try {
			return (int) StaticLocators.getTasks().getNumberOfEntities(tfc);
		} catch (TaskException e) {
		}
		return 0;
	}
	
}
