/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.menu;


import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.RepeatingView;
import org.apache.wicket.model.Model;
import org.biffo.dms.DmsWebApplication;
import org.biffo.dms.DmsWebApplicationPages;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.DmsWebApplicationPages.ApplicationPage;
import org.biffo.dms.entity.user.Users.UsersAccessType;

public class MenuPanel extends Panel {

	private ApplicationPage _acctPage = null;
	private static final long serialVersionUID = 1L;

	public MenuPanel(String id) {
		this(id, null);
	}

	public MenuPanel(String id, ApplicationPage acctPage) {
        super(id);
        if(DmsWebSession.get()!=null){
        	
        	DmsWebSession session = DmsWebSession.getSession();
        	UsersAccessType userAcessType = session.getUserAccessType();
        	_acctPage = acctPage;
        	RepeatingView listItems = new RepeatingView("listItems");
        	for(final ApplicationPage applicationPage : DmsWebApplicationPages.ApplicationPage.values()){
        		
        		MenuEntry navigationEntry = null;
        		boolean addPage=false;
    			switch (userAcessType){
    				case ADMIN:
    					if(applicationPage.getAccessType()==UsersAccessType.ADMIN || applicationPage.getAccessType()==UsersAccessType.USER)addPage=true;
    					break;
    				case USER:
    					if(applicationPage.getAccessType()==UsersAccessType.USER)addPage=true;
    					break;
    				default:
    					addPage=false;
    					break;
    			}
    			if(DmsWebApplication.getApplication().isNoConfig() && applicationPage!=ApplicationPage.CONFIG) {
    				addPage=false;
    			}else if(applicationPage==ApplicationPage.INBOX)addPage=false;
        		
        		if(addPage) {
        			navigationEntry = new MenuEntry(listItems.newChildId(), Model.of(applicationPage));
        			if(_acctPage.equals(applicationPage)) navigationEntry.setActive();
        			if(navigationEntry!=null)listItems.add(navigationEntry);
        		}
        		
        	}
        	
        	add(listItems);    
        }
	}   
	
}