/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.metrics;

 
 
import java.util.ArrayList;
import java.util.List;

import org.apache.wicket.extensions.markup.html.repeater.data.table.DefaultDataTable;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.Model;
import org.biffo.dms.dataprovider.metrics.DPMetrics;
import org.biffo.dms.dataprovider.metrics.Metric;
import org.biffo.dms.dataprovider.metrics.MetricType;
import org.biffo.dms.web.helpers.ResourceText;


public class MetricsPanel extends Panel{

 
	private static final long serialVersionUID = 1L;

	
	private DPMetrics  _qtdp;
	private DefaultDataTable<?, ?> _ddt;
	
	@Override
	protected void onBeforeRender() {
		if(_ddt!=null) {
			_qtdp.refresh();
		}
		super.onBeforeRender();
	}
	
	
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public MetricsPanel(String id, MetricType metricType) {
		super(id);
		
		Label lbl=null;
		switch (metricType) {
			case document:
				lbl = new Label("lblmetric", ResourceText.getText("lblDocuments", this));
				break;
			case opensearch:
				lbl = new Label("lblmetric", ResourceText.getText("lblOpenSearch", this));
				break;
			case tags:
				lbl = new Label("lblmetric", ResourceText.getText("lblTags", this));
				break;
		}
		add(lbl);


		_qtdp = new DPMetrics(metricType, MetricsPanel.this);
		
		List<IColumn<?,?>> columns = new ArrayList<IColumn<?,?>>();
		columns.add(new PropertyColumn<Metric, String>(new Model<String>("Key"), "Key","Key"));
		columns.add(new PropertyColumn<Metric, String>(new Model<String>("Value"), "Value","Value"));

		int size=(int) _qtdp.size();
		if(size<=0)size=1;
		
		_ddt = new DefaultDataTable("documents", columns, _qtdp, size);
		_ddt.setOutputMarkupId(true);
		add(_ddt);
	}
	
	 

}
