/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.modals;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.extensions.ajax.markup.html.modal.ModalWindow;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.PropertyModel;
import org.biffo.dms.web.helpers.ResourceText;


abstract public class AbstractModalDefaultPanel extends Panel{

	private static final long serialVersionUID = 1L;
	
	public enum ModalType{
		CUSTOM(0,null, null ),
		LOCK(1,"Markierte Benutzer sperren?", "&#xe897;" ),
		UNLOCK(2,"Markierte Benutzer entsperren?", "&#xe898;"),
		DELETE(3, "Markierte Einträge löschen?", "&#xe872;"),
		UNDELETE(4, "Markierte Einträge wiederherstellen?", "&#xe760;"),
		;
		
		private int _code;
	    private String _description;
	    private final String _imageCode;

	    ModalType(int code, String description, String imageCode){
	    	 _code = code;
	    	 _description = description;
	    	 _imageCode = imageCode;
	    	 
	    }
	    public int getCode(){return _code;}
	    public String getDescription(){return _description;}
	    public String getImageCode(){return _imageCode;}
	}
	
	@SuppressWarnings("unused")
	private String imageCode, lblText;
	private ModalType _modalType;
	private String _rcValue;

	public void setModalType(ModalType modalType) {
		_modalType = modalType;
		imageCode = modalType.getImageCode();
		lblText = modalType.getDescription();
	}
	
	public void setModalImageCode(String imageCode) {this.imageCode=imageCode;}
	public void setModalText(String modalText) {lblText = modalText;}
	public ModalType getModalType() {return _modalType;}
	public void setModalRCValue(String rcValue) {_rcValue=rcValue;}
	public String getModalRCValue() {return _rcValue;}
	
	
	public AbstractModalDefaultPanel(ModalWindow modalWindow) {
		super(modalWindow.getContentId());
		
		setModalType(ModalType.LOCK);
		
		add(new Label("lblimage", new PropertyModel<String>(this, "imageCode")).setEscapeModelStrings(false));
		add(new Label("lbltext", new PropertyModel<String>(this, "lblText")));
		
		AjaxLink<Void> alOK = new AjaxLink<Void>("ok"){

			private static final long serialVersionUID = 1L;

			@Override
			public void onClick(AjaxRequestTarget target){
				AbstractModalDefaultPanel.this.onOk(target);
			}
		};
		alOK.add(new Label("oktext",ResourceText.getText("btnOkTxt", this)));
		alOK.add(new AttributeModifier("title", ResourceText.getText("btnOkTitle", this)));
		add(alOK);
		
		
		
		AjaxLink<Void> alCancel = new AjaxLink<Void>("cancel"){

			private static final long serialVersionUID = 1L;

			@Override
			public void onClick(AjaxRequestTarget target){
				AbstractModalDefaultPanel.this.onCancel(target);
			}
		};
		
		alCancel.add(new Label("canceltext",ResourceText.getText("btnCancelTxt", this)));
		alCancel.add(new AttributeModifier("title", ResourceText.getText("btnCancelTitle", this)));
		add(alCancel);
		
	}

	public abstract void onOk(AjaxRequestTarget target);
	public abstract void onCancel(AjaxRequestTarget target);


}
