/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.modals;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.extensions.ajax.markup.html.modal.ModalWindow;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.biffo.dms.web.helpers.ResourceText;


abstract public class AbstractModalSavePanel extends Panel{

	private static final long serialVersionUID = 1L;
	
	public AbstractModalSavePanel(ModalWindow modalWindow) {
		super(modalWindow.getContentId());
		
		add(new Label("dialogText",ResourceText.getText("dialogText", this)));

		AjaxLink<Void> alVersioning = new AjaxLink<Void>("versioning"){

			private static final long serialVersionUID = 1L;

			@Override
			public void onClick(AjaxRequestTarget target){
				AbstractModalSavePanel.this.onVersioning(target);
			}
		};
		alVersioning.add(new Label("versioningtext",ResourceText.getText("btnVersioningTxt", this)));
		alVersioning.add(new AttributeModifier("title", ResourceText.getText("btnVersioningTitle", this)));
		add(alVersioning);
		
		
		AjaxLink<Void> alOverwrite = new AjaxLink<Void>("overwrite"){

			private static final long serialVersionUID = 1L;

			@Override
			public void onClick(AjaxRequestTarget target){
				AbstractModalSavePanel.this.onOverwrite(target);
			}
		};
		alOverwrite.add(new Label("overwritetext",ResourceText.getText("btnOverwriteTxt", this)));
		alOverwrite.add(new AttributeModifier("title", ResourceText.getText("btnOverwriteTitle", this)));
		add(alOverwrite);
		
		
		AjaxLink<Void> alCancel = new AjaxLink<Void>("cancel"){
			private static final long serialVersionUID = 1L;
			@Override
			public void onClick(AjaxRequestTarget target){
				AbstractModalSavePanel.this.onCancel(target);
			}
		};
		alCancel.add(new Label("canceltext",ResourceText.getText("btnCancelTxt", this)));
		alCancel.add(new AttributeModifier("title", ResourceText.getText("btnCancelTitle", this)));
		add(alCancel);
		
		
	}

	public abstract void onVersioning(AjaxRequestTarget target);
	public abstract void onOverwrite(AjaxRequestTarget target);
	public abstract void onCancel(AjaxRequestTarget target);


}
