/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.modules;

 
 

import java.util.Date;
import java.util.HashMap;
import java.util.Set;

import org.apache.wicket.Page;
import org.apache.wicket.PageReference;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.CheckBox;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.PropertyModel;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebApplication;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.infobox.InfoboxPanel.INFOBOX_STATUS;
import org.biffo.dms.entity.Helpers.EntityUUID;
import org.biffo.dms.entity.Variables.Variable;
import org.biffo.dms.entity.Variables.Impl.ApiModuleImpl;
import org.biffo.dms.web.helpers.PanelTitel;
import org.biffo.dms.web.helpers.ResourceText;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;

public class ModulePanel extends Panel{

	private static final String _className = "ModulePanel";
	private static final long serialVersionUID = 1L;
	private static final String toolbarCSS = "dataformToolbar";
	
	private TextField<String> _classname, _version, _jar,_type, _author, _url;
	private TextArea<String> _beschreibung;
	private Label _lblTitel;
	private CheckBox _firstPageOnly;

	private Form<?> _form;
	private Toolbar _toolBar;
	private ToolbarButton _btEdit, _btCancel, _btSave, _btOK, _btCheck;

	private Object _origin;
	
	private PanelTitel _panelTitel = new PanelTitel();
	
	private DmsWebSession _session = DmsWebSession.getSession();

	 
	private WebMarkupContainer wmcInfo;
	private InfoboxPanel infobox;
	
	private ApiModuleImpl _apiModuleImpl, _apiModuleImpl_Backup;
	
	public ModulePanel(String id, int idFTPServer,  Object origin) {
		this(id, null, origin);
	}
	
	public ModulePanel(String id, ApiModuleImpl apiModuleImpl,  Object origin) {
		super(id);
		_origin = origin;
		_apiModuleImpl = apiModuleImpl;
		_apiModuleImpl_Backup = apiModuleImpl;
		
		_form = new Form<Void>("form");
		_form.setOutputMarkupId(true);
		add(_form);
		
        addTextFields();
        addCheckBoxen();
        addAdditionalControls();
        
        addToolBar();
        addInfobox();
        
        setFormComponentsEditable(false);
		setOutputMarkupId(true);
		_panelTitel.setTitel(ResourceText.getText("titModule", this));
		refreshFormObjects();
	}
	
	private void addTextFields(){
		
		
		
		_classname = new TextField<String>("classname");
		_classname.setOutputMarkupId(true);
        _form.add(_classname); 
		
		_version = new TextField<String>("version");
		_version.setOutputMarkupId(true);
        _form.add(_version); 
		
		_jar = new TextField<String>("jar");
		_jar.setOutputMarkupId(true);
        _form.add(_jar); 
		
		_type = new TextField<String>("type");
		_type.setOutputMarkupId(true);
        _form.add(_type); 
		
		_author = new TextField<String>("author");
		_author.setOutputMarkupId(true);
        _form.add(_author); 
		
		_url = new TextField<String>("url");
		_url.setOutputMarkupId(true);
        _form.add(_url); 
		 
        
		_beschreibung= new TextArea<String>("beschreibung");
		_beschreibung.setOutputMarkupId(true);
        _form.add(_beschreibung);
	}
	
	private void addCheckBoxen(){
        _firstPageOnly = new CheckBox("firstpageonly");
        _firstPageOnly.setOutputMarkupId(true);
        _firstPageOnly.add(new AjaxFormComponentUpdatingBehavior("onclick") {
			private static final long serialVersionUID = 1L;

			@Override
            protected void onUpdate(AjaxRequestTarget target) {
                target.add(_firstPageOnly);
            }
        }); 
        _form.add(_firstPageOnly);
		 
	}
	
	private void addAdditionalControls(){
		_lblTitel = new Label("paneltitel", new PropertyModel<String>(this._panelTitel, "Titel"));
		_lblTitel.setOutputMarkupId(true);
		add(_lblTitel);
	}
	
	private void closeForm(AjaxRequestTarget target){
		if(_origin instanceof Panel){
			target.add(ModulePanel.this.replaceWith((Panel)_origin).getParent());
		}else if(_origin instanceof Page){
		}else if(_origin instanceof PageReference){
			PageReference pg = (PageReference)_origin;
			setResponsePage(pg.getPage());
		}
	}
	
	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};
		
		_btEdit = new ToolbarButton().setButtonType(ToolbarButtonType.EDIT).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btEdit.setAlternate(ResourceText.getText("tbAltEdit", this));
		_toolBar.add(_btEdit);
		
		_btCancel = new ToolbarButton().setButtonType(ToolbarButtonType.CANCEL).setToolbarState(ToolbarState.STATE_VIEW | ToolbarState.STATE_NEW | ToolbarState.STATE_CHANGE).setCssClass(toolbarCSS);
		_btCancel.setAlternate(ResourceText.getText("tbAltCancel", this));
		_toolBar.add(_btCancel);
		
		_btOK = new ToolbarButton(_form).setButtonType(ToolbarButtonType.OK).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btOK.setAlternate(ResourceText.getText("tbAltOk", this));
		_toolBar.add(_btOK);
		
		_btSave = new ToolbarButton(_form).setButtonType(ToolbarButtonType.SAVE).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btSave.setAlternate(ResourceText.getText("tbAltSave", this));
		_toolBar.add(_btSave);
 
		
		_btCheck = new ToolbarButton(_form).setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW | ToolbarState.WITH_EVENT);
		_btCheck.setUtf8Code("xf232");
		_btCheck.setAlternate(ResourceText.getText("tbAltCheck", this));
		_btCheck.setEventName("check");
		_btCheck.setCssClass(toolbarCSS);
		_toolBar.add(_btCheck);

		
		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		_form.add(_toolBar);
	}
	
	
	
	private void processEvent(String event, AjaxRequestTarget target) {
		if(event.equals("Edit")) edit(target);
		else if(event.equals("Cancel")) cancel(target);
		else if(event.equals("check")) checkModule(target);
		else if(event.equals("Save")) save(target, true);
		else if(event.equals("Ok")) ok(target);

		target.add(_form);
		target.add(_toolBar);
	}
	
	private boolean ok(AjaxRequestTarget target){
		if(save(target, false)) {
			_toolBar.setViewState(ToolbarState.STATE_VIEW);
			cancel(target);
			return true;
		}
		return false;
	} 
	
	private void edit(AjaxRequestTarget target){
		 try {
			 if(_apiModuleImpl.getId()!=-1){
				 _toolBar.setViewState(ToolbarState.STATE_CHANGE);
				 setFormComponentsEditable(true);
			 }
	 	}catch(Exception e) {
	 		infobox.refresh(target, ResourceText.getText("errChangeEditMode", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);	 		
	 	}
	}
	
	
	private boolean save(AjaxRequestTarget target, boolean withMessage) {
		try {
			
			HashMap<String, Variable> hmVariables = _apiModuleImpl.getVariables();
			Set<String> keys = hmVariables.keySet();
			if(_apiModuleImpl.getId()!=-1) {
				Date aenderungsDatum = new Date();
				for(String key: keys) {
					Variable var = hmVariables.get(key);
					var.setGeaendertAm(aenderungsDatum);
					var.setGeaendertVon(_session.getCurrentUser());
					var.setUUID(EntityUUID.createVariableUUID(var));
					StaticLocators.getVariables().updateEntity(var, true);
				}
				DmsException dmsInfo = new DmsException(2000, "I", 
						_className, 
						"save",
						ResourceText.getText("dmsInfoModuleChanged", this, _apiModuleImpl.getClassname()),
						null,
						_apiModuleImpl
						);
				StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
			}
			_apiModuleImpl_Backup = _apiModuleImpl;
			if(withMessage) {
				infobox.refresh(target, ResourceText.getText("infModuleSaved", this), INFOBOX_STATUS.success);
				infobox.setVisible(true);
				target.add(wmcInfo);
				target.add(_lblTitel);
			}
			return true;
		}catch(Exception e) {
			infobox.refresh(target, ResourceText.getText("errModuleSaved", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);
		}
		return false;
	}
	
	
	
	private boolean checkModule( AjaxRequestTarget target) {
		try {
			DmsWebApplication.getModules().loadModule(_apiModuleImpl);
			infobox.refresh(target, ResourceText.getText("infCheckModuleOK", this), INFOBOX_STATUS.success);
			infobox.setVisible(true);
			target.add(wmcInfo);
		}catch(Exception e) {
			infobox.refresh(target, ResourceText.getText("infCheckModuleErr", this)+"\n"+e, INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);
		}
		return true;
	}
	
	 
	
	
	
	
	private void cancel(AjaxRequestTarget target){
		 try {
			 if(_toolBar.getViewState()==ToolbarState.STATE_VIEW || _toolBar.getViewState()==ToolbarState.STATE_NEW ) {
				 closeForm(target);
			 }else {
				 try {
					 _apiModuleImpl = _apiModuleImpl_Backup;
					 refreshFormObjects();
					 _toolBar.setViewState(ToolbarState.STATE_VIEW);
					 setFormComponentsEditable(false);
					 target.add(_form);
					 target.add(_toolBar);
				 }catch(Exception e2) {
					 infobox.refresh(target, ResourceText.getText("errChangeCancel", this)+".\n"+e2.getMessage(), INFOBOX_STATUS.error);
					 infobox.setVisible(true);
					 target.add(wmcInfo);					 
				 }
			 } 
		 }catch(Exception e) {
			infobox.refresh(target, ResourceText.getText("errChangeCancel", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);			 
		 }

	}
	
	private void refreshFormObjects(){
		
        _classname.setDefaultModel(new PropertyModel<String>(_apiModuleImpl, "Classname"));
        _version.setDefaultModel(new PropertyModel<String>(_apiModuleImpl, "Version"));
        _jar.setDefaultModel(new PropertyModel<String>(_apiModuleImpl, "Jar"));
        _type.setDefaultModel(new PropertyModel<String>(_apiModuleImpl, "Type"));
        _author.setDefaultModel(new PropertyModel<String>(_apiModuleImpl, "Author"));
        _url.setDefaultModel(new PropertyModel<String>(_apiModuleImpl, "Url"));
        _beschreibung.setDefaultModel(new PropertyModel<String>(_apiModuleImpl, "Description"));
		_firstPageOnly.setDefaultModel(new PropertyModel<String>(_apiModuleImpl, "FirstpageOnly"));
	}
	
	private void setFormComponentsEditable(boolean editable){
		try{
			
			_classname.setEnabled(false);
			_version.setEnabled(false);
			_jar.setEnabled(false);
			_type.setEnabled(false);
			_author.setEnabled(false);
			_url.setEnabled(false);
			_beschreibung.setEnabled(false);
			_firstPageOnly.setEnabled(editable);
		}catch(Exception e){
		}
	}
	
	
	private void addInfobox() {
		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		
		infobox = new InfoboxPanel("infobox");
		infobox.setVisible(false);
		 
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
	} 
	
	 
	
}
