/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.modules;

import org.apache.wicket.Page;
import org.apache.wicket.PageReference;
import org.biffo.dms.DmsWebApplicationPages;
import org.biffo.dms.DmsWebApplicationPages.ApplicationPage;
import org.biffo.dms.entity.Variables.Impl.ApiModuleImpl;
import org.biffo.dms.web.modules.test.ModuleDocumentEntityTestPanel;
import org.biffo.dms.web.modules.test.ModuleDocumentFragmentsTestPanel;
import org.biffo.dms.web.template.DmsTemplate;
 

public class ModulesPage extends DmsTemplate {
 
	private static final long serialVersionUID = 1L;
	private final PageReference _pagefrom;
	
	public ModulesPage(){
		this(DmsWebApplicationPages.ApplicationPage.MODULES);
	}
	
	public ModulesPage(ApplicationPage appPage){
		super(appPage);
		_pagefrom = null;
		replace(new ModulesListPanel(CONTENT_ID));
	}
	
	public ModulesPage(PageReference pagefrom){
		super(DmsWebApplicationPages.ApplicationPage.MODULES);
		replace(new ModulesListPanel(CONTENT_ID));
		_pagefrom = pagefrom;
	}
	
	
	public ModulesPage(PageReference pagefrom, ApiModuleImpl moduleImpl){
		super(DmsWebApplicationPages.ApplicationPage.MODULES);
		if(moduleImpl.getType().equals("DocumentEntity")) {
			replace(new ModuleDocumentEntityTestPanel(CONTENT_ID, moduleImpl, pagefrom));
		}else {
			replace(new ModuleDocumentFragmentsTestPanel(CONTENT_ID, moduleImpl, pagefrom));
		}
		_pagefrom = pagefrom;
	}
	
	
	@Override
	protected void onConfigure() {
		super.onConfigure();
	}
	
	public Page getPageFrom(){
		if(null!=_pagefrom) return _pagefrom.getPage();
		return null;
	}
	
}
