/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.modules.test;

import java.util.ArrayList;
import java.util.List;

import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.DefaultDataTable;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.biffo.dms.dataprovider.config.DPDocumentFragmentsModuleTest;
import org.biffo.dms.entity.Fragments.Fragment;
import org.biffo.dms.entity.Variables.Impl.ApiModuleImpl;

public class FragmentsTestListPanel extends Panel{

 
	private static final long serialVersionUID = 1L;

	
	private DPDocumentFragmentsModuleTest _qtdp = new DPDocumentFragmentsModuleTest();
	private DefaultDataTable<?, ?> _ddt;

	private Form<?> _form;
	private ApiModuleImpl _apiModule;
	
	
	@SuppressWarnings({ "serial", "rawtypes", "unchecked" })
	public FragmentsTestListPanel(String id, ApiModuleImpl apiModule) {
		super(id);

		_apiModule = apiModule;
		_qtdp.setApiModule(_apiModule);

		List<IColumn<?,?>> columns = new ArrayList<IColumn<?,?>>();
		
		columns.add(new PropertyColumn<Fragment, String>(new Model<String>("Name"), "Fragmentname","Fragmentname"));
		columns.add(new PropertyColumn<Fragment, String>(new Model<String>("Wert"), "Fragmentvalue","Fragmentvalue"));
		columns.add(new PropertyColumn<Fragment, String>(new Model<String>("%"), "Probability","Probability"));

		columns.add(new AbstractColumn<Fragment, String>(new Model<String>("Seite"), "Page"){
			 
			private static final long serialVersionUID = 1L;

			@Override
		    public void populateItem(Item<ICellPopulator<Fragment>> item, String componentId, IModel<Fragment> rowModel){
				String label = "";
				if(rowModel.getObject().getPage()!=-1)label = ""+rowModel.getObject().getPage();
				Label lbl = new Label(componentId, label);
				item.add(lbl);
			}
		});
		columns.add(new AbstractColumn<Fragment, String>(new Model<String>("")){
			 
			private static final long serialVersionUID = 1L;

			@Override
		    public void populateItem(Item<ICellPopulator<Fragment>> item, String componentId, IModel<Fragment> rowModel){
				buildStatusCell(item, componentId, rowModel.getObject().isValid());
			}
		});
		
		
		_ddt = new DefaultDataTable("datatable", columns, _qtdp, 10){
			@Override
			protected void onPageChanged() {
				super.onPageChanged();
			}
		};
		
		
		_form = new Form("dataform");
		_form.add(_ddt);
		_form.setOutputMarkupId(true);
		add(_form);
	}
	
	private void buildStatusCell(Item<ICellPopulator<Fragment>> item, String componentId, boolean valid) {
		
		Label lbl = null;
		if(valid) {
			lbl = new Label(componentId, "<span class=\"statusType info\">&#xe2e6;</span>");
		}else {
			lbl = new Label(componentId, "<span class=\"statusType error\">&#xe000;</span>");
		}

		lbl.setEscapeModelStrings(false);
		lbl.setOutputMarkupId(true);
		item.add(lbl);
	}
	
	public void setContent(String content) {
		_qtdp.setContent(content);
		_qtdp.refresh();
	}
	
	
	
}
