/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.modules.test;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.wicket.Page;
import org.apache.wicket.PageReference;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.OnChangeAjaxBehavior;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.PropertyModel;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.infobox.InfoboxPanel.INFOBOX_STATUS;
import org.biffo.dms.entity.Document.Document;
import org.biffo.dms.entity.Document.DocumentOSFilterClass;
import org.biffo.dms.entity.Variables.Impl.ApiModuleImpl;
import org.biffo.dms.os.EntityFieldToOsFieldMapping;
import org.biffo.dms.os.EntityToOsMappingStrategy;
import org.biffo.dms.web.helpers.PanelTitel;
import org.biffo.dms.web.helpers.ResourceText;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;
import de.kleinekuh.os.rest.query.BooleanQuery;
import de.kleinekuh.os.rest.query.term.TermQuery;
import de.kleinekuh.os.rest.search.OsSearch;

public class ModuleDocumentFragmentsTestPanel   extends Panel{

 
	private static final long serialVersionUID = 1L;
	private static final String toolbarCSS = "dataformToolbar";
	
	private TextArea<String> _textinput;
	private Label _lblTitel;
	
	private Form<?> _form;
	private Toolbar _toolBar;
	private ToolbarButton _btRun, _btClear, _btCancel;
	private Object _origin;
	
	private PanelTitel _panelTitel = new PanelTitel();
	
	private ApiModuleImpl _apiModuleImpl;
	private TestDocumentEntity _testDocumentEntity = new TestDocumentEntity();
	
	private WebMarkupContainer wmcInfo;
	private InfoboxPanel infobox;
	
	private FragmentsTestListPanel _fp; 
	
	private String regexDoc = "doc\\[([0-9]+)\\]";
	private String regexPage = "page\\[([0-9]+)_([0-9]+)\\]";
	private Pattern patternDoc =Pattern.compile(regexDoc);
	private Pattern patternPage =Pattern.compile(regexPage);
	
	public class TestDocumentEntity implements Serializable{
		
		private static final long serialVersionUID = 1L;
		
		private String textinput;
		
		public TestDocumentEntity() {
		}
	
		public String getTextinput() {return textinput;}
		public void setTextinput(String textinput) {this.textinput = textinput;}
		
	}
	
	public ModuleDocumentFragmentsTestPanel(String id, ApiModuleImpl apiModuleImpl,  Object origin) {
		super(id);
		_origin = origin;
		_apiModuleImpl = apiModuleImpl;
		
		_form = new Form<Void>("form");
		_form.setOutputMarkupId(true);
		add(_form);
		
        addTextFields();
        addCheckBoxen();
        addAdditionalControls();
        
        addToolBar();
        addInfobox();
        
        setFormComponentsEditable(false);
		setOutputMarkupId(true);
		_panelTitel.setTitel(ResourceText.getText("titModule", this));
		refreshFormObjects();
		
		
		
	}
	
	private void addTextFields(){
	    
        _textinput= new TextArea<String>("textinput");
        _textinput.setOutputMarkupId(true);
        _form.add(_textinput);
        
        
        OnChangeAjaxBehavior onChange = new OnChangeAjaxBehavior() {

			private static final long serialVersionUID = 1L;

			@Override
	        protected void onUpdate(final AjaxRequestTarget target){
				try {
					
					String temp = _textinput.getDefaultModelObjectAsString();
					_testDocumentEntity.setTextinput(temp);	

					Matcher mtDoc = patternDoc.matcher(temp);
					Matcher mtPage = patternPage.matcher(temp);
					
				    if(mtDoc.matches()) {
				    	int docToLoad = Integer.valueOf(mtDoc.group(1)); 
				    	try {
				    		Document document = StaticLocators.getDocuments().getEntityById(docToLoad);
				    		clearAll(target);
				    		_testDocumentEntity.setTextinput(document.getFirstLines());
				    		target.add(_textinput);
				    		processEvent("run", target);
				    	}catch(Exception e) {
				    		infobox.refresh(target, ResourceText.getText("errDocumentLoad", ModuleDocumentFragmentsTestPanel.this, docToLoad), INFOBOX_STATUS.error);
							infobox.setVisible(true);
							target.add(wmcInfo);		
				    	}
				    }else if(mtPage.matches()) {
				    	int docToLoad = Integer.valueOf(mtPage.group(1));
				    	int pageToLoad = Integer.valueOf(mtPage.group(2));
				    	try {
				    		DocumentOSFilterClass dsfc  = new DocumentOSFilterClass();
				    		dsfc.setResultSize(1000);
				    		ArrayList<Document> entities = new ArrayList<Document>();
				    		entities.addAll(StaticLocators.getDocumentsDBOS().getOsFilteredEntities(-1, -1, "", dsfc, 
				    				getSearch(dsfc, docToLoad, pageToLoad), buildEntityMapping(), null));
				    		if(entities.size()>0) {
				    			_testDocumentEntity.setTextinput(entities.get(0).getResulttext());
				    			target.add(_textinput);
				    			processEvent("run", target);
				    		}else {
				    			throw new Exception();
				    		}
				    	}catch(Exception e) {
				    		infobox.refresh(target, ResourceText.getText("errDocumentPageLoad", ModuleDocumentFragmentsTestPanel.this, docToLoad, pageToLoad), INFOBOX_STATUS.error);
							infobox.setVisible(true);
							target.add(wmcInfo);		
				    	}
				    }else {
				    	_testDocumentEntity.setTextinput(_textinput.getDefaultModelObjectAsString());	
				    } 
				}catch(Exception e) {
					infobox.refresh(target, ResourceText.getText("errDocumentView", ModuleDocumentFragmentsTestPanel.this)+"\n"+e.getMessage(), INFOBOX_STATUS.error);
					infobox.setVisible(true);
					target.add(wmcInfo);		
				}
			}
		};
		_textinput.add(onChange);
	}
	
	private void addCheckBoxen(){}
	
	private void addAdditionalControls(){
		_lblTitel = new Label("paneltitel", new PropertyModel<String>(this._panelTitel, "Titel"));
		_lblTitel.setOutputMarkupId(true);
		add(_lblTitel);
		
		_fp = new FragmentsTestListPanel("fragments", _apiModuleImpl);
		_fp.setOutputMarkupId(true);
		_form.add(_fp);
		
	}
	
	private void closeForm(AjaxRequestTarget target){
		if(_origin instanceof Panel){
			target.add(ModuleDocumentFragmentsTestPanel.this.replaceWith((Panel)_origin).getParent());
		}else if(_origin instanceof Page){
		}else if(_origin instanceof PageReference){
			PageReference pg = (PageReference)_origin;
			setResponsePage(pg.getPage());
		}
	}
	
	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};
		
		_btCancel = new ToolbarButton().setButtonType(ToolbarButtonType.CANCEL).setToolbarState(ToolbarState.STATE_VIEW | ToolbarState.STATE_NEW | ToolbarState.STATE_CHANGE).setCssClass(toolbarCSS);
		_btCancel.setAlternate(ResourceText.getText("tbAltCancel", this));
		_toolBar.add(_btCancel);
		
		_btRun = new ToolbarButton(_form).setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW | ToolbarState.STATE_CHANGE |  ToolbarState.STATE_NEW);
		_btRun.setUtf8Code("xf8a3");
		_btRun.setAlternate(ResourceText.getText("tbAltRun", this));
		_btRun.setEventName("run");
		_btRun.setCssClass(toolbarCSS);
		_toolBar.add(_btRun);

		
		_btClear = new ToolbarButton(_form).setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW | ToolbarState.STATE_CHANGE |  ToolbarState.STATE_NEW);
		_btClear.setUtf8Code("xe0e9");
		_btClear.setAlternate(ResourceText.getText("tbAltClear", this));
		_btClear.setEventName("clear");
		_btClear.setCssClass(toolbarCSS);
		_toolBar.add(_btClear);
		
		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		_form.add(_toolBar);
	}
	
	private void processEvent(String event, AjaxRequestTarget target) {
		if(event.equals("run") && _testDocumentEntity.getTextinput()!=null && !_testDocumentEntity.getTextinput().isEmpty()) {
			testModule(target);
		}
		else if(event.equals("clear"))clearAll(target);
		else if(event.equals("Cancel")) cancel(target);
	}
	
	private void cancel(AjaxRequestTarget target){
		 try {
			 closeForm(target);
		 }catch(Exception e) {
			infobox.refresh(target, ResourceText.getText("errChangeCancel", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);			 
		 }

	}
	
	private void testModule( AjaxRequestTarget target) {
		try {
			_fp.setContent(_testDocumentEntity.getTextinput());
			target.add(_fp);
			
		}catch(Exception e) {}
	}
	
	private void clearAll(AjaxRequestTarget target) {

		_testDocumentEntity = new TestDocumentEntity();
		refreshFormObjects();
		
		_fp.setContent(null);
		
		
		target.add(_fp);
		target.add(_textinput);
	}
	
	
	private void setFormComponentsEditable(boolean editable){
		try{
			_textinput.setEnabled(true);
		}catch(Exception e){
		}
	}
	
	private void refreshFormObjects(){
		_textinput.setDefaultModel(new PropertyModel<String>(_testDocumentEntity, "Textinput"));
	}
	
	
	private void addInfobox() {
		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		
		infobox = new InfoboxPanel("infobox");
		infobox.setVisible(false);
		 
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
	}
	
	private OsSearch getSearch(DocumentOSFilterClass dsfc, int docid, int docpage) {

		BooleanQuery bq = new BooleanQuery();
		
		TermQuery tmid = new TermQuery();
		tmid.setField("1_docid");
		tmid.setValue(""+docid);
		bq.addMust(tmid);
		
		TermQuery tmpage = new TermQuery();
		tmpage.setField("3_docpage");
		tmpage.setValue(""+docpage);
		bq.addMust(tmpage);
		
		OsSearch search = new OsSearch();
		search.setSource(true);
		search.setSize(dsfc.getResultSize());
		search.setQuery(bq);
		return search;
		
	}
	

	private EntityToOsMappingStrategy buildEntityMapping() {
		EntityFieldToOsFieldMapping idField = new EntityFieldToOsFieldMapping("1_docid", "Id", true);
		EntityFieldToOsFieldMapping pageField = new EntityFieldToOsFieldMapping("3_docpage", "Page", false);
		EntityFieldToOsFieldMapping contentField = new EntityFieldToOsFieldMapping("4_doccontent", "Resulttext", true);

		EntityToOsMappingStrategy _strategy = new EntityToOsMappingStrategy(Document.class);
		_strategy.addFieldMapping(idField);
		_strategy.addFieldMapping(pageField);
		_strategy.addFieldMapping(contentField);
		return _strategy;
	}
	

}
