/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.server;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.panel.Panel;
import org.biffo.dms.entity.Variables.Impl.ApiModuleImpl;
import org.biffo.dms.entity.Variables.Impl.IVariablesImpl;
import org.biffo.dms.web.helpers.ResourceText;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;

 
public class ServerControlPanel extends Panel{

	
	private static final long serialVersionUID = 1L;

	private Toolbar _toolBar;
	private IVariablesImpl importProcess;
	private ToolbarButton _btStart, _btStop, _btSync, _btTest;
	private boolean isModule=false;
	
	public ServerControlPanel(String id, IVariablesImpl importProcess) {
		super(id);
		this.importProcess = importProcess;
		if(importProcess instanceof ApiModuleImpl) {
			isModule = true;
		}
		addToolBar();
		
	}
	
	private void addToolBar()  {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				if(event.equals("start")) {
					ServerControlPanel.this.OnServerStart(target, importProcess);
				}else if(event.equals("stop")) {
					ServerControlPanel.this.OnServerStop(target, importProcess);
				}else if(event.equals("sync")) {
					ServerControlPanel.this.OnServerSync(target, importProcess);
				}else if(event.equals("test")) {
					ServerControlPanel.this.OnModuleTest(target, importProcess);
				}
			}
		};
		
		_btStart = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
		_btStart.setCssClass("servercontrolpanel green");
		if(isModule)_btStart.setAlternate(ResourceText.getText("tbAltModuleStart", this));
		else _btStart.setAlternate(ResourceText.getText("tbAltProcessStart", this));
		_btStart.setUtf8Code("xf0d1");
		_btStart.setEventName("start");
		_toolBar.add(_btStart);

		
		_btStop = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
		_btStop.setCssClass("servercontrolpanel red");
		if(isModule)_btStop.setAlternate(ResourceText.getText("tbAltModuleStop", this));
		else _btStop.setAlternate(ResourceText.getText("tbAltProcessStop", this));
		_btStop.setUtf8Code("xef71");
		_btStop.setEventName("stop");
		_toolBar.add(_btStop);
		
		_btSync = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
		_btSync.setCssClass("servercontrolpanel blue");
		if(isModule)_btSync.setAlternate(ResourceText.getText("tbAltModuleSync", this));
		else _btSync.setAlternate(ResourceText.getText("tbAltProcessSync", this));
		_btSync.setUtf8Code("xe627");
		_btSync.setEventName("sync");
		_toolBar.add(_btSync);
		
		
		if(isModule) {
			_btTest = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
			_btTest.setCssClass("servercontrolpanel");
			_btTest.setUtf8Code("xf8a3");
			_btTest.setAlternate(ResourceText.getText("tbAltModuleTest", this));
			_btTest.setEventName("test");
			_toolBar.add(_btTest);
		}
		
		

		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		
		if(importProcess.isServerActive()) {
			_btStart.setVisible(false);
			_btStop.setVisible(true);
			_btSync.setVisible(true);
			if(isModule)_btTest.setVisible(true);
		}else {
			_btStart.setVisible(true);
			_btStop.setVisible(false);
			_btSync.setVisible(false);
			if(isModule)_btTest.setVisible(false);
		}
		add(_toolBar);
	}
	
	protected void OnServerStart(AjaxRequestTarget target, IVariablesImpl importProcess){}
	protected void OnServerStop(AjaxRequestTarget target, IVariablesImpl importProcess){}
	protected void OnServerSync(AjaxRequestTarget target, IVariablesImpl importProcess){}
	protected void OnModuleTest(AjaxRequestTarget target, IVariablesImpl importProcess){}


}
