/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.server.folder;


import java.util.Date;
import java.util.HashMap;
import java.util.Set;

import org.apache.wicket.Page;
import org.apache.wicket.PageReference;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.util.string.Strings;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebApplication;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.infobox.InfoboxPanel.INFOBOX_STATUS;
import org.biffo.dms.entity.Helpers.EntityUUID;
import org.biffo.dms.entity.Variables.Variable;
import org.biffo.dms.entity.Variables.VariableGroupType;
import org.biffo.dms.entity.Variables.Impl.FolderServerImpl;
import org.biffo.dms.helpers.ConfigHelpers;
import org.biffo.dms.web.helpers.ErrorLabel;
import org.biffo.dms.web.helpers.PanelTitel;
import org.biffo.dms.web.helpers.ResourceText;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;

public class FolderServerPanel extends Panel{


	private static final String _className ="FolderServerPanel";
	private static final long serialVersionUID = 1L;
	private static final String toolbarCSS = "dataformToolbar";
	
	private TextField<String> _uploadfolder, _foldername;
	private ErrorLabel _uploadfolder_info, _foldername_info;
	
	
	private TextArea<String> _beschreibung;
	private Label _lblTitel;

	private Form<?> _form;
	private Toolbar _toolBar;
	private ToolbarButton _btEdit, _btCancel, _btSave, _btOK;
	
	private Object _origin;
	
	private PanelTitel _panelTitel = new PanelTitel();
	
	private DmsWebSession _session = DmsWebSession.getSession();
	 
	private WebMarkupContainer wmcInfo;
	private InfoboxPanel infobox;
	
	private FolderServerImpl _folderServer, _folderServer_Backup;
	private boolean _isStarted=false;
	
	
	public FolderServerPanel(String id, int idFolderServer,  Object origin) {
		this(id, null, origin);
	}
	
	public FolderServerPanel(String id, FolderServerImpl folderServer,  Object origin) {
		super(id);
		_origin = origin;
		_folderServer = folderServer;
		_folderServer_Backup = folderServer;

        if(null==folderServer) {
        	_folderServer = new FolderServerImpl();
        }else {
        	_isStarted = DmsWebApplication.getProcessQueue().isRunning(_folderServer.getServerName());
        }
		
		_form = new Form<Void>("form");
		_form.setOutputMarkupId(true);
		add(_form);
		
        addTextFields();
        addCheckBoxen();
        addAdditionalControls();
        
        addToolBar();
        addInfobox();
        
        setFormComponentsEditable(false);
		setOutputMarkupId(true);
		if(-1==_folderServer.getId()) {
			_folderServer = new FolderServerImpl();
			_panelTitel.setTitel(ResourceText.getText("titNewFolderServer", this));
			_toolBar.setViewState(ToolbarState.STATE_NEW);
			setFormComponentsEditable(true);
		}else {
			_panelTitel.setTitel(ResourceText.getText("titEditFolderServer", this));
		}
	 
		
		refreshFormObjects();
	
	}
	
	private void addTextFields(){
		
		_foldername= new TextField<String>("foldername");
		_foldername.setOutputMarkupId(true);
		_form.add(_foldername);
		
		_foldername_info = new ErrorLabel("foldername_info");
        _form.add(_foldername_info);
                
        _uploadfolder= new TextField<String>("uploadfolder");
        _uploadfolder.setOutputMarkupId(true);
        _form.add(_uploadfolder);
        
        _uploadfolder_info = new ErrorLabel("uploadfolder_info");
        _form.add(_uploadfolder_info);
        
		_beschreibung= new TextArea<String>("beschreibung");
		_beschreibung.setOutputMarkupId(true);
        _form.add(_beschreibung);
	}
	
	private void addCheckBoxen(){

		 
        
	}
	
	private void addAdditionalControls(){
		
		_lblTitel = new Label("paneltitel", new PropertyModel<String>(this._panelTitel, "Titel"));
		_lblTitel.setOutputMarkupId(true);
		add(_lblTitel);
	}
	
	private void closeForm(AjaxRequestTarget target){
		if(_origin instanceof Panel){
			target.add(FolderServerPanel.this.replaceWith((Panel)_origin).getParent());
		}else if(_origin instanceof Page){
		}else if(_origin instanceof PageReference){
			PageReference pg = (PageReference)_origin;
			setResponsePage(pg.getPage());
		}
	}
	
	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};
		
		_btEdit = new ToolbarButton().setButtonType(ToolbarButtonType.EDIT).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btEdit.setAlternate(ResourceText.getText("tbAltEdit", this));
		if(!_isStarted)_toolBar.add(_btEdit);
		
		_btCancel = new ToolbarButton().setButtonType(ToolbarButtonType.CANCEL).setToolbarState(ToolbarState.STATE_VIEW | ToolbarState.STATE_NEW | ToolbarState.STATE_CHANGE).setCssClass(toolbarCSS);
		_btCancel.setAlternate(ResourceText.getText("tbAltCancel", this));
		_toolBar.add(_btCancel);
		
		_btOK = new ToolbarButton(_form).setButtonType(ToolbarButtonType.OK).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btOK.setAlternate(ResourceText.getText("tbAltOk", this));
		_toolBar.add(_btOK);
		
		_btSave = new ToolbarButton(_form).setButtonType(ToolbarButtonType.SAVE).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btSave.setAlternate(ResourceText.getText("tbAltSave", this));
		_toolBar.add(_btSave);
		
		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		
		_form.add(_toolBar);
	}
	
	private void processEvent(String event, AjaxRequestTarget target) {
		if(event.equals("Edit")) edit(target);
		else if(event.equals("Cancel")) cancel(target);
		else if(event.equals("Save")) save(target, true);
		else if(event.equals("Ok")) ok(target);

		target.add(_form);
		target.add(_toolBar);
	}
	
	private boolean checkShowError(FolderServerImpl serverImpl, AjaxRequestTarget target) {
		
		boolean error=false;
		if(serverImpl==null) return false;
		if(Strings.isEmpty(serverImpl.getServerName())){
			
			_foldername_info.setErrorText(ResourceText.getText("errTextServerNameWrong", this));
			_foldername_info.setVisible(true);
			error = true;
		}else if(ConfigHelpers.checkServerNameExists(serverImpl.getId(), serverImpl.getServerName(), VariableGroupType.FOLDERS)){
			_foldername_info.setErrorText(ResourceText.getText("errTextServerNameExists", this));
			_foldername_info.setVisible(true);
			error = true;
		}else {
			_foldername_info.setVisible(false);
		}
		
		if(Strings.isEmpty(_folderServer.getServerFolder())){
			_uploadfolder_info.setErrorText(ResourceText.getText("errTextServerFolderWrong", this));
			_uploadfolder_info.setVisible(true);
			error = true;
		}else if(ConfigHelpers.checkUploadFolderExists(_folderServer.getId(), _folderServer.getVariableByKey("folder").getId(), _folderServer.getServerFolder(), VariableGroupType.FOLDERS)){
			_uploadfolder_info.setErrorText(ResourceText.getText("errTextServerFolderUsed", this));
			_uploadfolder_info.setVisible(true);
			error=true;
		}else if(!ConfigHelpers.checkFileSystemFolderExists(_folderServer.getServerFolder())) {
			_uploadfolder_info.setErrorText(ResourceText.getText("errTextServerFolderNotExists", this));
			_uploadfolder_info.setVisible(true);
			error=true;
		}else if(!ConfigHelpers.isWritable(_folderServer.getServerFolder())) {
			_uploadfolder_info.setErrorText(ResourceText.getText("errTextServerFolderNotWritable", this));
			_uploadfolder_info.setVisible(true);
			error=true;
		}else {
			_uploadfolder_info.setVisible(false);
		}

		target.add(_uploadfolder_info);
	 
		return error;
		
	}
	
	
	private boolean save(AjaxRequestTarget target, boolean withMessage) {
		try {
			
			if(checkShowError(_folderServer, target))return false;
			
			HashMap<String, Variable> hmVariables = _folderServer.getVariables();
			Set<String> keys = hmVariables.keySet();
			if(_folderServer.getId()!=-1) {
				Date aenderungsDatum = new Date();
				for(String key: keys) {
					Variable var = hmVariables.get(key);
					var.setGeaendertAm(aenderungsDatum);
					var.setGeaendertVon(_session.getCurrentUser());
					var.setUUID(EntityUUID.createVariableUUID(var));
					StaticLocators.getVariables().updateEntity(var, true);
				}
				DmsException dmsInfo = new DmsException(2000, "I", 
						_className, 
						"save",
						ResourceText.getText("dmsInfoFolderServerChanged", this, _folderServer.getServerName()),
						null,
						_folderServer
						);
				StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
			}else {
				
				Date erstellungsDatum = new Date();
				for(String key: keys) {
					Variable var = hmVariables.get(key);
					var.setErstelltAm(erstellungsDatum);
					var.setGeaendertAm(var.getErstelltAm());
					var.setErstelltVon(_session.getCurrentUser());
					var.setGeaendertVon(var.getErstelltVon());
					var.setUUID(EntityUUID.createVariableUUID(var));
					int varID = StaticLocators.getVariables().getEntityId(var);
					if(-1==varID) {
						varID = StaticLocators.getVariables().addEntity(var);
						if(var.getVariableName().equals("folder"))_folderServer.setId(varID);
					}else {
						StaticLocators.getVariables().updateEntity(var, true);
					}
				}
				DmsException dmsInfo = new DmsException(2000, "I", 
						_className, 
						"save",
						ResourceText.getText("dmsInfoFolderServerCreated", this, _folderServer.getServerName()),
						null,
						_folderServer
						);
				StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
				
			}
			
			
			_folderServer_Backup = _folderServer;
			if(withMessage) {
				infobox.refresh(target, ResourceText.getText("infFolderServerSaved", this), INFOBOX_STATUS.success);
				infobox.setVisible(true);
				target.add(wmcInfo);
				target.add(_lblTitel);
			}
			return true;
		}catch(Exception e) {
			infobox.refresh(target, ResourceText.getText("errFolderServerSaved", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);
		}
		return false;
	}
	
	
	
	
	private void cancel(AjaxRequestTarget target){
		 try {
			 if(_toolBar.getViewState()==ToolbarState.STATE_VIEW || _toolBar.getViewState()==ToolbarState.STATE_NEW ) {
				 closeForm(target);
			 }else {
				 try {
					 _folderServer = _folderServer_Backup;
					 refreshFormObjects();
					 _toolBar.setViewState(ToolbarState.STATE_VIEW);
					 setFormComponentsEditable(false);
					 target.add(_form);
					 target.add(_toolBar);
				 }catch(Exception e2) {
					 infobox.refresh(target, ResourceText.getText("errChangeCancel", this)+".\n"+e2.getMessage(), INFOBOX_STATUS.error);
					 infobox.setVisible(true);
					 target.add(wmcInfo);					 
				 }
			 } 
		 }catch(Exception e) {
			infobox.refresh(target, ResourceText.getText("errChangeCancel", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);			 
		 }

	}
	
	private void edit(AjaxRequestTarget target){
		 try {
			 if(_folderServer.getId()!=-1){
				 _toolBar.setViewState(ToolbarState.STATE_CHANGE);
				 setFormComponentsEditable(true);
			 }
	 	}catch(Exception e) {
	 		infobox.refresh(target, ResourceText.getText("errChangeEditMode", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);	 		
	 	}
	}
	
	 
		
	private boolean ok(AjaxRequestTarget target){
		if(save(target, false)) {
			_toolBar.setViewState(ToolbarState.STATE_VIEW);
			cancel(target);
			return true;
		}
		return false;
	} 
	
	private void refreshFormObjects(){
		_foldername.setDefaultModel(new PropertyModel<String>(_folderServer, "ServerName"));
	    _uploadfolder.setDefaultModel(new PropertyModel<String>(_folderServer, "ServerFolder"));
		_beschreibung.setDefaultModel(new PropertyModel<String>(_folderServer, "Description"));
	}
	
	private void setFormComponentsEditable(boolean editable){
		try{
			_foldername.setEnabled(editable);
			_uploadfolder.setEnabled(editable);
			_beschreibung.setEnabled(editable);
		}catch(Exception e){
		}
	}
	
	
	private void addInfobox() {
		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		
		if(_isStarted) {
			infobox = new InfoboxPanel("infobox", ResourceText.getText("warFolderServerStarted", this), INFOBOX_STATUS.warning);
			infobox.setVisible(true);
		}else {
			infobox = new InfoboxPanel("infobox");
			infobox.setVisible(false);
		}
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
	} 
	
	
	
}
