/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.tags;

import java.util.Date;

import org.apache.wicket.Page;
import org.apache.wicket.PageReference;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.util.string.Strings;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebApplication;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.Types.EntityType;
import org.biffo.dms.components.colorpicker.ColorPicker;
import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.infobox.InfoboxPanel.INFOBOX_STATUS;
import org.biffo.dms.components.tagbox.TagsBoxPanel;
import org.biffo.dms.entity.Helpers.EntityUUID;
import org.biffo.dms.entity.Tags.Tag;

import org.biffo.dms.helpers.DmsDBHelpersTag;
import org.biffo.dms.helpers.TagObjectTypes;
import org.biffo.dms.lock.Lock;
import org.biffo.dms.models.TagModel;
import org.biffo.dms.web.helpers.ErrorLabel;
import org.biffo.dms.web.helpers.PanelTitel;
import org.biffo.dms.web.helpers.ResourceText;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;

 
public class TagPanel extends Panel{

 
	private static final String _className = "TagPanel";
	private static final long serialVersionUID = 1L;
	private static final String toolbarCSS = "dataformToolbar";
	
	private Form<?> _form;
	private Toolbar _toolBar;
	private ToolbarButton _btEdit, _btCancel, _btOk, _btSave;
	
	private final Object _origin;
	private Page _originPage;
	
	private TextField<String>  _tagname;
	private TextArea<String> _beschreibung;
	private ErrorLabel _tagname_info;
	
	private TagsBoxPanel _tagBox;
	private ColorPicker _colorpickerbg, _colorpickerfg;
	
	private PanelTitel _panelTitel = new PanelTitel();
	private Label _lblTitel;
	
	private Tag _tag;
	private int _tagId;
	private TagObjectTypes _tagObjectTypes;

	private DmsWebSession _session = (DmsWebSession)DmsWebSession.getSession();
	
	private WebMarkupContainer wmcInfo;
	private InfoboxPanel infobox;
	
	private Lock _lock = new Lock();
	private int _documentId=-1;
	
	public TagPanel(String id, int tagId,  Object origin) {
		this(id, tagId, origin, null, -1);
	}
	
	
	public TagPanel(String id, int tagId,  Object origin, Tag tag, int documentId) {
		super(id);

		_origin = origin;
		_tagId = tagId;
		_documentId =documentId;
		
		if(origin!=null && origin instanceof PageReference) {
			_originPage = ((PageReference)_origin).getPage();
 		}
		 
		
		if(tagId==-1) {
			_tag = new Tag();
			_tag.setTagFgColor("ffffff");
			_tag.setTagBgColor("5a76a9");
			_tag.setTagValue(ResourceText.getText("txtTagDefaultValue", this));
			if(null!=tag) {
				_tag.setTagValue(tag.getTagValue());
				_tag.setBeschreibung(tag.getBeschreibung());
			}
			
		}else {
			_tag = new TagModel(tagId).getObject();
		}
		_tagObjectTypes = new TagObjectTypes(_tag.getStatusTypes());

		_form = new Form<Void>("form");
		_form.setOutputMarkupId(true);
		add(_form);
		
        addTextFields();
        addCheckBoxen();
        addDropDownBoxen();
        addAdditionalControls();
        addToolBar();
        addInfobox();
        
		refreshFormObjects();
		setFormComponentsEditable(false);
		setOutputMarkupId(true);
		
		if(_tagId==-1) {
			_panelTitel.setTitel(ResourceText.getText("titNewTag", this));
			_toolBar.setViewState(ToolbarState.STATE_NEW);
			setFormComponentsEditable(true);
		}else {
			_panelTitel.setTitel(ResourceText.getText("titEditTag", this));
		}
		buildLock(_tagId);
	}
	
	
	private void addTextFields(){
		
        _tagname= new TextField<String>("tagname");
        _tagname.add(new AjaxFormComponentUpdatingBehavior("change") {
			
			private static final long serialVersionUID = 1L;
			
			@Override
			protected void onUpdate(AjaxRequestTarget target) {
				if(getComponent().getDefaultModelObject()!=null)_tag.setTagValue(getComponent().getDefaultModelObject().toString());
				else _tag.setTagValue(null);
				target.add(_tagBox);
			}
		});
        _tagname.setOutputMarkupId(true);
        _form.add(_tagname);
        
        _tagname_info = new ErrorLabel("tagname_info");
        _form.add(_tagname_info);
        
		_beschreibung= new TextArea<String>("beschreibung");
		_beschreibung.setOutputMarkupId(true);
        _form.add(_beschreibung);
	}
	
	
	
	
	private void addCheckBoxen(){}
	
	private void addDropDownBoxen(){
	}
	
	private void addAdditionalControls(){
		_colorpickerbg =  new ColorPicker("colorpickerbg", new PropertyModel<String>(_tag, "TagBgColor")){
			private static final long serialVersionUID = 1L;
			@Override
			protected void OnChange(AjaxRequestTarget target, String event, String value) {
				_tagBox.getTags().get(0).setTagBgColor(value);
				target.add(_tagBox);
			}
		}; ;
		_colorpickerbg.setOutputMarkupId(true);
		_form.add(_colorpickerbg);

		_colorpickerfg =  new ColorPicker("colorpickerfg", new PropertyModel<String>(_tag, "TagFgColor")){
			private static final long serialVersionUID = 1L;
			@Override
			protected void OnChange(AjaxRequestTarget target, String event, String value) {
				_tagBox.getTags().get(0).setTagFgColor(value);
				target.add(_tagBox);
			}
		}; 
		_colorpickerfg.setOutputMarkupId(true);
		_form.add(_colorpickerfg);
		
		
		_tagBox = new TagsBoxPanel("tagbox");
		_tagBox.setOutputMarkupId(true);
		_form.add(_tagBox);
		
		_lblTitel = new Label("paneltitel", new PropertyModel<String>(this._panelTitel, "Titel"));
		_lblTitel.setOutputMarkupId(true);
		add(_lblTitel);

	}
	
	private void closeForm(AjaxRequestTarget target){
		if(_origin instanceof Panel){
			target.add(this.replaceWith((Panel)_origin).getParent());
		}else if(_origin instanceof Page){
		}else if(_origin instanceof PageReference){
			if(_originPage!=null)setResponsePage(_originPage);
			else if(((PageReference) _origin).getPage()!=null){
				setResponsePage(((PageReference) _origin).getPage());
			}
		}
	}
	
	private void refreshFormObjects(){
		_tagname.setDefaultModel(new PropertyModel<String>(_tag, "TagValue"));
		_beschreibung.setDefaultModel(new PropertyModel<String>(_tag, "Beschreibung"));
		
		_colorpickerbg.setDefaultModel(new PropertyModel<String>(_tag, "TagBgColor"));
		_colorpickerfg.setDefaultModel(new PropertyModel<String>(_tag, "TagFgColor"));
		_tagBox.setTag(_tag);
		
	}
	
	
	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};
		
		
		_btEdit = new ToolbarButton().setButtonType(ToolbarButtonType.EDIT).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btEdit.setAlternate(ResourceText.getText("tbAltEdit", this));
		_toolBar.add(_btEdit);
		
		_btCancel = new ToolbarButton().setButtonType(ToolbarButtonType.CANCEL).setToolbarState(ToolbarState.STATE_VIEW | ToolbarState.STATE_NEW | ToolbarState.STATE_CHANGE).setCssClass(toolbarCSS); 
		_btCancel.setAlternate(ResourceText.getText("tbAltCancel", this));
		_toolBar.add(_btCancel);
		
		_btOk = new ToolbarButton(_form).setButtonType(ToolbarButtonType.OK).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btOk.setAlternate(ResourceText.getText("tbAltOk", this));
		_toolBar.add(_btOk);
		
		_btSave = new ToolbarButton(_form).setButtonType(ToolbarButtonType.SAVE).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btSave.setAlternate(ResourceText.getText("tbAltSave", this));
		_toolBar.add(_btSave);
		
		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		_form.add(_toolBar);
		
	}
	
	private void processEvent(String event, AjaxRequestTarget target) {
		if(event.equals("Save")) save(target, true);
		else if(event.equals("Ok")) ok(target);
		else if(event.equals("Cancel")) cancel(target);
		else if(event.equals("Edit")) edit(target);
		
		target.add(_form);
		target.add(_toolBar);
	}
	
	
	private void cancel(AjaxRequestTarget target){
		 try {
			 DmsWebApplication.getLocks().removeLock(_tagId, EntityType.TAG);
			 if(_toolBar.getViewState()==ToolbarState.STATE_VIEW || _toolBar.getViewState()==ToolbarState.STATE_NEW ) {
				 closeForm(target);
			 }else {
				 try {
					 refreshFormObjects();
					 _toolBar.setViewState(ToolbarState.STATE_VIEW);
					 setFormComponentsEditable(false);
				 }catch(Exception e2) {
					 infobox.refresh(target, ResourceText.getText("errChangeCancel", this)+".\n"+e2.getMessage(), INFOBOX_STATUS.error);
					 infobox.setVisible(true);
					 target.add(wmcInfo);
				 }
			 } 
		 }catch(Exception e) {
			 infobox.refresh(target, ResourceText.getText("errChangeCancel", this)+".\n"+e.getMessage(), INFOBOX_STATUS.error);
			 infobox.setVisible(true);
			 target.add(wmcInfo);
		 }
	}
	
	
	private void setFormComponentsEditable(boolean editable){
		try{
			_tagname.setEnabled(editable);
			_beschreibung.setEnabled(editable);
			_colorpickerbg.setEnabled(editable);
			_colorpickerfg.setEnabled(editable);
			
		}catch(Exception e){
		}
	}
	
	
	private void edit(AjaxRequestTarget target){
		 try {
			 
			 DmsWebApplication.getLocks().aquireLock(_tagId, _session.getCurrentUserId(), EntityType.TAG);
			 if(_tag.getId()!=-1){
				 _toolBar.setViewState(ToolbarState.STATE_CHANGE);
				 setFormComponentsEditable(true);
			 }
	 	}catch(Exception e) {
	 		infobox.refresh(target, ResourceText.getText("errChangeEditMode", this)+".\n"+e.getMessage(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);
	 	}
	}
 
	
	private boolean ok(AjaxRequestTarget target){
		if(save(target, false)) {
			_toolBar.setViewState(ToolbarState.STATE_VIEW);
			cancel(target);
			return true;
		}
		return false;
	} 
	
 
	
	 
	
	
	
	
	private boolean save(AjaxRequestTarget target, boolean withMessage) {
		try {
			_tag.setStatusTypes(_tagObjectTypes.getStatusTypes());
			_tag.setTagName(_tag.getTagValue());
			_tag.setTagGroup("NA");
			_tag.setUUID(EntityUUID.createTagUUID(_tag));
			_tag.setGeaendertAm(new Date());
			_tag.setGeaendertVon(_session.getCurrentUser());
			_tag.setIdTenant(_session.getCurrentTenantId());
			
			
			if(checkShowError(_tag, target))return false;
			
			if(_tag.getId()==-1) {
				_tag.setErstelltAm(_tag.getGeaendertAm());
				_tag.setErstelltVon(_tag.getGeaendertVon());
				StaticLocators.getTags().addEntity(_tag);
				DmsException dmsInfo = new DmsException(2000, "I", 
						_className,
						"save",
						ResourceText.getText("dmsInfoTagCreated", this, _tag.getTagValue()),
						null,
						_tag
						);
				StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
				_panelTitel.setTitel(ResourceText.getText("titEditTag", this));
				
				_tagId=_tag.getId();
			}else {
				StaticLocators.getTags().updateEntity(_tag, false);
				DmsException dmsInfo = new DmsException(2000, "I", 
						_className,
						"save",
						ResourceText.getText("dmsInfoTagChanged", this, _tag.getTagValue()),
						null,
						_tag
						);
				StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
				_panelTitel.setTitel(ResourceText.getText("titEditTag", this));
				_tagId = _tag.getId();
			}
			if(_documentId!=-1) {
				try {
					DmsDBHelpersTag.addTagToDocument(_documentId, _tag, _session.getCurrentUser(), _session.getCurrentTenantId());
				}catch(Exception e) {
					DmsException dmsException = new DmsException(1000, "E", this.getClass().getSimpleName(), "save", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, DmsWebSession.getSession());
				}
			}	
			_toolBar.setViewState(ToolbarState.STATE_CHANGE);
			refreshFormObjects();
			
			if(withMessage) {
				 infobox.refresh(target, ResourceText.getText("infTagSaved", this), INFOBOX_STATUS.success);
				infobox.setVisible(true);
				target.add(wmcInfo);
				target.add(_lblTitel);
			}
			return true;
			
		}catch(Exception e) {
			infobox.refresh(target, ResourceText.getText("errTagSaved", this)+"\n"+e.getMessage(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);
		}
		return false;
	}
	
	private boolean checkShowError(Tag tag, AjaxRequestTarget target) {
		
		boolean error=false;
		if(tag==null) return false;
		
		if(Strings.isEmpty(tag.getTagValue())){
			_tagname_info.setErrorText(ResourceText.getText("errTxtTagValueEmpty", this));
			_tagname_info.setVisible(true);
			error = true;
			
		}else {
			_tagname_info.setVisible(false);
		}

		if(Strings.isEmpty(tag.getTagName())){
			_tagname_info.setErrorText(ResourceText.getText("errTxtTagValueEmpty", this));
			_tagname_info.setVisible(true);
			error = true;
		}else {
			_tagname_info.setVisible(false);
		}

		if(!error) {
			try {
				int tempTagId = StaticLocators.getTags().getEntityId(_tag);
				if(tempTagId!=-1 && tempTagId != tag.getId()) {
					_tagname_info.setErrorText(ResourceText.getText("errTxtTagValueExist", this));
					_tagname_info.setVisible(true);
					error = true;
				}else {
					_tagname_info.setVisible(false);
				}
			}catch(Exception e) {
			}
		}
		target.add(_tagname_info);
		return error;
		
	}
	
	 
		
	private void addInfobox() {
		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		
		infobox = new InfoboxPanel("infobox");
		infobox.setVisible(false);
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
	} 
	
	private void buildLock(int id) {
		_lock = new Lock();
		_lock.setEntityId(id);
		_lock.setEntityType(EntityType.TAG);
		_lock.setUserId(_session.getCurrentUserId());
	}
	
	
}

 
