/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.template;

import org.apache.wicket.Component;
import org.apache.wicket.markup.html.WebPage;
import org.apache.wicket.markup.html.basic.Label;
import org.biffo.dms.DmsWebApplication;
import org.biffo.dms.DmsWebApplicationPages.ApplicationPage;
import org.biffo.dms.DmsWebSession;
//import org.biffo.dms.config.DmsConfigPage;
import org.biffo.dms.web.header.HeaderPanel;
import org.biffo.dms.web.menu.MenuPanel;

import de.kleinekuh.components.toolbar.Toolbar;

public class DmsTemplate extends WebPage{

 
	private static final long serialVersionUID = 1L;

	public static final String CONTENT_ID = "contentComponent";

	private Component headerPanel;
	private Component menuPanel;
	

	public DmsTemplate(){
		add(new Toolbar("toolbar"));
		add(headerPanel = new HeaderPanel("headerPanel"));
		add(menuPanel = new MenuPanel("menuPanel", ApplicationPage.HOME));
		add(new Label(CONTENT_ID, "Put your content here"));
	}
	
	
	public DmsTemplate(ApplicationPage acctPage){
		add(new Toolbar("toolbar"));
		add(headerPanel = new HeaderPanel("headerPanel"));
		add(menuPanel = new MenuPanel("menuPanel", acctPage));
		add(new Label(CONTENT_ID, "Put your content here"));
	}
	
	
	public Component getHeaderPanel() {
		return headerPanel;
	}

	public Component getMenuPanel() {
		return menuPanel;
	}
	
	@Override
	protected void onConfigure() {
		super.onConfigure();
		if(!DmsWebSession.get().isSignedIn()) DmsWebApplication.getApplication().restartResponseAtSignInPage();
	}
	
}
