/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.uploads;

import java.util.ArrayList;
import java.util.List;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.DefaultDataTable;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.components.upload.UploadPanel;
import org.biffo.dms.dataprovider.config.DPWebImporters;
import org.biffo.dms.entity.Logentries.LogEntry;
import org.biffo.dms.entity.Logentries.LogEntryFilterClass;
import org.biffo.dms.entity.Variables.Impl.ImportServerImpl;

public class UploadPanelViewList extends Panel{

	private static final long serialVersionUID = 1L;
	
	private DPWebImporters _qtdp = new DPWebImporters();
	private DefaultDataTable<?, ?> _ddt;
	private DmsWebSession _session = DmsWebSession.getSession();
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	public UploadPanelViewList(String id) {
		super(id);
		 
		List<IColumn<?,?>> columns = new ArrayList<IColumn<?,?>>();
		columns.add(new AbstractColumn<ImportServerImpl, String>(Model.of("")){

			private static final long serialVersionUID = 1L;

			@Override
			public void populateItem(Item<ICellPopulator<ImportServerImpl>> item, String componentId, IModel<ImportServerImpl> rowModel) {
				try{
					UploadPanel uploadPanel= new UploadPanel(componentId, rowModel.getObject().getId()) {

						private static final long serialVersionUID = 1L;
						
						protected void OnUploadFinish(AjaxRequestTarget target, int processId) {
							
							int uploads=0;
							int dublicates=0;
							try {
								
								LogEntryFilterClass lfc = new LogEntryFilterClass();
								lfc.setIdUser(_session.getCurrentUserId());
								lfc.setIdProcess(processId);
								List<LogEntry> lstLogEntries = StaticLocators.getLogEntries().getEntities(-1, -1, "", lfc);
								for(int i=0;i<lstLogEntries.size();i++) {
									if(lstLogEntries.get(i).getMessageCode()==63)dublicates++;
									else if(lstLogEntries.get(i).getMessageCode()==62)uploads++;
								}
								
							}catch(Exception e) {
							}
							UploadPanelViewList.this.OnUploadPanelViewListFinished(target, processId, uploads, dublicates);
						};
						
					};
					uploadPanel.setOutputMarkupId(true);
					item.add(uploadPanel);
				}catch(Exception e){ 
					item.add(new Label(componentId, ""));
				}
			}
		});
		int numberOfImporters = (int) _qtdp.size();
		if(numberOfImporters==0)numberOfImporters=1;
		_ddt = new DefaultDataTable("datatable", columns, _qtdp, numberOfImporters);
		_ddt.setOutputMarkupId(true);
		add(_ddt);
		
	}
	
	
	public int getNumberOfImporters() {
		return (int) _qtdp.size();
	}
	
	
	protected void OnUploadPanelViewListFinished(AjaxRequestTarget target, int processId, int  uploads, int dublicates){}
	
	
	

	

}
