/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.users;

import java.io.Serializable;
import java.util.Arrays;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;


import org.apache.wicket.Page;
import org.apache.wicket.PageReference;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.CheckBox;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.IChoiceRenderer;
import org.apache.wicket.markup.html.form.PasswordTextField;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.util.string.Strings;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebApplication;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.infobox.InfoboxPanel.INFOBOX_STATUS;
import org.biffo.dms.components.select2.SelectData;
import org.biffo.dms.components.select2.ISelectData;
import org.biffo.dms.entity.Helpers.EntityUUID;
import org.biffo.dms.entity.Helpers.SaltedMD5;
import org.biffo.dms.entity.Relation.RelationType;
import org.biffo.dms.entity.Variables.Impl.IVariablesImpl;
import org.biffo.dms.entity.user.Usergroups.UserGroup;
import org.biffo.dms.entity.user.Userrelation.UserRelation;
import org.biffo.dms.entity.user.Userrelation.UserRelationFilterClass;
import org.biffo.dms.entity.user.Userrelation.UserRelationType;
import org.biffo.dms.entity.user.Users.ISessionParameters;
import org.biffo.dms.entity.user.Users.User;
import org.biffo.dms.entity.user.Users.UsersAccessType;
import org.biffo.dms.helpers.DmsDBHelpersUser;
import org.biffo.dms.models.UserModel;
import org.biffo.dms.session.parameters.DmsWebSessionParameters;
import org.biffo.dms.web.helpers.ErrorLabel;
import org.biffo.dms.web.helpers.PanelTitel;
import org.biffo.dms.web.helpers.ResourceText;
import org.biffo.dms.web.usergroups.UserGroupSelectorPanel;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;
 
public class UserPanel extends Panel{

	private static final String _className="UserSelectorPanel";
	private static final long serialVersionUID = 1L;
	private static final String toolbarCSS = "dataformToolbar";
	
	public enum UserPanelType{
		full,
		change_pw
	}
	
	
	public class EntryHelper implements Serializable{

		private static final long serialVersionUID = 1L;
		
		private String _compareBenutzerEmail;
		private String _compareBenutzerPassword;
		
		public EntryHelper(User user) {
			setUser(user);
		}
		
		public void setUser(User user) {
			if(null!=user) {
				_compareBenutzerEmail = user.getBenutzerEmail();
				_compareBenutzerPassword=user.getBenutzerPassword();
			}
		}

		public String getCompareBenutzerEmail() {return _compareBenutzerEmail;}
		public void setCompareBenutzerEmail(String _compareBenutzerEmail) {this._compareBenutzerEmail = _compareBenutzerEmail;}

		public String getCompareBenutzerPassword() {return _compareBenutzerPassword;}
		public void setCompareBenutzerPassword(String _compareBenutzerPassword) {this._compareBenutzerPassword = _compareBenutzerPassword;}
		
	}
	
	private DropDownChoice<UsersAccessType> _accesstype;
	private TextField<String> _benutzername, _benutzeremail;
	private PasswordTextField _benutzerpassword, _benutzerpasswordcompare;
	private TextArea<String> _beschreibung;
	private CheckBox _gesperrt; 
	private Label _lblTitel;
	
	private ErrorLabel _benutzername_info, _benutzeremail_info, _benutzerpassword_info;
	private ErrorLabel _benutzerpasswordcompare_info, _accesstype_info;

	private UserGroupSelectorPanel _userGroups;
	
 	private Form<?> _form;
	private Toolbar _toolBar;
	private ToolbarButton _btEdit, _btCancel, _btSave, _btOK;

	private Object _origin;
	
	private PanelTitel _panelTitel = new PanelTitel();

	private UserModel _userModel;
	private EntryHelper _entryHelper=null;
	
	private int _userId;
	
	private DmsWebSession _session = DmsWebSession.getSession();
	private UserPanelType _userPanelType;
 
	private WebMarkupContainer wmcInfo;
	private InfoboxPanel infobox;
	private boolean _isStarted=false;
	
	private List<ISelectData> lstData = new LinkedList<ISelectData>();
	
	public UserPanel(String id, int userId,  Object origin) {
		this(id, userId, origin, UserPanelType.full);
	}
	
	
	public UserPanel(String id, int userId,  Object origin, UserPanelType userPanelType ) {
		super(id);
		_userPanelType = userPanelType;
		_origin = origin;
		_userId = userId;
		
		if(userId==-1){
			_userModel = new UserModel(new User());
		}else{
			_userModel = new UserModel(userId);
			
			List<IVariablesImpl> serverImpls = new LinkedList<IVariablesImpl>();
			User user = _userModel.getObject();
			if(user.getAccessType()!=UsersAccessType.ADMIN && user.getAccessType()!=UsersAccessType.USER) {
				DmsDBHelpersUser.retrieveServerProcessesFromServerUser(serverImpls, user, RelationType.FTP_USER_TO_FTP_SERVER);
				DmsDBHelpersUser.retrieveServerProcessesFromServerUser(serverImpls, user, RelationType.SFTP_USER_TO_SFTP_SERVER);
				DmsDBHelpersUser.retrieveServerProcessesFromServerUser(serverImpls, user, RelationType.SMTP_USER_TO_SMTP_SERVER);
				for(int i=0;i<serverImpls.size();i++) {
					if(DmsWebApplication.getProcessQueue().isRunning(serverImpls.get(i).getServerName())){
						_isStarted = true;
					}
				}
			}
		}
		_entryHelper = new EntryHelper(_userModel.getObject());
		RefreshDataList();
		
		_form = new Form<Void>("form");
		_form.setOutputMarkupId(true);
		add(_form);
		
        addTextFieldsUser();
        addCheckBoxen();
        
        addDropDownBoxen();
        addAdditionalControls();
        
        addToolBar();
        addInfobox();
        
        refreshFormObjects();
		setFormComponentsEditable(false);
		setOutputMarkupId(true);
		
		
		if(_userPanelType==UserPanelType.full) {
			if(_userId==-1) {
				_panelTitel.setTitel(ResourceText.getText("titNewUser", this));
				_toolBar.setViewState(ToolbarState.STATE_NEW);
				setFormComponentsEditable(true);
			}else {
				_panelTitel.setTitel(ResourceText.getText("titEditUser", this));
			}
		}else {
			_panelTitel.setTitel(ResourceText.getText("titChangePW", this));
			_toolBar.setViewState(ToolbarState.STATE_CHANGE);
			setFormComponentsEditable(true);
		}
	        
	}
	
	private void setUserAdmin() {
		if(_session.getCurrentUser().equals(_userModel.getObject().getBenutzerName())) {
			_benutzername.setEnabled(false);
		}
	}
	
	
	private void RefreshDataList(){
		try{
			List<UserGroup> availabelUserGroups = StaticLocators.getUserGroups().getEntities(0, 100, " order by name", null);
			for(UserGroup ug : availabelUserGroups) {
				boolean found=false;
				for(ISelectData data : lstData) {
					if(data.getId()==ug.getId()) {
						data.setSelected(false);
						found=true;
					}
				}
				if(!found) {
					lstData.add(new SelectData(ug.getId(), ug.getName(), false, false));
				}
			}
			if(_userId!=-1) {
				UserRelationFilterClass urfc = new UserRelationFilterClass();
				urfc.setIdrelation2(_userId);
				urfc.setUserRelationType(UserRelationType.GROUP_TO_USER);
			
				List<UserRelation> entities = StaticLocators.getUserRelations().getEntities(0, 100, "", urfc);
				if(null!=entities && entities.size()>0){
					for(UserRelation entity : entities){
						for(ISelectData data : lstData) {
							if(data.getId()==entity.getIdrelation1()) {
								data.setSelected(true);
							}
						}
					}
				}
			}
		}catch(Exception e){
			DmsException dmsException = new DmsException(1000, "E", _className, "RefreshDataList", null, e);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
		return;
	} 
	
	
	
	
	private void addTextFieldsUser(){
		
		_benutzername = new TextField<String>("benutzername");
		_benutzername.setOutputMarkupId(true);
        _form.add(_benutzername); 

        _benutzername_info = new ErrorLabel("benutzername_info");
        _form.add(_benutzername_info);
		
        _benutzerpassword= new PasswordTextField("benutzerpassword");
		_benutzerpassword.setOutputMarkupId(true);
		_benutzerpassword.setRequired(false);
		_benutzerpassword.setResetPassword(false);
        _form.add(_benutzerpassword);
        
        _benutzerpassword_info = new ErrorLabel("benutzerpassword_info");
        _form.add(_benutzerpassword_info);
        
        _benutzerpasswordcompare= new PasswordTextField("benutzerpasswordcompare");
        _benutzerpasswordcompare.setOutputMarkupId(true);
        _benutzerpasswordcompare.setRequired(false);
        _benutzerpasswordcompare.setResetPassword(false);
        _form.add(_benutzerpasswordcompare);
        
        _benutzerpasswordcompare_info = new ErrorLabel("benutzerpasswordcompare_info");
        _form.add(_benutzerpasswordcompare_info);
        
         
		_benutzeremail= new TextField<String>("benutzeremail");
		_benutzeremail.setOutputMarkupId(true);
        _form.add(_benutzeremail);
        
        _benutzeremail_info = new ErrorLabel("benutzeremail_info");
        _form.add(_benutzeremail_info);
        
		_beschreibung= new TextArea<String>("beschreibung");
		_beschreibung.setOutputMarkupId(true);
        _form.add(_beschreibung);
	}
	

	
	
	
	private void addCheckBoxen(){

		_gesperrt = new CheckBox("gesperrt");
		_gesperrt.setOutputMarkupId(true);
		_gesperrt.add(new AjaxFormComponentUpdatingBehavior("onclick") {
			private static final long serialVersionUID = 1L;

			@Override
            protected void onUpdate(AjaxRequestTarget target) {
                target.add(_gesperrt);
            }
        }); 
        _form.add(_gesperrt);
	}
 
	private void addDropDownBoxen(){

		 _accesstype = new DropDownChoice<UsersAccessType>("accesstype",  Arrays.asList(UsersAccessType.values()), new IChoiceRenderer<UsersAccessType>() {

			private static final long serialVersionUID = 1L;

			public Object getDisplayValue(UsersAccessType accessType) {
				return accessType.getDescription();
			}

			public String getIdValue(UsersAccessType accessType, int index) {
				return ""+accessType.getCode();
			}
		});
		
		
		_accesstype.add(new AjaxFormComponentUpdatingBehavior("onchange") {
 
			private static final long serialVersionUID = 1L;

			@Override
			protected void onUpdate(AjaxRequestTarget target) {
				target.add(_accesstype);				
			}
		});
		_accesstype_info = new ErrorLabel("accesstype_info");
		_form.add(_accesstype_info);

		_accesstype.setOutputMarkupId(true);
	    _form.add(_accesstype);
	     
	}
	
	private void addAdditionalControls(){
        _userGroups = new UserGroupSelectorPanel("usergroups",  lstData);
        _userGroups.setOutputMarkupId(true);
        _form.add(_userGroups);
        
		_lblTitel = new Label("paneltitel", new PropertyModel<String>(this._panelTitel, "Titel"));
		_lblTitel.setOutputMarkupId(true);
		add(_lblTitel);
	}
	
	private void closeForm(AjaxRequestTarget target){
		if(_origin instanceof Panel){
			target.add(UserPanel.this.replaceWith((Panel)_origin).getParent());
		}else if(_origin instanceof Page){
		}else if(_origin instanceof PageReference){
			PageReference pg = (PageReference)_origin;
			setResponsePage(pg.getPage());
		}
	}
	
	
	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};

		_btEdit = new ToolbarButton().setButtonType(ToolbarButtonType.EDIT).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btEdit.setAlternate(ResourceText.getText("tbAltEdit", this));
		if(_userPanelType==UserPanelType.full && !_isStarted) _toolBar.add(_btEdit);
		
		_btCancel = new ToolbarButton().setButtonType(ToolbarButtonType.CANCEL).setToolbarState(ToolbarState.STATE_VIEW | ToolbarState.STATE_NEW | ToolbarState.STATE_CHANGE).setCssClass(toolbarCSS);
		_btCancel.setAlternate(ResourceText.getText("tbAltCancel", this));
		_toolBar.add(_btCancel);
		
		_btOK = new ToolbarButton(_form).setButtonType(ToolbarButtonType.OK).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btOK.setAlternate(ResourceText.getText("tbAltOk", this));
		if(_userPanelType==UserPanelType.full) _toolBar.add(_btOK);
		
		_btSave = new ToolbarButton(_form).setButtonType(ToolbarButtonType.SAVE).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btSave.setAlternate(ResourceText.getText("tbAltSave", this));
		_toolBar.add(_btSave);
		
		
		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		
		_form.add(_toolBar);
	}
	
	private void processEvent(String event, AjaxRequestTarget target) {
		if(event.equals("Save")) save(target, true);
		else if(event.equals("Ok")) ok(target);
		else if(event.equals("Cancel")) cancel(target);
		else if(event.equals("Edit")) edit(target);
		target.add(_form);
		target.add(_toolBar);
	}
	
	private void cancel(AjaxRequestTarget target){
		 try {
			 if(_toolBar.getViewState()==ToolbarState.STATE_VIEW 
					 || _toolBar.getViewState()==ToolbarState.STATE_NEW 
					 || _userPanelType==UserPanelType.change_pw) {
				 closeForm(target);
			 }else {
				 try {
					 _userModel = new UserModel(_userId);
					 refreshFormObjects();
					 RefreshDataList();
					 _toolBar.setViewState(ToolbarState.STATE_VIEW);
					 setFormComponentsEditable(false);
					 target.add(_userGroups);
					 target.add(_form);
					 target.add(_toolBar);
				 }catch(Exception e2) {
					 infobox.refresh(target, ResourceText.getText("errChangeCancel", this)+".\n"+e2.getMessage(), INFOBOX_STATUS.error);
					 infobox.setVisible(true);
					 target.add(wmcInfo);					 
				 }
			 } 
		 }catch(Exception e) {
			infobox.refresh(target, ResourceText.getText("errChangeCancel", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);			 
		 }

	}
	
	
	private void edit(AjaxRequestTarget target){
		 try {
			 if(_userModel.getObject().getId()!=-1){
				 _toolBar.setViewState(ToolbarState.STATE_CHANGE);
				 setFormComponentsEditable(true);
			 }
	 	}catch(Exception e) {
			infobox.refresh(target, ResourceText.getText("errChangeEditMode", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);	 		
	 	}
	}	
	
	
 
	private boolean ok(AjaxRequestTarget target){
		if(save(target, false)) {
			_toolBar.setViewState(ToolbarState.STATE_VIEW);
			cancel(target);
			return true;
		}
		return false;
	} 
	
	private boolean checkShowError(User user, AjaxRequestTarget target) {
		
		boolean error=false;
		if(user==null) return false;
		
		if(Strings.isEmpty(user.getBenutzerName())){
			_benutzername_info.setErrorText(ResourceText.getText("errTextUserEmpty", this));
			_benutzername_info.setVisible(true);
			error = true;
		}else {
			_benutzername_info.setVisible(false);
		}
		if(Strings.isEmpty(user.getBenutzerEmail())){
			_benutzeremail_info.setErrorText(ResourceText.getText("errTextEmailEmpty", this));
			_benutzeremail_info.setVisible(true);
			error = true;
		}else {
			_benutzeremail_info.setVisible(false);
		}
		
		if(Strings.isEmpty(user.getBenutzerPassword())){
			_benutzerpassword_info.setErrorText(ResourceText.getText("errTextPasswordEmpty", this));
			_benutzerpassword_info.setVisible(true);
			error = true;
		}else {
			_benutzerpassword_info.setVisible(false);
		}
		
		if(Strings.isEmpty(_entryHelper.getCompareBenutzerPassword())){
			_benutzerpasswordcompare_info.setErrorText(ResourceText.getText("errTextPasswordCompareEmpty", this));
			_benutzerpasswordcompare_info.setVisible(true);
			error = true;
		}else {
			_benutzerpasswordcompare_info.setVisible(false);
		}
		
		if(!Strings.isEmpty(user.getBenutzerPassword()) && !Strings.isEmpty(_entryHelper.getCompareBenutzerPassword())){
			if(!user.getBenutzerPassword().equals(_entryHelper.getCompareBenutzerPassword())) {
				String text=ResourceText.getText("errTextPasswordCompare", this);
				_benutzerpassword_info.setErrorText(text);
				_benutzerpassword_info.setVisible(true);
				_benutzerpasswordcompare_info.setErrorText(text);
				_benutzerpasswordcompare_info.setVisible(true);
				error = true;
			}else {
				_benutzerpassword_info.setVisible(false);
				_benutzerpasswordcompare_info.setVisible(false);
			}
		}
		
		if(user.getAccessType()==null) {
			_accesstype_info.setErrorText(ResourceText.getText("errTextAccessTypeNotChoosen", this));
			_accesstype_info.setVisible(true);
			error = true;
		}else {
			_accesstype_info.setVisible(false);
		}
	
		
		
		target.add(_benutzername_info);
		target.add(_benutzeremail_info);		
		target.add(_benutzerpassword_info);
		target.add(_benutzerpasswordcompare_info);
		target.add(_accesstype_info);
		
		return error;
		
	}
	
	
 
	private boolean save(AjaxRequestTarget target, boolean withMessage) {
		try {
			
			User user = _userModel.getObject();
			if(checkShowError(user, target))return false;
			
			user.setUUID(EntityUUID.createUserUUID(user));
			user.setAenderungsDatum(new Date());
			user.setGeaendertVon(_session.getCurrentUser());
			
			String pw = user.getBenutzerPassword();
			if(!pw.startsWith("200:")) user.setBenutzerPassword(SaltedMD5.generateStorngPasswordHash(pw));
			 
			if(user.getId()==-1){
				user.setErstellungsDatum(user.getAenderungsDatum());
				user.setErstelltVon(user.getGeaendertVon());
			}
			saveUser(user);
			
			_toolBar.setViewState(ToolbarState.STATE_CHANGE);
			refreshFormObjects();
			
			if(withMessage) {
				if(_userPanelType==UserPanelType.change_pw)infobox.refresh(target, ResourceText.getText("infPasswordChanged", this), INFOBOX_STATUS.success);
				else infobox.refresh(target, ResourceText.getText("infUserSaved", this), INFOBOX_STATUS.success);
				infobox.setVisible(true);
				target.add(wmcInfo);
				target.add(_lblTitel);
			}
			return true;
		}catch(Exception e) {
			if(_userPanelType==UserPanelType.change_pw)infobox.refresh(target, ResourceText.getText("errPasswordSaved", this), INFOBOX_STATUS.error);
			else infobox.refresh(target, ResourceText.getText("errUserSaved", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);			 
		}
		return false;
		
	}
	
	
	private void saveUser(User user) throws Exception{
		
		if(user.getId()==-1){
			_userId = StaticLocators.getUsers().getEntityId(user);
			if(_userId==-1){
				_panelTitel.setTitel(ResourceText.getText("titEditUser", this));
				user.setGesperrt(false);
				ISessionParameters xi = new DmsWebSessionParameters(-1);
				user.setSessionParameters(xi);
				StaticLocators.getUsers().addEntity(user);
				DmsException dmsInfo = new DmsException(2000, "I", 
						_className,
						"saveUser",
						ResourceText.getText("dmsInfoUserCreated", this, user.getBenutzerName()),
						null,
						user
						);
				StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
				_userId = user.getId();
			}else{
				throw new Exception(ResourceText.getText("errUserExists", this));
			}
			_userModel = new UserModel(_userId);
			_entryHelper.setUser(_userModel.getObject());
		}else{
			User userTemp = StaticLocators.getUsers().getEntityById(user.getId());
			if(user.getSessionParameters()==null || !(user.getSessionParameters() instanceof DmsWebSessionParameters)) {
				user.setSessionParameters(new DmsWebSessionParameters(user.getId()));
			}
			if(userTemp.isDifferent(user)) {
				StaticLocators.getUsers().updateEntity(user, true);
				DmsException dmsInfo = new DmsException(2000, "I", 
						_className,
						"saveUser",
						ResourceText.getText("dmsInfoUserChanged", this, user.getBenutzerName()),
						null,
						user
						);
				StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
				_userModel = new UserModel(_userId);
				_entryHelper.setUser(_userModel.getObject());
			}
		}
	}
	
	
	
	
	
	
	@SuppressWarnings("unused")
	private void saveUserGroups(User user, List<ISelectData> selectedUserGroups) throws Exception{

		if(_userId==-1) return;
			
		UserRelationFilterClass urfc = new UserRelationFilterClass();
		urfc.setIdrelation2(_userId);
		
		long numberOfEntities = StaticLocators.getUserRelations().getNumberOfEntities(urfc);
		if(numberOfEntities>0) {
			List<UserRelation> entities = StaticLocators.getUserRelations().getEntities(0, numberOfEntities, "", urfc);
			for(int i=0;i<entities.size();i++) {
				UserRelation entity = entities.get(i);
				boolean delete = true;
				for(ISelectData selectedUserGroup : selectedUserGroups){
					if(selectedUserGroup.getId()==entity.getIdrelation1()) {
						delete = false;
						break;
					}
				}
				if(delete) {
					StaticLocators.getUserRelations().deleteEntityById(entity.getId());
					UserGroup ug = StaticLocators.getUserGroups().getEntityById(entity.getIdrelation1());
					DmsException dmsInfo = new DmsException(2000, "I", 
							_className,
							"saveUserGroups",
							ResourceText.getText("dmsInfoUsergroupRemoved", this, ug.getName(), user.getBenutzerName()),
							null,
							entity
							);
					StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
				}
			}
		}
		

		if(null!=selectedUserGroups && selectedUserGroups.size()>0){
			for(ISelectData entity : selectedUserGroups){
				UserRelation userRelation = new UserRelation();
				userRelation.setErstelltVon(_session.getCurrentUser());
				userRelation.setErstellungsDatum(new Date());
				userRelation.setGeaendertVon(userRelation.getErstelltVon());
				userRelation.setAenderungsDatum(userRelation.getErstellungsDatum());
				userRelation.setIdrelation1(entity.getId());
				userRelation.setIdrelation2(user.getId());
				userRelation.setUserRelationType(UserRelationType.GROUP_TO_USER);
				int idTemp = StaticLocators.getUserRelations().getEntityId(userRelation);
				if(idTemp==-1) {
					StaticLocators.getUserRelations().addEntity(userRelation);
					UserGroup ug = StaticLocators.getUserGroups().getEntityById(entity.getId());
					DmsException dmsInfo = new DmsException(2000, "I", 
							_className,
							"saveUserGroups",
							ResourceText.getText("dmsInfoUsergroupAdded", this, ug.getName(), user.getBenutzerName()),
							null,
							entity
							);
					StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
					
					
				}
			}
		}
	} 
	
	
	
	private void refreshFormObjects(){
	    
	    _benutzername.setDefaultModel(new PropertyModel<String>(_userModel, "BenutzerName"));
		_benutzerpassword.setDefaultModel(new PropertyModel<String>(_userModel, "BenutzerPassword"));
        _benutzerpasswordcompare.setDefaultModel(new PropertyModel<String>(_entryHelper, "CompareBenutzerPassword"));
        _benutzeremail.setDefaultModel(new PropertyModel<String>(_userModel, "BenutzerEmail"));
		_beschreibung.setDefaultModel(new PropertyModel<String>(_userModel, "Beschreibung"));
		_gesperrt.setDefaultModel(new PropertyModel<Boolean>(_userModel, "Gesperrt"));
		_accesstype.setDefaultModel(new PropertyModel<UsersAccessType>(_userModel, "AccessType"));
	}
	
	private void setFormComponentsEditable(boolean editable){
		try{
			
			if(_userPanelType==UserPanelType.full) {
				_benutzername.setEnabled(editable);
				_benutzeremail.setEnabled(editable);
				_benutzerpassword.setEnabled(editable);
				_benutzerpasswordcompare.setEnabled(editable);
				_beschreibung.setEnabled(editable);
				_gesperrt.setEnabled(editable);
				_accesstype.setEnabled(editable);
				_userGroups.setDisabled(true);
				setUserAdmin();
			}else {
				_benutzeremail.setEnabled(editable);
				_benutzerpassword.setEnabled(editable);
				_benutzerpasswordcompare.setEnabled(editable);
				_benutzername.setEnabled(false);
				_beschreibung.setEnabled(false);
				_gesperrt.setEnabled(false);
				_accesstype.setEnabled(false);
				_userGroups.setDisabled(true);
			}
				
		}catch(Exception e){
		}
	}
	
 
 
	private void addInfobox() {
		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		
		if(_isStarted) {
			infobox = new InfoboxPanel("infobox", ResourceText.getText("warLinkedServerStarted", this), INFOBOX_STATUS.warning);
			infobox.setVisible(true);
		}else {
			infobox = new InfoboxPanel("infobox");
			infobox.setVisible(false);
		}
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
	} 
	
	
	
}
