/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.users;

import java.util.LinkedList;
import java.util.List;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.json.JSONArray;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.panel.Panel;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.components.select2.ISelectData;
import org.biffo.dms.components.select2.Select2;

 
public class UserSelectorPanel extends Panel{

	private static final String _className="UserSelectorPanel";
	private static final long serialVersionUID = 1L;
	
	private Form<?> _form;
	private Select2 _usersSelector;
	private List<ISelectData> lstData = new LinkedList<ISelectData>();
	
	
	public void setDisabled(boolean disabled) {
		_usersSelector.getSettings().setDisabled(disabled);
	}
	
	public UserSelectorPanel(String id) {
		this(id, null);
	}
	
	public UserSelectorPanel(String id, final List<ISelectData> lstData) {
		super(id);
		if(null!=lstData)this.lstData = lstData;
		_form = new Form<Void>("form") {
			private static final long serialVersionUID = 1L;
			@Override
			protected void onSubmit() {
				super.onSubmit();
			}
		};
		_form.setOutputMarkupId(true);
		add(_form);
		
		_usersSelector = new Select2("usersselector") {
			 
			private static final long serialVersionUID = 1L;
			@Override
			protected void OnChange(AjaxRequestTarget target, String event, String value) {
				super.OnChange(target, event, value);
				try {
					List<Integer> selectedIds = new LinkedList<Integer>();
					JSONArray savedContent = new JSONArray(value);
					if(null!=savedContent && savedContent.length()>0) {
						for(int i=0;i<savedContent.length();i++) {
							String obj =(String) savedContent.getJSONObject(i).get("id");
							selectedIds.add(Integer.valueOf(obj));
						}
					}
					for(int i=0;i<lstData.size();i++) {
						if(!selectedIds.contains(lstData.get(i).getId())) {
							lstData.get(i).setSelected(false);
						}else {
							lstData.get(i).setSelected(true);
						}
					}
				}catch(Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "OnChange", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, DmsWebSession.getSession());
				}
			}
		};
		_usersSelector.setOutputMarkupId(true);
		_usersSelector.getSettings().setMultiple(true);
		_usersSelector.getSettings().setData(lstData);
		_form.add(_usersSelector);
		
	}
	
	
	public List<ISelectData> getList(){
		return this.lstData;
	}
	
	
	public List<ISelectData> getSelectionList(){
		
		List<ISelectData> lstRC = new LinkedList<>();
		for(int i=0;i<lstData.size();i++) {
			if(lstData.get(i).getSelected())lstRC.add(lstData.get(i));
		}
		return lstRC;
	}
	
	public List<ISelectData> getDataSelection(boolean onlySelected){
		if(!onlySelected) return lstData;
		List<ISelectData> rc = new LinkedList<ISelectData>();
		for(ISelectData entity : lstData) {
			if(entity.getSelected()) rc.add(entity);
		}
		return rc;
	}


}
