/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.users;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;


import org.apache.wicket.AttributeModifier;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.DefaultDataTable;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.filter.FilterForm;
import org.apache.wicket.extensions.model.AbstractCheckBoxModel;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.components.custommodal.CustomModalWindow;
import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.infobox.InfoboxPanel.INFOBOX_STATUS;
import org.biffo.dms.components.pagging.DmsPagingNavigator;
import org.biffo.dms.dataprovider.users.DPUsers;
import org.biffo.dms.entity.Relation.RelationType;
import org.biffo.dms.entity.Variables.Impl.IVariablesImpl;
import org.biffo.dms.entity.user.Userrelation.UserRelation;
import org.biffo.dms.entity.user.Userrelation.UserRelationFilterClass;
import org.biffo.dms.entity.user.Userrelation.UserRelationType;
import org.biffo.dms.entity.user.Users.User;
import org.biffo.dms.entity.user.Users.UsersAccessType;
import org.biffo.dms.helpers.DmsDBHelpersUser;
import org.biffo.dms.session.parameters.SessionParameterTypes;
import org.biffo.dms.session.parameters.StandardListParameters;
import org.biffo.dms.web.administration.AdministrationPage;
import org.biffo.dms.web.administration.AdministrationTabedPanel.TabContent;
import org.biffo.dms.web.helpers.ResourceText;
import org.biffo.dms.web.htmlhelpers.CheckBoxColumn;
import org.biffo.dms.web.htmlhelpers.LinkColumn;
import org.biffo.dms.web.modals.AbstractModalDefaultPanel;
import org.biffo.dms.web.modals.AbstractModalDefaultPanel.ModalType;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;
 

 
public class UsersPanelViewList extends Panel{

	private static final String _className = "UsersPanelViewList";
	private static final long serialVersionUID = 1L;
	private static final String toolbarCSS = "viewListToolbar";
	
	
	private static final SimpleDateFormat _sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
	private final static SimpleDateFormat _sdf2 = new SimpleDateFormat("dd.MM.yyyy HH:mm");
	
	private DPUsers _qtdp = new DPUsers();
	private DefaultDataTable<?, ?> _ddt;
	private Set<User> _selectionList = new HashSet<User>();
	private Toolbar _toolBar;
	private ToolbarButton _btAll, _btNone, _btRefresh, _btAdd, _btDelete, _btLock, _btUnlock;
	private FilterForm<?> _form;
	
	private DmsWebSession _session = DmsWebSession.getSession();
	private StandardListParameters _thisParameters;
	

	private WebMarkupContainer wmcInfo;
	private InfoboxPanel infobox;
	private DmsPagingNavigator _dpn;
	
	private CustomModalWindow modalDefault = new CustomModalWindow("modalDefault");
	private AbstractModalDefaultPanel _amdp;
	
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public UsersPanelViewList(String id) {
		super(id);
		
		initParameters();
		setOutputMarkupId(true);
		addInfobox();
		addModalDefault();
		_form = new FilterForm("dataform", _qtdp);
		_form.setOutputMarkupId(true);
		add(_form);
		
		
		List<IColumn<?,?>> columns = new ArrayList<IColumn<?,?>>();
		columns.add(new CheckBoxColumn<User>(Model.of("")) {

			private static final long serialVersionUID = 1L;

			@Override
			protected IModel<Boolean> newCheckBoxModel(final IModel<User> rowModel) {
				return new AbstractCheckBoxModel() {

					private static final long serialVersionUID = 1L;
					@Override
					public void unselect() {
						_selectionList.remove(rowModel.getObject());
					}
					@Override
					public void select() {
						_selectionList.add(rowModel.getObject());
					}
					@Override
					public boolean isSelected() {
						return _selectionList.contains(rowModel.getObject());
					}
					@Override
					public void detach() {
						rowModel.detach();
					}
				};
			}
		});
		
		columns.add(new LinkColumn(ResourceText.getModelText("colUserName", this), "BenutzerName", "BenutzerName") {
			 
			private static final long serialVersionUID = 1L;
			
			
 
			public void onClick(Item item, String componentId, IModel model) {
				User fn = (User)model.getObject();
				UserPanel panel = new  UserPanel(UsersPanelViewList.this.getId(), fn.getId(),  UsersPanelViewList.this);
				UsersPanelViewList.this.replaceWith(panel);
			}
		});
		
		columns.add(new PropertyColumn<User, String>(null, "AccessType.Description"));
		columns.add(new AbstractColumn<User, String>(ResourceText.getModelText("colUserLastAccessDate", this), "LastAccessDate"){
			 
			private static final long serialVersionUID = 1L;

			@Override
			public void populateItem(Item<ICellPopulator<User>> item, String componentId, IModel<User> rowModel) {
				String temp = "NA";
				try{
					if(null!=rowModel.getObject().getLastAccessDate()){
						temp = _sdf.format(rowModel.getObject().getLastAccessDate());
					}
				}catch(Exception e){}
				item.add(new Label(componentId, temp));
			}
		});
		
		columns.add(new AbstractColumn<User, String>(new Model<String>("")){
			 
			private static final long serialVersionUID = 1L;

			@Override
			public void populateItem(Item<ICellPopulator<User>> item, String componentId, IModel<User> rowModel) {
				if(rowModel.getObject().getAccessType()==UsersAccessType.ADMIN 
						|| rowModel.getObject().getAccessType()==UsersAccessType.USER) {
					buildUserLocked(item,componentId, rowModel.getObject().isGesperrt());
				}else {
					item.add(new Label(componentId, ""));
				}
			}
			
		});
		
		columns.add(new AbstractColumn<User, String>(ResourceText.getModelText("colUserBeschreibung", this), "Beschreibung"){
			 
			private static final long serialVersionUID = 1L;

			@Override
			public void populateItem(Item<ICellPopulator<User>> item, String componentId, IModel<User> rowModel) {
				String temp = "NA";
				try{
					temp = rowModel.getObject().getBeschreibung();
					if(null!=temp){
						if(temp.length()>40) temp = temp.substring(0, 40);
					}else{
						temp="NA";
					}
				}catch(Exception e){}
				item.add(new Label(componentId, temp));
			}
			
		});
		
		
		
		
		columns.add(new AbstractColumn<User, String>(ResourceText.getModelText("colUserAenderungsDatum", this), "AenderungsDatum"){
			private static final long serialVersionUID = 1L;
			@Override
			public void populateItem(Item<ICellPopulator<User>> item, String componentId, IModel<User> rowModel){
				String label = "NA";
				try{
					label = _sdf2.format(rowModel.getObject().getAenderungsDatum());
				}catch(Exception e){}
				Label lbl = new Label(componentId, label);
				item.add(lbl);
			}
		});
		
		columns.add(new AbstractColumn<User, String>(Model.of("Verbunden mit")){
			private static final long serialVersionUID = 1L;
			@Override
			public void populateItem(Item<ICellPopulator<User>> item, String componentId, IModel<User> rowModel){
				String label = "NA";
				try{
					
					User user = rowModel.getObject();
					List<IVariablesImpl> serverImpls = new LinkedList<IVariablesImpl>();
					if(user.getAccessType()!=UsersAccessType.ADMIN && user.getAccessType()!=UsersAccessType.USER) {
						DmsDBHelpersUser.retrieveServerProcessesFromServerUser(serverImpls, user, RelationType.FTP_USER_TO_FTP_SERVER);
						DmsDBHelpersUser.retrieveServerProcessesFromServerUser(serverImpls, user, RelationType.SFTP_USER_TO_SFTP_SERVER);
						DmsDBHelpersUser.retrieveServerProcessesFromServerUser(serverImpls, user, RelationType.SMTP_USER_TO_SMTP_SERVER);
					}
					if(serverImpls.size()>0) {
						StringBuilder sb = new StringBuilder();
						for(int i=0;i<serverImpls.size();i++) {
							if(i>0)sb.append(", ");
							sb.append(serverImpls.get(i).getServerName());
						}
						label = sb.toString();
					}
				}catch(Exception e){}
				Label lbl = new Label(componentId, label);
				item.add(lbl);
			}
		});
		
		
		
		
		_ddt = new DefaultDataTable("datatable", columns, _qtdp, 10){

			private static final long serialVersionUID = 1L;

			@Override
			protected void onPageChanged() {
				super.onPageChanged();
				_thisParameters.setCurrentPageNumber(_ddt.getCurrentPage());
				_thisParameters.setSortParam(_qtdp.getSort());
			}
		};
		_ddt.setOutputMarkupId(true);
		_ddt.setCurrentPage(_thisParameters.getCurrentPageNumber());
		addToolBar(); 
		_form.add(_ddt);
		
		_dpn = new DmsPagingNavigator("navigation", _ddt, false, false, false, false);
		_dpn.setOutputMarkupId(true);
		add(_dpn);
		
	}
	
	
	 private void lockEntities(AjaxRequestTarget target, boolean lock){
		try{
			Date changeDate = new Date();
			for(User entity : _selectionList) {
				
				
				if(entity.getId()!=_session.getCurrentUserId()) {

					if(lock) {
						entity.setGesperrt(true);
					}else {
						entity.setGesperrt(false);
					}
					entity.setAenderungsDatum(changeDate);
					entity.setGeaendertVon(_session.getCurrentUser());
					StaticLocators.getUsers().updateEntity(entity, true);
					
					DmsException dmsInfo=null;
					if(entity.isGesperrt()) {
						dmsInfo = new DmsException(2000, "I", 
							_className,
							"lockEntities",
							ResourceText.getText("dmsInfoUserLocked", this, entity.getBenutzerName()),
							null,
							entity
						);
					}else {
						dmsInfo = new DmsException(2000, "I", 
							_className,
							"lockEntities",
							ResourceText.getText("dmsInfoUserUnlocked", this, entity.getBenutzerName()),
							null,
							entity
						);
					}
					StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
				}
			}
			_selectionList = new HashSet<User>();
			if(lock)infobox.refresh(target, ResourceText.getText("infUserLocked", this), INFOBOX_STATUS.success);
			else infobox.refresh(target, ResourceText.getText("infUserUnlocked", this), INFOBOX_STATUS.success);
			infobox.setVisible(true);
			target.add(wmcInfo);
		}catch(Exception e){
			if(lock)infobox.refresh(target, ResourceText.getText("errUserLocked", this)+"\n"+e.getMessage(), INFOBOX_STATUS.error);
			else infobox.refresh(target, ResourceText.getText("errUserUnlocked", this)+"\n"+e.getMessage(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);
		} 
	}
	 
	 

	
	 
	
	
	
	
	
	private void deleteEntities(AjaxRequestTarget target){
		try{
			
			List<User> entitiesToDelete = new LinkedList<User>();
			for(User entityToDelete : _selectionList) {
				if(entityToDelete.getId()!=_session.getCurrentUserId()) {
					if(entityToDelete.getAccessType()!=UsersAccessType.ADMIN 
							&& entityToDelete.getAccessType()!=UsersAccessType.USER) {
						if(!DmsDBHelpersUser.isUserLinkedToServer(entityToDelete)){
							entitiesToDelete.add(entityToDelete);
						}
					}else {
						entitiesToDelete.add(entityToDelete);
					}
				}
			}
			
			for(User entityToDelete : entitiesToDelete) {
				DmsDBHelpersUser.deleteUser(entityToDelete, _session.getCurrentUser(), _session.getCurrentTenantId());
				DmsException dmsInfo = new DmsException(2000, "I", 
						_className,
						"deleteEntities",
						ResourceText.getText("dmsInfoUserDeleted", this, entityToDelete.getBenutzerName()),
						null,
						entityToDelete.getBenutzerName()
						);
				StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
			}

			if(entitiesToDelete.size()==_selectionList.size()) {
				infobox.refresh(target, ResourceText.getText("infUserDeleted", this), INFOBOX_STATUS.success);	
			}else {
				infobox.refresh(target, ResourceText.getText("warnUserDeleted", this), INFOBOX_STATUS.warning);
			}
			_selectionList = new HashSet<User>();
		}catch(Exception e){
			infobox.refresh(target, ResourceText.getText("errUserDeleted", this)+"\n"+e.toString(), INFOBOX_STATUS.error);
		} 
		infobox.setVisible(true);
		target.add(wmcInfo);
	}
	
	
	private void addToolBar() {
		
		
		
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};
		
		_btAll= new ToolbarButton().setButtonType(ToolbarButtonType.ALL).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btAll.setAlternate(ResourceText.getText("tbAltAll", this));
		_toolBar.add(_btAll);
				
		_btNone=new ToolbarButton().setButtonType(ToolbarButtonType.NONE).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btNone.setAlternate(ResourceText.getText("tbAltNone", this));
		_toolBar.add(_btNone);
		
		_btAdd=new ToolbarButton().setButtonType(ToolbarButtonType.ADD).setToolbarState(ToolbarState.STATE_VIEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btAdd.setAlternate(ResourceText.getText("tbAltAdd", this));
		_toolBar.add(_btAdd);
		
		_btDelete=new ToolbarButton().setButtonType(ToolbarButtonType.DELETE).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btDelete.setAlternate(ResourceText.getText("tbAltDelete", this));
		_toolBar.add(_btDelete);
		
		_btRefresh=new ToolbarButton().setButtonType(ToolbarButtonType.REFRESH).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btRefresh.setAlternate(ResourceText.getText("tbAltRefresh", this));
		_toolBar.add(_btRefresh);

		_btLock = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
		_btLock.setUtf8Code("xe897");
		_btLock.setAlternate(ResourceText.getText("tbAltLock", this));
		_btLock.setEventName("lock");
		_btLock.setCssClass(toolbarCSS);
		_toolBar.add(_btLock);
		
		_btUnlock = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
		_btUnlock.setUtf8Code("xe898");
		_btUnlock.setEventName("unlock");
		_btUnlock.setAlternate(ResourceText.getText("tbAltUnlock", this));
		_btUnlock.setCssClass(toolbarCSS);
		_toolBar.add(_btUnlock);

		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		add(_toolBar);
		
	}

	
	public Set<User> getSelection(){
		return _selectionList;
	}
	
	private void fselectAll(boolean select){
		long itemsPerPage = _ddt.getItemsPerPage();
		if(_qtdp.size()<itemsPerPage)itemsPerPage=_qtdp.size();
		long currentPage = _ddt.getCurrentPage();
		Iterator<?> iter =_ddt.getDataProvider().iterator(itemsPerPage*currentPage, itemsPerPage);
		while(iter.hasNext()) {
			User ql =(User)iter.next();
			if(ql.isChangeable()==null || ql.isChangeable()) {
				if(select){
					_selectionList.add(ql);
				}else{
					_selectionList.remove(ql);
				}
			}
		}
	}
	
	private void processEvent(String event, AjaxRequestTarget target) {
		if(event.equals("All")) {
			fselectAll(true);
			target.add(_ddt);
		}else if(event.equals("None")) {
			fselectAll(false);
			target.add(_ddt);
		}else if(event.equals("Delete")) {
			if(_selectionList.size()>0)showModal(target, ModalType.DELETE); 
		}else if(event.equals("Refresh")) {
			_qtdp.size();
			target.add(_dpn);
			target.add(_ddt);
		}else if(event.equals("Add")) {
			AdministrationPage ap = new AdministrationPage(getPage().getPageReference(), TabContent.USERS, true);
			setResponsePage(ap);
		}else if(event.equals("unlock")) {
			if(_selectionList.size()>0)showModal(target, ModalType.UNLOCK);
		}else if(event.equals("lock")) {
			if(_selectionList.size()>0)showModal(target, ModalType.LOCK);
		}
	}
	
	
	@SuppressWarnings("unused")
	private String getUserGroups(User user) {
		try {
			
			StringBuilder sb = new StringBuilder();
			UserRelationFilterClass urfc = new UserRelationFilterClass();
			urfc.setIdrelation2(user.getId());
			urfc.setUserRelationType(UserRelationType.GROUP_TO_USER);
			List<UserRelation> entities = StaticLocators.getUserRelations().getEntities(0, 100, "", urfc);
			if(entities!=null && !entities.isEmpty()) {
				int x=0;
				for(int i=0;i<entities.size();i++) {
					try {
						if(x>0)sb.append(",");
						sb.append(StaticLocators.getUserGroups().getEntityById(entities.get(i).getIdrelation1()).getName());
						x++;
					}catch(Exception er) {}
				}
				return sb.toString();
			}
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", _className, "getUserGroups", null, e, user);
			StaticLocators.getMessageHandler().createMessage(dmsException, DmsWebSession.getSession());
		}
		return "";
	}
	
	private void buildUserLocked(Item<ICellPopulator<User>> item, String componentId, boolean islocked) {
		
		Label lbl = null;
		if(islocked) {
			lbl=new Label(componentId, "<span class=\"statusType\" title=\""+ResourceText.getText("lblTitleUserLocked", this)+"\">&#xe897;</span>");
		}else {
			lbl=new Label(componentId, "<span class=\"statusType\" title=\""+ResourceText.getText("lblTitleUserNotLocked", this)+"\">&#xe898;</span>");
		}
		lbl.setEscapeModelStrings(false);
		item.add(lbl);
		item.add(new AttributeModifier("style", "text-align:center;font-size:20px;border:0px;width:10px"));
	}
	
	private void addInfobox() {
		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		
		infobox = new InfoboxPanel("infobox");
		infobox.setVisible(false);
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
	} 
	
	private void initParameters() {
		_thisParameters = (StandardListParameters) _session.getSessionParameters().getParametersByType(SessionParameterTypes.USERS);
		if(_thisParameters.getSortParam()!=null) _qtdp.setSort(_thisParameters.getSortParam());
	}
	
	
	private void showModal(AjaxRequestTarget target, ModalType modalType) {

		if(modalType==ModalType.DELETE) {
			_amdp.setModalType(ModalType.DELETE);
			_amdp.setModalText(ResourceText.getText("titDialogDelete", this));
			modalDefault.show(target);
		}else if(modalType==ModalType.LOCK) {
			_amdp.setModalType(ModalType.LOCK);
			_amdp.setModalText(ResourceText.getText("titDialogLock", this));
			modalDefault.show(target);
		}else if(modalType==ModalType.UNLOCK) {
			_amdp.setModalType(ModalType.UNLOCK);
			_amdp.setModalText(ResourceText.getText("titDialogUnlock", this));
			modalDefault.show(target);
		}
		
	}
	
	
	private void addModalDefault() {

		modalDefault.setAutoSize(false);
		modalDefault.setHeightUnit("px");
		modalDefault.setInitialHeight(80);
		modalDefault.setInitialWidth(400);
		modalDefault.setResizable(false);
		modalDefault.showUnloadConfirmation(false);
    	
    	_amdp = new AbstractModalDefaultPanel(modalDefault) {
 
			private static final long serialVersionUID = 1L;

			@Override
			public void onOk(AjaxRequestTarget target) {
				if(_amdp.getModalType()==ModalType.DELETE) {
					deleteEntities(target);	
				}else if(_amdp.getModalType()==ModalType.LOCK) {
					lockEntities(target, true);
				}else if(_amdp.getModalType()==ModalType.UNLOCK) {
					lockEntities(target, false);
				}
				modalDefault.close(target);
				_ddt.size();
				target.add(_dpn);
				target.add(_ddt);
			}
			
			@Override
			public void onCancel(AjaxRequestTarget target) {
				modalDefault.close(target);
				fselectAll(false);
				target.add(_dpn);
				target.add(_ddt);
			}
		};
		modalDefault.setContent(_amdp);
    	add(modalDefault);
    	
    }
	
	 
	
	
	
	
}
