/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.variables;



import java.util.Arrays;
import java.util.Date;

import org.apache.wicket.Page;
import org.apache.wicket.PageReference;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.IChoiceRenderer;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.util.string.Strings;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.infobox.InfoboxPanel.INFOBOX_STATUS;
import org.biffo.dms.entity.Helpers.EntityUUID;
import org.biffo.dms.entity.Variables.Variable;
import org.biffo.dms.entity.Variables.VariableGroupType;
import org.biffo.dms.entity.Variables.VariableType;
import org.biffo.dms.models.VariableModel;
import org.biffo.dms.web.helpers.ErrorLabel;
import org.biffo.dms.web.helpers.PanelTitel;
import org.biffo.dms.web.helpers.ResourceText;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;
 
public class VariablePanel extends Panel{

	private static final String _className="VariablePanel";
	private static final long serialVersionUID = 1L;
	private static final String toolbarCSS = "dataformToolbar";
	
	private TextField<String> _variablename, _variablegroup, _variablevalue;
	private TextArea<String> _beschreibung;
	private DropDownChoice<VariableType> _variabletype;
	private DropDownChoice<VariableGroupType> _variablegrouptype;
	private Label _lblTitel;
	
	private ErrorLabel _variablename_info, _variablegroup_info, _variablevalue_info, _variabletype_info, _variablegrouptype_info;
	
 	private Form<?> _form;
	private Toolbar _toolBar;
	private ToolbarButton _btEdit,  _btCancel, _btSave, _btOK;

	private Object _origin;
	
	private PanelTitel _panelTitel = new PanelTitel();
	
	private int _variableId;
	private VariableModel _variableModel;
	
	private WebMarkupContainer wmcInfo;
	private InfoboxPanel infobox;
	
	
	private DmsWebSession _session = DmsWebSession.getSession();
	
	public VariablePanel(String id, int variableId,  Object origin) {
		super(id);

		_origin = origin;
		_variableId = variableId;
		
		if(-1==_variableId){
			_variableModel = new VariableModel(new Variable());
		}else{
			_variableModel = new VariableModel(_variableId);
		}
		
		_form = new Form<Void>("dataform");
		_form.setOutputMarkupId(true);
		add(_form);
 
		
        addTextFields();
        addCheckBoxen();
        
        addDropDownBoxen();
        addAdditionalControls();

        addToolBar();
        addInfobox();
        
        refreshFormObjects();
		setFormComponentsEditable(false);
		setOutputMarkupId(true);
		
		if(-1==_variableId){
			_panelTitel.setTitel(ResourceText.getText("titNewVariable", this));
			_toolBar.setViewState(ToolbarState.STATE_NEW);
			setFormComponentsEditable(true);
		}else {
			_panelTitel.setTitel(ResourceText.getText("titEditVariable", this));
		}
        
	}
	
	
	private void addTextFields(){

		_variablegroup = new TextField<String>("variablegroup");
		_variablegroup.setOutputMarkupId(true);
        _form.add(_variablegroup);
        
        _variablegroup_info = new ErrorLabel("variablegroup_info");
        _variablegroup_info.setOutputMarkupId(true);
        _form.add(_variablegroup_info);

        _variablename = new TextField<String>("variablename");
        _variablename.setOutputMarkupId(true);
        _form.add(_variablename);

        _variablename_info = new ErrorLabel("variablename_info");
        _variablename_info.setOutputMarkupId(true);
        _form.add(_variablename_info);
        
        _variablevalue= new TextField<String>("variablevalue");
        _variablevalue.setOutputMarkupId(true);
        _form.add(_variablevalue);
        
        _variablevalue_info = new ErrorLabel("variablevalue_info");
        _variablevalue_info.setOutputMarkupId(true);
        _form.add(_variablevalue_info);
        
        _beschreibung= new TextArea<String>("beschreibung");
        _beschreibung.setOutputMarkupId(true);
        _form.add(_beschreibung);
	    
	}
	
	
	
	
	private void addCheckBoxen(){}
	
	private void addDropDownBoxen(){
		
		_variabletype = new DropDownChoice<VariableType>("variabletype",  Arrays.asList(VariableType.values()), new IChoiceRenderer<VariableType>() {

			private static final long serialVersionUID = 1L;

			public Object getDisplayValue(VariableType type) {
				return type.getDescription();
			}

			public String getIdValue(VariableType type, int index) {
				return ""+type.getCode();
			}
		});
		
		_variabletype.add(new AjaxFormComponentUpdatingBehavior("onchange") {
 			private static final long serialVersionUID = 1L;
			@Override
			protected void onUpdate(AjaxRequestTarget target) {
				target.add(_variabletype);				
			}
		});
		
		_variabletype.setOutputMarkupId(true);
	    _form.add(_variabletype);
	    
	    _variabletype_info = new ErrorLabel("variabletype_info");
	    _variabletype_info.setOutputMarkupId(true);
        _form.add(_variabletype_info);

        
	    _variablegrouptype = new DropDownChoice<VariableGroupType>("variablegrouptype",  Arrays.asList(VariableGroupType.values()), new IChoiceRenderer<VariableGroupType>() {

			private static final long serialVersionUID = 1L;

			public Object getDisplayValue(VariableGroupType type) {
				return type.getDescription();
			}

			public String getIdValue(VariableGroupType type, int index) {
				return ""+type.getCode();
			}
		});
		
	    _variablegrouptype.add(new AjaxFormComponentUpdatingBehavior("onchange") {
 			private static final long serialVersionUID = 1L;
			@Override
			protected void onUpdate(AjaxRequestTarget target) {
				target.add(_variablegrouptype);				
			}
		});
	    _variablegrouptype.setOutputMarkupId(true);
	    _form.add(_variablegrouptype);
	    
	    _variablegrouptype_info = new ErrorLabel("variablegrouptype_info");
	    _variablegrouptype_info.setOutputMarkupId(true);
        _form.add(_variablegrouptype_info);
	    
	}
	
	private void addAdditionalControls(){
		_lblTitel = new Label("paneltitel", new PropertyModel<String>(this._panelTitel, "Titel"));
		_lblTitel.setOutputMarkupId(true);
		add(_lblTitel);
	}
	
	private void closeForm(AjaxRequestTarget target){
		if(_origin instanceof Panel){
			target.add(VariablePanel.this.replaceWith((Panel)_origin).getParent());
		}else if(_origin instanceof Page){
		}else if(_origin instanceof PageReference){
			PageReference pg = (PageReference)_origin;
			setResponsePage(pg.getPage());
		}
	}
	
	
	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};

		_btEdit = new ToolbarButton().setButtonType(ToolbarButtonType.EDIT).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btEdit.setAlternate(ResourceText.getText("tbAltEdit", this));
		_toolBar.add(_btEdit);
		
		_btCancel = new ToolbarButton().setButtonType(ToolbarButtonType.CANCEL).setToolbarState(ToolbarState.STATE_VIEW | ToolbarState.STATE_NEW | ToolbarState.STATE_CHANGE).setCssClass(toolbarCSS);
		_btCancel.setAlternate(ResourceText.getText("tbAltCancel", this));
		_toolBar.add(_btCancel);
		
		_btOK = new ToolbarButton(_form).setButtonType(ToolbarButtonType.OK).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btOK.setAlternate(ResourceText.getText("tbAltOk", this));
		_toolBar.add(_btOK);
		
		_btSave = new ToolbarButton(_form).setButtonType(ToolbarButtonType.SAVE).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btSave.setAlternate(ResourceText.getText("tbAltSave", this));
		_toolBar.add(_btSave);
		
		
		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		
		_form.add(_toolBar);
	}
	
	private void processEvent(String event, AjaxRequestTarget target) {
		if(event.equals("Save")) save(target, true);
		else if(event.equals("Ok")) ok(target);
		else if(event.equals("Cancel")) cancel(target);
		else if(event.equals("Edit")) edit(target);

		target.add(_form);
		target.add(_toolBar);
	}
	
	private void cancel(AjaxRequestTarget target){
		 try {
			 if(_toolBar.getViewState()==ToolbarState.STATE_VIEW || _toolBar.getViewState()==ToolbarState.STATE_NEW ) {
				 closeForm(target);
			 }else {
				 try {
					 _variableModel = new VariableModel(_variableId);
					 refreshFormObjects();
					 _toolBar.setViewState(ToolbarState.STATE_VIEW);
					 setFormComponentsEditable(false);
					 target.add(_form);
					 target.add(_toolBar);
				 }catch(Exception e2) {
					 infobox.refresh(target, ResourceText.getText("errChangeCancel", this)+".\n"+e2.getMessage(), INFOBOX_STATUS.error);
					 infobox.setVisible(true);
					 target.add(wmcInfo);
				 }
			 } 
		 }catch(Exception e) {
			 infobox.refresh(target, ResourceText.getText("errChangeCancel", this)+".\n"+e.getMessage(), INFOBOX_STATUS.error);
			 infobox.setVisible(true);
			 target.add(wmcInfo);
		 }
	}
	
	
	private void edit(AjaxRequestTarget target){
		 try {
			 
			 if(_variableModel.getObject().getId()!=-1){
				 _toolBar.setViewState(ToolbarState.STATE_CHANGE);
				 setFormComponentsEditable(true);
			 }
	 	}catch(Exception e) {
			infobox.refresh(target, ResourceText.getText("errChangeEditMode", this)+".\n"+e.getMessage(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);
	 	}
	}	
	
	
 
	private boolean ok(AjaxRequestTarget target){
		if(save(target, false)) {;
			_toolBar.setViewState(ToolbarState.STATE_VIEW);
			cancel(target);
			return true;
		}
		return false;
	}
	
	private boolean checkShowError(Variable variable, AjaxRequestTarget target) {
		
		boolean error=false;
		if(variable==null) return false;
		
		if(Strings.isEmpty(variable.getVariableGroup())){
			_variablegroup_info.setErrorText(ResourceText.getText("errTxtVariableGroupEmpty", this));
			_variablegroup_info.setVisible(true);
			error = true;
		}else {
			_variablegroup_info.setVisible(false);
		}
		
		
		if(Strings.isEmpty(variable.getVariableName())){
			_variablename_info.setErrorText(ResourceText.getText("errTxtVariableNameEmpty", this));
			_variablename_info.setVisible(true);
			error = true;
		}else {
			_variablename_info.setVisible(false);
		}
		
		if(Strings.isEmpty(variable.getVariableValue())){
			_variablevalue_info.setErrorText(ResourceText.getText("errTxtVariableValueEmpty", this));
			_variablevalue_info.setVisible(true);
			error = true;
		}else {
			_variablevalue_info.setVisible(false);
		}
		
		if(variable.getVariableType()==null) {
			_variabletype_info.setErrorText(ResourceText.getText("errTxtVariableTypeEmpty", this));
			_variabletype_info.setVisible(true);
			error = true;
		}else {
			_variabletype_info.setVisible(false);
		}

		if(variable.getVariableGroupType()==null) {
			_variablegrouptype_info.setErrorText(ResourceText.getText("errTxtVariableSelectEmpty", this));
			_variablegrouptype_info.setVisible(true);
			error = true;
		}else {
			_variablegrouptype_info.setVisible(false);
		}

 
		return error;
		
	}
	
 
	private boolean save(AjaxRequestTarget target, boolean withMessage) {
		try {
			
			Variable variable = _variableModel.getObject();
			if(checkShowError(variable, target))return false;
			
			if(variable.getVariableType()==null)variable.setVariableType(VariableType.STRING);
			
			variable.setIdTenant(_session.getCurrentTenantId());
			variable.setUUID(EntityUUID.createVariableUUID(variable));
			variable.setGeaendertAm(new Date());
			variable.setGeaendertVon(_session.getCurrentUser());
			
			if(variable.getId()==-1) {
				variable.setErstelltAm(variable.getGeaendertAm());
				variable.setErstelltVon(variable.getGeaendertVon());
			}
			saveVariable(variable);
			
			if(_toolBar.getViewState()==ToolbarState.STATE_NEW)_toolBar.setViewState(ToolbarState.STATE_CHANGE);
			refreshFormObjects();
			if(withMessage) {
				infobox.refresh(target, ResourceText.getText("infVariableSaved", this), INFOBOX_STATUS.success);
				infobox.setVisible(true);
				target.add(wmcInfo);
				target.add(_lblTitel);
			}
			return true;
		}catch(Exception e) {
			infobox.refresh(target, ResourceText.getText("errVariableSaved", this) +"\n"+e.getMessage(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);
		}
		return false;
	}
	
	private void saveVariable(Variable variable) throws Exception{
		
		if(variable.getId()==-1){
			_variableId = StaticLocators.getVariables().getEntityId(variable);
			if(_variableId==-1){
				_panelTitel.setTitel(ResourceText.getText("titEditVariable", this));
				StaticLocators.getVariables().addEntity(variable);
				
				DmsException dmsInfo = new DmsException(2000, "I", 
						_className,
						"saveVariable",
						ResourceText.getText("dmsInfoVariableCreated", this, variable.getVariableName()+"/"+variable.getVariableGroup())
						);
				StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
				_variableId = variable.getId();
			}else{
				throw new Exception(ResourceText.getText("errVariableExists", this));
			}
			_variableModel = new VariableModel(_variableId);
		}else{
			Variable variableTemp = StaticLocators.getVariables().getEntityById(variable.getId());
			if(variableTemp.isDifferent(variable)) {
				StaticLocators.getVariables().updateEntity(variable, true);
				DmsException dmsInfo = new DmsException(2000, "I", 
						_className,
						"saveVariable",
						ResourceText.getText("dmsInfoVariableChanged", this, variable.getVariableName()+"/"+variable.getVariableGroup())
						);
				StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
				_variableModel = new VariableModel(_variableId);
			}
		}
	}
	
	private void refreshFormObjects(){
		_variablename.setDefaultModel(new PropertyModel<String>(_variableModel, "VariableName"));
		_variablevalue.setDefaultModel(new PropertyModel<String>(_variableModel, "VariableValue"));
		_variablegroup.setDefaultModel(new PropertyModel<String>(_variableModel, "VariableGroup"));
		_beschreibung.setDefaultModel(new PropertyModel<String>(_variableModel, "Beschreibung"));
		
		_variablegrouptype.setDefaultModel(new PropertyModel<VariableGroupType>(_variableModel, "VariableGroupType"));
		_variabletype.setDefaultModel(new PropertyModel<VariableType>(_variableModel, "VariableType"));
		
	}
	
	private void setFormComponentsEditable(boolean editable){
		_variablename.setEnabled(editable); 
		_variablegroup.setEnabled(editable);
		_beschreibung.setEnabled(editable);
		_variablevalue.setEnabled(editable);
		
		_variablegrouptype.setEnabled(editable);
		_variabletype.setEnabled(editable);
	}
	
	
 
	private void addInfobox() {
		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		
		infobox = new InfoboxPanel("infobox");
		infobox.setVisible(false);
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
	} 

	
	
	
}
